/* 
 * File:   app_config.h
 * Author: chrism
 *
 * Created on 26 October 2017, 12:04
 */

#ifndef APP_CONFIG_H
#define	APP_CONFIG_H

#ifdef	__cplusplus
extern "C" {
#endif
         
#include "system/ports/sys_ports.h"

//------------------------------------------------------------------------------
// Application launch address
//------------------------------------------------------------------------------    
    
//! The address of the main program in memory (see the main apps linker file _RESET_ADDR)    
#define APP_LAUNCH_ADDRESS          0x9D020000    
    
//------------------------------------------------------------------------------
// Software Version / Model 
//------------------------------------------------------------------------------    

#define SOFTWARE_VERSION_MAJOR      0
#define SOFTWARE_VERSION_MINOR      1
    
//------------------------------------------------------------------------------
// Software Version SA20/SA10
//------------------------------------------------------------------------------    

#define SOFTWARE \
    X(SA30, SA30\0)

// Now use the input data to define the enum
#define X(a, b) SOFTWARE_MODEL_##a,
	typedef enum {SOFTWARE} SOFTWARE_MODEL_ENUM;
#undef X
    
// Redefine the macro and use to create an array of strings
#define X(a, b) #b,
	static char *SOFTWARE_VERSION_STRINGS[] = {SOFTWARE};
#undef X
    
SOFTWARE_MODEL_ENUM SOFTWARE_MODEL;
char* SOFTWARE_MODEL_STRING;   

//! Used for checking data loded from eeproms (see settings/upgrade_metadata)
#define SOFTWARE_MODEL_STRING_LENGTH    5

//------------------------------------------------------------------------------
// Application Commands
// Commands used by the application.  These are used to map to RS232 commands and IR
// These are used by the STRUCTURED MESSAGE command    
//------------------------------------------------------------------------------    

//--------------------------------------------------------------------------
// Application Commands - Enum of available Input Mode change types
//--------------------------------------------------------------------------

/**
 * Enum of available commands
 */
typedef enum
{
    APP_COMMANDS_SOFTWARE_VERSION           = 0x04,
    APP_COMMANDS_AMX_DUET                   = 0xC0,
            
    APP_COMMANDS_UPGRADE_LOAD_START         = 0xB9,
    APP_COMMANDS_UPGRADE_LOAD_DATA          = 0xBA,
    APP_COMMANDS_UPGRADE_LOAD_END           = 0xBB,
    APP_COMMANDS_ERASE_FLASH                = 0xBC,

    //! Command used for logger messages - outgoing messages only
//    APP_COMMANDS_WATCHDOG_STATUS            = 0xF2,
    APP_COMMANDS_LOGGER_MESSAGE             = 0xF3,   
    APP_COMMANDS_SET_LOGGER_LEVEL           = 0xF4,   
} APP_COMMANDS;

//--------------------------------------------------------------------------
// Application Modes
//--------------------------------------------------------------------------

typedef enum
{
    //! Default running mode - generally displays the Splash screen 
    APP_CONTEXT_MODE_DEFAULT,

    //! The user has taken an action that displays the menu
    APP_CONTEXT_MODE_MENU,

    //! The user is editing a specific menu option
    APP_CONTEXT_MODE_MENU_EDIT,

    //! A process is being invoked  as a response to selecting a menu item
    APP_CONTEXT_MODE_PROCESS,

    //! The app is in a standby state
    APP_CONTEXT_MODE_STANDBY,

    //! The app is performing a BIAS test
    APP_CONTEXT_MODE_BIAS_TEST,
} APP_CONTEXT_MODE;
    
//--------------------------------------------------------------------------
// Application Strings
//--------------------------------------------------------------------------

#define STRINGS \
    X(OFF, Off)\
    X(ON, On)\

// Now use the input data to define the enum
#define X(a, b) APP_STRINGS_ENUM_##a,
	typedef enum {STRINGS} APP_STRINGS_ENUM;
#undef X
    
// Redefine the macro and use to create an array of strings
#define X(a, b) #b,
	static char *APP_STRINGS[] = {STRINGS};
#undef X

//------------------------------------------------------------------------------
// Flash Memory
//------------------------------------------------------------------------------  

#define FLASH_MEMORY_MAX_CLIENTS            2
    
//! The device name to be supplied to the SYS_FS_Mount function
//  This should match the device name supplied in the Harmony configuration    
#define FLASH_MEMORY_DEVICE_NAME            "/dev/mtda1"    

//! The mount name to be suppled to the SYS_FS_Mount function
//  This should match the mount name supplied in the Harmony configuration    
#define FLASH_MEMORY_MOUNT_NAME             "/mnt/Drive1"
    
//! The number of requests that can be queued at any point
#define FLASH_MEMORY_REQUEST_QUEUE_SIZE     5
    
//! Define client ids for the modules using the flash memory
//  These should be ascending from 0    
#define FLASH_MEMORY_CLIENT_SETTINGS        0    
#define FLASH_MEMORY_CLIENT_UPGRADE         1  

//! The maximum number of files that can be associated with a single client
#define FLASH_MEMORY_MAX_CLIENT_FILES       3
    
//------------------------------------------------------------------------------
// Logger
//------------------------------------------------------------------------------    

/**
 * Define logger enabled to enable logging, comment out if logging not required
 */
#define LOGGER_ENABLED 1
    
/**
 * Logger message codes
 */
typedef enum
{
    LOGGER_CODE_DEBUG,
    LOGGER_CODE_SET_LOGGER_LEVEL,
    LOGGER_CODE_INITIALISATION,
            
    LOGGER_CODE_FLASH_INITIALISATION,
    LOGGER_CODE_FLASH_REQUEST_OVERRUN,
            
    LOGGER_CODE_USART_OPEN_ERROR,
    LOGGER_CODE_USART_READ_ERROR,
    LOGGER_CODE_USART_WRITE_ERROR,

    LOGGER_CODE_UPGRADE,
            
    LOGGER_CODE_TEST_RGB_OFF = 0xDE,
    LOGGER_CODE_TEST_RGB_RED = 0xDF,
    LOGGER_CODE_TEST_RGB_GREEN = 0xE0,
    LOGGER_CODE_TEST_RGB_BLUE = 0xE1,
    LOGGER_CODE_TEST_RGB_ORANGE = 0xE2,
    LOGGER_CODE_TEST_RGB_PURPLE = 0xE3,
    LOGGER_CODE_TEST_RGB_WHITE = 0xE4,
            
} LOGGER_CODES;
  
//------------------------------------------------------------------------------
// RGB LED
//------------------------------------------------------------------------------  

// LED Ports all defined as Out/Low/Digital 
// Note settings are for AMP    
#define RGB_LED_BLUE_PORTS_ID               PORTS_ID_0
#define RGB_LED_BLUE_PORT_CHANNEL           PORT_CHANNEL_D
#define RGB_LED_BLUE_PIN                    PORTS_BIT_POS_4
#define RGB_LED_RED_PORTS_ID                PORTS_ID_0
#define RGB_LED_RED_PORT_CHANNEL            PORT_CHANNEL_D 
#define RGB_LED_RED_PIN                     PORTS_BIT_POS_0 
#define RGB_LED_GREEN_PORTS_ID              PORTS_ID_0
#define RGB_LED_GREEN_PORT_CHANNEL          PORT_CHANNEL_D
#define RGB_LED_GREEN_PIN                   PORTS_BIT_POS_2

//------------------------------------------------------------------------------
// Timers
//------------------------------------------------------------------------------    

/** Timer reset type
 */
typedef enum
{
    TIMERS_RESET_NONE = 0,              
    TIMERS_RESET_INACTIVE = 1,
    TIMERS_RESET_IMMEDIATE = 2,
 } TIMERS_RESET;

 /**
  * Types of timer duration 
  */
 typedef enum
 {
    TIMERS_HANDLE_1MS,
    TIMERS_HANDLE_10US,
 } TIMERS_HANDLE;
 
/**
 * Timer X code 1: Name, 2: Timeout, 3: Reset type, 4: Base duration
 */
#define TIMER_ITEMS \
    X(TEST, 5000, TIMERS_RESET_IMMEDIATE, TIMERS_HANDLE_1MS)\
    X(RGB_LED, 500, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(RGB_PWM, 1, TIMERS_RESET_IMMEDIATE, TIMERS_HANDLE_10US)\
    X(UPGRADE_LAUNCH_APP, 1000, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\

//// Now use the input data to define the enum
//#define X(a, b, c, d) APP_TIMER_INDEX_##a,
//	typedef enum {TIMER_ITEMS APP_TIMER_INDEX_MAX} APP_TIMER_INDEX;
//#undef X
//
//// Now use the input data to define the enum
//#define X(a, b, c, d) APP_TIMER_TIMEOUT_MS_##a,
//	typedef enum {TIMER_ITEMS} APP_TIMER_TIMEOUT;
//#undef X
//   
//// Now use the input data to define the enum
//#define X(a, b, c, d) b,
//    static int APP_TIMER_TIMEOUT_VALUES[] = { TIMER_ITEMS };
//#undef X
//   
//// Now use the input data to define the enum
//#define X(a, b, c, d) c,
//    static int APP_TIMER_TIMEOUT_MODES[] = { TIMER_ITEMS };
//#undef X
//   
//// Now use the input data to define the enum
//#define X(a, b, c, d) d,
//    static int APP_TIMER_TIMEOUT_HANDLES[] = { TIMER_ITEMS };
//#undef X
//
///**
// * Defines the number of timers to be managed
// */
//#define TIMERS_NUM_TIMERS APP_TIMER_INDEX_MAX
    
//------------------------------------------------------------------------------
// Upgrade
//------------------------------------------------------------------------------    

//! Required to allow upgrade functionality to work
#define UPGRADE_METADATA_INCLUDE_UPGRADE_FUNCTIONALITY
    
#define USE_PAGE_ERASE          1
#define USE_QUAD_WORD_WRITE     1

    //! Calculated from Linker file / memory map
#define PROGRAM_FLASH_END_ADDRESS (APP_LAUNCH_ADDRESS + 0x200000 - (0x20000 + 0x480) - 1) // (0x9D000000+BMXPFMSZ-1)

/* APP_FLASH_BASE_ADDRESS and APP_FLASH_END_ADDRESS reserves program Flash for the application*/ 
/* Rule: 
 		1)The memory regions kseg0_program_mem, kseg0_boot_mem, exception_mem and 
 		kseg1_boot_mem of the application linker script must fall with in APP_FLASH_BASE_ADDRESS 
 		and APP_FLASH_END_ADDRESS
 		 
 		2)The base address and end address must align on  4K address boundary */
 		
#define APP_FLASH_BASE_ADDRESS 	APP_LAUNCH_ADDRESS 
#define APP_FLASH_END_ADDRESS   PROGRAM_FLASH_END_ADDRESS    
    
//! A unique identifier for the module
#define UPGRADE_METADATA_CLIENT_ID              FLASH_MEMORY_CLIENT_UPGRADE   

//! Name associated with storage
#define UPGRADE_STORAGE_NAME                    "UPGRADE_INFO"
    
//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------    

#ifdef	__cplusplus
}
#endif

#endif	/* APP_CONFIG_H */

