/* 
 * File:   app_context.h
 * Author: chrism
 *
 * Created on 26 October 2017, 11:45
 */

#ifndef APP_CONTEXT_H
#define	APP_CONTEXT_H

#include "utilities/structured_message.h"

#ifdef	__cplusplus
extern "C" {
#endif

//------------------------------------------------------------------------------
// Constants
//-------------------------------------------------------------------------------

    typedef enum
    {
        APP_STATE_INIT = 0,
        APP_STATE_CHECK_FOR_UPGRADE,
        APP_STATE_LOAD_UPGRADE,
        APP_STATE_ROLLBACK_UPGRADE,
        APP_STATE_JUMP_TO_APP,
        APP_STATE_RESTART,
    } APP_STATE;

//------------------------------------------------------------------------------
// Interface
//-------------------------------------------------------------------------------

    /**
     * Returns the current application state
     * @return 
     */
    APP_STATE app_context_get_app_state();
    
    /**
     * Gets the current mode
     */
    APP_CONTEXT_MODE app_context_get_mode();
    
    /**
     * Initialise the application context
     */
    void app_context_initialise();
    
    /**
     * Used to identify whether the code is entering the running state for the first time
     * Note: Can only be called once as this will set the state to NOT_LOADING
     * @return 
     */
    bool app_context_loading();

    /**
     * Sets the current application state
     * @param state - The application state to change to 
     */
    APP_STATE app_context_set_app_state(APP_STATE state);

    /**
     * Sets the current mode
     */
     void app_context_set_mode(APP_CONTEXT_MODE mode);
     
#ifdef	__cplusplus
}
#endif

#endif	/* APP_CONTEXT_H */

