/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    app_initialise.h

  @Summary
    Brief description of the file.

  @Description
    Contains application level initialisation functions called in app.c
 */
/* ************************************************************************** */

#ifndef _APP_INITIALISE_H    /* Guard against multiple inclusion */
#define _APP_INITIALISE_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/logger.h"
#include "utilities/rs232.h"
#include "utilities/settings.h"
#include "utilities/structured_message.h"
#include "utilities/timers.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************
    
    /**
     * Initialise flash memory and allocate space for clients
     */
    void initialise_flash_memory();
    
    /**
     * Initialise logger functions
     * @param message
     */
    void initialise_logger(LOGGER_CALLBACK callback);

    /**
     * Setup RGB LED 
     */
    void initialise_rgb_led();
    
    /**
     * Initialise USART
     * @param callback
     */
    void initialise_rs232(STRUCTURED_MESSAGE_CALLBACK callback);

    /**
     * Initialise settings module and define individual settings
     */
    void initialise_settings(SETTINGS_INITIALISE_CALLBACK callback);
    
    /**
     * This method is where default application settings should be set.  This is called 
     * the first time the eeprom is accessed for the settings in  order to set any 
     * default values so that they are stored.
     */
    void initialise_settings_defaults();

    /**
     * Initialise timers
     */
    void initialise_timers();

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
