/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    app_commands.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "app_commands.h"
#include "app_commands_menu.h"
#include "app_config.h"
#include "utilities/app_commands_utilities.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/**
 * Gets the software version
 */
static void app_commands_software_version(STRUCTURED_MESSAGE* message)
{
    if (APP_COMMAND_VALUE_REQUEST_STATUS == (int16_t)message->data[0])
    {
        message->data_length = 2;
        message->data[0] = SOFTWARE_VERSION_MAJOR;
        message->data[1] = SOFTWARE_VERSION_MINOR;
        message->response = STATUS_UPDATE;
    }
    else
    {
        message->response = COMMAND_NOT_RECOGNISED;
    }

    app_commands_send_message(message);
}

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**
 * Process a structured message
 * This command will map the messages command and data to a function
 */
void app_commands_process_message(STRUCTURED_MESSAGE* message)
{
    //TODOapp_context_pre_process_message(message);
    
    logger_suppress_command_logging(true);
    
    // Execute the command
    //?  Change to a function mapping???/?
    switch(message->command)
    {
        case APP_COMMANDS_SOFTWARE_VERSION:
        {
            app_commands_software_version(message);
            break;
        }

        case APP_COMMANDS_UPGRADE_LOAD_START:
        {
            app_commands_upgrade_upload_start(message);
            break;
        }

        case APP_COMMANDS_UPGRADE_LOAD_DATA:
        {
            //logger_log_debugf(0, "app_commands_upgrade_upload_data --------------------------------------------------------------");
            app_commands_upgrade_upload_data(message);
            break;
        }

        case APP_COMMANDS_UPGRADE_LOAD_END:
        {
            app_commands_upgrade_upload_end(message);
            break;
        }

        case APP_COMMANDS_ERASE_FLASH:
        {
            flash_memory_erase();
            break;
        }
        
        default:
        {
            message->response = COMMAND_NOT_RECOGNISED;
            
            app_commands_send_message(message);

            break;
        }
    }
    
    logger_suppress_command_logging(false);
}

/* *****************************************************************************
 End of File
 */
