/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "app_context.h"
#include "app_config.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

typedef struct
{
    //! Sets whether then application is being loaded for the first time
    bool loading;

    // The current state active state of the app - decides whether were in a menu or not
    APP_CONTEXT_MODE mode;
    
    //! Main application state machine state
    APP_STATE state;
} APP_CONTEXT_DATA;

static APP_CONTEXT_DATA app_context_data;

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Returns the current application state
 * @return 
 */
APP_STATE app_context_get_app_state()
{
    return app_context_data.state;
}
    
/**-----------------------------------------------------------------------------
 * Gets the current mode - modes are used during the running state to identify 
 * what type of processing is underway (normal, tests, processes)
 */
APP_CONTEXT_MODE app_context_get_mode()
{
    return app_context_data.mode;
}    

/**-----------------------------------------------------------------------------
 * Initialise the application context
 */
void app_context_initialise()
{
    app_context_data.loading = true;
    app_context_set_app_state(APP_STATE_INIT);
}
    
/**-----------------------------------------------------------------------------
 * Used to identify whether the code is entering the running state for the first time
 * Note: Can only be called once as this will set the state to NOT_LOADING
 * @return 
 */
bool app_context_loading()
{
    bool result = app_context_data.loading;
    app_context_data.loading = true;
    return result;
}

/**-----------------------------------------------------------------------------
 * Sets the current application state
 * @param state - The application state to change to 
 */
APP_STATE app_context_set_app_state(APP_STATE state)
{
    app_context_data.state = state;
}
     
/**-----------------------------------------------------------------------------
 * Sets the current mode
 */
 void app_context_set_mode(APP_CONTEXT_MODE mode)
 {
     if (app_context_data.mode != mode)
     {
        app_context_data.mode = mode;
        
        logger_log_infof(0, "APP: Entering mode %d", mode);
     }
 }   
 
/* *****************************************************************************
 End of File
 */
