/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "app_initialise.h"
#include "app_config.h"
#include "system_definitions.h"
#include "utilities/flash_memory.h"
//#include "utilities/power.h"
#include "utilities/rgb_led.h"
#include "utilities/settings.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */
    
/**-----------------------------------------------------------------------------
 * Initialise flash memory and allocate space for clients
 */
void initialise_flash_memory()
{
    flash_memory_initialise();
}

/**-----------------------------------------------------------------------------
 * Sets up the logger 
 */
void initialise_logger(LOGGER_CALLBACK callback)
{
    logger_initialise();
    logger_set_callback(callback);
}

/**-----------------------------------------------------------------------------
 * Setup RGB LED 
 */
void initialise_rgb_led()
{
    rgb_led_initialise();

    // Set the power LED
    rgb_led_set(RGB_LED_COLOUR_RED);    
}

/**-----------------------------------------------------------------------------
 * Initialise rs232 functions
 */
void initialise_rs232(STRUCTURED_MESSAGE_CALLBACK callback)
{
    logger_log_info(LOGGER_CODE_INITIALISATION, "Initialising RS232");
    
    // Initialise RS232 port
    rs232_initialise(DRV_USART_INDEX_0, callback);
}

/**-----------------------------------------------------------------------------
 * Initialise timers
 */
void initialise_timers()
{
    logger_log_info(LOGGER_CODE_INITIALISATION, "Initialising Timers");
    
    // Initialise the timers module
    // Look in timer1msCallback for the timers_process() method
    timers_initialise();
    
    // Add timer modules
    // note DRV_TMR_INDEX_N - Timer Instance N
    // We have allocated Timer 0 to the System Timers Service MEANING that we need
    // 3 timer instances setting up in Harmony
    short timer_1ms_handle = timers_add_timer(DRV_TMR_INDEX_1, 1000);
    short timer_10us_handle = timers_add_timer(DRV_TMR_INDEX_2, 100000);
    
    int x;
    for (x = 0; x < TIMER_INDEX_MAX; x++)
    {
        switch(TIMER_TIMEOUT_HANDLES[x])
        {
            case TIMERS_HANDLE_10US:
            {
                timers_set_timeout(timer_10us_handle, x, TIMER_TIMEOUT_VALUES[x], TIMER_TIMEOUT_MODES[x]);
                break;
            }
            
            default:
            {
                timers_set_timeout(timer_1ms_handle, x, TIMER_TIMEOUT_VALUES[x], TIMER_TIMEOUT_MODES[x]);
                break;
            }
        }
    }

    //TODOtimers_add_expired_event(TIMER_INDEX_RGB_PWM, rgb_led_process_interrupt);
    timers_reset(TIMER_INDEX_RGB_PWM);      // Start the PWM timer now the interrupt has been set
}

/* *****************************************************************************
 End of File
 */
