/*******************************************************************************
  MPLAB Harmony Project Main Source File

  Company:
    Microchip Technology Inc.
  
  File Name:
    main.c

  Summary:
    This file contains the "main" function for an MPLAB Harmony project.

  Description:
    This file contains the "main" function for an MPLAB Harmony project.  The
    "main" function calls the "SYS_Initialize" function to initialize the state 
    machines of all MPLAB Harmony modules in the system and it calls the 
    "SYS_Tasks" function from within a system-wide "super" loop to maintain 
    their correct operation. These two functions are implemented in 
    configuration-specific files (usually "system_init.c" and "system_tasks.c")
    in a configuration-specific folder under the "src/system_config" folder 
    within this project's top-level folder.  An MPLAB Harmony project may have
    more than one configuration, each contained within it's own folder under
    the "system_config" folder.
 *******************************************************************************/

// DOM-IGNORE-BEGIN
/*******************************************************************************
Copyright (c) 2013-2014 released Microchip Technology Inc.  All rights reserved.

//Microchip licenses to you the right to use, modify, copy and distribute
Software only when embedded on a Microchip microcontroller or digital signal
controller that is integrated into your product or third party product
(pursuant to the sublicense terms in the accompanying license agreement).

You should refer to the license agreement accompanying this Software for
additional information regarding your rights and obligations.

SOFTWARE AND DOCUMENTATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF ANY KIND,
EITHER EXPRESS OR IMPLIED, INCLUDING WITHOUT LIMITATION, ANY WARRANTY OF
MERCHANTABILITY, TITLE, NON-INFRINGEMENT AND FITNESS FOR A PARTICULAR PURPOSE.
IN NO EVENT SHALL MICROCHIP OR ITS LICENSORS BE LIABLE OR OBLIGATED UNDER
CONTRACT, NEGLIGENCE, STRICT LIABILITY, CONTRIBUTION, BREACH OF WARRANTY, OR
OTHER LEGAL EQUITABLE THEORY ANY DIRECT OR INDIRECT DAMAGES OR EXPENSES
INCLUDING BUT NOT LIMITED TO ANY INCIDENTAL, SPECIAL, INDIRECT, PUNITIVE OR
CONSEQUENTIAL DAMAGES, LOST PROFITS OR LOST DATA, COST OF PROCUREMENT OF
SUBSTITUTE GOODS, TECHNOLOGY, SERVICES, OR ANY CLAIMS BY THIRD PARTIES
(INCLUDING BUT NOT LIMITED TO ANY DEFENSE THEREOF), OR OTHER SIMILAR COSTS.
 *******************************************************************************/
// DOM-IGNORE-END


// *****************************************************************************
// *****************************************************************************
// Section: Included Files
// *****************************************************************************
// *****************************************************************************

#include <stddef.h>                     // Defines NULL
#include <stdbool.h>                    // Defines true
#include <stdlib.h>                     // Defines EXIT_FAILURE
#include "system/common/sys_module.h"   // SYS function prototypes

/**
 * Application specific configuration bits
 * The configuration bits are normally defined in system_init.c.  To enable the 
 * FileSystemBootloader to be used with a number of different applications each 
 * with their own set of configuration bits the declaration of those has been moved 
 * to the top of main.c
 * 
 * Each application should do the following:
 * - Add a new configuration with the same name as the new application e.g. "SA30"
 * - Create a directory under include/config which has the same name e.g. include/config/SA30
 * - Add a header file "configuration_bits.h" in the new directory, and put the 
 *   #pragma directives defining the bits into it (these can be copied from the 
 *   main applications system_init.c
 * - Comment out the Configuration Bits in system_config/system)init.c
 * - Create a Logical File structure in the project matching the physical structure
 *   and add the "configuration_bits.h" file.
 * - Ensure that the configuration_bits.h is excluded from all other configurations
 * - With the new configuration active load properties and define a new pre-processor
 *   macro "APP_CONFIG_app_name" e.g. APP_CONFIG_SA30 (see Conf:[app] -> XC32 ->
 *   xc32-gcc -> Preprocessor and Macros)
 * - In main.c add a check for the new macro, and if it exists #include the new
 *   configuration_bits.h file.
 * - In the applications properties Conf:[app] -> Loading section, add the 
 *   FileSystemBootloader as a loadable project and ensure the new configuration
 *   is selected.
 */
#ifdef APP_CONFIG_SA30
#include "config/SA30/configuration_bits.h"
#endif

#ifdef APP_CONFIG_SA750
#include "config/SA750/configuration_bits.h"
#endif

#ifdef APP_CONFIG_ST60
#include "config/ST60/configuration_bits.h"
#endif

#ifdef APP_CONFIG_ST60_DISPLAY
#include "config/ST60.Display/configuration_bits.h"
#endif


// *****************************************************************************
// *****************************************************************************
// Section: Main Entry Point
// *****************************************************************************
// *****************************************************************************

int main ( void )
{
    /* Initialize all MPLAB Harmony modules, including application(s). */
    SYS_Initialize ( NULL );


    while ( true )
    {
        /* Maintain state machines of all polled MPLAB Harmony modules. */
        SYS_Tasks ( );

    }

    /* Execution should not come here during normal operation */

    return ( EXIT_FAILURE );
}


/*******************************************************************************
 End of File
*/

