/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _ALBUM_ART_CACHE_H    /* Guard against multiple inclusion */
#define _ALBUM_ART_CACHE_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include <stdint.h>
#include "utilities/tft_control.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

#define ALBUM_ART_MAX_URL_LENGTH    32

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Adds image data to the cache for a given url
     * @param url
     * @param data
     * @param len
     * @param offset
     */
    void album_art_cache_add_image_data(char* url, uint8_t* data, uint16_t len, uint16_t offset);
    
    /**
     * 
     * @param url
     * @param ctrl
     */
    void album_art_cache_allocate_image(char* url, TFT_CONTROL* tft_control);
    
    /**
     * 
     * @param url
     */
    bool album_art_cache_draw_image(char* url);
    
    /**
     * Gets a reference to a pointer to the image data, and the length of the data
     * @param url - a pointer to a string containing the URL
     * @param data
     * @param len
     */
//    void album_art_cache_get_image_data(char* url, uint8_t* data, uint16_t* len);
    
    /**
     * Called when the last packet of image data is added to the current image
     * @param url
     */
    void album_art_cache_finalise_image_data(char* url);
    
    /**
     * Returns true if the cache has album art available for the current url
     * @param url
     * @return 
     */
    bool album_art_cache_has_url(char* url);
    
    /**
     * Tests whether a url is empty
     * @param url
     * @return 
     */
    bool album_art_cache_is_empty_url(char* url);
    
    /**
     * Compares 2 urls 
     * @param url1
     * @param url2
     * @return 
     */
    bool album_art_cache_urls_are_equal(char* url1, char* url2);
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
