/* 
 * File:   buttons.h
 * Author: chrism
 *
 * Created on 30 October 2017, 13:22
 */

#ifndef BUTTONS_H
#define	BUTTONS_H

#include "app_config.h"

#ifdef	__cplusplus
extern "C" {
#endif

    /**
     * General button state enum
     */
    typedef enum
    {
        BUTTON_STATE_ON         = 0x00,
        BUTTON_STATE_OFF        = 0x01,
        BUTTON_STATE_UNDEFINED  = 0xF0,
    } BUTTON_STATE;

    /**
     * Defines a new button
     */
    void buttons_add_button(uint8_t index, uint8_t port, uint8_t channel, uint8_t pin, uint8_t max_count);
    
    /**
     * Returns true is the selected button/switch has been moved
     * @param index
     * @return 
     */
    bool buttons_changed(uint8_t index);
    
    /**
     * Gets the state of the supplied button
     */
    BUTTON_STATE buttons_get_state(uint8_t index);
    
    /**
     * Returns true is the selected button has been pushed in the current processing cycle
     * @param index
     * @return 
     */
    bool buttons_get_pushed(uint8_t index);
    
    /**
     * Returns true is the selected button is currently signalling as being down
     * @param index
     * @return 
     */
    bool buttons_get_raw_value(uint8_t index);
    
    /**
     * Returns true is the selected button has been released in the current processing cycle
     * @param index
     * @return 
     */
    bool buttons_get_released(uint8_t index);
    
    /**
     * Initialises the buttons module
     */
    void buttons_initialise();
    
    /**
     * Processes buttons
     */
    void buttons_process();
    
    /**
     * Sets the sets of all buttons
     */
    void buttons_reset();
    
#ifdef	__cplusplus
}
#endif

#endif	/* BUTTONS_H */

