/* ************************************************************************** */
/** Descriptive File Name

  @Company
    ARCAM

  @File Name
    display_constants.h

  @Summary
    Brief description of the file.

  @Description
 * The Display module presents a common interface to the rest of the application 
 * encapsulating display specific functionality.
 * 
 * The constants in the file can be used by the supporting modules as well as the 
 * main module (oled_vfd_emulation, vfd)
 */
/* ************************************************************************** */

#ifndef _DISPLAY_CONSTANTS_H    /* Guard against multiple inclusion */
#define _DISPLAY_CONSTANTS_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

/* This section lists the other files that are included in this file.
 */

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    /**
     * Constants for use with Special character functions
     */
    #define DISPLAY_SPECIAL_CHARACTER_DEFS \
        X(SPACE, =' ')\
        X(A, ='A')\
        X(B, ='B')\
        X(C, ='C')\
        X(D, ='D')\
        X(E, ='E')\
        X(F, ='F')\
        X(G, ='G')\
        X(H, ='H')\
        X(I, ='I')\
        X(J, ='J')\
        X(K, ='K')\
        X(L, ='L')\
        X(M, ='M')\
        X(N, ='N')\
        X(O, ='O')\
        X(P, ='P')\
        X(Q, ='Q')\
        X(R, ='R')\
        X(S, ='S')\
        X(T, ='T')\
        X(U, ='U')\
        X(V, ='V')\
        X(W, ='W')\
        X(X, ='X')\
        X(Y, ='Y')\
        X(Z, ='Z')\
        X(0, ='0')\
        X(1, ='1')\
        X(2, ='2')\
        X(3, ='3')\
        X(4, ='4')\
        X(5, ='5')\
        X(6, ='6')\
        X(7, ='7')\
        X(8, ='8')\
        X(9, ='9')\
        X(COMMA, =',')\
        X(HYPHEN, ='-')\
        X(COLON, =':')\
        X(FULL_STOP, ='.')\
        X(EXCLAM, ='!')\
        X(HOME, =27)\
        X(UP, =173)\
        X(DOWN, =175)\
        X(OK, =23)\
        X(BACK, =20)\
        X(LEFT, =172)\
        X(RIGHT, =174)\
        X(EMPTY_BLOCK, =1)\
        X(BLOCK, =127)\
        X(BLOCK_GREY_1, =6)\
        X(BLOCK_GREY_2, =7)\
        X(HEADPHONES, =8)\
        X(DEGREES, =9)\
        X(EDIT_ORIGINAL, =10)\
        X(DIRAC_LOGO, =11)\
        X(ANALOGUE, =12)\
        X(DIGITAL, =13)\
        X(USB_RECORD, =14)\
        X(UPGRADE_AVAILABLE, =15)\
 
    /**
     * Special keys
     */
#define X(a, b) DISPLAY_SPECIAL_CHARACTERS_##a b,
	typedef enum {DISPLAY_SPECIAL_CHARACTER_DEFS} DISPLAY_SPECIAL_CHARACTERS;
#undef X

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // Brightness state enumeration
    typedef enum
    {
        DISPLAY_BRIGHTNESS_OFF,
        DISPLAY_BRIGHTNESS_DIMMED,
        DISPLAY_BRIGHTNESS_FULL,
    } DISPLAY_BRIGHTNESS;
    
    /**
     * Identifies the line to write to (line1 = top, line 2 = bottom)
     * @param immediate_refresh
     */
    typedef enum
    {
        DISPLAY_LINE_1,
        DISPLAY_LINE_2,
    } DISPLAY_LINE;
  
    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

/* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
