/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
 Display message functions for use solely on the main MCU side

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _DISPLAY_MESSAGE_CONTROLLER_H    /* Guard against multiple inclusion */
#define _DISPLAY_MESSAGE_CONTROLLER_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/display_message_data.h"
#include "utilities/display_message.h"
#include "utilities/menu.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Used as a default for menu_add_build_screen_data_function that generates the
     * screen data based on the type of menu item (Option List, Edit)
     * @param menu_id
     * @param screen_data
     */
    void display_message_build_screen_data(MENU_ID menu_id);
    
    /**
     * Checks the state of handshaking with the Display
     */
    void display_message_check_handshake();
    
    /**
     * Called when the host receives a message from the display
     */
    void display_message_complete_handshake();
    
    /**
     * Generates the appropriate display_messages based on the current item, and 
     * navigation type
     * @param menu
     * @param navigation_type
     */
    void display_message_display_screen(MENU_ID menu_id, DISPLAY_MESSAGE_SCREEN_NAVIGATION navigation_type);

    /**
     * Can be used in menu quit functions to determine how the menu is being navigated away from
     * @return 
     */
    DISPLAY_MESSAGE_SCREEN_NAVIGATION display_message_get_current_navigation_type();

    /**
     * Gets the page size of the MENU_ITEM (set by menu_define_screen_data)
     * @param menu_item
     * @return 
     */
    uint8_t display_message_get_page_size(MENU_ITEM* menu_item);
    
    /**
     * Called whenever a message is sent to initiate handshaking if neccessary
     */
    void display_message_initiate_handshake();
    
    /**
     * Sends a Screen Action message
     * @param action
     * @param screen_data_id
     */
    void display_message_send_screen_action(
            DISPLAY_MESSAGE_SCREEN_ACTION action, 
            DISPLAY_MESSAGE_DATA_ID screen_data_id);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _DISPLAY_MESSAGE_H */

/* *****************************************************************************
 End of File
 */
