/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    display_test.h

  @Summary
    Brief description of the file.

  @Description
 Used to run the display test animation
 */
/* ************************************************************************** */

#ifndef _DISPLAY_TEST_H    /* Guard against multiple inclusion */
#define _DISPLAY_TEST_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif

#include "app_context.h"

    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    /**
     * Enum of different display test cycle types
     */
    typedef enum
    {
        DISPLAY_TEST_MODE_FULL,
        DISPLAY_TEST_MODE_LOGO,
    } DISPLAY_TEST_MODE;
    
    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************
    
    /**
     * Draw the bespoke Balance display
     */
    void display_test_draw(APP_COMMAND_MENU menu);

    /**
     * Invoked when the display test ends in test mode
     */
    void display_test_end();

    /**
     * Runs the display test sequence
     * To be  used as an input to app_commands_start_process
     */
    void display_test_run(MENU_ID menu_id);

    /**
     * Sets the mode for the display test - Note this is reset to DISPLAY_TEST_MODE_FULL
     * at the end of each run
     * @param mode
     */
    void display_test_set_mode(DISPLAY_TEST_MODE mode);
    
    void set_redraw(void);
    bool get_redraw(void);
    bool display_test_ended(void);
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _DISPLAY_TEST_H */

/* *****************************************************************************
 End of File
 */
