/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    encoder.h

  @Summary
    Encapsulates rotary encoder functionality.

  @Description
    This file adapts the code from the rotary.c/h files
 */
/* ************************************************************************** */

#ifndef _ENCODER_MULTI_H    /* Guard against multiple inclusion */
#define _ENCODER_MULTI_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdint.h>
#include <stdbool.h>
#include "app_config.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */
   
    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    typedef struct
    {
        uint16_t current_value;
        uint16_t max_value;
        uint16_t min_value;
       
        // Indicates whether the encoder has moved within the range in the current cycle
        // This should only be used when using min/max values
        bool moved;
        
        // An indicator of how much the encoder has moved during the current cycle
        // This is not dependent on min/max values
        int delta;

    } ENCODER_STATE;
    
            
       
    typedef struct
    {
        PORTS_MODULE_ID index;
        PORTS_CHANNEL channel;
        PORTS_BIT_POS bitPos;
    }ENCODER_PIN_DEF;
    
    typedef struct
    {
        ENCODER_PIN_DEF pin_A;
        ENCODER_PIN_DEF pin_B;
    }ENCODER_PIN_DEFS;
    
    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Adapted from rotary.c get_rotary()
     * @param state
     * @return - an indication of whether the rotary has changed or not
     */
    bool encoder_get_state(ENCODER_STATE* state, int encoder_id);
    
    
    /**
     * Get the current wrap state
     * @return 
     */
    bool encoder_get_wrap(int encoder_id);
    
    /**
     * Get the current wrap state
     * @return 
     */
    bool input_encoder_get_wrap(int encoder_id);

    /**
     * Initialise the encoder
     */
    void encoder_initialise(ENCODER_PIN_DEFS encoder_pin_defs[], int nr_encoders);
 
    /**
    * Perform encoder processing
     * 
    * @return - true if samples processed, false if not
    */
    bool encoder_process(int encoder_id);

    /**
     * Adapted from rotary.c get_rotary()
     * @param state array of encoder states, array_size
     * @return - an indication of whether the encoders have changed or not
     */
    bool encoder_process_state(ENCODER_STATE state[APP_ENCODERS_ENCODERS_MAX], uint8_t actual_nr_encoders);
    
     /**
     * Read the current encoder state from the ports
     */
    void encoder_read(int encoders_id);
    
        
    /**
     * Update the encoder state
     * @param state
     * @param wrap
     */
    void encoder_set(ENCODER_STATE state, int encoder_id, bool wrap);
    
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
