/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _I2C_BIT_BASH_H    /* Guard against multiple inclusion */
#define _I2C_BIT_BASH_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include <stdint.h>
#include "system_definitions.h"
#include "driver/i2c/drv_i2c.h"
#include "utilities/i2c_data.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */
    
    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************
    
    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Define a specific module
     * Must be called after i2c_bit_bash_initialise and before i2c_bit_bash_open
     * @param module
     */
    void i2c_bit_bash_add_module(I2C_BIT_BASH_MODULE* module);
    
    /**
     * Initialise the I2C bit bash system.  Must be called before i2c_bit_bash_add_module
     * (or not after at least)
     */
    void i2c_bit_bash_initialise();
    
    /**
     * Open a module
     * @return 
     */
    DRV_HANDLE i2c_bit_bash_open(const SYS_MODULE_INDEX   drvIndex,
                                 const DRV_IO_INTENT      ioIntent);
    
    /**
     * Processes I2C Bit Bash requests
     */
    void i2c_bit_bash_process();

    /**
     * Reads bytes from the slave
     */
    DRV_I2C_BUFFER_HANDLE i2c_bit_bash_receive(DRV_HANDLE handle,
                                               uint16_t device_address,
                                               void* read_buffer,
                                               size_t read_size,
                                               void* callback_context);
    
    /**
     * Sets the event handler to be called when a message is processed
     * @param handle
     * @param eventHandler
     * @param context - Ignored
     */
    void i2c_bit_bash_set_event_handler(const DRV_HANDLE handle,
                                        const DRV_I2C_BUFFER_EVENT_HANDLER event_handler,
                                        const uintptr_t context);

    /**
     * Transmit a block of data to an address
     */
    DRV_I2C_BUFFER_HANDLE i2c_bit_bash_transmit(DRV_HANDLE handle,
                                                uint16_t device_address,
                                                void* write_buffer,
                                                size_t write_size,
                                                void* callback_context);

    /**
     * Transmit a block of data to an address then reads bytes from the slave
     */
    DRV_I2C_BUFFER_HANDLE i2c_bit_bash_transmit_receive(DRV_HANDLE handle,
                                                uint16_t device_address,
                                                void* write_buffer,
                                                size_t write_size,
                                                void* read_buffer,
                                                size_t read_size,
                                                void* callback_context);
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
