/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
 Holds common declares fro the I2C modules
 */
/* ************************************************************************** */

#ifndef _I2C_DATA_H    /* Guard against multiple inclusion */
#define _I2C_DATA_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */


/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */

//! Defines for different slave implementations    
#define I2C_SLAVE_TYPE_PLIB         0
#define I2C_SLAVE_TYPE_BIT_BASH     1

//------------------------------------------------------------------------------
// i2c_bit_bash.h - Bit Bash data
//------------------------------------------------------------------------------
    
    //! Underling pin definition
    typedef struct
    {
        uint8_t port;
        uint8_t channel;
        uint8_t pin;
    } I2C_BIT_BASH_PIN;
    
    /**
     * Used to define module info
     */
    typedef struct
    {
        //! The module index
        SYS_MODULE_INDEX index;
        
        //! Pin definition for the SCL pin
        I2C_BIT_BASH_PIN scl_pin;
        
        //! Pin definition for the SDA pin
        I2C_BIT_BASH_PIN sda_pin;
        
    } I2C_BIT_BASH_MODULE;

//------------------------------------------------------------------------------
//  i2c_slave.h
//------------------------------------------------------------------------------
    
    typedef uint32_t I2C_SLAVE_MODULE_ID;
    
    /**
     * Event states
     * @param index
     */
    typedef enum
    {
        I2C_SLAVE_STATE_IDLE,
                
        I2C_SLAVE_STATE_READ_ADDRESS,
                
        I2C_SLAVE_STATE_RECEIVE_START,
        I2C_SLAVE_STATE_SEND_START,
        I2C_SLAVE_STATE_RECEIVING,
        I2C_SLAVE_STATE_SENDING,
        I2C_SLAVE_STATE_RECEIVE_END,
        I2C_SLAVE_STATE_SEND_END,
    } I2C_SLAVE_STATE;

    /**
     * Event used in I2C callbacks
     * @param index
     */
    typedef struct
    {
        /** 
         * The data associated with the event (if any))
         * For I2C_SLAVE_STATE_READ/WRITE_START this will contain the address
         * For I2C_SLAVE_STATE_READING this will contain the data read
         * For I2C_SLAVE_STATE_WRITING this should be populated with the data to send (0 if there is no data);
         */
        uint8_t* data;
        
        // The current processing state
        I2C_SLAVE_STATE state;
    } I2C_SLAVE_EVENT;
    
    /**
     * Callback function template
     */
    typedef void (*I2C_SLAVE_CALLBACK)(I2C_SLAVE_EVENT* event);

    /**
     * 
     */
    typedef struct
    {
        //! The 7bit address without the read / write bit
        // e.g. 0x38 (00111000) equates to a write address of 0x70 (01110000) and 
        // a read address 0x71 (01110001)
        uint8_t address_7bit;

        //! The Id of the module e.g. I2C_ID_2
        I2C_SLAVE_MODULE_ID module_id;
        
        I2C_SLAVE_CALLBACK callback;
        
        // Bit bash implementation
        I2C_BIT_BASH_MODULE bit_bash;

    } I2C_SLAVE_MODULE;

    /**
     * 
     */
    typedef struct
    {
        //! Contains address/callback info
        I2C_SLAVE_MODULE module_info;

        // Set to true if this has been initialised by i2c_slave_add_module
        bool in_use;

        //! Holds the current processing state
        I2C_SLAVE_STATE state;
        
        //! True when the I2C line is open..
        bool open;
        
        //! The bit position in the current read/write (Bit bashed versions only)
        uint8_t bit_pos;
        
        // The current state of the SDA line
        bool sda_state;
        
        // The current state of the SCL line
        bool scl_state;
        
        bool scl_change;
        
        bool sda_change;
        
        //! holder for the data in a bit bash read/write
        uint8_t data;

    } I2C_SLAVE_RUNTIME_MODULE;


// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************


// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************


    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
