/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Based on the example described in https://www.microchip.com/forums/m1074497.aspx
 
 */
/* ************************************************************************** */

#ifndef _I2C_SLAVE_H    /* Guard against multiple inclusion */
#define _I2C_SLAVE_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdint.h>
#include "system_definitions.h"
#include "utilities\i2c_data.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************
    
    /**
     */
    void i2c_slave_add_module(uint8_t index, I2C_SLAVE_MODULE* module);
    
    /**
     * Processes I2C BUS interrupt messages.
     */
    void i2c_slave_idle_interrupt(uint8_t index);

    /**
     * Initialises the I2C Slave module
     */
    void i2c_slave_initialise();
    
    /**
     * Starts processing on the module
     */
    DRV_HANDLE i2c_slave_open(uint8_t index);
    
    /**
     * Processes I2C Slave interrupt messages.
     */
    void i2c_slave_process_interrupt(uint8_t index);
    
    //--------------------------------------------------------------------------
    //  Internal functions 
    //--------------------------------------------------------------------------
    
    /**
     * Used internally:
     * Obtains the module
     * @param index
     * @return 
     */
    I2C_SLAVE_RUNTIME_MODULE* i2c_slave_get_module(uint8_t index);

    /**
     * Used internally:
     * Initialises the common elements of the I2C Slave module, this is implemented in i2c_slave_common.c
     */
    void i2c_slave_initialise_data();
 
    /**
     * Used internally:
     * Invokes the module callback
     * @param module
     */
    void i2c_slave_invoke_callback(I2C_SLAVE_RUNTIME_MODULE* module);
   
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _I2C_SLAVE_H */

/* *****************************************************************************
 End of File
 */
