/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _INPUT_CONTROL_ANALOGUE_H    /* Guard against multiple inclusion */
#define _INPUT_CONTROL_ANALOGUE_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/input_control_defs.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************
    
// a. Sample Rate, b. String representation, c. Index for get_sample_rate RS232 message    
// d. The sample rate as an integer
#define PHONO_INPUTS \
	X(MM, MM)\
	X(MC, MC)\

// Now use the input data to define the enum
#define X(a, b) PHONO_INPUT_##a,
	typedef enum {PHONO_INPUTS PHONO_INPUT_MAX} PHONO_INPUT;
#undef X
    
// Redefine the macro and use to create an array of strings
#define X(a, b) #b,
	static char *PHONO_INPUT_STRINGS[] = { PHONO_INPUTS };
#undef X
    
    /**
     * Callback for configuration function
     * @param address
     * @param value
     */
    typedef void (*INPUT_CONTROL_ANALOGUE_CONFIGURE_CALLBACK) ();

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Sets pins for analogue direct on/off
     * @param on
     */
    void input_control_adc_reset(bool on);
 
    /**
     * Used to define DAC configuration values.
     * Note: This works at a register level rather than the individual settings
     * @param register_ident - the register identifier
     * @param value - th value to be written
     */
    void input_control_add_adc_setting(uint8_t address, uint8_t value);
    
    /**
     * Starts the process of applying settings to the dac
     * Note the define I2C_REQUEST_QUEUE_SIZE must be at least as big as this
     */
    void input_control_apply_adc_settings();
    
    /**
     * Configures the analogue module
     * @return 
     */
    bool input_control_analogue_configure();
    
    /**
     * True when the ADC has been configured
     * @return 
     */
    bool input_control_analogue_configured();
    
    /**
     * Initialise the input control module
     * @return - True if successfully initialised
     */
    bool input_control_analogue_initialise();
    
    /**
     * 
     * @param channel
     */
    void input_control_analogue_set_callbacks(INPUT_CONTROL_ANALOGUE_CONFIGURE_CALLBACK configure_callback);

    /**
     * Sets analogue direct mod on/off
     * @param on
     */
    bool input_control_get_analogue_direct();
    
    /**
     * Sets analogue direct setting for a given input channel
     * @param on
     */
    bool input_control_get_input_analogue_direct(uint8_t index);
    
    /**
     * Gets the phono input to be MM or MC
     */
    PHONO_INPUT input_control_get_phono_input();

    /**
     * Sets analogue direct mod on/off
     * @param on
     */
    void input_control_set_analogue_direct(bool on);
    
    /**
     * Sets analogue direct setting for a given input channel
     * @param on
     */
    void input_control_set_input_analogue_direct(uint8_t index, bool on);
    
    /**
     * Sets enabled setting for a given input channel
     * @param on
     */
    void input_control_set_input_enabled(uint8_t index, bool on);
   
    /**
     * Sets the phono input to be MM or MC
     * @param type
     */
    void input_control_set_phono_input(PHONO_INPUT type);
    
    /**
     * Switch to a specific input on the analogue input control
     * @param definition - the definition of the control to switch to
     */
    void input_control_switch_analog(INPUT_CONTROL_DEFINITION* definition);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
