/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
 Handles the display of the menu system on the screen

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _MENU_DISPLAY_H    /* Guard against multiple inclusion */
#define _MENU_DISPLAY_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/menu.h"

    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */
   
// menu buttons
#ifndef BIT
#define BIT(N)                      (1 << (N))
#endif
#define BIT_HOME                    BIT(3)
#define BIT_UP                      BIT(4)
#define BIT_DOWN                    BIT(5)
#define BIT_OK                      BIT(6)
#define BIT_BACK                    BIT(7)

// buttons for left_right menu
#define BIT_LEFT                    BIT(4)
#define BIT_RIGHT                   BIT(5)    

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    typedef enum
    {
        MENU_KEYS_NONE  =   0,
        MENU_KEYS_HOME  =   BIT(3),
        MENU_KEYS_UP    =   BIT(4),
        MENU_KEYS_DOWN  =   BIT(5),
        MENU_KEYS_OK    =   BIT(6),
        MENU_KEYS_BACK  =   BIT(7),

        MENU_KEYS_LEFT  =   BIT(4),
        MENU_KEYS_RIGHT =   BIT(5),

        MENU_KEYS_ALL   =   MENU_KEYS_HOME | MENU_KEYS_UP | MENU_KEYS_DOWN | MENU_KEYS_OK | MENU_KEYS_BACK,
    } MENU_KEYS;

    /**
     * Enum for soft keys, options are up/down left/right
     */
    typedef enum
    {
        MENU_SOFTKEYS_NONE,
        MENU_SOFTKEYS_UP_DOWN,
        MENU_SOFTKEYS_LEFT_RIGHT
    } MENU_SOFTKEYS;

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Draw the standard navigation buttons for the given menu item
     * @param item
     */
    void menu_display_draw_generic_navigation(MENU_ITEM* item, MENU_SOFTKEYS edit_navigation_keys, bool enter_branch);

    /**
     * Draws a standard navigation menu with title on top line and navigation buttons 
     * on the bottom
     */
    void menu_display_draw_generic_menu();
    
    /**
     * Initialises the menu display module
     * @param top_menu_index - the index of the top meny - e.g. SYSTEM SETTINGS
     */
    void menu_display_initialise(int top_menu_index);
    
    /**
     * Process the menu display
     */
    void menu_display_process();

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
