/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    rs232.h

  @Summary
 Handles communication with the RS232 port

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _RS232_H    /* Guard against multiple inclusion */
#define _RS232_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdint.h>
#include <stddef.h>
#include <stdbool.h>
#include "utilities/display_message.h"
#include "system/common/sys_module.h"   


/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************
    
    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************
    
    /**
     * RS232 module initialisation
     */
    void rs232_display_initialise(const SYS_MODULE_INDEX module, DISPLAY_MESSAGE_CALLBACK display_callback);

    /**
     * RS232 processing
     */
    void rs232_display_process();
    
    /**
     * Resets the messsge, drops all outstanding buffers...
     */
    void rs232_display_reset();
    
    /**
     * Sends a buffer of data over the RS232 connection
     * @param buffer
     * @param len
     */
    void rs232_display_send_buffer(char* buffer, size_t len);

    /**
     * Send a message out
     */
//    void rs232_display_send_message(DISPLAY_MESSAGE* message);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
