/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _SPECIAL_SWITCHES_H    /* Guard against multiple inclusion */
#define _SPECIAL_SWITCHES_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/adc.h"
#include "utilities/structured_message.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif

    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Callback function template
     */
    typedef void (*SPECIAL_SWITCH_CALLBACK)(STRUCTURED_MESSAGE* message);
    
    /**
     * Structure holding a definition for a combination of switches where if they 
     * are all held down together a function is invoked
     */
    typedef struct
    {
        ADC_SWITCH switch1;
        ADC_SWITCH switch2;
        ADC_SWITCH switch3;
        
        //! Pointer to an SPECIAL_SWITCH_CALLBACK function
        SPECIAL_SWITCH_CALLBACK function;
    } SPECIAL_SWITCH_COMBINATION;

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

/**
 * Returns the number of currently active switches.
 * @param switch_combination
 */
uint8_t special_switches_active_button_count();
    
/**
 * Add a definition for a switch combination where if all the switches are depressed 
 * a function will be invoked
 * @param switch1
 * @param switch2
 * @param switch3
 */
void special_switches_add(SPECIAL_SWITCH_COMBINATION* switch_combination);

/**
 * Gets the switch info for the supplied switch - if the switch is not active it returns NULL
 */
ADC_SWITCH_INFO* special_switches_get_info_for_switch(ADC_SWITCH adc_switch);

/**
 * Initialises the speaicl switches module
 */
void special_switches_initialise();

/**
 * Tests the current ADC state for the special switch combinations
 * @return True is a combination is matched, False if not
 */
bool special_switches_test(STRUCTURED_MESSAGE* message);
    
#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
