/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    standby.h

  @Summary
    Brief description of the file.

  @Description
 Manages the different states in standby mode 
 */
/* ************************************************************************** */

#ifndef _STANDBY_H    /* Guard against multiple inclusion */
#define _STANDBY_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */


/**
 * State form managing stand-by states
 */
typedef enum
{
    STANDBY_STATE_ENTRY,
    STANDBY_STATE_ENTER_SLEEP,
    STANDBY_STATE_SLEEP,
    STANDBY_STATE_ACTIVE,
} STANDBY_STATE;

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

/**
 * Gets the current standby state
 * @return 
 */
STANDBY_STATE standby_get_state();

/**
 * Use when leaving standby mode to reset standby state to STANDBY_STATE_ENTRY
 */
void standby_reset();

/**
 * Use with STANDBY_STATE_ENTRY to determine which default standby state to move to
 * @param can_sleep -  generally supplied by power_network_standby()
 */
void standby_set_initial_state(bool can_idle, bool can_sleep, STANDBY_STATE default_state);

/**
 * Directly sets the Standby state
 * @return 
 */
void standby_set_state(STANDBY_STATE state);

/**
 * Set the standby state into an active high power state
 * @param state
 */
void standby_set_state_active();

/**
 * Puts the unit into sleep mode if set in the appropriate standby mode
 */
void standby_try_sleep();

/**
 * Checks to see whether the state can be put back into sleep state
 */
void standby_update_state(bool allow_idle, bool allow_sleep);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
