/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _USB_AUDIO_H    /* Guard against multiple inclusion */
#define _USB_AUDIO_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include <stdint.h>
#include "usb/usb_audio_v2_0.h"
#include "usb/usb_device_audio_v2_0.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************


    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Returns a value identifying whether USB Audio is active or not;
     * @return 
     */
    bool usb_audio_get_active();
    
    /**
     * Gets USB Audio configured state
     */
    bool usb_audio_get_configured();

    /**
     * Initialise the usb_audio module
     */
    bool usb_audio_initialise();
    
    /**
     * Perform USB Audio processing
     */
    void usb_audio_process();
    
    /**
     * Allows the code to effectivly turn USB processing On/Off.  Because it is 
     * processor intensive other activity can cause packets to be dropped and for 
     * the USB Audio module to become stuck.
     * @param active
     */
    void usb_audio_set_active(bool active);
    
    /**
     * This function is called from the DMA Channel 0 interrupt handler in system_interrupt.c
     * to synchronise up the I2S LRCLK when the audio stream is restarted.  Otherwise, the left/right
     * channels may become swapped.
     */
    void usb_audio_sync_LR_clock();
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _USB_AUDIO_H */

/* *****************************************************************************
 End of File
 */
