/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _USB_AUDIO_BUFFER_H    /* Guard against multiple inclusion */
#define _USB_AUDIO_BUFFER_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include <stdint.h>
#include "usb/usb_audio_v2_0.h"
#include "usb/usb_device_audio_v2_0.h"
#include "utilities\usb_audio_defs.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Clears down the buffer
     */
    void usb_audio_buffer_clear();

    /**
     * Used to indicate that USB read queue data is available
     */
    void usb_audio_buffer_clear_data_available(uint32_t index);

    /**
     * Configures the buffer
     */
    void usb_audio_buffer_configure(uint32_t sample_rate);

    /**
     * Determines whether the read queue has unread data available
     */
    bool usb_audio_buffer_data_available(uint32_t index);

    /**
     * Determines whether the read queue is ready for re-use
     */
    bool usb_audio_buffer_data_clear(uint32_t index);

    /**
     * Gets the length of the available data in the buffer
     */
    uint16_t usb_audio_buffer_get_data_length(uint32_t index);

    /**
     * Returns the maximum number of samples that can be used.  Initialised by the
     * usb_audio_buffer_configure function
     */
    uint32_t usb_audio_buffer_get_max_samples();

    /**
     * Gets the maximum buffer size
     * @return 
     */
    uint32_t usb_audio_buffer_get_max_size();

    /**
     * Gets the minimum buffer size
     * @return 
     */
    uint32_t usb_audio_buffer_get_min_size();

    /**
     * Gets a pointer to the read handle
     */
    USB_DEVICE_AUDIO_V2_TRANSFER_HANDLE* usb_audio_buffer_get_read_handle(uint32_t index);

    /**
     * Gets a pointer to the read buffer
     */
    uint8_t* usb_audio_buffer_get_read_buffer(uint32_t index);
    
    /**
     * Gets the number of unprocessed reads
     * @return 
     */
    uint32_t usb_audio_buffer_get_unprocessed_reads();
    
    /**
     * Checks whether the supplied handle is the same as the one for the supplied 
     * buffer index
     */
    bool usb_audio_buffer_is_handle(uint32_t index, USB_DEVICE_AUDIO_V2_TRANSFER_HANDLE handle);

    /**
     * Indicates that the slot has data that has been read in it
     */
    void usb_audio_buffer_set_data_available(uint32_t index, uint16_t length);
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
