/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _USB_AUDIO_PORT_H    /* Guard against multiple inclusion */
#define _USB_AUDIO_PORT_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include <stdint.h>
#include "usb/usb_audio_v2_0.h"
#include "usb/usb_device_audio_v2_0.h"
#include "utilities\usb_audio_defs.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************
    
    /**
     * Attache the USB port
     */
    void usb_audio_port_attach();
    
    /**
     * Wrap control receive message
     */
    USB_DEVICE_CONTROL_TRANSFER_RESULT usb_audio_port_control_receive(void * data, size_t length);

    /**
     * Wrap control send message
     * @param data
     * @param length
     * @return 
     */
    USB_DEVICE_CONTROL_TRANSFER_RESULT usb_audio_port_control_send(void * data, size_t length);
    
    /**
     * Detach the USB port
     */
    void usb_audio_port_detach();

    /**
     * Gets the index of the next buffer to be read into from the USB Host
     * @return 
     */
    uint32_t usb_audio_port_get_read_idx();
    
    /**
     * tests whether the read error limit is exceeded
     */
    bool usb_audio_port_error_limit_exceeded();
    
    /**
     * Initialise the USB port
     */
    bool usb_audio_port_initialise(USB_AUDIO_DEVICE_EVENT_HANDLER device_handler, 
            USB_AUDIO_EVENT_HANDLER audio_handler);
    
    /**
     * Opens the port for reading
     */
    bool usb_audio_port_open();
    
    /**
     * Reads the next audio buffer
     */
    USB_DEVICE_AUDIO_V2_RESULT usb_audio_port_queue_read();

    /**
     * Reads all the audio buffers
     */
    USB_DEVICE_AUDIO_V2_RESULT usb_audio_port_read_buffers();
    
    /**
     *  Resets the port
     */
    void usb_audio_port_reset();
    
    /**
     * Sends the USB host an error status
     */
    void usb_audio_port_send_control_status_error();

    /**
     * Sends the USB host an OK status
     */
    void usb_audio_port_send_control_status_ok();   

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
