/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/album_art_cache.h"
#include <stdio.h>
#include <string.h>
#include "utilities/display_message.h"
#include "utilities/image_cache.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

#define ALBUM_ART_IMAGE_BUFFER_SIZE     16384//24200   // Change to be max size used by 5707 - 16384    // Needs review - must be same size as largest image...
#define ALBUM_ART_MAX_IMAGES            1
/**
 */
typedef struct 
{
    IMAGE_CACHE_CODED_IMAGE_HEADER header;
    
    uint8_t buffer[ALBUM_ART_IMAGE_BUFFER_SIZE]  __attribute__ ((aligned(2)));		
    
} ALBUM_ART_DATA;

typedef struct
{
    ALBUM_ART_DATA cache[ALBUM_ART_MAX_IMAGES];
} ALBUM_ART_CACHE_DATA;

static ALBUM_ART_CACHE_DATA album_art_cache_data = {
    
};

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Adds image data to the cache for a given url
 * @param url
 * @param data
 * @param len
 * @param offset
 */
void album_art_cache_add_image_data(char* url, uint8_t* data, uint16_t len, uint16_t offset)
{
    //logger_log_debugf(0, "ALBUM ART album_art_cache_add_image_data %d %d -  %X %X %X %X --", len, offset, data[0], data[1], data[2], data[3]);
    
    ALBUM_ART_DATA* album_art = &album_art_cache_data.cache[0];
    
    uint16_t x;
    for (x = 0; x < len; x++)
    {
        album_art->buffer[offset + x] = data[x];
    }
    
    // Update the size of the album art
    album_art->header.size = offset + len;
}

/**-----------------------------------------------------------------------------
* 
* @param url
* @param ctrl
*/
void album_art_cache_allocate_image(char* url, TFT_CONTROL* tft_control)
{
    // This would allocate an album art structure for the URL but just use the one for starters
    ALBUM_ART_DATA* album_art = &album_art_cache_data.cache[0];
    album_art->header.buffer_offset = 0; // Initialise the buffer offset
    album_art->header.buffer_ptr = (uint8_t*)album_art->buffer;
    album_art->header.ready_to_process = false;
    album_art->header.tft_control = tft_control;
}
    
/**-----------------------------------------------------------------------------
 * 
 * @param url
 */
bool album_art_cache_draw_image(char* url)
{
    bool result = true;
    
    ALBUM_ART_DATA* album_art = &album_art_cache_data.cache[0];
    if (album_art->header.ready_to_process) 
    {
        //logger_log_debugf(0, "ALBUM ART Drawing %s --", url);
        result = image_cache_process_image(&album_art->header);
        
        //This should reduce the number of draws but sometimes the album art is cleared when it shouldnt be
        //album_art->header.ready_to_process = false;
    }
    
    return result;
}
    
/**-----------------------------------------------------------------------------
 * Called when the last packet of image data is added to the current image
 * @param url
 */
void album_art_cache_finalise_image_data(char* url)
{
    ALBUM_ART_DATA* album_art = &album_art_cache_data.cache[0];
    album_art->header.ready_to_process = true;
}

//
///**-----------------------------------------------------------------------------
// * Gets a reference to a pointer to the image data, and the length of the data
// * @param url - a pointer to a string containing the URL
// * @param data
// * @param len
// */
//void album_art_cache_get_image_data(char* url, uint8_t* data, uint16_t* len)
//{
//}

/**-----------------------------------------------------------------------------
 * Returns true if the cache has album art available for the current url
 * @param url
 * @return 
 */
bool album_art_cache_has_url(char* url)
{
    bool result = false;
    
    return result;
}
    
/**-----------------------------------------------------------------------------
 * Tests whether a url is empty
 * @param url
 * @return 
 */
bool album_art_cache_is_empty_url(char* url)
{
    char empty_url[DISPLAY_MESSAGE_ALBUM_ART_URL_LENGTH];
    memset((uint8_t*)empty_url, 0, DISPLAY_MESSAGE_ALBUM_ART_URL_LENGTH);
    
    return album_art_cache_urls_are_equal(url, empty_url);
}
    
/**-----------------------------------------------------------------------------
 * Compares 2 urls 
 * @param url1
 * @param url2
 * @return 
 */
bool album_art_cache_urls_are_equal(char* url1, char* url2)
{
    return memcmp(url1, url2, DISPLAY_MESSAGE_ALBUM_ART_URL_LENGTH) == 0;
}
    
/* *****************************************************************************
 End of File
 */
