/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

/* This section lists the other files that are included in this file.
 */
#include "utilities/colour.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

typedef struct
{
    // Array to hold the color data
    uint8_t colour_cache[sizeof(COLOUR) * COLOUR_INDEX_MAX];
} COLOUR_DATA;

static COLOUR_DATA colour_data = {
    
};

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * 
 * @param index
 * @param red
 * @param green
 * @param blue
 */
static void colour_add(COLOUR_INDEX index, uint8_t red, uint8_t green, uint8_t blue)
{
    colour_update(index, red, green, blue);
//    uint8_t* colour = &colour_data.colour_cache[sizeof(COLOUR) * (uint8_t)index];
//    // Red
//    *colour = red;
//    colour++;
//    *colour = green;
//    colour++;
//    *colour = blue;
}

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * 
 * @param index
 * @return 
 */
COLOUR* colour_get(COLOUR_INDEX index)
{
    COLOUR* result = NULL;
    
    if (index < COLOUR_INDEX_MAX)
    {
        result = (COLOUR*)&colour_data.colour_cache[sizeof(COLOUR) * (uint8_t)index];
    }
    
    return result;
}

/**-----------------------------------------------------------------------------
 * 
 */
void colour_initialise()
{
    //TODO This will kick off a request to the filesystem to retrieve the colours from the eeprom
    // but for now we will load a hard coded version
    int x;
    for (x = 0; x < COLOUR_INDEX_MAX; x++)
    {
        colour_add(x, COLOURS[x].red, COLOURS[x].green, COLOURS[x].blue);
    }
}
     
/**-----------------------------------------------------------------------------
 * To saturate we mix the foreground with the background based on the amount of the foreground to be displayed
 * @param colour
 * @param saturation
 */
void colour_saturate(COLOUR* colour, uint8_t saturation, COLOUR* background)
{
    if (colour != NULL)
    {
        // Formula is 
        // value = foreground_colour * saturation + backgound * 0xFF - saturation 
        colour->red = (uint8_t)(((colour->red * (uint16_t)saturation) + (background->red * (0xFF - (uint16_t)saturation))) / 0x00FF);
        colour->green = (uint8_t)(((colour->green * (uint16_t)saturation) + (background->green * (0xFF - (uint16_t)saturation))) / 0x00FF);
        colour->blue = (uint8_t)(((colour->blue * (uint16_t)saturation) + (background->blue * (0xFF - (uint16_t)saturation))) / 0x00FF);
    }
}

/**-----------------------------------------------------------------------------
 * Sets the supplied colour reference to the colours associated with the supplied index
 * @param colour
 * @param index
 */
void colour_set(COLOUR* colour, COLOUR_INDEX index)
{
    if ((colour != NULL) && (index < COLOUR_INDEX_MAX))
    {
        uint8_t* colour_cache = &colour_data.colour_cache[sizeof(COLOUR) * (uint8_t)index];

        colour->red = *colour_cache;
        colour_cache++;
        colour->green = *colour_cache;
        colour_cache++;
        colour->blue = *colour_cache;
    }
}

/**-----------------------------------------------------------------------------
 * Update the colour at the given index
 * @param index
 * @param red
 * @param green
 * @param blue
 */
void colour_update(COLOUR_INDEX index, uint8_t red, uint8_t green, uint8_t blue)
{
    uint8_t* colour = &colour_data.colour_cache[sizeof(COLOUR) * (uint8_t)index];
    // Red
    *colour = red;
    colour++;
    *colour = green;
    colour++;
    *colour = blue;
}

/* *****************************************************************************
 End of File
 */
