/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
 Shares common functionality between i2c_slave and i2c_slave_bit_bash
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/i2c_slave.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/**
 * 
 */
typedef struct
{
    // Represents each slave module
    I2C_SLAVE_RUNTIME_MODULE modules[I2C_SLAVE_NUM_MODULES];
} I2C_SLAVE_DATA;

static I2C_SLAVE_DATA i2c_slave_data;

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */


/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */
    
/**-----------------------------------------------------------------------------
 * Note - these MUST be added before
 */
void i2c_slave_add_module(uint8_t index, I2C_SLAVE_MODULE* module)
{
    if (index < I2C_SLAVE_NUM_MODULES)
    {
        // Copy the data into the I2C_SLAVE_RUNTIME_MODULE modules data.
        memcpy(&i2c_slave_data.modules[index], module, sizeof(I2C_SLAVE_MODULE));
        
        i2c_slave_data.modules[index].in_use = true;
        
        logger_log_debugf(LOGGER_CODE_I2C, "I2C SLAVE Added Module %d %d --", index, module->module_id);
    }
}
    
/**-----------------------------------------------------------------------------
 * 
 * @param index
 * @return 
 */
I2C_SLAVE_RUNTIME_MODULE* i2c_slave_get_module(uint8_t index)
{
    I2C_SLAVE_RUNTIME_MODULE* result = NULL;
    
    if (index < I2C_SLAVE_NUM_MODULES)
    {
        result = &i2c_slave_data.modules[index];
    }

    return result;
}

/**-----------------------------------------------------------------------------
 * Processes I2C BUS interrupt messages.
 */
void i2c_slave_idle_interrupt(uint8_t index)
{
    I2C_SLAVE_RUNTIME_MODULE* module = &i2c_slave_data.modules[index];
    module->state = I2C_SLAVE_STATE_IDLE;
}

/**-----------------------------------------------------------------------------
 * Initialises the common elements of the I2C Slave module
 */
void i2c_slave_initialise_data()
{
    int x;
    for (x = 0; x < I2C_SLAVE_NUM_MODULES; x++)
    {
        I2C_SLAVE_RUNTIME_MODULE* module = &i2c_slave_data.modules[x];
   
        module->in_use = false;
        module->state = I2C_SLAVE_STATE_IDLE;
        
        module->open = false;
        module->bit_pos = 0;
        module->scl_state = true;
        module->sda_state = true;
    }
}

/**-----------------------------------------------------------------------------
 * 
 * @param module
 */
void i2c_slave_invoke_callback(I2C_SLAVE_RUNTIME_MODULE* module)
{
    if (module != NULL)
    {
        // Invoke the callback
        if (module->module_info.callback != NULL)
        {
            I2C_SLAVE_EVENT event = 
            {
                .data = &module->data,
                .state = module->state,
            };

            module->module_info.callback(&event);
        }
    }
}

/* *****************************************************************************
 End of File
 */
