/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    stateful_process.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/stateful_process.h"
#include <stdio.h>

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

typedef struct
{
    /**
     * A simple structure that can be used to hold state when in APP_CONTEXT_MODE_PROCESS
     * mode.
     */
    PROCESS_INFO process_info;
    
} STATEFUL_PROCESS_DATA;

static STATEFUL_PROCESS_DATA process_data;

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Invokes the function to be called at the end of the process - should be called in 
 * the code that implements a process
 */
void stateful_process_end()
{
    if (stateful_process_get_info()->end_function != NULL)
    {
        stateful_process_get_info()->end_function();
    }    
}

/**-----------------------------------------------------------------------------
 * Gets a pointer to a common processing info block that can be used to store 
 * state details when in APP_CONTEXT_MODE_PROCESS mode
 * Note: Originally this was put onto individual menu items which had the problem
 * that when the menu was changed during processing the process info needed copying
 * It may be necessary to go back to that model if ever we need to run two 
 * processes simultaneously
 */
PROCESS_INFO* stateful_process_get_info()
{
    return &process_data.process_info;
}
    
/**-----------------------------------------------------------------------------
 * Initialises the process info for the start of a process
 * @param process_function
 */
void stateful_process_initialise_info(void (*process_function)(), void (*end_function)())
{
    // Set up the process info
    PROCESS_INFO* info = stateful_process_get_info();
    info->state = 0;
    info->process_function = process_function; 
    info->end_function = end_function; 
}

/**-----------------------------------------------------------------------------
 * Invokes the process function
 */
void stateful_process_invoke()
{
    if (stateful_process_get_info()->process_function != NULL)
    {
        stateful_process_get_info()->process_function();
    }
}

/* *****************************************************************************
 End of File
 */
