/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/tft_display.h"
#include "app_config.h"
#include "utilities/colour.h"
#include "utilities/display_message_display.h"
#include "utilities/logger.h"
#include "utilities/timers.h"
#include "utilities/tft_control.h"
#include "utilities/font_manager.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

//------------------------------------------------------------------------------
// Rectangle handling
//------------------------------------------------------------------------------

#define MAX_INVALIDATED_RECTS   16

//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------

typedef enum 
{
    TFT_DISPLAY_STATE_INITIALISE,
    TFT_DISPLAY_STATE_RESET,
    TFT_DISPLAY_STATE_SOFT_RESET,
    TFT_DISPLAY_STATE_SET_PLL,
    TFT_DISPLAY_STATE_LOCK_PLL,
    TFT_DISPLAY_STATE_CONFIGURE,
    TFT_DISPLAY_STATE_RUNNING,
            
} TFT_DISPLAY_STATE;

//! Commands
#define TFT_COMMAND_ITEMS \
    X(NOP, 0x00, Nop, true)\
    X(SOFT_RESET, 0x01, Soft Reset, true)\
    X(GET_ADDRESS_MODE, 0x0B, Get Address Mode, false)\
    X(SET_DISPLAY_ON, 0x29, Set Display On, true)\
    X(SET_COLUMN_ADDRESS, 0x2A, Set Column Address, true)\
    X(SET_PAGE_ADDRESS, 0x2B, Set Page Address, true)\
    X(WRITE_MEMORY_START, 0x2C, Write Memory Start, true)\
    X(READ_MEMORY_START, 0x2E, Read Memory Start, false)\
    X(SET_ADDRESS_MODE, 0x36, Set Address Mode, true)\
    X(READ_DDB, 0xA1, Read DDB, false)\
    X(SET_LCD_MODE, 0xB0, Set LCD Mode, true)\
    X(GET_LCD_MODE, 0xB1, Get LCD Mode, false)\
    X(SET_HORZ_PERIOD, 0xB4, Set Horz Period, true)\
    X(GET_HORZ_PERIOD, 0xB5, Get Horz Period, false)\
    X(SET_VERT_PERIOD, 0xB6, Set Vert Period, true)\
    X(GET_VERT_PERIOD, 0xB7, Get Vert Period, false)\
    X(SET_GPIO_CONF, 0xB8, Set GPIO Conf, true)\
    X(GET_GPIO_CONF, 0xB9, Get GPIO Conf, false)\
    X(SET_GPIO_VALUE, 0xBA, Set GPIO Value, true)\
    X(GET_GPIO_STATUS, 0xBB, Get GPIO Status, false)\
    X(SET_POST_PROC, 0xBC, Set Post Proc, true)\
    X(GET_POST_PROC, 0xBD, Set Post Proc, false)\
    X(SET_PWM_CONF, 0xBE, Set PWM Conf, true)\
    X(GET_PWM_CONF, 0xBF, Get PWM Conf, false)\
    X(SET_DBC_CONF, 0xD0, Set DBC Conf, true)\
    X(GET_DBC_CONF, 0xD1, Get DBC Conf, false)\
    X(SET_PLL, 0xE0, Set PLL, true)\
    X(SET_PLL_MN, 0xE2, Set PLL MN, true)\
    X(GET_PLL_MN, 0xE3, Get PLL MN, false)\
    X(SET_LSHIFT_FREQ, 0xE6, Set LShift Freq, true)\
    X(GET_LSHIFT_FREQ, 0xE7, Get LShift Freq, false)\
    X(SET_PIXEL_DATA_INTERFACE, 0xF0, Set Pixel Interface, true)\
    X(GET_PIXEL_DATA_INTERFACE, 0xF1, Get Pixel Interface, false)\

// Now use the input data to define the enum
#define X(a, b, c, d) TFT_COMMAND_##a,
	typedef enum {TFT_COMMAND_ITEMS} TFT_COMMAND;
#undef X
    
// Redefine the macro and use to create an array of strings
#define X(a, b, c, d) b,
	static uint16_t TFT_COMMAND_VALUES[] = { TFT_COMMAND_ITEMS };
#undef X
    
// Redefine the macro and use to create an array of strings
#define X(a, b, c, d) #c,
	static char *TFT_COMMAND_STRINGS[] = {TFT_COMMAND_ITEMS};
#undef X
    
// Redefine the macro and use to create an array of strings
#define X(a, b, c, d) d,
	static bool TFT_COMMAND_WRITE[] = { TFT_COMMAND_ITEMS };
#undef X

//#define NOP                         0x00
//#define GET_POWER_MODE              0x0A
//#define GET_DISPLAY_MODE            0x0D
//#define GET_TEAR_EFFECT_STATUS      0x0E
//#define ENTER_SLEEP_MODE            0x10
//#define EXIT_SLEEP_MODE             0x11
//#define ENTER_PARTIAL_MODE          0x12
//#define ENTER_NORMAL_MODE           0x13
//#define EXIT_INVERT_MODE            0x20
//#define ENTER_INVERT_MODE           0x21
//#define SET_GAMMA_CURVE             0x26
//#define SET_DISPLAY_OFF             0x28
//#define SET_DISPLAY_ON              0x29
//#define SET_COLUMN_ADDRESS          0x2A
//#define SET_PAGE_ADDRESS            0x2B
//#define WRITE_MEMORY_START          0x2C
//#define READ_MEMORY_START           0x2E 
//#define SET_PARTIAL_AREA            0x30
//#define SET_SCROLL_AREA             0x33
//#define SET_TEAR_OFF                0x34
//#define SET_TEAR_ON                 0x35
//#define SET_ADDRESS_MODE            0x36
//#define SET_SCROLL_START            0x37
//#define EXIT_IDLE_MODE              0x38
//#define ENTER_IDLE_MODE             0x39
//#define WRITE_MEMORY_CONTINUE       0x3C
//#define READ_MEMORY_CONTINUE        0x3E
//#define SET_TEAR_SCANLINE           0x44
//#define GET_SCANLINE                0x45
//#define READ_DDB                    0xA1
//#define SET_LCD_MODE                0xB0
//#define GET_LCD_MODE                0xB1
//#define SET_POST_PROC               0xBC
//#define GET_POST_PROC               0xBD
//#define SET_PWM_CONF                0xBE
//#define GET_PWM_CONF                0xBF
//#define SET_LCD_GEN0                0xC0
//#define GET_LCD_GEN0                0xC1    
//#define SET_LCD_GEN1                0xC2
//#define GET_LCD_GEN1                0xC3
//#define SET_LCD_GEN2                0xC4
//#define GET_LCD_GEN2                0xC5
//#define SET_LCD_GEN3                0xC6
//#define GET_LCD_GEN3                0xC7
//#define SET_GPIO0_ROP               0xC8
//#define GET_GPIO0_ROP               0xC9
//#define SET_GPIO1_ROP               0xCA
//#define GET_GPIO1_ROP               0xCB
//#define SET_GPIO2_ROP               0xCC
//#define GET_GPIO2_ROP               0xCD
//#define SET_GPIO3_ROP               0xCE
//#define GET_GPIO3_ROP               0xCF
//#define SET_DBC_CONF                0xD0
//#define GET_DBC_CONF                0xD1
//#define SET_DBC_TH                  0xD4
//#define GET_DBC_TH                  0xD5
//#define GET_PLL_STATUS              0xE4
//#define SET_DEEP_SLEEP              0xE5







typedef struct
{
    //! True if the display has been setup
    bool initialised;
    
    //! Array of invalidated rects
    BOUNDS invalidated_rects[MAX_INVALIDATED_RECTS];
    
    //! The number of invalidated rects
    uint8_t invalidated_rect_count;
    
    //! Indicated wther the screen is currently in a clear state or whether it
    // has stuff drawn on it
    bool is_clear;
    
    //! The screen buffer
    PIXEL screen_buffer[TFT_DISPLAY_HEIGHT][TFT_DISPLAY_WIDTH]; 
    
    //! Current display state
    TFT_DISPLAY_STATE state;
} TFT_DISPLAY_DATA;

static TFT_DISPLAY_DATA tft_display_data = 
{
    .initialised = false,
    .invalidated_rect_count = 0,
    .is_clear = true,
    .state = TFT_DISPLAY_STATE_INITIALISE,
};

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

static uint8_t tft_display_get_brightness();
static void tft_display_send_command(TFT_COMMAND command, uint8_t* parameters, uint8_t count);
static void tft_display_set_backlight(bool on);
static void tft_display_set_command_mode();
static void tft_display_set_data_mode();
static void tft_display_set_dimensions();
static void tft_display_set_ram_address(BOUNDS* bounds);
static void tft_display_write_data(uint16_t data);

/**-----------------------------------------------------------------------------
 * 
 * @param bounds1
 * @param bounds2
 * @return 
 */
static bool tft_display_bounds_are_equal(BOUNDS* bounds1, BOUNDS* bounds2)
{
    return ((bounds1->top == bounds2->top) &&
            (bounds1->left == bounds2->left) &&
            (bounds1->bottom == bounds2->bottom) &&
            (bounds1->right == bounds2->right));
}

/**
 * 
 * @param bounds
 * @param dest
 */
static void tft_display_bounds_copy(BOUNDS* bounds, BOUNDS* dest)
{
    dest->top = bounds->top;
    dest->left = bounds->left;
    dest->bottom = bounds->bottom;
    dest->right = bounds->right;
}

/**-----------------------------------------------------------------------------
 * 
 * @param on
 */
static void tft_display_chip_select(bool on)
{
    PLIB_PORTS_PinWrite(TFT_DISPLAY_CS_PORTS_ID, 
                  TFT_DISPLAY_CS_CHANNEL, 
                  TFT_DISPLAY_CS_PIN,
                  !on);
}

/**-----------------------------------------------------------------------------
 * 
 */
static void tft_display_configure()
{
    uint8_t command;
    uint8_t parameters[8];
     
    parameters[0] = 0x00;
    tft_display_send_command(TFT_COMMAND_SET_DISPLAY_ON, parameters, 1);
    
    //SET LCD MODE  SET TFT 18Bits MODE
    parameters[0] = 0x20;    //SET TFT MODE & hsync+Vsync+DEN MODE
    parameters[1] = 0x80;    //SET TFT MODE & hsync+Vsync+DEN MODE
    parameters[2] = 0x01;    //SET horizontal size=320-1 HightByte
    parameters[3] = 0xDF;    //SET horizontal size=320-1 LowByte
    parameters[4] = 0x00;    //SET vertical size=240-1 HightByte
    parameters[5] = 0x7F;    //SET vertical size=240-1 LowByte
    parameters[6] = 0x00;    //SET even/odd line RGB seq.=RGB
    tft_display_send_command(TFT_COMMAND_SET_LCD_MODE, parameters, 7);

    //command = SET_PIXEL_DATA_INTERFACE;
    //parameter[0] = 0x00;       //SET pixel data I/F format=8bit
    parameters[0] = 0x02;       //SET pixel data I/F format=16bit
    tft_display_send_command(TFT_COMMAND_SET_PIXEL_DATA_INTERFACE, parameters, 1);
        
    //command = SET_ADDRESS_MODE;
    //parameter[0] = 0x08;   // SET read from frame buffer to the display is RGB
    //parameters[0] = 0x42;//0x40;   // SET read from frame buffer to the display is RGB
    // No idea why we need 0x02 (Flipped Horizontal) as it should really be 0x00 but this 
    // appears to work??
    parameters[0] = 0x02;//0x40;   
    tft_display_send_command(TFT_COMMAND_SET_ADDRESS_MODE, parameters, 1);
    
    //command = SET_PLL_MN;
    parameters[0] = 0x1D;
    parameters[1] = 0x02;
    parameters[2] = 0x54;  // should be 04
    tft_display_send_command(TFT_COMMAND_SET_PLL_MN, parameters, 3);
    
    //command = SET_LSHIFT_FREQ; //SET PCLK freq=4.94MHz  ; pixel clock frequency
    parameters[0] = 0x01;		//00
	parameters[1] = 0x99;		//ce	
	parameters[2] = 0x9a;		//94
    tft_display_send_command(TFT_COMMAND_SET_LSHIFT_FREQ, parameters, 3);
    
    //command = SET_HORI_PERIOD; //SET HBP,
    parameters[0] = 0x02;		//SET HSYNC Total = 440
	parameters[1] = 0x13;
	parameters[2] = 0x00;			//SET HBP = 68
	parameters[3] = 0x2b;
	parameters[4] = 0x05;			//SET VBP 16 = 15 + 1
	parameters[5] = 0x00;			//SET Hsync pulse start position
	parameters[6] = 0x00;
	parameters[7] = 0x00;			//SET Hsync pulse subpixel start position
    tft_display_send_command(TFT_COMMAND_SET_HORZ_PERIOD, parameters, 8);
     
    // data sheet value
    //command = SET_VERT_PERIOD;
    parameters[0] = 0x01;
    parameters[1] = 0x24;
    parameters[2] = 0x00;
    parameters[3] = 0x10;
    parameters[4] = 0x02;
    parameters[5] = 0x02;
    parameters[6] = 0x05;
    tft_display_send_command(TFT_COMMAND_SET_VERT_PERIOD, parameters, 7);

    //command = SET_COLUMN_ADDRESS;   //SET column address
	parameters[0] = 0x00;   //SET start column address=0
	parameters[1] = 0x00;
	parameters[2] = 0x01;   //SET end column address=479
	parameters[3] = 0xDF;
    tft_display_send_command(TFT_COMMAND_SET_COLUMN_ADDRESS, parameters, 4);
    
    //command = SET_PAGE_ADDRESS; //SET page address
	parameters[0] = 0x00;   //SET start page address=0
	parameters[1] = 0x00;
	parameters[2] = 0x00;   //SET end page address=127
	parameters[3] = 0x7F;
    tft_display_send_command(TFT_COMMAND_SET_PAGE_ADDRESS, parameters, 4);

    //command = SET_GPIO_CONF;   //SET GPIO
	parameters[0] = 0x0F;      //SET I/O
	parameters[1] = 0x01;
    tft_display_send_command(TFT_COMMAND_SET_GPIO_CONF, parameters, 2);
    
    
	//command = SET_GPIO_VALUE;   //SET GPIO
    parameters[0] = 0x01;      //SET I/O
    tft_display_send_command(TFT_COMMAND_SET_GPIO_VALUE, parameters, 1);
    
    //read back test
    //command = GET_GPIO_STATUS;   //SET GPIO
    parameters[0] = 0x00;      
    tft_display_send_command(TFT_COMMAND_GET_GPIO_STATUS, parameters, 1);
    
    //command = SET_DISPLAY_ON;   //SET display on
    parameters[0] = 0x00;
    tft_display_send_command(TFT_COMMAND_SET_DISPLAY_ON, parameters, 0);    

    //
    tft_display_set_brightness(0x80);
}

/**-----------------------------------------------------------------------------
 * Draw the specified rectangle on the screen
 */
static void tft_display_draw_rect(BOUNDS* bounds)
{
    uint16_t row;
    uint16_t column;

    uint8_t x;
    uint8_t colour_idx = 0;
    uint16_t data;
    bool high_byte = true;
    PIXEL* current_pixel;
    
    //logger_log_debugf(0, "TFT Draw Buffer %d %d %d %d --", bounds->top, bounds->left, bounds->bottom, bounds->right);
    
    tft_display_set_ram_address(bounds);
    
    for (row = bounds->top; row <= bounds->bottom; row++)
    {
        for (column = bounds->left; column <= bounds->right; column++)
        {
            current_pixel = &tft_display_data.screen_buffer[row][column];
            
            for (x = 0; x < 3; x++)
            {
                if (high_byte)
                {
                    switch(colour_idx)
                    {
                        case 0:
                            data = current_pixel->red;
                            break;
                        case 1:
                            data = current_pixel->green;
                            break;
                        case 2:
                            data = current_pixel->blue;
                            break;
                    }
                    data <<= 8;
                }
                else
                {
                    
                    switch(colour_idx)
                    {
                        case 0:
                            data |= current_pixel->red;
                            break;
                        case 1:
                            data |= current_pixel->green;
                            break;
                        case 2:
                            data |= current_pixel->blue;
                            break;
                    }
 
                    tft_display_write_data(data);
                    data = 0;
                }

                colour_idx = (colour_idx + 1) % 3; // increment colour
                high_byte = !high_byte; // toggle byte selection
            }
        }
    }
    
    // Write the last pixel
    if (!high_byte)
    {
        tft_display_write_data(data);
    }
}

/**-----------------------------------------------------------------------------
 * 
 */
static void tft_display_lock_pll()
{
    uint8_t parameters[1];
    parameters[0] = 0x03; // lock PLL
    tft_display_send_command(TFT_COMMAND_SET_PLL, parameters, 1);
}

/**-----------------------------------------------------------------------------
 * 
 * @param data
 */
static uint16_t tft_display_read_data()
{
    uint16_t result = 0;
    
    tft_display_chip_select(true);
    
    result = DRV_PMP0_Read();
    
    tft_display_chip_select(false);
    
    return result;
}

/**-----------------------------------------------------------------------------
 * 
 * @param on
 */
static void tft_display_reset(bool on)
{
    PLIB_PORTS_PinWrite(TFT_DISPLAY_RESET_PORTS_ID, 
                  TFT_DISPLAY_RESET_CHANNEL, 
                  TFT_DISPLAY_RESET_PIN,
                  !on);
}

/**-----------------------------------------------------------------------------
 * 
 * @param command
 */
static void tft_display_send_command(TFT_COMMAND command, uint8_t* parameters, uint8_t count)
{
    uint16_t data = TFT_COMMAND_VALUES[command];
    
    tft_display_set_command_mode();
    timers_delay_ms(1);
    
    tft_display_write_data(data);
    
    tft_display_set_data_mode();
    timers_delay_ms(1);
    
    int x;
        
    //logger_log_debugf(0, "TFT Send Command %d 0X%X %s %d %d --", command, TFT_COMMAND_VALUES[command], TFT_COMMAND_STRINGS[command], count, TFT_COMMAND_WRITE[command]);
    for (x = 0; x < count; x++)
    {
        if (!TFT_COMMAND_WRITE[command])
        {
            data = tft_display_read_data();
            parameters[x] = (uint8_t)data;  //TODo - verify this is returning the value
        }
        else // write data
        {
            data = (uint16_t)parameters[x];
            tft_display_write_data(data);
        }
    }

}

/**-----------------------------------------------------------------------------
 * 
 * @param on
 */
static void tft_display_set_backlight(bool on)
{
    PLIB_PORTS_PinWrite(TFT_DISPLAY_BACKLIGHT_PORTS_ID, 
                  TFT_DISPLAY_BACKLIGHT_CHANNEL, 
                  TFT_DISPLAY_BACKLIGHT_PIN,
                  !on);
}

/**-----------------------------------------------------------------------------
 * 
 */
static void tft_display_set_command_mode()
{
    PLIB_PORTS_PinClear(TFT_DISPLAY_DC_PORTS_ID, 
                  TFT_DISPLAY_DC_CHANNEL, 
                  TFT_DISPLAY_DC_PIN);
}

/**-----------------------------------------------------------------------------
 * 
 */
static void tft_display_set_data_mode()
{
    PLIB_PORTS_PinSet(TFT_DISPLAY_DC_PORTS_ID, 
                  TFT_DISPLAY_DC_CHANNEL, 
                  TFT_DISPLAY_DC_PIN);
}

/**-----------------------------------------------------------------------------
 * 
 */
static void tft_display_set_dimensions()
{
}

/**-----------------------------------------------------------------------------
 * 
 */
static void tft_display_set_ram_address(BOUNDS* bounds)
{
    uint8_t parameters[5];
    
    //command = SET_COLUMN_ADDRESS;
    parameters[0] = min(bounds->left, bounds->right) >> 8;      //0x00; // start column address = 0
    parameters[1] = min(bounds->left, bounds->right) & 0xFF;    //0x00; 
    parameters[2] = max(bounds->left, bounds->right) >> 8;     //(TFT_DISPLAY_WIDTH - 1) >> 8; // set end column address
    parameters[3] = max(bounds->left, bounds->right) & 0xFF;   //((TFT_DISPLAY_WIDTH - 1) & 0xFF);
    
    tft_display_send_command(TFT_COMMAND_SET_COLUMN_ADDRESS, parameters, 4);
    
    //command = SET_PAGE_ADDRESS;
    parameters[0] = bounds->top >> 8;       //0x00; // start page address = 0
    parameters[1] = bounds->top & 0xFF;     //0x00; 
    parameters[2] = bounds->bottom >> 8;    //((TFT_DISPLAY_HEIGHT - 1) >> 8) + 0x00; // set page column address
    parameters[3] = bounds->bottom & 0xFF;  //(TFT_DISPLAY_HEIGHT - 1 - 0x10) & 0xFF;
    tft_display_send_command(TFT_COMMAND_SET_PAGE_ADDRESS, parameters, 4);
    
    //command = WRITE_MEMORY_START;
    tft_display_send_command(TFT_COMMAND_WRITE_MEMORY_START, parameters, 0);
}

/**-----------------------------------------------------------------------------
 * 
 * @param on
 */
static void tft_display_set_read(bool on)
{
    PLIB_PORTS_PinWrite(TFT_DISPLAY_READ_PORTS_ID, 
                  TFT_DISPLAY_READ_CHANNEL, 
                  TFT_DISPLAY_READ_PIN,
                  !on);
}

/**-----------------------------------------------------------------------------
 * 
 */
static void tft_display_set_pll()
{
    uint8_t parameters[1];
    parameters[0] = 0x01; // start PLL
    tft_display_send_command(TFT_COMMAND_SET_PLL, parameters, 1);
}

/**-----------------------------------------------------------------------------
 * 
 */
static void tft_display_soft_reset()
{
    uint8_t parameters[1];
    parameters[0] = 0x00; // 
    tft_display_send_command(TFT_COMMAND_SOFT_RESET, parameters, 1);
}

/**-----------------------------------------------------------------------------
 * 
 * @param on
 */
static void tft_display_set_write(bool on)
{
    PLIB_PORTS_PinWrite(TFT_DISPLAY_WRITE_PORTS_ID, 
                  TFT_DISPLAY_WRITE_CHANNEL, 
                  TFT_DISPLAY_WRITE_PIN,
                  !on);
}

/**-----------------------------------------------------------------------------
 * 
 * @param on
 */
static void tft_display_power(bool on)
{
    PLIB_PORTS_PinWrite(TFT_DISPLAY_POWER_PORTS_ID, 
                  TFT_DISPLAY_POWER_CHANNEL, 
                  TFT_DISPLAY_POWER_PIN,
                  !on);
}

/**-----------------------------------------------------------------------------
 * 
 * @param data
 */
static void tft_display_write_data(uint16_t data)
{
    tft_display_chip_select(true);
    
    DRV_PMP0_Write(data);
    
    tft_display_chip_select(false);
}

/* *
 ************************************************************************* */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * 
 * @param bounds
 * @return 
 */
uint16_t tft_display_bounds_area(BOUNDS* bounds)
{
    return tft_display_bounds_height(bounds) * tft_display_bounds_width(bounds); 
}

/**-----------------------------------------------------------------------------
 * 
 * @param bounds
 * @return 
 */
uint16_t tft_display_bounds_height(BOUNDS* bounds)
{
    return bounds->bottom - bounds->top + 1;
}

/**-----------------------------------------------------------------------------
 * 
 * @param bounds
 * @return 
 */
uint16_t tft_display_bounds_width(BOUNDS* bounds)
{
    return bounds->right - bounds->left + 1;
}

/**-----------------------------------------------------------------------------
 */
 void tft_display_clear()
{
    //logger_log_debugf(LOGGER_CODE_TFT_DISPLAY, "TFT Clear Display --");
    
    // set the buffer to the background colour
    PIXEL background;
    background.red = 0x30;
    background.green = 0x30;
    background.blue = 0X30;
    
    int row;
    int column;
    for (row = 0; row < TFT_DISPLAY_HEIGHT; row++)
    {
        for (column = 0; column < TFT_DISPLAY_WIDTH; column++)
        {
            PIXEL* pixel = &tft_display_data.screen_buffer[row][column];
            colour_set(pixel, COLOUR_INDEX_BACKGROUND);
            
            // Draw a grid for development
//            if ((row % 10 == 0) || (column % 10 == 0))
//            {
//                colour_set(pixel, COLOUR_INDEX_PROGRESS_BAR_BACKGROUND);                
//            }
        }
    }
    
    tft_display_invalidate_rect(&FULL_SCREEN_BOUNDS);
    
    // Done after th invalidate as that sets it to false
    tft_display_data.is_clear = true;
}

/**-----------------------------------------------------------------------------
 * 
 */
void tft_display_draw()
{
    if (tft_display_data.invalidated_rect_count > 0)
    {
        // Switch the backlight on
        if (tft_display_get_brightness() > 0)
        {
            tft_display_set_backlight(true);  
        }
    }
    
    // Review scroll groups 
    
    int x;
    for (x = 0; x < tft_display_data.invalidated_rect_count; x++)
    {
        tft_display_draw_rect(&tft_display_data.invalidated_rects[x]);
    }
    
    // Reset the invalidated rect count
    tft_display_data.invalidated_rect_count = 0;
}
 
/**-----------------------------------------------------------------------------
 * 
 * @param pixels
 * @param bounds
 * @param colour
 */
void tft_display_draw_character(PIXEL* pixels, 
        BOUNDS* pixel_bounds, 
        int16_t top_offset, 
        int16_t left_offset, 
        FONT_ID font_id, 
        char* character,
        COLOUR_INDEX foreground_colour,
        COLOUR_INDEX background_colour)
{
    uint8_t character_height = font_manager_get_font_height(font_id);
    uint8_t character_width = font_manager_get_character_width(font_id, character);
    uint8_t pixels_per_byte = font_manager_get_pixels_per_byte(font_id);
    uint16_t pixels_width = tft_display_bounds_width(pixel_bounds);
    uint16_t pixels_height = tft_display_bounds_height(pixel_bounds);

    // Define the bounds that the character will be draw in (relative to the control bounds))
    BOUNDS character_bounds;
    character_bounds.top = top_offset;
    character_bounds.left = left_offset;
    character_bounds.bottom = character_bounds.top + character_height - 1;
    character_bounds.right = character_bounds.left + character_width - 1;

    COLOUR* background = colour_get(background_colour);
    
    // get the font representation of the data
    char* character_data = (char*)font_manager_get_character_data(font_id, character);

    if (character_data != NULL)
    {

//logger_log_debugf(0, "TFT CTRL %d %d %d %d = %d %d %d = %d %d  --",
//    character_bounds.top,
//    character_bounds.left,
//    character_bounds.bottom,
//    character_bounds.right,
//    pixels_per_byte,  pixels_width, pixels_height, character_height, character_width);    
//logger_process();
    
        // We attempt 
        int16_t row = 0;
        int16_t column = 0;
        for (row = character_bounds.top; row <= character_bounds.bottom; row++)
        {
            uint16_t pixel_row_offset = (row * pixels_width);

            // keep track of which column in the character we are drawing
            // this will go from 0-8/pixels_per byte
            uint8_t column_count = 0;

            for (column = character_bounds.left; column <= character_bounds.right; column++)
            {
                // Only draw the pixel if its within the bounds of the pixels
                if (((row >= 0) && (row < pixels_height)) &&
                    ((column >= 0) && (column < pixels_width)))
                {
                    PIXEL* pixel = &pixels[pixel_row_offset + column];

                    // Get the saturation (intensity) for the current pixel)
                    uint8_t saturation = font_manager_get_pixel_saturation(font_id, character_data, column_count);

                    if (saturation > 0x00)
                    {
                        // This will need some work for non-black backgrounds to merge
                        // the colour with the background , otherwise it tends to black which does no look nice....
                        colour_set(pixel, foreground_colour);
                        colour_saturate(pixel, saturation, background);
                    }
                    else
                    {
                        // TODO Check if colour is transparent - if so do nothing.....
                        colour_set(pixel, background_colour);
                    }
                }

                column_count++;

                // Check if we need to move to the next character if we have used up the current chracter data
                // But NOT if were at the end of the row (as that will be done below))
                if ((column < character_bounds.right) && (column_count % pixels_per_byte == 0))
                {   
                    // move to the next character
                    character_data++;

                    // reset the column counter
                    column_count = 0;
                }
            }

            // Move on character data at end of row
            character_data++;

            // Check if we are below the bottom row and break.....
            if (row >= pixels_height - 1)
            {
                break;
            }
        }
    }
    else
    {
        tft_display_draw_rectangle(pixels, &character_bounds, foreground_colour);
    }
}
 
/**-----------------------------------------------------------------------------
 * 
 * @param pixels
 * @param bounds
 * @param colour
 */
void tft_display_draw_unicode_character(PIXEL* pixels, 
        BOUNDS* pixel_bounds, 
        int16_t top_offset, 
        int16_t left_offset, 
        FONT_ID font_id, 
        UNICODE_CHAR* character,
        COLOUR_INDEX foreground_colour,
        COLOUR_INDEX background_colour)
{
    uint8_t character_height = font_manager_get_font_height(font_id);
    uint8_t character_width = font_manager_get_unicode_character_width(font_id, character);
    uint8_t pixels_per_byte = font_manager_get_pixels_per_byte(font_id);
    uint16_t pixels_width = tft_display_bounds_width(pixel_bounds);
    uint16_t pixels_height = tft_display_bounds_height(pixel_bounds);

    // Define the bounds that the character will be draw in (relative to the control bounds))
    BOUNDS character_bounds;
    character_bounds.top = top_offset;
    character_bounds.left = left_offset;
    character_bounds.bottom = character_bounds.top + character_height - 1;
    character_bounds.right = character_bounds.left + character_width - 1;

    COLOUR* background = colour_get(background_colour);
    
    // get the font representation of the data
    char* character_data = (char*)font_manager_get_unicode_character_data(font_id, character);

    if (character_data != NULL)
    {

//logger_log_debugf(0, "TFT CTRL %d %d %d %d = %d %d %d = %d %d  --",
//    character_bounds.top,
//    character_bounds.left,
//    character_bounds.bottom,
//    character_bounds.right,
//    pixels_per_byte,  pixels_width, pixels_height, character_height, character_width);    
//logger_process();
    
        // We attempt 
        int16_t row = 0;
        int16_t column = 0;
        for (row = character_bounds.top; row <= character_bounds.bottom; row++)
        {
            uint16_t pixel_row_offset = (row * pixels_width);

            // keep track of which column in the character we are drawing
            // this will go from 0-8/pixels_per byte
            uint8_t column_count = 0;

            for (column = character_bounds.left; column <= character_bounds.right; column++)
            {
                // Only draw the pixel if its within the bounds of the pixels
                if (((row >= 0) && (row < pixels_height)) &&
                    ((column >= 0) && (column < pixels_width)))
                {
                    PIXEL* pixel = &pixels[pixel_row_offset + column];

                    // Get the saturation (intensity) for the current pixel)
                    uint8_t saturation = font_manager_get_pixel_saturation(font_id, character_data, column_count);

                    if (saturation > 0x00)
                    {
                        // This will need some work for non-black backgrounds to merge
                        // the colour with the background , otherwise it tends to black which does no look nice....
                        colour_set(pixel, foreground_colour);
                        colour_saturate(pixel, saturation, background);
                    }
                    else
                    {
                        // TODO Check if colour is transparent - if so do nothing.....
                        colour_set(pixel, background_colour);
                    }
                }

                column_count++;

                // Check if we need to move to the next character if we have used up the current chracter data
                // But NOT if were at the end of the row (as that will be done below))
                if ((column < character_bounds.right) && (column_count % pixels_per_byte == 0))
                {   
                    // move to the next character
                    character_data++;

                    // reset the column counter
                    column_count = 0;
                }
            }

            // Move on character data at end of row
            character_data++;

            // Check if we are below the bottom row and break.....
            if (row >= pixels_height - 1)
            {
                break;
            }
        }
    }
    else
    {
        tft_display_draw_rectangle(pixels, &character_bounds, foreground_colour);
    }
}

/**-----------------------------------------------------------------------------
 * 
 * @param bounds
 */
void tft_display_draw_rectangle(PIXEL* pixels, BOUNDS* bounds, COLOUR_INDEX colour_idx)
{
    // Clear the background - change to a draw rectangle
     uint16_t x;
     for (x = 0; x < (bounds->bottom - bounds->top + 1) * (bounds->right - bounds->left + 1); x++)
     {
         colour_set(&pixels[x], colour_idx);
     }
}
    
/**-----------------------------------------------------------------------------
 * Sets the brightness in the range 0-128
 */
static uint8_t tft_display_get_brightness()
{
    uint8_t parameters[5];
        
    tft_display_send_command(TFT_COMMAND_GET_POST_PROC, parameters, 5);
//    logger_log_debugf(0, "TFT DISPLAY POST PROC %d %d %d %d %d %d --",
//            parameters[0],
//            parameters[1],
//            parameters[2],
//            parameters[3],
//            parameters[4],
//            parameters[5]);
    
    return parameters[2];
}

/**-----------------------------------------------------------------------------
 * 
 * @param on
 */
bool tft_display_get_backlight()
{
    return PLIB_PORTS_PinGet(TFT_DISPLAY_BACKLIGHT_PORTS_ID, 
                  TFT_DISPLAY_BACKLIGHT_CHANNEL, 
                  TFT_DISPLAY_BACKLIGHT_PIN);//TODO needs setting to low (!on) when Pete had done the backlight stuff
}

/**-----------------------------------------------------------------------------
 * Gets the character index at the supplied pixel position
 * @param bounds
 * @param pixels
 * @param invalidate
 */
uint16_t tft_display_get_character_at_pos(FONT_ID font_id, char* text, uint16_t pos)
{
    uint16_t result = 0;
    uint16_t text_width = 0;
    
    int x;
    for (x = 0; x < strlen(text); x++)
    {
        text_width += font_manager_get_character_width(font_id, &text[x]);
        if (text_width >= pos + 1)
        {
            result = x;
            break;
        }
    }
    
    return result;
}

/**-----------------------------------------------------------------------------
 * Gets the character index at the supplied pixel position
 * @param bounds
 * @param pixels
 * @param invalidate
 */
uint16_t tft_display_get_unicode_character_at_pos(FONT_ID font_id, UNICODE_CHAR* text, uint16_t pos)
{
    uint16_t result = 0;
    uint16_t text_width = 0;
    
    int x;
    for (x = 0; x < font_manager_get_unicode_text_length(text); x++)
    {
        text_width += font_manager_get_unicode_character_width(font_id, &text[x]);
        if (text_width >= pos + 1)
        {
            result = x;
            break;
        }
    }
    
    return result;
}

/**-----------------------------------------------------------------------------
 * 
 * @param bounds
 */
uint16_t tft_display_get_text_width(FONT_ID font_id, char* text)
{
    uint16_t result = 0;
    
    int x;
    for (x = 0; x < strlen(text); x++)
    {
        uint16_t width = font_manager_get_character_width(font_id, &text[x]);
        if (width > 0)
        {
            result += width;
        }
        else
        {
            result += font_manager_get_font_average_width(font_id);
        }
    }
    
    return result;
}

/**-----------------------------------------------------------------------------
 * 
 * @param bounds
 */
uint16_t tft_display_get_unicode_text_width(FONT_ID font_id, UNICODE_CHAR* text)
{
    uint16_t result = 0;
    
    int x;
    for (x = 0; x < font_manager_get_unicode_text_length(text); x++)
    {
        uint16_t width = font_manager_get_unicode_character_width(font_id, &text[x]);
        if (width > 0)
        {
            result += width;
        }
        else
        {
            result += font_manager_get_font_average_width(font_id);
        }
    }
    
    return result;
}

/**-----------------------------------------------------------------------------
 * Initialise
 */
void tft_display_initialise()
{
    logger_log_debugf(LOGGER_CODE_TFT_DISPLAY, "TFT DISPLAY Initialise --");
    
    // Clear down the display buffer
    
    // Switch the backlight on
    //tft_display_set_backlight(true);  
    tft_display_set_backlight(false);  
    
    // Set Chip select high
    tft_display_chip_select(false);
    
    tft_display_reset(false);
    
    timers_reset(TIMER_INDEX_TFT_DISPLAY_INITIALISE);
}

/**-----------------------------------------------------------------------------
 * We are assuming that each control will only draw on its own part of the display and
 * will not overlap with other controls.  This means we are eother invalidating separte rectangles or 
 * the whole screen which makes RECT comparison easier
 */ 
void tft_display_invalidate_rect(BOUNDS* bounds)
{
    tft_display_data.is_clear = false;

    if (tft_display_bounds_are_equal(bounds, &FULL_SCREEN_BOUNDS))
    {
        // The new rect is full screen
        BOUNDS* dest = &tft_display_data.invalidated_rects[0];
        tft_display_bounds_copy(bounds, dest);
        tft_display_data.invalidated_rect_count = 1;
        //logger_log_debugf(0, "TFT Invalidate FULL SCREEN Rect %d %d %d %d --", bounds->top, bounds->left, bounds->bottom, bounds->right);
    }
    else if ((tft_display_data.invalidated_rect_count > 0) && 
             (tft_display_bounds_are_equal(&tft_display_data.invalidated_rects[0], &FULL_SCREEN_BOUNDS)))
    {
        // The existing rect is full screen, so do nothing
        //logger_log_debugf(0, "TFT Invalidate Already FULL SCREEN so Ignore %d %d %d %d --", bounds->top, bounds->left, bounds->bottom, bounds->right);
    }
    else if (++tft_display_data.invalidated_rect_count <= MAX_INVALIDATED_RECTS)
    {
        //logger_log_debugf(0, "TFT Invalidate Rect %d %d %d %d --", bounds->top, bounds->left, bounds->bottom, bounds->right);
        
        BOUNDS* dest = &tft_display_data.invalidated_rects[tft_display_data.invalidated_rect_count - 1];
        
        dest->top = max(bounds->top, 0);
        dest->left = max(bounds->left, 0);
        dest->bottom = min(bounds->bottom, TFT_DISPLAY_HEIGHT - 1);
        dest->right = min(bounds->right, TFT_DISPLAY_WIDTH - 1);
    }
}
    
/**-----------------------------------------------------------------------------
 * Indicates whether the display has been cleared down or not - used to determine whether
 * to redraw the screen
 */
bool tft_display_is_clear()
{
    return tft_display_data.is_clear;
}

//////////////////////////////////////////////////////////////////////////
//
//
//
//void tft_display_reset_bodge()
//{
//    tft_display_data.state = TFT_DISPLAY_STATE_INITIALISE;
//}
//
//
///////////////////////////////////////////////////////////////////////////////

/**-----------------------------------------------------------------------------
 * 
 */
void tft_display_process()
{
    switch (tft_display_data.state)
    {
        case TFT_DISPLAY_STATE_INITIALISE:
        {
            logger_log_debugf(LOGGER_CODE_TFT_DISPLAY, "TFT DISPLAY TFT_DISPLAY_STATE_INITIALISE --");
            if (timers_has_expired(TIMER_INDEX_TFT_DISPLAY_INITIALISE))
            {
                tft_display_reset(true);
                
                // Set PIN G15 to low
                tft_display_power(true);
                
                timers_reset(TIMER_INDEX_TFT_DISPLAY_INITIALISE);
                
                tft_display_data.state = TFT_DISPLAY_STATE_RESET;
            }
            break;
        }

        case TFT_DISPLAY_STATE_RESET:
        {
            if (timers_has_expired(TIMER_INDEX_TFT_DISPLAY_INITIALISE))
            {
                logger_log_debugf(LOGGER_CODE_TFT_DISPLAY, "TFT DISPLAY TFT_DISPLAY_STATE_RESET --");
                tft_display_reset(false);
                timers_reset(TIMER_INDEX_TFT_DISPLAY_INITIALISE);
                
                tft_display_data.state = TFT_DISPLAY_STATE_SOFT_RESET;
            }
            break;
        }
        
        case TFT_DISPLAY_STATE_SOFT_RESET:
        {
            if (timers_has_expired(TIMER_INDEX_TFT_DISPLAY_INITIALISE))
            {
                logger_log_debugf(LOGGER_CODE_TFT_DISPLAY, "TFT DISPLAY TFT_DISPLAY_STATE_SOFT_RESET --");
                tft_display_soft_reset();
                timers_reset(TIMER_INDEX_TFT_DISPLAY_INITIALISE);
                
                tft_display_data.state = TFT_DISPLAY_STATE_SET_PLL;
            }
            break;
        }
        
        case TFT_DISPLAY_STATE_SET_PLL:        
        {
            if (timers_has_expired(TIMER_INDEX_TFT_DISPLAY_INITIALISE))
            {
                logger_log_debugf(LOGGER_CODE_TFT_DISPLAY, "TFT DISPLAY TFT_DISPLAY_STATE_SET_PLL --");
                tft_display_set_pll();
                timers_reset(TIMER_INDEX_TFT_DISPLAY_INITIALISE);
                
                tft_display_data.state = TFT_DISPLAY_STATE_LOCK_PLL;
            }
            break;
        }
        
        case TFT_DISPLAY_STATE_LOCK_PLL:        
        {
            if (timers_has_expired(TIMER_INDEX_TFT_DISPLAY_INITIALISE))
            {
                logger_log_debugf(LOGGER_CODE_TFT_DISPLAY, "TFT DISPLAY TFT_DISPLAY_STATE_LOCK_PLL --");
                tft_display_lock_pll();
                timers_reset(TIMER_INDEX_TFT_DISPLAY_INITIALISE);
                
                tft_display_data.state = TFT_DISPLAY_STATE_CONFIGURE;
            }
            break;
        }
        
        case TFT_DISPLAY_STATE_CONFIGURE:        
        {
            if (timers_has_expired(TIMER_INDEX_TFT_DISPLAY_INITIALISE))
            {
                logger_log_debugf(LOGGER_CODE_TFT_DISPLAY, "TFT DISPLAY TFT_DISPLAY_STATE_CONFIGURE --");
                tft_display_configure();
                timers_reset(TIMER_INDEX_TFT_DISPLAY_INITIALISE);

                tft_display_clear();
                tft_display_draw();
                
                tft_display_data.state = TFT_DISPLAY_STATE_RUNNING;
            }
            break;
        }
        
        case TFT_DISPLAY_STATE_RUNNING:        
        {
            // update scrolling
            tft_display_draw();
            break;
        }
        
        default:
        {
            break;
        }    
    }
}
    
/**-----------------------------------------------------------------------------
 * Sets the brightness in the range 0-128
 */
void tft_display_set_brightness(uint8_t value)
{
    uint8_t parameters[4];
    
    // Note: Using Post processing to adjust brightness - tried PWM (see below)
    // nut didn't work immediately
    value = min(value, 0x80);

	parameters[0] = value / 2;   
	parameters[1] = value;
	parameters[2] = value / 2;   
	parameters[3] = 0x01;//value == 0xFF ? 0x01 : 0x00;   // Enable / Disable
    tft_display_send_command(TFT_COMMAND_SET_POST_PROC, parameters, 4);
    
    if (value > 0)
    {
        tft_display_set_backlight(true);  
    }
    else
    {
// BUG 3800 - 2 lines below removed so that the current sceen is not cleared down and is redraw after
// the brightness is set back - this seems o contradict the message below so not sure it if will jhave knock on consequences..        
//        tft_display_clear();    // Clear down the display so we don't see what is currently on it when its displayed again
//        tft_display_draw();
        tft_display_set_backlight(false);  
    }
       
//    tft_display_send_command(TFT_COMMAND_GET_POST_PROC, parameters, 5);
//    logger_log_debugf(0, "TFT DISPLAY POST PROC %d %d %d %d %d %d --",
//            parameters[0],
//            parameters[1],
//            parameters[2],
//            parameters[3],
//            parameters[4],
//            parameters[5]);
       
    // PWM Attempt
// 	parameters[0] = 0x0E;   
//	parameters[1] = 0xFF;
//	parameters[2] = 0x09;   
//	parameters[3] = 0x10;
//	parameters[4] = 0x00;
//	parameters[5] = 0x00;
//    tft_display_send_command(TFT_COMMAND_SET_PWM_CONF, parameters, 6);
//    
// 	parameters[0] = 0x09;   
//    tft_display_send_command(TFT_COMMAND_SET_DBC_CONF, parameters, 1);
           
//    tft_display_send_command(TFT_COMMAND_GET_PWM_CONF, parameters, 6);
//    logger_log_debugf(0, "TFT DISPLAY PWN CONF %d %d %d %d %d %d --",
//            parameters[0],
//            parameters[1],
//            parameters[2],
//            parameters[3],
//            parameters[4],
//            parameters[5]);

}

/**-------------------------------------------------------------------------
 * Writes a buffer (as an array) of PIXEL DATA to the screen buffer - needs a call to tft_display_draw afterwards
 * Assumes the buffer length is the same size as the rectangle area
 * If invalidate = false then it is assumed that the area will be invalidated manually
 */
void tft_display_write_buffer(BOUNDS* bounds, PIXEL* pixels, bool invalidate)
{
    uint16_t x = 0;
    uint16_t row;
    uint16_t column;
    PIXEL* current_pixel;
    
    uint16_t width = bounds->right - bounds->left + 1;
    
    //logger_log_debugf(0, "TFT Write Buffer %d %d %d %d --", bounds->top, bounds->left, bounds->bottom, bounds->right);
    //logger_process();

    for (row = bounds->top; row <= bounds->bottom; row++)
    {
        for (column = bounds->left; column <= bounds->right; column++)
        {
            if (((row >= 0) && (row < TFT_DISPLAY_HEIGHT)) &&
                ((column >= 0) && (column < TFT_DISPLAY_WIDTH)))    
            {
                // Note display draws fro right to left, top to bottom
                current_pixel = &tft_display_data.screen_buffer[row][column];  // Note to flip bounds->right - column + bounds->left
                current_pixel->red = pixels[x].red;
                current_pixel->green = pixels[x].green;
                current_pixel->blue = pixels[x].blue;            
            }
            x++;
        }
    }
    
    //logger_log_debugf(0, "TFT Write Buffer B %d %d %d %d --", bounds->top, bounds->left, bounds->bottom, bounds->right);
    //logger_process();
    
    if (invalidate)
    {
        tft_display_invalidate_rect(bounds);
    }
}

/* *****************************************************************************
 End of File
 */
