/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    volume_hw_es9038.c

  @Summary
    Implements volume_hw functions (see volume.h) for the ES9038 DAC.(ST60)

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

/* This section lists the other files that are included in this file.
 */

#include "utilities/volume.h"
#include "app_config.h"
#include "utilities\input_control_dac.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Used to perform any specific initialisation of the Volume hardware
 */
void volume_hw_initialise()
{
    // No initialisation required
}
     
/**-----------------------------------------------------------------------------
 * Used to ensure that the Volume component (On the SA30) stays alive.
 */
void volume_hw_keep_alive(uint8_t current_volume)
{
    // Do nothing
}
   
/**-----------------------------------------------------------------------------
 * Used in volume mute to allow the hardware to do mute specific things
 */
void volume_hw_mute()
{
    // Set the volume to zero in an attemt to stop the double un-mute when ramping up the volume in an unmute
    logger_log_debugf(0, "VOLUME MUTING HW --");
    volume_hw_set_volume(0, 0);
}
  
/**-----------------------------------------------------------------------------
 * Used to set the volume.  The volumes passed are the mapped value from the 
 * volume_mappings and are in the range 1-247 (1 being silence, 247 being max volume)
 * @param left
 * @param right
 */
void volume_hw_set_volume(uint8_t left, uint8_t right)
{
    uint8_t left_adjusted = max(left, 0x06); // min so we don't roll round when subtracting 5 below
    uint8_t right_adjusted = max(right, 0x06);
    
    // BUG 3661 - Add 2.5dB to DAC
    //TODO This is ST60 specific so will need generalising if used again....
    left_adjusted -= 0x05;
    right_adjusted -= 0x05;
    
    // Convert the volume into the range 0x08 (max volume) to 0xFE (min volume)
    input_control_set_dac_volume(0xFF - left_adjusted, 0xFF - right_adjusted);
}

/* *****************************************************************************
 End of File
 */
