/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    volume_hw_pga2311.c

  @Summary
    Implements volume_hw functions (see volume.h) for the PGA2311.(SA30)

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

/* This section lists the other files that are included in this file.
 */

#include "utilities/volume.h"
#include "app_config.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */


/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Disable Chip Select pin
 */
static void disable_transfer()
{
#if VOLUME_PINS_ENABLED
    PLIB_PORTS_PinSet(VOLUME_CS_PORTS_ID, 
                      VOLUME_CS_PORT_CHANNEL, 
                      VOLUME_CS_PIN);
#endif
}

/**-----------------------------------------------------------------------------
 * Enable CS pin
 */
static void enable_transfer()
{
#if VOLUME_PINS_ENABLED
    PLIB_PORTS_PinClear(VOLUME_CS_PORTS_ID, 
                        VOLUME_CS_PORT_CHANNEL, 
                        VOLUME_CS_PIN);
#endif
}

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Used to perform any specific initialisation of the Volume hardware
 */
void volume_hw_initialise()
{
#if VOLUME_PINS_ENABLED   

    // Setup the pins
    disable_transfer();

    PLIB_PORTS_PinSet(VOLUME_SCLK_PORTS_ID, 
                      VOLUME_SCLK_PORT_CHANNEL, 
                      VOLUME_SCLK_PIN);
    PLIB_PORTS_PinClear(VOLUME_MOSI_PORTS_ID, 
                        VOLUME_MOSI_PORT_CHANNEL, 
                        VOLUME_MOSI_PIN);

#endif
}
      
/**-----------------------------------------------------------------------------
 * Used to ensure that the Volume component (On the SA30) stays alive.
 */
void volume_hw_keep_alive(uint8_t current_volume)
{
    volume_set_volume(current_volume);
}
    
/**-----------------------------------------------------------------------------
 * Used in volume mute to allow the hardware to do mute specific things
 */
void volume_hw_mute()
{
    // Do nothing
}
  
/**-----------------------------------------------------------------------------
 * Used to set the volume 
 * @param left
 * @param right
 */
void volume_hw_set_volume(uint8_t left, uint8_t right)
{
#if VOLUME_PINS_ENABLED
    
 	uint16_t leftright = left | (right << 8);
 	
	// enable transfer
    enable_transfer();

//    logger_log_debugf(0, "VOLUME val: %d    l: %d r: %d      lr:%d bits:%d%d%d%d%d%d%d%d %d%d%d%d%d%d%d%d --",
//            value,
//            left,
//            right,
//            leftright,
//            ((leftright >> 15) & 1) == 1,
//            ((leftright >> 14) & 1) == 1,
//            ((leftright >> 13) & 1) == 1,
//            ((leftright >> 12) & 1) == 1,
//            ((leftright >> 11) & 1) == 1,
//            ((leftright >> 10) & 1) == 1,
//            ((leftright >> 9) & 1) == 1,
//            ((leftright >> 8) & 1) == 1,
//            ((leftright >> 7) & 1) == 1,
//            ((leftright >> 6) & 1) == 1,
//            ((leftright >> 5) & 1) == 1,
//            ((leftright >> 4) & 1) == 1,
//            ((leftright >> 3) & 1) == 1,
//            ((leftright >> 2) & 1) == 1,
//            ((leftright >> 1) & 1) == 1,
//            ((leftright >> 0) & 1) == 1
//            );
    
  	int pos;
	for (pos = 15; pos >= 0; pos--)
	{
		// set data from msb
		//ioport_set_pin_level(SPI_MOSI, BIT_NR_IS_SET(leftright, 15 - count));
        PLIB_PORTS_PinWrite(VOLUME_MOSI_PORTS_ID, 
                            VOLUME_MOSI_PORT_CHANNEL, 
                            VOLUME_MOSI_PIN,
                            ((leftright >> pos) & 1) == 1);
		
		// set clock low
		//ioport_set_pin_level(SPI_SCLK, false);
        PLIB_PORTS_PinClear(VOLUME_SCLK_PORTS_ID, 
                            VOLUME_SCLK_PORT_CHANNEL, 
                            VOLUME_SCLK_PIN);
		
		timers_delay_us(1);
		
		// set clock high
		//ioport_set_pin_level(SPI_SCLK, true);
        PLIB_PORTS_PinSet(VOLUME_SCLK_PORTS_ID, 
                          VOLUME_SCLK_PORT_CHANNEL, 
                          VOLUME_SCLK_PIN);
		
		timers_delay_us(1);
	}

	// terminate transfer
	disable_transfer();
    
#endif   
}

/* *****************************************************************************
 End of File
 */
