
//-------------------------------------------------------------
// app_config.h
//-------------------------------------------------------------

//! A list of modules that support events.  Each module must have a unique id. in the range 0 -> EVENT_MODULES_MAX - 1
typedef enum
{
	EVENT_MODULE_INPUT,
	EVENT_MODULE_VOLUME,
	EVENT_MODULE_MAX,
} EVENT_MODULES;

//! Define the number of modules using events
#define EVENT_MODULES_NUM_MODULES	EVENT_MODULES_MAX

//-------------------------------------------------------------
// example_module.h
//-------------------------------------------------------------

//!  Define the event types used by the module.  These are specicif to the module and 
//  used solely by the module and its event subscribers.
typedef enum
{
	MODULE_EVENT_TYPE_ONE,
	MODULE_EVENT_TYPE_TWO,
	MODULE_EVENT_TYPE_MAX
} MODULE_EVENT_TYPE;

//-------------------------------------------------------------
// example_module.c
//-------------------------------------------------------------

typedef struct
{
	//! An Array of event subscription structures.  These ara managd by the events module when
	// invoking events.  There MUST be enough decalred to handle all the concurrent event 
	// subscriptions for the module
	EVENT_SUBSCRIPTIONS event_subscriptions[the max number of subscriptions to events that this module supports];

	//! Determine whether the process is asynchronous or not
	bool async_process;
}
EXAMPLE_MODULE_DATA;

static EXAMPLE_MODULE_DATA example_module_data = 
{
	.async_process = true,
};

/**-------------------------------------------------------------
 * The callback invoked by the completion of the set method asyn process
 */
static void example_module_set_callback()
{
	events_raise(EVENT_MODULE_ID, event_type, context_data);
}

/**-------------------------------------------------------------
 *
 */
 void example_module_set(uint8_t value, EVENT_CALLBACK_INFO* callback_info)
{
	// Set the value

	// If we want a full blown event raising as a result of this function call
	//------------------------------------------------------------------------
	// Define the subsciption as a transient subscription
	// Subscribe to the event
	event_subscribe(EXAMPLE_MODULE_ID, 
					events_define_subscription(event_type, callback_info, false));

	// Check async - In real life this will either be asynchronous or not...
	if (!example_module_data.async_process)
	{
		events_raise(EVENT_MODULE_ID, event_type, context_data);
	}

	// Whereas if we just want to invoke the callback
	//------------------------------------------------
	// Rather than return a response Invoke the callback
	// An example of the callback expecting a result
	uint8_t callback_result = 0;
	events_invoke(EXAMPLE_MODULE_ID,
					event_type,
					(void*)&callback_result,
					callback_info);
	if (callback_result != 0)
	{
		logger_log_message(0, "The Callback Result is %d --", callback_result);
	}
}

/**-------------------------------------------------------------
 *
 */
void example_module_init()
{
	// Register the events in the EVENTS structure, Volume is being used as an example
	events_register(EVENT_MODULE_VOLUME, module_data.event_subscriptions, max_num_subscriptions);
}

//-------------------------------------------------------------
// example_other_module.c
//-------------------------------------------------------------

typedef struct
{
	uint8_t some_event_data;
} EXAMPLE_OTHER_MODULE_DATA;

static EXAMPLE_OTHER_MODULE_DATA example_other_module_data;

/**-------------------------------------------------------------
 *
 */
void* example_other_module_event_callback(EVENT* event)
{
	uint8_t result = 0;
	
	// Do something event related...
	
	return (void*)result;
}

/**-------------------------------------------------------------
 *
 */
void example_other_module_initialise()
{
	event_subscribe(EXAMPLE_MODULE_ID, 
					events_define_subscription(MODULE_EVENT_TYPE_ONE, 
											   events_define_callback(example_other_module_event_callback, 
																	 (void*)example_other_module_data.some_event_data), 
											   true);
}

/**-------------------------------------------------------------
 *
 */
void example_other_module_set_something()
{
	example_module_set(100, 
					   events_define_callback(example_other_module_event_callback, 
											 (void*)example_other_module_data.some_event_data));
}

//-------------------------------------------------------------
// events.h
//-------------------------------------------------------------

//! Typedef for the module id
typedef uint8_t EVENT_MODULE_ID;

//! An identifier used for individual events.  For a given module
// this will be bin the range 0 -> n-1. As it is used to directly reference the 
// EVENT in the EVENTS list of event by ordinal position.
typedef uint8_t EVENT_TYPE;

//! Describes an event
typedef struct
{
	EVENT_MODULE_ID module_id;

	// A unique identifier for the event type in the set of events used for the curent module.  These should range 0->n-1.
	EVENT_TYPE type;

	// Any data that is context specific to this event being raise
	void* context_data;

	//!  Any data that is associated with the subscription
	void* subscription_data;
} EVENT;

//! Template for an Event callback
// Note it taks a pointer to an EVENT structure and rturns a void* pointr which can be cast 
// depending on the event type.
typedef void* (*EVENTS_CALLBACK)(EVENT* event);

//! Describes the info needed fo a callback
typedef struct
{
	// The callback to be executed when the event is raised
	EVENTS_CALLBACK callback;

	// Subscription specific data to be passed when the event is raised.
	void* data;  
} EVENT_CALLBACK_INFO;

//! Describes a subscription to an event
typedef struct
{
	// Defines the type of event the subscription is for
	EVENT_TYPE type;

	//! Identifs whether the event is persistnt or transient.  If transient
	// it will be removed once its callback has been executed.
	bool persistent;

	// The info needed for the callback to be executed when the event is raised
	EVENT_CALLBACK_INFO callback_info;

	//! Used internally to determin whether he subsription is actually in use
	bool in_use;
} EVENT_SUBSCRIPTION;

//! Container for a modules events.
typedef struct
{
	EVENT_MODULE_ID module_id;

	EVENT_SUBSCRIPTION* subscriptions;

	uint8_t num_subscriptions;
} EVENTS;

void events_raise(EVENT_MODULE_ID module_id, EVENT_TYPE event_type, void* context_data);
void events_subscribe(EVENT_MODULE_ID module_id, EVENT_TYPE event_type, EVENT_SUBSCRIPTION subscription);


//-------------------------------------------------------------
// events.c
//-------------------------------------------------------------

typedef struct
{
	// An array of EVENTS structure one per module
	EVENTS events[EVENT_MODULES_NUM_MODULES];
	
} EVENTS_DATA;

static EVENTS_DATA events_data = {};

/**-----------------------------------------------------------------------------
 * Define  a transient callback info stucture
 */
EVENT_CALLBACK_INFO* events_define_callback(EVENTS_CALLBACK callback, void* data)
{
	EVENT_CALLBACK_INFO result;

	result.callback = callback;
	result.data = data;

	return &result;
}

/**-----------------------------------------------------------------------------
 * Defines a transient event subscription.  Basically subsciptions are only persistent if
 * they are added by other modules wanting to be informed whenever an event is raised.  If 
 * used in a set methid they should be transient and removed once the callback has ben invoked.
 */
EVENT_SUBSCRIPTION* events_define_subscription(EVENT_TYPE type, EVENTS_CALLBACK_INFO* callback_info, bool persistent)
{
	EVENT_SUBSCRIPTION result;

	result=>event_type = event_type;
	result->persistent = persistent;
	memcpy(result->callback_info, callback_info, sizeof(callback_info));

	return &result;
}

/**-----------------------------------------------------------------------------
 * This is called by the module when invoking one of its events.
 data is data that is relevant to the event bing raised at this point in time
 */
void events_invoke(EVENT_MODULE_ID module_id, EVENT_TYPE event_type, void* context_data, EVENT_CALLBACK_INFO* callback_info)
{
	if (callback_info.callback != NULL)
	{
		// Set up the event
		EVENT event;
		event.module_id = module_id;
		event.event_type = event_type;
		event.context_data = context_data;
		event.subscription_data = callback_info.data;

		result = callback_info.callback(&event);
	}
}

/**-----------------------------------------------------------------------------
 * This is called by the module when invoking one of its events.
 data is data that is relevant to the event bing raised at this point in time
 */
void events_raise(EVENT_MODULE_ID module_id, EVENT_TYPE event_type, void* context_data)
{
	if (module_id < EVENT_MODULES_NUM_MODULES)
	{
		EVENTS* events = &events_data.events[module_id];

		// Set up the event
		EVENT event;
		event.module_id = module_id;
		event.event_type = event_type;
		event.context_data = context_data;

		// Find a free subscription slot
		uint8_t x;
		bool found = false;

		for (x = 0; x < events->num_subscriptions; x++)
		{
			EVENT_SUBSCRIPTION* subscription = &events->subscriptions[x];
			if (subscription->in_use && (subscription->type == event_type))
			{
				if (subscription->callback_info.callback != NULL)
				{
					// Updat the event with the current subscription data
					event.subscription_data = subscription->callback_info.data;
					subscription->callback_info.callback(&event);

					// Free if not persistent
					if (!subscription->persistent)
					{
						subscription->in_use = false;
					}
				}

				found = true;
			}
		}

		if (!found)
		{
			logger_log_warningf(LOGGER_CODE_EVENTS, "EVENTS - No subscriptions for event type spcified %d/%d --", module_id, event_type);
		}
	}
	else
	{
		logger_log_warningf(LOGGER_CODE_EVENTS, "EVENTS - Invoke Invalid module spcified %d --", module_id);
	}
}

/**-----------------------------------------------------------------------------
 * Register the modules event list in the modules EVENTS structure
 */
void events_register(EVENT_MODULE_ID module_id, EVENT* events_list, uint8_t num_events, EVENT_SUBSCRIPTION* subscription_list, uint8_t num_subscriptions)
{
	if (module_id < EVENT_MODULES_NUM_MODULES)
	{
		EVENTS*	events = &events_data.events[module_id];
		if (events != NULL)
		{
			events->module_id = module_id;
			events->subscriptions = subscription_list;
			events->num_subscriptions = num_subscriptions;
		}
	}
	else
	{
		logger_log_warningf(LOGGER_CODE_EVENTS, "EVENTS - Register - Invalid module spcified %d --", module_id);
	}
}

/**-----------------------------------------------------------------------------
 * Subscribe to an event
 */
void events_subscribe(EVENT_MODULE_ID module_id, EVENT_SUBSCRIPTION* event_subscription)
{
	if (module_id < EVENT_MODULES_NUM_MODULES)
	{
		EVENTS*	events = &events_data.events[module_id];
		if (events != NULL)
		{
			// Find a free subscription slot
			uint8_t x;
			bool found = false;
			for (x = 0; x < events->num_subscriptions; x++)
			{
				EVENT_SUBSCRIPTION* subscription = &events->subscriptions[x];
				if (subscription->in_use)
				{
					memcpy(subscription, event_subscription, sizeof(EVENT_SUBSCRIPTION));
					subscription->in_use = true;
					found = true;
					break;
				}
			}

			if (!found)
			{
				logger_log_warningf(LOGGER_CODE_EVENTS, "EVENTS - Insufficient space to subscribe to event %d/%d --", module_id, event_type);
			}
		}
	}
	else
	{
		logger_log_warningf(LOGGER_CODE_EVENTS, "EVENTS - Invalid module %d --", module_id);
	}
}

/**-----------------------------------------------------------------------------
 * Unsubscribe from an event -  note it only makes a comparison on event type and callback, so
 * if there ar more than one subscriotion with differing data they will bot be removed
 */
void events_unsubscribe(EVENT_MODULE_ID module_id, EVENT_SUBSCRIPTION* event_subscription)
{
	if (module_id < EVENT_MODULES_NUM_MODULES)
	{
		EVENTS*	events = &events_data.events[module_id];
		if (events != NULL)
		{
			// Find a free subscription slot
			uint8_t x;
			bool found = false;
			for (x = 0; x < events->num_subscriptions; x++)
			{
				EVENT_SUBSCRIPTION* subscription = &events->subscriptions[x];
				if (subscription->in_use)
				{
					if ((subscription->event_type == event_subscription->event_type) && (subscription->callback_info.callback == event_subscription->callback_info.callback))
					subscription->in_use = false;
					found = true;
				}
			}

			if (!found)
			{
				logger_log_warningf(LOGGER_CODE_EVENTS, "EVENTS - Insufficient space to subscribe to event %d/%d --", module_id, event_type);
			}
		}
	}
	else
	{
		logger_log_warningf(LOGGER_CODE_EVENTS, "EVENTS - Invalid module %d --", module_id);
	}
}