

//-------------------------------------------------------------
// app_config.h
//-------------------------------------------------------------

#define MODULE_RUNTIME_HW_INTERFACE	1


//-------------------------------------------------------------
// app_specific_module_code.c
// This si stuff thatvis application specific but which MUST 
// know about the different HW implementations
//-------------------------------------------------------------

#include "module/private/module_data.h"
#include "module/private/module_hw_impl1.h"
#include "module/private/module_hw_impl2.h"

void app_module_initialise()
{
	// Define which functions to use based on a runtime value
	MODULE_HW_FUNCTIONS* hw_functions = module_data_get_hw_functions();
	
	switch (runtime value)
	{
		case 1:
		{
			hw_functions->state_func = module_hw_impl1_state_func;
			break;
		}

		case 2:
		{
			hw_functions->state_func = module_hw_impl2_state_func;
			break;
		}
	}
	
	module_initialise();
}

//-------------------------------------------------------------
// module_data.c
//-------------------------------------------------------------

typdef void (MODULE_GET_STATE_FUNC*) ();

typedef struct
{
	MODULE_GET_STATE_FUNC state_func;
} MODULE_HW_FUNCTIONS;

typedef struct
{
	MODULE_HW_FUNCTIONS module_hw_functions;
} MODULE_DATA;

static MODULE_DATA module_data;

/**
 * Get a reference to the modules hardware functions
 */
MODULE_HW_FUNCTIONS* module_data_get_hw_functions()
{
	return &module_data.module_hw_functions;
}

//-------------------------------------------------------------
// module.c
//-------------------------------------------------------------

void module_get_state()
{
	module_hw_get_state();
}

//-------------------------------------------------------------
// module_hw.h
//-------------------------------------------------------------

/**
 * In a simple implementation where ther is only 1 hardware implementation 
 * in the configuration this can map to a function in the _hw.c filelength
 */
void module_hw_get_state();

//-------------------------------------------------------------
// module_hw.c
//-------------------------------------------------------------

void module_hw_get_state()
{
	MODULE_HW_FUNCTIONS* hw_functions = module_data_get_hw_functions();
	if (hw_functions->state_func != NULL)
	{
		hw_functions->state_func();
	}
}

//-------------------------------------------------------------
// module_hw_impl2.h
//-------------------------------------------------------------

#include "module_hw.h"

void module_hw_impl2_get_state()
{
	// Return the stat of the module
}


//-------------------------------------------------------------
// module_hw.h
//-------------------------------------------------------------

void module_hw_get_state();


//-------------------------------------------------------------
// module_hw_impl1.h -either module_hw_impl1 or module_hw_impl2 
// are include - Not both
//-------------------------------------------------------------

#include "module_hw.h"

void module_hw_get_state()
{
	// Return the stat of the module
}

//-------------------------------------------------------------
// module_hw_impl2.h
//-------------------------------------------------------------

#include "module_hw.h"

void module_hw_get_state()
{
	// Return the stat of the module
}
