/* ************************************************************************** */
/** Descriptive File Name

  @Company
    ARCAM

  @File Name
    adc.h

  @Summary
    Public Interface to the adc module

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef ADC_H    /* Guard against multiple inclusion */
#define ADC_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/adc/adc_defs.h"
#include "app_config.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


// *****************************************************************************
// *****************************************************************************
// Section: Constants                                                        
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/**
 * Required to allocate an input for the ADC module.  This should be called once for
 * each input.
 */
ADC_INPUT_DEFINITION* adc_add_input(ADC_INPUT_ID input);

/**
 * Performs the mV calculation
 * @param input
 * @return 
 */
inline uint64_t adc_calculate_mV(ADC_INPUT_DEFINITION* input)
{
    return (input->current_value * 3300) / ADC_DATA_RESOLUTION;    
}

/**
 *  Initialises the adc module
 * @return 
 */
void adc_initialise();

/**
 * Performs a cycle of adc processing
 * @return 
 */
bool adc_process();
    
/**
 * Gets Whether the input is currently active or not 
 * @return - Whether the input is currently active or not 
 */
void adc_set_input_active(ADC_INPUT_ID input, bool active);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* ADC_H */

/* *****************************************************************************
 End of File
 */
