/* ************************************************************************** */
/** Descriptive File Name

  @Company
    ARCAM

  @File Name
    adc.h

  @Summary
    Public Interface to the adc module

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef ADC_POT_H    /* Guard against multiple inclusion */
#define ADC_POT_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/adc/adc_defs.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


// *****************************************************************************
// *****************************************************************************
// Section: Constants                                                        
// *****************************************************************************
// *****************************************************************************

//! Defines the maximum number of event subscriptions that can be made to the events 
// in this module 
#define ADC_POT_MAX_EVENT_SUBSCRIPIONS  1

// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/**
 * Required to allocate an input for the ADC module.  This should be called once for
 * each input.  There is an assumption that if we want to use the center functions then
 * the number of steps is un-even
 */
void adc_pot_add_input(ADC_INPUT_ID input, uint16_t step_count, ADC_INPUT_STATE state);

/**
 * Returns the current step value for the pot
 * @param input
 * @return 
 */
uint16_t adc_pot_get_current_step(ADC_INPUT_ID input);

/**
 * Returns true if the pot has changed in the last scan
 * @param input
 * @return 
 */
bool adc_pot_has_changed(ADC_INPUT_ID input);

/**
 * Gets the offset of the current step from the calculated center of the range
 * @param input
 * @return 
 */
int16_t adc_pot_get_offset_from_center(ADC_INPUT_ID input);

/**
 * Initialises the ADC Buttons
 */
void adc_pot_initialise();

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* ADC_H */

/* *****************************************************************************
 End of File
 */
