/* ************************************************************************** */
/** Descriptive File Name

  @Company
    ARCAM

  @File Name
    adc.h

  @Summary
    Public Interface to the adc module

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef ADC_VOLTAGE_H    /* Guard against multiple inclusion */
#define ADC_VOLTAGE_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/adc/adc_defs.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


// *****************************************************************************
// *****************************************************************************
// Section: Constants                                                        
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/**
 * Required to allocate an input for the ADC module.  This should be called once for
 * each input.
 */
void adc_voltage_add_input(ADC_INPUT_ID input, ADC_INPUT_STATE state);

/**
 * Gets the calculated voltage on the supplied input in mV
 * @param input_index - The index of the input to be reported on
 * @return - The calculated voltage in mV.  
 * 
 * Note : This will return values for all input types..
 */
uint32_t adc_voltage_get(ADC_INPUT_ID input);

/**
 * Initialises the ADC Buttons
 */
void adc_voltage_initialise();

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* ADC_H */

/* *****************************************************************************
 End of File
 */
