/* ************************************************************************** */
/** Descriptive File Name

  @Company
    ARCAM

  @File Name
    adc_data.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _ADC_DATA_H    /* Guard against multiple inclusion */
#define _ADC_DATA_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/adc/adc_defs.h"
#include "app_config.h"
#include "utilities/events.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */

//! Defines the maximum number o event subscriptions that can be made to the events 
// in this module - this is set to 3 one for each event as its only the input_control module 
// accesses them
#define ADC_MAX_EVENT_SUBSCRIPIONS    1//***TBD***

// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************

//! private data type available only to the adc module.  If this needs
// to be public then move to the _defs.h
typedef struct
{
    //! Hods information on each of the ADC Inputs
    ADC_INPUT_DEFINITION inputs[ADC_INPUT_COUNT];
    
    //!  adc_hw functions.  These should be stored in a static data structure by
    // the hw module and supplied using spdif_rx_define_functions
    //ADC_FUNCTION_DATA functions;
    
	//! An Array of event subscription structures.  These ara managed by the events module when
	// invoking events.  There MUST be enough declared to handle all the concurrent event 
	// subscriptions for the module.  
    // In this case it will be 1 for each event as its only th spdif_rx module subscribing
	EVENT_SUBSCRIPTION event_subscriptions[ADC_MAX_EVENT_SUBSCRIPIONS];    
} ADC_DATA;

// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/**
 * Used to find an ADC_INPUT_DEFINITION.  If not found it returns the next unused 
 * ADC_INPUT_DEFINITION
 * @param input
 * @return 
 */
ADC_INPUT_DEFINITION* adc_data_allocate_input(ADC_INPUT_ID input);

/**
 * Returns a reference to the ADC_DATA structure
 * @return 
 */    
ADC_DATA* adc_data_get();

/**
 * Gets a reference to the supplied input  definition
 * @param input
 * @return 
 */
ADC_INPUT_DEFINITION* adc_data_get_input_definition(ADC_INPUT_ID input);

/**
 * Gets a reference to the supplied input  definition basd on its ordinal position
 * @param input
 * @return 
 */
ADC_INPUT_DEFINITION* adc_data_get_input_definition_by_index(uint8_t index);

/**
 * Initialises the adc data module
 */
void adc_data_initialise();

/* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _ADC_DATA_H */

/* *****************************************************************************
 End of File
 */
