/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _ALBUM_ART_H    /* Guard against multiple inclusion */
#define _ALBUM_ART_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */

// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/**
 * Used to move the write pointer on by however many bytes have been written to the album art data
 */    
void album_art_advance_write_pointer(uint16_t len, bool complete);

/**
 * Resets the album art URL
 */
void album_art_clear_url();

/**
 * Returns a pointer to the current write position
 */    
uint8_t* album_art_get_write_pointer();

/**
 * Used to check if the album art module is currently dealing with the supplied URL
 */    
bool album_art_has_url(char* url);

/**
 * Sets up the album art module to start receiving new album art.  If album art is 
 * currently being populated then a cancellation will be triggered.
 */
bool album_art_prepare_for_url(char* url, size_t len);

/**
 * Processes album art that has been transferred from the avatar, sends it onto the display
 */
void album_art_process();

/**
 * Sets the Ready To Receive flag on the album art
 */
void album_art_set_ready_to_receive();

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
