/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _ANALOGUE_DIRECT_FEATURE_H    /* Guard against multiple inclusion */
#define _ANALOGUE_DIRECT_FEATURE_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include <stdint.h>
#include "device_configuration.h"

#if APP_FEATURE_ANALOGUE_DIRECT == APP_FEATURE_ON    
#include "utilities/analogue_direct.h"
#endif

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif

// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/**
 * Gets current analogue direct state
 * @param on
 */
static inline bool analogue_direct_feature_get_active()
{
    bool result = false;
#if APP_FEATURE_ANALOGUE_DIRECT == APP_FEATURE_ON    
    result = analogue_direct_get_active();
#endif        
    return result;
}

/**
 * Set to true the feature is available
 */
static inline bool analogue_direct_feature_is_present()
{
    bool result = false;
#if APP_FEATURE_ANALOGUE_DIRECT == APP_FEATURE_ON    
    result = true;
#else
    return result;
#endif    
}

/**
 * Sets pins for analogue direct on/off
 * @param on
 */
static inline void analogue_direct_feature_set_active(bool on)
{
#if APP_FEATURE_ANALOGUE_DIRECT == APP_FEATURE_ON    
    analogue_direct_set_active(on);
#endif        
}

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
