/* ************************************************************************** */
/** Descriptive File Name

  @Company
    ARCAM

  @File Name
    app_commands_utilities.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _APP_COMMANDS_UTILITIES_H    /* Guard against multiple inclusion */
#define _APP_COMMANDS_UTILITIES_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdint.h>
#include "utilities/menu.h"
#include "utilities/structured_message.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */

// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/**
 * Sets up the message to display the default screen
 * @param message
 * @return 
 */
void app_commands_display_default_screen(STRUCTURED_MESSAGE* message);
 
/**
 * Get the index of a value in an array of int[]
 * @param value
 * @param array
 * @param array_len
 * @return 
 */
int8_t app_commands_get_array_index_of(int value, int* array, int array_len);

/**
 * Sets up the message to get the default screen
 * @param message
 * @return 
 */
MENU_ID app_commands_get_default_screen();

/**
 * Calculates the next position in an array based on the direction and array length
 * @param idx
 * Note: This version is making an assumption that direction is +/- 1 and not a larger number 
 * when wrapping round,
 * @param direction
 * @param array_len
 */
void app_commands_get_next_array_pos(int8_t* idx, int8_t direction, int array_len);

/**
 * Generate a structure message that can be used to generate a message.
 * @param value
 * @param array
 * @param array_len
 * @return 
 */    
 void app_commands_invoke_message(uint8_t command, STRUCTURED_MESSAGE_CALLBACK status_proc, uint8_t action);

/**
 * Provides a standard set of logic to update the message action parameter 
 * based upon the value of the message.  This can be used in edit functions to save 
 * having to interrogate the message, but can be ignored if neccessary?
 * Assumes data[0] contains the value of a command message
 * So if the message is absolute and the value is 0xF0 it becomes GetStatus
 * If it is 0xF1 it becomes increment
 * If it is 0xF2 it becomes decrement
 * @param message
 */

/**
 * Performs standard processing on a pointer to a value
 * @param message       - the message
 * @param current_value - the current value to be updated
 * @param value         - the value used for update
 * @param increment     - the value to increment/decrement by
 */
void app_commands_process_message_value(STRUCTURED_MESSAGE* message, 
        int16_t* current_value, 
        int16_t* value,
        int16_t increment);

/**
 * Performs standard processing on a pointer to a value based on values in an int array
 * @param message       - the message
 * @param current_value - the current value to be updated
 * @param value         - the value used for update
 * @param array         - a pointer to an int[] array
 * @param array_len     - the number of elements in  the array
 */
void app_commands_process_message_value_array(STRUCTURED_MESSAGE* message, 
        int16_t* current_value, 
        int16_t* value,
        int* array,
        int array_len);

/**
 * Boolean version of process_message_value.  Includes ranbge checking
 * @param message
 * @param current_value
 * @param value
 * @param increment
 * @return true if processed, false if not
 */
bool app_commands_process_message_value_bool(STRUCTURED_MESSAGE* message, 
        int8_t* current_value, 
        int8_t* value,
        bool send_message);

/**
 * Generate a structure message that can be used to generate a request staus message.
 * @param value
 * @param array
 * @param array_len
 * @return 
 */    
 void app_commands_request_status(uint8_t command, STRUCTURED_MESSAGE_CALLBACK status_proc);

/**
 * Uses data[0] from the message to identify whether the message is a standard REques Status/Increment/Decrement 
 * type of message of pure data.
 * @param message
 */
void app_commands_review_message_action(STRUCTURED_MESSAGE* message);

/**
 * Sends a response message to either the RS232 or Ethernet port dependent on the 
 * message source
 * @param message
 */
void app_commands_send_message(STRUCTURED_MESSAGE* message);

/**
 * Start a process
 */
void app_commands_start_process(void (*process_function)(MENU_ID), void (*end_function)());

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
