/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Arcam

  @File Name
    cn_detect.h

  @Summary
 Used to monitor Change Notifications in shutdown mode.
 * Makes use of the CN_DETECT timer which needs defining in app_config.h e.g.
 * 
 *     X(CN_DETECT, 1000, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
 * 
 * Also i've found that some Change Notification sources need the following type 
 * of line adding in App_Init
 * 
 *    PLIB_PORTS_PinChangeNoticeEnable(PORTS_ID_0, CN13);

 * Note also cn_detect_interrupt_handler() needs putting into the appropriate interrupt handler

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _CN_DETECT_H    /* Guard against multiple inclusion */
#define _CN_DETECT_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include <stdint.h>

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif

    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Disables CN event monitoring
     */
    void cn_detect_disable();
    
    /**
     * Enables monitoring of CN events.  This can be run multiple times, however 
     * only the first time will set the initial pause
     * @param pause_ms = The pause in milliseconds after which the cn_detect_get() 
     * message will respond positivly
     * @return - true if the pause is set, false if not
     */
    bool cn_detect_enable(uint16_t pause_ms);
    
    /**
     * Used to determine whether a Change Notification has occurred
     * @return 
     */
    bool cn_detect_get();
    
    /**
     * This should be added to the change notification interrupt handler generated by 
     * Harmony
     */
    void cn_detect_interrupt_handler();
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
