/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _COLOUR_H    /* Guard against multiple inclusion */
#define _COLOUR_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

/* This section lists the other files that are included in this file.
 */
#include "app_config.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    typedef struct
    {
        uint8_t red;
        uint8_t green;
        uint8_t blue;
    } COLOUR;

    
//! Uses COLOUR_ITEMS defined in app_config.h/device_configuration.h
// Now use the input data to define the enum
#define X(a, b, c, d) COLOUR_INDEX_##a,
	typedef enum {COLOUR_ITEMS COLOUR_INDEX_MAX} COLOUR_INDEX;
#undef X
   
// Now use the input data to define the enum
#define X(a, b, c, d) b, c, d,
    static COLOUR COLOURS[] = { COLOUR_ITEMS };
#undef X

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * 
     * @param index
     * @return 
     */
    COLOUR* colour_get(COLOUR_INDEX index);

    /**
     * 
     */
    void colour_initialise();
    
    /**
     * 
     * @param colour
     * @param saturation
     */
    void colour_saturate(COLOUR* colour, uint8_t saturation, COLOUR* background);
    
    /**
     * Sets the supplied colour reference to the colours associated with the supplied index
     * @param colour
     * @param index
     */
    void colour_set(COLOUR* colour, COLOUR_INDEX index);

    /**
     * Update the colour at the given index
     * @param index
     * @param red
     * @param green
     * @param blue
     */
    void colour_update(COLOUR_INDEX index, uint8_t red, uint8_t green, uint8_t blue);
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
