/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    encoder.h

  @Summary
    Encapsulates rotary encoder functionality.

  @Description
    This file adapts the code from the rotary.c/h files
 */
/* ************************************************************************** */

#ifndef _ENCODER_H    /* Guard against multiple inclusion */
#define _ENCODER_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdint.h>
#include <stdbool.h>
#include "app_config.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */
   
    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    typedef struct
    {
        uint16_t current_value;
        uint16_t max_value;
        uint16_t min_value;
       
        // Indicates whether the encoder has moved within the range in the current cycle
        // This should only be used when using min/max values
        bool moved;
        
        // An indicator of how much the encoder has moved during the current cycle
        // This is not dependent on min/max values
        int delta;

    } ENCODER_STATE;

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Adapted from rotary.c get_rotary()
     * @param state
     * @return - an indication of whether the rotary has changed or not
     */
    bool encoder_get_state(ENCODER_STATE* state);

    /**
     * Get the current wrap state
     * @return 
     */
    bool encoder_get_wrap();

    /**
     * Initialise the encoder
     */
    void encoder_initialise();
 
    /**
    * Perform ADC processing
     * 
    * @return - true if samples processed, false if not
    */
    bool encoder_process(void);

    /**
     * Adapted from rotary.c get_rotary()
     * @param state
     * @return - an indication of whether the rotary has changed or not
     */
    bool encoder_process_state(ENCODER_STATE* state);
  
    /**
     * Read the current encoder state from the ports
     */
    void encoder_read(void);
    
    /**
     * Update the encoder state
     * @param state
     * @param wrap
     */
    void encoder_set(ENCODER_STATE state, bool wrap);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
