/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    encoder_multi.h

  @Summary
    Encapsulates rotary encoder functionality.

  @Description
    This file adapts the code from the rotary.c/h files
 */
/* ************************************************************************** */

#ifndef _ENCODER_MULTI_H    /* Guard against multiple inclusion */
#define _ENCODER_MULTI_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdint.h>
#include <stdbool.h>
#include "app_config.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

#define X(a, b, c, d, e, f, g) ENCODER_INDEX_##a,
	typedef enum {ENCODER_ITEMS ENCODER_INDEX_MAX} ENCODER_INDEX;
#undef X
    
    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    typedef struct
    {
        uint16_t current_value;
        uint16_t max_value;
        uint16_t min_value;
       
        // Indicates whether the encoder has moved within the range in the current cycle
        // This should only be used when using min/max values
        bool moved;
        
        // An indicator of how much the encoder has moved during the current cycle
        // This is not dependent on min/max values
        int delta;

    } ENCODER_STATE;
    
    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Disables the change notice ports for all encoders
     */
    void encoder_change_notice_disable();
    
    /**
     * Enables the change notice ports for all encoders
     */
    
    void encoder_change_notice_enable();
    
    /**
     * Adapted from rotary.c get_rotary()
     * @param state
     * @return - an indication of whether the rotary has changed or not
     */
    bool encoder_get_state(ENCODER_STATE* state, ENCODER_INDEX index);
    
    
    /**
     * Get the current wrap state
     * @return 
     */
    bool encoder_get_wrap(ENCODER_INDEX index);
    
    /**
     * Get the current wrap state
     * @return 
     */
    bool input_encoder_get_wrap(ENCODER_INDEX index);

    /**
     * Initialise the encoder
     */
    void encoder_initialise();
 
    /**
    * Perform encoder processing
     * 
    * @return - true if samples processed, false if not
    */
    bool encoder_process(ENCODER_INDEX index);

    /**
     * Adapted from rotary.c get_rotary()
     * @param state array of encoder states, array_size
     * @return - an indication of whether the encoders have changed or not
     */
    bool encoder_process_state(ENCODER_STATE state[ENCODER_INDEX_MAX]);
    
     /**
     * Read the current encoder state from the ports
     */
    void encoder_read(ENCODER_INDEX index);
    
        
    /**
     * Update the encoder state
     * @param state
     * @param wrap
     */
    void encoder_set(ENCODER_STATE state, ENCODER_INDEX index, bool wrap);
    
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
