/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _HDMI_ARC_H    /* Guard against multiple inclusion */
#define _HDMI_ARC_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include "utilities/hdmi_arc_data.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/**
 * Mute states
 */
#define HDMI_ARC_CONTROL_STATE_ITEMS \
    X(OFF, Off)\
    X(AUTO, Auto)\

// Now use the input data to define the enum
#define X(a, b) HDMI_ARC_CONTROL_STATE_##a,
	typedef enum {HDMI_ARC_CONTROL_STATE_ITEMS HDMI_ARC_CONTROL_STATE_MAX} HDMI_ARC_CONTROL_STATE;
#undef X
    
// Redefine the macro and use to create an array of strings
#define X(a, b) #b,
	static char *HDMI_ARC_CONTROL_STATE_STRINGS[] = { HDMI_ARC_CONTROL_STATE_ITEMS };
#undef X

// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

    /**
     * Gets the current setting deciding whether HDMI arc can be used to control
     * audio volume
     * @return 
     */
    HDMI_ARC_CONTROL_STATE hdmi_arc_get_audio_control_setting();

    /**
     * Gets the current setting deciding whether HDMI arc can be used to control
     * power state
     * @return 
     */
    HDMI_ARC_CONTROL_STATE hdmi_arc_get_power_control_setting();
    
    /**
     * Initialise the HDMI ARC module
     */
    bool hdmi_arc_initialise(HDMI_ARC_MESSAGE_CALLBACK callback);
    
    /**
     * Initialises the supplied HDMI ARC Message readying it for populating
     * @param message
     */
    void hdmi_arc_message_initialise(HDMI_ARC_MESSAGE* message);

    /**
     * Can be used to determine whether the display message has been fully populated
     * @param message
     * @return 
     */
    bool hdmi_arc_message_is_complete(HDMI_ARC_MESSAGE* message);

    /**
     * Can be used to determine whether the display message is currently being populated
     * @param message
     * @return 
     */
    bool hdmi_arc_message_is_processing(HDMI_ARC_MESSAGE* message);
    
    /**
     * Process a byte of data read from the RX channel
     * @param data_byte
     */
    void hdmi_arc_message_process_byte(HDMI_ARC_MESSAGE* message, unsigned char data_byte);

    /**
     * Used to reset the ARC chip
     */
    void hdmi_arc_reset(bool on);

    /**
     * Sets the current setting deciding whether HDMI arc can be used to control
     * audio volume
     * @return 
     */
    void hdmi_arc_set_audio_control_setting(HDMI_ARC_CONTROL_STATE value);

    /**
     * Sets the current setting deciding whether HDMI arc can be used to control
     * power state
     * @return 
     */
    void hdmi_arc_set_power_control_setting(HDMI_ARC_CONTROL_STATE value);
   
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
