/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _HDMI_ARC_COMMANDS_H    /* Guard against multiple inclusion */
#define _HDMI_ARC_COMMANDS_H

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities\hdmi_arc.h"
#include "crypto/crypto.h"
#include "crypto/src/md5.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */

//! Defines the maximum size of data in an Upgrade Data packet (not the full size of the packet)    
#define HDMI_ARC_UPLOAD_DATA_MAX_PACKET_SIZE    496 //(512 - other data in the message)
    
// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

    /**
     * Informs the hdmi arc that the devices ARC Stream state has changed
     * @param state
     */
    void hdmi_arc_send_arc_stream_message(HDMI_ARC_STREAM_STATE state);

    /**
     * Informs the hdmi arc that the devices mute state has changed
     * @param state
     */
    void hdmi_arc_send_cec_name_message(char* cec_name, uint8_t len);

    /**
     * Informs the hdmi arc that the devices mute state has changed
     * @param state
     */
    void hdmi_arc_send_mute_message(HDMI_ARC_MUTE_STATE state);

    /**
     * Informs the hdmi arc that the devices power state has changed
     * @param state
     */
    void hdmi_arc_send_power_message(HDMI_ARC_POWER_STATE state);

    /**
     * Informs the hdmi arc that the devices power state has changed
     * @param state
     */
    void hdmi_arc_send_system_ready_message(HDMI_ARC_SYSTEM_READY_STATE state);

    /**
     * Sends a request for the HDMI ARC Transmitter version
     */
    void hdmi_arc_send_transmitter_version_request();

    /**
     * Informs the hdmi arc that upgrade data is available
     */
    void hdmi_arc_send_upgrade_start_message();

    /**
     * Sends and Upgrade data packet to the ARC MCU
     */
    void hdmi_arc_send_upgrade_data_message(uint32_t total_size, uint32_t offset, uint8_t* data, uint16_t len);

    /**
     * Informs the hdmi arc that the devices volume has changed
     * @param state
     */
    void hdmi_arc_send_volume_message(uint8_t volume);
    
    /**
     * Used to suppress ARC messages being sent while a message is being processed 
     * by the arc module (set to solve problem where volume mapping caused rounding issues) 
     * @param on
     */
    void hdmi_arc_suppress_messages(bool on);
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
