/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _HDMI_ARC_PROCESSING_H    /* Guard against multiple inclusion */
#define _HDMI_ARC_PROCESSING_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities\hdmi_arc_data.h"
#include <stdbool.h>

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Gets the current HDMI ARC Format as a string
     * @param buffer
     * @param len
     */
    void hdmi_arc_get_arc_format_string(char* buffer, uint8_t len);

    /**
     * Gets the Major software version
     * @param buffer
     * @param len
     */
    uint8_t hdmi_arc_get_firmware_version_major();

    /**
     * Gets the Minor software version
     * @param buffer
     * @param len
     */
    uint8_t hdmi_arc_get_firmware_version_minor();

    /**
     * Gets the current HDMI ARC Format as a string
     * @param buffer
     * @param len
     */
    void hdmi_arc_get_firmware_version_string(char* buffer, uint8_t len);

    /**
     * Gets the Major software version
     * @param buffer
     * @param len
     */
    uint8_t hdmi_arc_get_rx_firmware_version_major();

    /**
     * Gets the Minor software version
     * @param buffer
     * @param len
     */
    uint8_t hdmi_arc_get_rx_firmware_version_minor();

    /**
     * Gets the current HDMI ARC Format as a string
     * @param buffer
     * @param len
     */
    void hdmi_arc_get_rx_firmware_version_string(char* buffer, uint8_t len);
    
    /**
     * Used to determine whether the firmware version has been retrieved from the ARC module 
     * @return 
     */
    bool hdmi_arc_has_firmware_version();
    
    /**
     * Initiates a fresh ARC handshake
     */
    void hdmi_arc_initiate_handshake();
    
    /**
     * Callback for RS232 connection
     * @param message
     */
    void hdmi_arc_processing_callback(HDMI_ARC_MESSAGE* message);
    
    /**
     * HDMI ARC Message processing initialisation
     */
    void hdmi_arc_processing_initialise(HDMI_ARC_MESSAGE_CALLBACK callback);
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _HDMI_ARC_PROCESSING_H */

/* *****************************************************************************
 End of File
 */
