/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _INPUT_ADC_H    /* Guard against multiple inclusion */
#define _INPUT_ADC_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include "utilities/input_adc/input_adc_defs.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */


// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************


// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/**
 * Configures the SPDIF RX module.  Note this includes a callback to the 
 * application to define which hardware implementation will be used
 */
void input_adc_configure();

/**
 * Used to define the functions for the particular adc.  Called in app_initialise
 * and is passed the hw version of define functions
 * @param function_data
 */
void input_adc_define_functions(INPUT_ADC_FUNCTION_DATA* function_data);

/**
 * Performs cyclic INPUT ADC processing
 */
bool input_adc_initialise();

/**
 * Set to true when all settings are applied
 */
bool input_adc_is_configured();

/**
 * Used to reapply the ADC settings
 */
void input_adc_reapply_settings();

/**
 * Sets pins for the adc
 * @param on
 */
void input_adc_reset(bool on);

/**
 * Switch to a specific input on the digital input control
 * @param definition - the definition of the control to switch to
 */
void input_adc_switch(INPUT_CONTROL_DEFINITION* definition);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
