/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _INPUT_ADC_DEFS_H    /* Guard against multiple inclusion */
#define _INPUT_ADC_DEFS_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include <stddef.h>
#include "utilities/input_control_defs.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */


// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************

//------------------------------------------------------------------------------
// INPUT ADC Hardware Events
// Note: These are implemented by bot the input_adc and inpt_adc_hw levels
//------------------------------------------------------------------------------

/**
 * a: Enum name b: Event Module Description
 */
#define INPUT_ADC_EVENT_ITEMS \
    X(SETTINGS_APPLIED, Input ADC Settings Applied)\

//! Uses EVENT_MODULE_ITEMS defined in app_config.h
// Define enum
#define X(a, b) INPUT_ADC_EVENT_##a,
	typedef enum {INPUT_ADC_EVENT_ITEMS} INPUT_ADC_EVENT;
#undef X
    
// Create array of Event module names
#define X(a, b) #b,
	static char *INPUT_ADC_EVENT_NAMES[] = { INPUT_ADC_EVENT_ITEMS };
#undef X
       
//! Structure for passing Switch event data
typedef struct
{
    // Indicates whther they have been applied or are about to be applied (true = applied))
    bool applied;
} INPUT_ADC_SETTINGS_APPLIED_EVENT_DATA;
         
//------------------------------------------------------------------------------
// INPUT ADC Hardware Functions
//------------------------------------------------------------------------------

/**
 * Template for the configure function - used by the hardware layer
 */
typedef void (*INPUT_ADC_CONFIGURE_FUNCTION)();     

/**
 * Template for the initialise function - used by the hardware layer
 */
typedef bool (*INPUT_ADC_INITIALISE_FUNCTION)();     

/**
 * Template for the reapply settings function - used by the hardware layer
 */
typedef void (*INPUT_ADC_REAPPLY_SETTINGS_FUNCTION)();     

/**
 * Template for the reset function - used by the hardware layer
 */
typedef void (*INPUT_ADC_RESET_FUNCTION)(bool on);     

//! Struct used by the application to decide which INPUT ADC to use
// This is defined by th input_adc_hw_nnn_define_callbacks method which is called in 
// app_initialise depends on which input adc is being used.
typedef struct
{
    //! Used to configure the input_adc module
    INPUT_ADC_CONFIGURE_FUNCTION configure_function;
    
    //! handles the module initialisation
    INPUT_ADC_INITIALISE_FUNCTION initialise_function;
    
    //! Used for reaplying the settings 
    INPUT_ADC_REAPPLY_SETTINGS_FUNCTION reapply_settings_function;
    
    //! Used to put the adc in or out of reset
    INPUT_ADC_RESET_FUNCTION reset_function;
     
} INPUT_ADC_FUNCTION_DATA;

// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************


    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
