/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _INPUT_ADC_FUNCTION_H    /* Guard against multiple inclusion */
#define _INPUT_ADC_FUNCTION_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include "utilities/input_control_defs.h"
#include "device_configuration.h"

#if APP_FEATURE_INPUT_ADC == APP_FEATURE_ON    
#include "utilities/input_adc/input_adc.h"
#endif

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif

// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/**
 * Configures the SPDIF RX module.  Note this includes a callback to the 
 * application to define which hardware implementation will be used
 */
static inline void input_adc_feature_configure()
{
#if APP_FEATURE_INPUT_ADC == APP_FEATURE_ON    
    input_adc_configure();
#endif    
}

/**
 * Performs cyclic INPUT ADC processing
 */
static inline bool input_adc_feature_initialise()
{
#if APP_FEATURE_INPUT_ADC == APP_FEATURE_ON    
    return input_adc_initialise();
#else
    return true;
#endif    
}

/**
 * Set to true when all settings are applied
 */
static inline bool input_adc_feature_is_configured()
{
#if APP_FEATURE_INPUT_ADC == APP_FEATURE_ON    
    return input_adc_is_configured();
#else
    return false;
#endif    
}

/**
 * Set to true the feature is available
 */
static inline bool input_adc_feature_is_present()
{
    bool result = false;
#if APP_FEATURE_INPUT_ADC == APP_FEATURE_ON    
    result = true;
#else
    return result;
#endif    
}

/**
 * Used to reapply the ADC settings
 */
static inline void input_adc_feature_reapply_settings()
{
#if APP_FEATURE_INPUT_ADC == APP_FEATURE_ON    
    input_adc_reapply_settings();
#endif    
}

/**
 * Sets pins for the adc
 * @param on
 */
static inline void input_adc_feature_reset(bool on)
{
#if APP_FEATURE_INPUT_ADC == APP_FEATURE_ON    
    input_adc_reset(on);
#endif    
}

/**
 * Switch to a specific input on the digital input control
 * @param definition - the definition of the control to switch to
 */
static inline void input_adc_feature_switch(INPUT_CONTROL_DEFINITION* definition)
{
#if APP_FEATURE_INPUT_ADC == APP_FEATURE_ON    
    input_adc_switch(definition);
#endif    
}

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
