/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _INPUT_ADC_DATA_H    /* Guard against multiple inclusion */
#define _INPUT_ADC_DATA_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/input_adc/input_adc_defs.h"
#include "utilities/events.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */

//! Defines the maximum number o event subscriptions that can be made to the events 
// in this module - this is set to 3 one for each event as its only the input_control module 
// accesses them
#define INPUT_ADC_MAX_EVENT_SUBSCRIPIONS    1

// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************

//! private data type available only to the input_adc module.  If this needs
// to be public then move to the _defs.h
typedef struct
{
    //!  inpuit_adc_hw functions.  These should be stored in a static data structure by
    // the hw module and supplied using spdif_rx_define_functions
    INPUT_ADC_FUNCTION_DATA functions;
    
	//! An Array of event subscription structures.  These ara managed by the events module when
	// invoking events.  There MUST be enough declared to handle all the concurrent event 
	// subscriptions for the module.  
    // In this case it will be 1 for each event as its only th spdif_rx module subscribing
	EVENT_SUBSCRIPTION event_subscriptions[INPUT_ADC_MAX_EVENT_SUBSCRIPIONS];    
    
    //! Set to true is all the settings have been applied
    bool configured;
} INPUT_ADC_DATA;

// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/**
 * Returns a reference t th SPDIF_RX_DATA structure
 * @return 
 */    
INPUT_ADC_DATA* input_adc_data_get();

/**
 * 
 */
bool input_adc_data_is_configured();

/**
 */
void input_adc_data_set_configured(bool configured);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
