/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _INPUT_CONTROL_USB_H    /* Guard against multiple inclusion */
#define _INPUT_CONTROL_USB_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/input_control_defs.h"
#include <stdbool.h>

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Enables / Disables the USB ports for EMC purposes (See Pete email 20/08/2019 13:28)
     * @param on
     */
    void input_control_usb_enable(bool on);
    
    /**
     * Returns whether USB record is enabled (though not necessarily active
     * depending on input)
     * @return 
     */
    bool input_control_usb_get_record();
    
    /**
     * Initialise the input control module
     */
    bool input_control_usb_initialise();
    
    /**
     * Switch to usb input
     * @param definition - the definition of the control to switch to
     */
    void input_control_switch_usb(INPUT_CONTROL_DEFINITION* definition);
    
    /**
     * Sets USB Record function On/Off
     */
    void input_control_usb_set_record(bool on);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
