/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _MENU_APP_H    /* Guard against multiple inclusion */
#define _MENU_APP_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/structured_message.h"
#include "utilities/menu.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */

//--------------------------------------------------------------------------
// Enum of available Menu Navigation options
//--------------------------------------------------------------------------
typedef enum
{
    MENU_NAVIGATION_UP      = 0x00, // Navigates up a level
    MENU_NAVIGATION_OK      = 0x01, // Handles OK command -Navigates down a level or puts into edit mode
    MENU_NAVIGATION_LEFT    = 0x02, // Navigates left 
    MENU_NAVIGATION_RIGHT   = 0x03, // Navigates right 
} MENU_NAVIGATION;

// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************

// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/**
 * Handle menu navigation - goes to a specific menu
 */
void menu_app_handle_switch(STRUCTURED_MESSAGE* message);

/**
 * Handle menu navigation - goes to a relative menu
 */
void menu_app_handle_navigation(STRUCTURED_MESSAGE* message);

/**
 * Can be used to reset the inactivity timeout in cases where its been adjusted (DIRAC/ANALOGUE_DIRECT_NO_NAVIGTION)
 * @param menu
 */
MENU_ID menu_app_reset_inactivity_timeout(MENU_ID menu);

/**
 * Handle a change to a value during menu edit mode
 * Note: This invokes a context specific method for a given menu item
 */
void menu_app_value_change(STRUCTURED_MESSAGE* message);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
