/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    mutes.h

  @Summary
    Handles simple I/O muting in the application.

  @Description
    Handles simple GPIO I/O muting in the application.
 */
/* ************************************************************************** */

#ifndef _MUTES_H    /* Guard against multiple inclusion */
#define _MUTES_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "app_config.h"
#include "system/ports/sys_ports.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    //! Use MUTE_ITEMS defined in app_config.h
// Now use the input data to define the enum
#define X(a, b, c, d, e, f) MUTE_INDEX_##a,
	typedef enum {MUTE_ITEMS MUTE_INDEX_MAX} MUTE_INDEX;
#undef X

/**
 * Mute states
 */
#define MUTE_STATE_ITEMS \
    X(ON, On)\
    X(OFF, Off)\

// Now use the input data to define the enum
#define X(a, b) MUTE_STATE_##a,
	typedef enum {MUTE_STATE_ITEMS MUTE_STATE_MAX} MUTE_STATE;
#undef X
    
// Redefine the macro and use to create an array of strings
#define X(a, b) #b,
	static char *MUTE_STATE_STRINGS[] = { MUTE_STATE_ITEMS };
#undef X

    
    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************
    
    /**
     * Perform mute initialisation
     */
    void mutes_initialise();
    
    /**
     * Mutes the specified mute index
     * 
     * @param mute_index - The index of the mute
     */
    void mutes_mute(uint8_t mute_index);
    
    /**
     * Returns whether the mute is currently muted
     * 
     * @param mute_index - The index of the mute
     */
    MUTE_STATE mutes_state(uint8_t mute_index);
    
    /**
     * Unmutes the specified mute index
     * 
     * @param mute_index - The index of the mute
    */
    void mutes_unmute(uint8_t mute_index);
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
