/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    output_control_streamer.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _OUTPUT_CONTROL_H    /* Guard against multiple inclusion */
#define _OUTPUT_CONTROL_H

#include <stdbool.h>
#include "utilities/system_fault.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************
    
    /**
     * Initialise the output control module
     */
    void output_control_initialise(SYSTEM_FAULT_CALLBACK fault_callback);    
    
    /**
     * Mutes Speaker, Preamp, and Power AMp outputs
     * @param include_power_amp - True for SA10, false SA20
     */
    void output_control_mute_analogue_outputs();
    
    /**
     * Mutes Speaker, Preamp, and Power AMp outputs
     * @param include_power_amp - True for SA10, false SA20
     */
    void output_control_mute_digital_outputs();
    
    /**
     * Mutes Speaker, Preamp, and Power AMp outputs
     * @param include_power_amp - True for SA10, false SA20
     */
    void output_control_mute_outputs();
    
    /**
     * Perform regular output control tasks
     */
    void output_control_process();
    
#ifdef MQA_ENABLED

    /**
     * Reviews the state of the digital outputs based on the current state of the MQA Decoder settings
     * when unmuting
     */
    void output_control_review_mqa_outputs();
#endif
    
    /**
     * Unmutes 
     */
    void output_control_unmute_analogue_outputs();
    
    /**
     * Unmutes 
     */
    void output_control_unmute_digital_outputs();
    
    /**
     * Unmutes 
     */
    void output_control_unmute_outputs();
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
