/* ************************************************************************** */
/** Descriptive File Name

  @Company
    ARCAM

  @File Name
    pins.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _PINS_H    /* Guard against multiple inclusion */
#define _PINS_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

/* This section lists the other files that are included in this file.
 */

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif

#include <stdint.h>
#include <stdbool.h>
    
    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************
    
    /**
     * Add a pin definition
     * @param index
     * @param port
     * @param channel
     * @param pin
     */
    void pins_add_pin(uint8_t port, uint8_t channel, uint8_t pin, uint8_t mask, bool set_as_input);
    
    /**
     * initialises the pins module
     */
    void pins_initialise();
    
    /**
     * Clears all the pins (saves state first)
     */
    void pins_set_all_low(uint8_t mask);
    
    /**
     * Restores the pins to the most recent saved state
     */
    void pins_restore_state();

    /**
     * Saves the current state of all the pins
     */
    void pins_save_state();

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
