/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    rgb_led.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _RGB_LED_H    /* Guard against multiple inclusion */
#define _RGB_LED_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include <stdint.h>
#include "app_config.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */
    
//#define APP_BLUE_LED_PORTS_ID               PORTS_ID_0
//#define APP_BLUE_LED_PORT_CHANNEL           PORT_CHANNEL_A
//#define APP_BLUE_LED_PIN                    PORTS_BIT_POS_2
//#define APP_RED_LED_PORTS_ID                PORTS_ID_0
//#define APP_RED_LED_PORT_CHANNEL            PORT_CHANNEL_F
//#define APP_RED_LED_PIN                     PORTS_BIT_POS_13
//#define APP_GREEN_LED_PORTS_ID              PORTS_ID_0
//#define APP_GREEN_LED_PORT_CHANNEL          PORT_CHANNEL_F
//#define APP_GREEN_LED_PIN                   PORTS_BIT_POS_12

// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************

    typedef enum
    {
        RGB_LED_COLOUR_OFF,
        RGB_LED_COLOUR_RED,
        RGB_LED_COLOUR_GREEN,
        RGB_LED_COLOUR_BLUE,
        RGB_LED_COLOUR_ORANGE,
        RGB_LED_COLOUR_PURPLE,
        RGB_LED_COLOUR_WHITE,
    } RGB_LED_COLOUR;

// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

    /**
     * Can be used to adjust the default value settings for a colour - used for
     * example to match the White colour with the CDS50
     * @param colour
     * @param red
     * @param green
     * @param blue
     */
    void rgb_led_adjust_colour(RGB_LED_COLOUR colour, uint8_t red, uint8_t green, uint8_t blue);
    
    /**
     * Flash the led
     */
    void rgb_led_flash();

    /**
     * Gets the current RGB LED colour
     * @return 
     */
    RGB_LED_COLOUR rgb_led_get_colour();
    
    /**
     * Returns whether the led is flashing or not
     */
    bool rgb_led_get_flash();
    
    /**
     * Initialises the RGB LEd module
     */
    void rgb_led_initialise();
    
    /**
     * Set the colors of the led - can be a combination of Red-Green-Blue
     * @param value - an Or'd combination of RGB_LED_COLOR constants
     */
    void rgb_led_set(RGB_LED_COLOUR value);

    /**
     * Used to process the flashing led
     */
    void rgb_led_process();
    
    /**
     * Used as an interrupt event timer
     */
    void rgb_led_process_interrupt();

    /**
     * Stop the LED flashing
     */
    void rgb_led_flash_stop();
    
    /**
     * Sets the RGB colour of the led
     * @param red
     * @param green
     * @param blue
     */
    void rgb_led_set_colour(uint8_t red, uint8_t green, uint8_t blue);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
