/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _SPDIF_RX_DATA_H    /* Guard against multiple inclusion */
#define _SPDIF_RX_DATA_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/spdif_rx/spdif_rx_defs.h"
#include "utilities/events.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */

//! Defines the maximum number of event subscriptions that can be made to the events 
// in this module - this is set to 3 one for each event as its only the input_control module 
// accesses them
#define SPDIF_RX_MAX_EVENT_SUBSCRIPIONS  3

// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************

//! private data type available only to the spdif_rx module.  If this needs
// to be public then move to the _defs.h
typedef struct
{
    SAMPLE_RATE sample_rate;
    
    SAMPLE_RATE steady_sample_rate;
           
    // Required to ensure only one sample_rate request is active at a time
    bool calculating_sample_rate;
    
    //! Returns true if its a mix
    bool in_switch;
    
    //!  spdif_rx_hw functions.  These should be stored in a static data structure by
    // the hw module and supplied using spdif_rx_define_functions
    SPDIF_RX_FUNCTION_DATA functions;
    
	//! An Array of event subscription structures.  These ara managed by the events module when
	// invoking events.  There MUST be enough declared to handle all the concurrent event 
	// subscriptions for the module.  
    // In this case it will be 1 for each event as its only th spdif_rx module subscribing
	EVENT_SUBSCRIPTION event_subscriptions[SPDIF_RX_MAX_EVENT_SUBSCRIPIONS];    
    
} SPDIF_RX_DATA;

// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/**
 * Returns a reference t th SPDIF_RX_DATA structure
 * @return 
 */    
SPDIF_RX_DATA* spdif_rx_data_get();

/**
 * Gets whether the SPDIF_RX sample rate is currently being calculated
 * @param sample_rate
 */
bool spdif_rx_data_get_calculating_sample_rate();

/**
 * Returns true if the spdif_rx module is in a switch
 * @return 
 */
bool spdif_rx_data_get_in_switch();

/**
 * Gets the SPDIF_RX sample rate e
 * @param sample_rate
 */
SAMPLE_RATE spdif_rx_data_get_sample_rate();

/**
 * Sets whether the SPDIF_RX sample rate is bing calculated (spdif_rx use ONLY)
 * @param sample_rate
 */
void spdif_rx_data_set_calculating_sample_rate(bool calculating);

/**
 * Sets the spdif_rx module switch indicator
 * @return 
 */
void spdif_rx_data_set_in_switch(bool in);

/**
 * Sets the SPDIF_RX sample rate either when calculated or when an error occurs
 * @param sample_rate
 */
void spdif_rx_data_set_sample_rate(SAMPLE_RATE sample_rate);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
