/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _SPDIF_RX_H    /* Guard against multiple inclusion */
#define _SPDIF_RX_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include "utilities/spdif_rx/spdif_rx_defs.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */


// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************


// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/**
 * Configures the SPDIF RX module.  Note this includes a callback to the 
 * application to define which hardware implementation will be used
 */
void spdif_rx_configure();

/**
 * Used to define the functions for the particular spdif receiver.  Called in app_initialise
 * and is passed the hw version of define functions
 * @param function_data
 */
void spdif_rx_define_functions(SPDIF_RX_FUNCTION_DATA* function_data);

/**
 * Gets the current smoothed sample rate
 * @return 
 */
SAMPLE_RATE spdif_rx_get_sample_rate();
    
/**
 * Handles interrupt on the SPDIF RX Interrupt pins
 * @param definition
 */
void spdif_rx_handle_interrupt(uint8_t channel);

/**
 * Returns true if the spdif_rx module is in a switch
 * @return 
 */
bool spdif_rx_in_switch();

/**
 * Performs cyclic SPDIF_RX processing
 */
bool spdif_rx_initialise();

/**
 * Performs cyclic SPDIF_RX processing
 */
void spdif_rx_process();

/**
 * Requests that th SPDIF_RX module refresh its sample rate
 */
void spdif_rx_refresh_sample_rate(/* May be needs a callback when complete */);

/**
 * Put SPDIF RX in or out of reset
 * @param on
 */
void spdif_rx_reset(bool on);

/**
 * Sets the SPDIF Rx ADC Mode for Master/Slave and Power
 * @param mode
 */
void spdif_rx_set_adc_mode(SPDIF_RX_ADC_MODE mode);

/**
 * Switch to a specific input on the digital input control
 * @param definition - the definition of the control to switch to
 */
void spdif_rx_switch(INPUT_CONTROL_DEFINITION* definition);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
