/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _SPDIF_RX_HW_AK4113_H    /* Guard against multiple inclusion */
#define _SPDIF_RX_HW_AK4113_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/spdif_rx/spdif_rx_defs.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */


// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************


// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/**
 * Configure the ak4113 ADC Mode
 */
void spdif_rx_hw_ak4113_adc_mode(SPDIF_RX_ADC_MODE mode);

/**
 * Configure the ak4113
 */
void spdif_rx_hw_ak4113_configure();

/**
 * Defines the SPDIF_RX callbacks used when using the AK4113.  To be used as
 * the parameter to spdif_rx_define_callbacks() function
 * @return 
 */    
SPDIF_RX_FUNCTION_DATA* spdif_rx_hw_ak4113_define_functions();
 
/**
 * Initialise the AK4113
 */
bool spdif_rx_hw_ak4113_initialise();
    
/**
 * Handles interrupt on the SPDIF RX Interrupt pins
 * @param definition
 */
void spdif_rx_hw_ak4113_interrupt(uint8_t channel);
 
/**
 * Perform AK4113 processing
 */
void spdif_rx_hw_ak4113_process();

/**
 * Perform AK4113 reset
 */
void spdif_rx_hw_ak4113_reset(bool on);     

/**
 * Trigger AK4113 sample rate calculation 
 */
bool spdif_rx_hw_ak4113_sample_rate();

/**
 * Perform AK4113 switch 
 */
void spdif_rx_hw_ak4113_switch(INPUT_CONTROL_DEFINITION* definition);    

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
