/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _SPDIF_RX_HW_TIPCM9211_H    /* Guard against multiple inclusion */
#define _SPDIF_RX_HW_TIPCM9211_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/spdif_rx/spdif_rx_defs.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */


// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************


// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/**
 * Configure the tipcm9211 ADC Mode
 */
void spdif_rx_hw_tipcm9211_adc_mode(SPDIF_RX_ADC_MODE mode);

/**
 * Configure the tipcm9211
 */
void spdif_rx_hw_tipcm9211_configure();

/**
 * Defines the SPDIF_RX callbacks used when using the TIPCM9211.  To be used as
 * the parameter to spdif_rx_define_callbacks() function
 * @return 
 */    
SPDIF_RX_FUNCTION_DATA* spdif_rx_hw_tipcm9211_define_functions();
 
/**
 * Initialise the TIPCM9211
 */
bool spdif_rx_hw_tipcm9211_initialise();
    
/**
 * Handles interrupt on the SPDIF RX Interrupt pins
 * @param definition
 */
void spdif_rx_hw_tipcm9211_interrupt(uint8_t channel);
 
/**
 * Perform TIPCM9211 processing
 */
void spdif_rx_hw_tipcm9211_process();

/**
 * Perform TIPCM9211 reset
 */
void spdif_rx_hw_tipcm9211_reset(bool on);     

/**
 * Trigger TIPCM9211 sample rate calculation 
 */
bool spdif_rx_hw_tipcm9211_sample_rate();

/**
 * Perform TIPCM9211 switch 
 */
void spdif_rx_hw_tipcm9211_switch(INPUT_CONTROL_DEFINITION* definition);    

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
