/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    system_fault.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _SYSTEM_FAULT_H    /* Guard against multiple inclusion */
#define _SYSTEM_FAULT_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include "app_config.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    //! Fault description length 16 as it will be displayed on the VFD
#define SYSTEM_FAULT_MAX_DESCRIPTION_LEN    16
    
//! Defines the number of historic shutdown codes that are retained (including the current one))
#ifndef SHUTDOWN_CODE_HISTORY_LENGTH    
#define SHUTDOWN_CODE_HISTORY_LENGTH        10   
#endif    
    
    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************
    
    /**
     */
    typedef struct
    {
        //! A code identifying the type of fault
        SYSTEM_FAULT_CODE code;
        
        //! A description of the fault
        char description[SYSTEM_FAULT_MAX_DESCRIPTION_LEN];
    } SYSTEM_FAULT;
    
    /**
     * Callback function template
     */
    typedef void (*SYSTEM_FAULT_CALLBACK)(SYSTEM_FAULT* fault);

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************
   
    /**
     * Gets the current System Fault 
     */
    SYSTEM_FAULT system_fault_get();
    
    /**
     * Gets the current System Fault 
     */
    SYSTEM_FAULT_CODE system_fault_get_last_shutdown_code();

    /**
     * Initialise the system fault context
     */
    void system_fault_initialise();
    
    /**
     * Saves the current system fault into settings.  Returns true once the settings have 
     * been successfully saved
     */
    bool system_fault_save_to_settings();
    
    /**
     * Sets the current system fault
     */
    void system_fault_set(SYSTEM_FAULT* fault);
    
    /**
     * Sets the current System Fault 
     */
    void system_fault_set_last_shutdown_code(SYSTEM_FAULT_CODE code);
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
