/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _TFT_DISPLAY_H    /* Guard against multiple inclusion */
#define _TFT_DISPLAY_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdint.h>
#include <stdbool.h>
#include "utilities/colour.h"
#include "utilities/font_manager.h"
#include "utilities/tft_control.h"
#include "utilities/tft_display_data.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

#define TFT_DISPLAY_HEIGHT  128
#define TFT_DISPLAY_WIDTH   480
    
static BOUNDS FULL_SCREEN_BOUNDS = { 0, 0, TFT_DISPLAY_HEIGHT - 1, TFT_DISPLAY_WIDTH - 1 };
    
//
//    //------------------------------------------------------------------------------
//    // Pixel handling
//    //------------------------------------------------------------------------------
//
//    typedef COLOUR PIXEL;
//    
//    //------------------------------------------------------------------------------
//    // Rectangle handling
//    //------------------------------------------------------------------------------
//
//    struct BOUNDS
//    {
//        int16_t top;
//        int16_t left;
//        int16_t bottom;
//        int16_t right;
//    };
//    
//    typedef struct BOUNDS BOUNDS;

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * 
     * @param bounds
     * @return 
     */
    uint16_t tft_display_bounds_area(BOUNDS* bounds);

    /**
     * 
     * @param bounds
     * @return 
     */
    uint16_t tft_display_bounds_height(BOUNDS* bounds);

    /**
     * 
     * @param bounds
     * @return 
     */
    uint16_t tft_display_bounds_width(BOUNDS* bounds);
    
    /**
     */
    void tft_display_clear();

    /**
     * 
     */
    void tft_display_draw();
 
    /**
     * 
     * @param pixels
     * @param bounds
     * @param colour
     */
    void tft_display_draw_character(PIXEL* pixels, 
            BOUNDS* pixel_bounds, 
            int16_t top_offset, 
            int16_t left_offset, 
            FONT_ID font_id, 
            char* character,
            COLOUR_INDEX foreground_colour,
            COLOUR_INDEX background_colour);
    
    /**
     * 
     * @param pixels
     * @param bounds
     * @param colour
     */
    //void tft_display_draw_character(PIXEL* pixels, BOUNDS* pixel_bounds, int16_t top_offset, int16_t left_offset, uint8_t font_id, char* character)
    
    /**
     * 
     * @param bounds
     */
    void tft_display_draw_rectangle(PIXEL* pixels, BOUNDS* bounds, COLOUR_INDEX colour_idx);

    /**-----------------------------------------------------------------------------
     * 
     * @param on
     */
    bool tft_display_get_backlight();
    
    /**
     * Gets the character at the supplied pixel position
     * @param bounds
     * @param pixels
     * @param invalidate
     */
    uint16_t tft_display_get_character_at_pos(FONT_ID, char* text, uint16_t pos);
    
    /**
     * Returns the width of the text
     * @param bounds
     */
    uint16_t tft_display_get_text_width(FONT_ID font_id, char* text);
    
    /**
     * Initialise
     */
    void tft_display_initialise();

    /**
     * We are assuming that each control will only draw on its own part of the display and
     * will not overlap with other controls.  This means we are eother invalidating separte rectangles or 
     * the whole screen which makes RECT comparison easier
     */ 
    void tft_display_invalidate_rect(BOUNDS* bounds);
    
    /**
     * Indicates whether the display has been cleared down or not - used to determine whether
     * to redraw the screen
     */
    bool tft_display_is_clear();
    
    /**
     * 
     */
    void tft_display_process();
    
    /**
     * Sets the brightness in the range 0-128
     */
    void tft_display_set_brightness(uint8_t value);

    /**
     * Writes a buffer (as an array) of PIXEL DATA to the screen buffer - needs a call to tft_display_draw afterwards
     * Assumes the buffer length is the same size as the rectangle area
     * If invalidate = false then it is assumed that the area will be invalidated manually
     */
    void tft_display_write_buffer(BOUNDS* bounds, PIXEL* pixels, bool invalidate);
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
