/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _UPGRADE_H    /* Guard against multiple inclusion */
#define _UPGRADE_H

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdio.h>
#include <stdbool.h>

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */

#define UPGRADE_WRITE_COMPLETE      -1
#define UPGRADE_WRITE_ERROR         -2

#define UPGRADE_MAX_HEX_RECORD_SIZE     100
    
typedef unsigned int    UINT;
typedef unsigned char   UINT8;


// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************


// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************

/**
 * Use to determine whether an upgrade can proceed
 * @return 
 */
bool upgrade_can_proceed();

/**
 * Erase the current flash memory
 */
void upgrade_erase_flash();

/**
 * Initialises the upgrade process - must be called at the start of any upgrade
 */
void upgrade_initialise();

/**
 * Writes a buffer from a hex file into flash
 * @param buffer
 * @param num_bytes
 * @return 
 */
UINT upgrade_write_buffer(UINT8* buffer, size_t num_bytes);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
