/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _USB_AUDIO_CODEC_H    /* Guard against multiple inclusion */
#define _USB_AUDIO_CODEC_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include <stdint.h>
#include <stdio.h>
#include "utilities\usb_audio_defs.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************
    
    /**
     * Add an I2S write request
     */
    bool usb_audio_codec_add_write(uint32_t index);
    
    /**
     * Gets the combined size of the available buffers
     */
    size_t usb_audio_codec_get_buffer_size();

    /**
     * Gets the index of the buffer from which the next codec write will be performed
     */
    uint32_t usb_audio_codec_get_read_index();
    
    /**
     * Initialises the codec
     */
    bool usb_audio_codec_initialise();
    
    /**
     * Opens the Codec I2S connection
     */
    bool usb_audio_codec_open();

    /**
     * Resets the codec for writing 
     */
    void usb_audio_codec_reset();
    
    /**
     * Sets up the codec sample rate
     */
    void usb_audio_codec_set_sample_rate(uint32_t sample_rate);
    
    /**
     * Attempts to write data to the audio codex from the current write index
     * @return - true if write success
     */
    bool usb_audio_codec_try_write();
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
