/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _USB_AUDIO_DEFS_H    /* Guard against multiple inclusion */
#define _USB_AUDIO_DEFS_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include <stdint.h>
#include "usb/usb_audio_v2_0.h"
#include "usb/usb_device_audio_v2_0.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */
    
#define USB_MAX_BAD_PACKETS                     20
#define USB_LEVEL1_ERROR                        1      // mute, init, submit read request
#define USB_LEVEL2_ERROR                        2      // mute, init, detach/attach usb, wait for config

/**
 * Level errors
 */
#define USB_AUDIO_LEVEL1_ERROR                  USB_LEVEL1_ERROR      // mute, init, submit read request
#define USB_AUDIO_LEVEL2_ERROR                  USB_LEVEL2_ERROR      // mute, init, detach/attach usb, wait for config

#define USB_AUDIO_BUFFER_QUEUE_MAX_COUNT        QUEUE_SIZE_TX_IDX0   
#define USB_AUDIO_MAX_FREQ                      DRV_I2S_BAUD_RATE
#define USB_AUDIO_MAX_SAMPLES                   (USB_AUDIO_MAX_FREQ / 1000)
#define NO_OF_SAMPLES_IN_A_USB_FRAME            1024 //192+8 does not seem to be enough! (USB_AUDIO_MAX_SAMPLES + 8) /* +1 sample for buffer overrun tuning */

#define USB_AUDIO_SAMPLING_RATE_8KHZ                        8000
#define USB_AUDIO_SAMPLING_RATE_32KHZ                       32000
#define USB_AUDIO_SAMPLING_RATE_44_1KHZ                     44100
#define USB_AUDIO_SAMPLING_RATE_48KHZ                       48000
#define USB_AUDIO_SAMPLING_RATE_88_2KHZ                     88200
#define USB_AUDIO_SAMPLING_RATE_96KHZ                       96000    
#define USB_AUDIO_SAMPLING_RATE_176_4KHZ                    176400    
#define USB_AUDIO_SAMPLING_RATE_192KHZ                      192000    

// These specifiy the limits of the DMA Tx buffer (usually 64 long queue) where the USB will be told to queue up/slowdown    
#define USB_AUDIO_BUFFER_QUEUE_TUNE_UPPER_LIMIT_32          55//63  
#define USB_AUDIO_BUFFER_QUEUE_TUNE_LOWER_LIMIT_32          40//50     

#define USB_AUDIO_BUFFER_QUEUE_TUNE_UPPER_LIMIT_44_1        55//63  
#define USB_AUDIO_BUFFER_QUEUE_TUNE_LOWER_LIMIT_44_1        40//50     

#define USB_AUDIO_BUFFER_QUEUE_TUNE_UPPER_LIMIT_48          55//63  
#define USB_AUDIO_BUFFER_QUEUE_TUNE_LOWER_LIMIT_48          40//45     

#define USB_AUDIO_BUFFER_QUEUE_TUNE_UPPER_LIMIT_88_2        55//63  
#define USB_AUDIO_BUFFER_QUEUE_TUNE_LOWER_LIMIT_88_2        40//45     

#define USB_AUDIO_BUFFER_QUEUE_TUNE_UPPER_LIMIT_96          55//63  
#define USB_AUDIO_BUFFER_QUEUE_TUNE_LOWER_LIMIT_96          40//45      

#define USB_AUDIO_BUFFER_QUEUE_TUNE_UPPER_LIMIT_176_4       55//63  
#define USB_AUDIO_BUFFER_QUEUE_TUNE_LOWER_LIMIT_176_4       40//50    

#define USB_AUDIO_BUFFER_QUEUE_TUNE_UPPER_LIMIT_192         55//63  
#define USB_AUDIO_BUFFER_QUEUE_TUNE_LOWER_LIMIT_192         40//50     

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    typedef enum
    {
        USB_AUDIO_MCLK_MULTIPLIER_128FS = 128,                        
        USB_AUDIO_MCLK_MULTIPLIER_192FS = 192,                                    
        USB_AUDIO_MCLK_MULTIPLIER_256FS = 256,            
        USB_AUDIO_MCLK_MULTIPLIER_384FS = 384,
        USB_AUDIO_MCLK_MULTIPLIER_512FS = 512,            
        USB_AUDIO_MCLK_MULTIPLIER_768FS = 768,                        
        USB_AUDIO_MCLK_MULTIPLIER_1152FS = 1152     
    } USB_AUDIO_MCLK_MULTIPLIER;

    /**
     * Audio device callback function template.  This is used to relay device events
     * to the application
     */
    typedef void (*USB_AUDIO_DEVICE_EVENT_HANDLER)
    (
        USB_DEVICE_EVENT event, 
        void* event_data, 
        uintptr_t context
    );

    /**
     * Audio event callback function template
     */
    typedef void (*USB_AUDIO_EVENT_HANDLER)
    (
        USB_DEVICE_AUDIO_V2_INDEX index,
        USB_DEVICE_AUDIO_V2_EVENT event,
        void * data,
        uintptr_t context
    );
    
    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
