/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _USB_AUDIO_FEEDBACK_H    /* Guard against multiple inclusion */
#define _USB_AUDIO_FEEDBACK_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include <stdint.h>
#include "usb/usb_audio_v2_0.h"
#include "usb/usb_device_audio_v2_0.h"
#include "utilities/usb_audio_defs.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Feedback rate controller
     */
    typedef enum
    {
        USB_AUDIO_FEEDBACK_STEADY,
        USB_AUDIO_FEEDBACK_SPEEDUP,
        USB_AUDIO_FEEDBACK_SLOWDOWN,
    } USB_AUDIO_FEEDBACK_RATE;
    
    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Called in response to a write complete event
     * @param data
     */
    void usb_audio_feedback_complete(USB_DEVICE_AUDIO_V2_EVENT_DATA_READ_COMPLETE* data);

    /**-----------------------------------------------------------------------------
     * Sends a request to the USB host to adjust the transfer rate
     */
    void usb_audio_feedback_send_rate_request();

    /**
     * Sets up feedback rates and buffer limits
     */
    void usb_audio_feedback_setup_rates(uint32_t sample_rate, uint32_t max_audio_samples);

    /**
     * Populates the supplied buffer (uint8_t[4]) with he appropriate feedback rate
     * @param copy_to
     * @param rate
     */
    void usb_audio_feedback_set_rate(USB_AUDIO_FEEDBACK_RATE rate);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
