/* ************************************************************************** */
/** Descriptive File Name

  @Company
 ARCAM

  @File Name
    uuid.h

  @Summary
 Manages UUID's.  The generated UUID are Version 5 UUIDs

  @Description
 Given the UUID must be unique we are aiming to use the tcpip mac address as the namespace
 */
/* ************************************************************************** */

#ifndef _UUID_H    /* Guard against multiple inclusion */
#define _UUID_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stddef.h>
#include <stdbool.h>
#include <stdint.h>
#include <string.h>

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */


    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    typedef struct UUID 
    {
        uint32_t data1;
        uint16_t data2;
        uint16_t data3;
        uint16_t data4;
        uint16_t data5;
        uint32_t data6;
    } UUID;
    
    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Sets the supplied uuid to 00000000-0000-0000-0000-000000000000
     * @param uuid
     */
    void uuid_empty(UUID* uuid);
    
    /**
     * Can be used to test whetger a UUID is empty (00000000-0000-0000-0000-000000000000)
     * @param uuid
     * @return - true is empty , false if not
     */
    bool uuid_is_empty(UUID* uuid);
    
    /*
     * Initialises the UUID module with a given namespace
     */
    void uuid_initialise(const char* namespace_string, size_t ns_len);
    
    /*
     * Returns true is uuid_initialise has been called
     */
    bool uuid_is_initialised();
    
    /**
     * Generates a new UUID and places it in the supplied UUID variable
     * @param uuid
     */
    void uuid_generate(UUID* uuid, char* name, size_t name_len);
    
    /**
     * Generates a string version of the UUID in the format XXXXXXXX-XXXX-XXXX-XXXX-XXXXXXXXXXXX
     * The buffer must be at least 36 chars long
     */
    void uuid_to_string(UUID* uuid, char* buffer);
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
