/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    vfd.h

  @Summary
 Manages communication with a VFD

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _VFD_H    /* Guard against multiple inclusion */
#define _VFD_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif

#include "utilities/display/display_defs.h"
#include <stddef.h>
#include <stdint.h>
#include <stdbool.h>

    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    /**
     * Highlight options
     */
    typedef enum
    {
        VFD_HIGHLIGHT_MODE_INVERT,
        VFD_HIGHLIGHT_MODE_INVERT_FLASH,
        VFD_HIGHLIGHT_MODE_TOGGLE_BOTTOM_LINE,
        VFD_HIGHLIGHT_MODE_TOGGLE_CYCLE_LINE,   // RUBBISH!!
        VFD_HIGHLIGHT_MODE_CYCLE_LINE,
        VFD_HIGHLIGHT_MODE_FLASH_BLOCK,
        VFD_HIGHLIGHT_MODE_CYCLE_FILL_DROP,
        VFD_HIGHLIGHT_MODE_MAX,
    } VFD_HIGHLIGHT_MODE;
   
    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Clear the VFD display
     */
    void vfd_clear();

    /**
     * Clears the VFD into reset mode - 
     */
    void vfd_clear_reset();

    /**
     * Gets the height of a character in pixels
     * @return 
     */
    size_t vfd_get_character_height();

    /**
     * Gets the width of a character in pixels
     * @return 
     */
    size_t vfd_get_character_width();
    
    /**
     * Highlights the character at the specified ordinal position in the line
     * @param line - the line to copy
     * @param character -  the character to highlight
     * @param highlight_spaces - determines whether space are highlighted
     * @return - the character code that can be used to reference the highlighted 
     * version of the character
     */
    char vfd_get_highlight_character(DISPLAY_LINE line, const char* character, bool highlight_spaces);

    /**
     * Returns the length of the display in characters
     * @return 
     */
    size_t vfd_get_line_length();   
    
    /**
     * Returns a character code 
     * @param line - the line to add the special character to 
     * @param character - a character selected from the DISPLAY_SPECIAL_CHARACTERS enum
     * @return 
     */
    char vfd_get_special_character(DISPLAY_LINE line, char* buffer, uint8_t buffer_height, uint8_t buffer_width,  uint16_t index, int offset_h, int offset_v);
    
    /**
     * Returns the character code of the special character in the given slot
     * @param slot - the special character slot number 
     * @return 
     */
    char vfd_get_special_character_in_slot(uint8_t slot);

    /**
     * Puts the VFD into reset mode - used during power up/down to prevent spurious 
     * characters being displayed
     */
    void vfd_hold_in_reset();
    
    /**
     * Called by display_initialise()
     */
    void vfd_initialise();

    /**
     * Refresh the VFD display
     */
    void vfd_refresh();

    /**
     * Set the VFD display brightness
     */
    void vfd_set_brightness(DISPLAY_BRIGHTNESS brightness);
    
    /**
     * Sets the highlight mode
     * @param mode- a value from the VFD_HIGHLIGHT_MODE enum
     */
    void vfd_set_highlight_mode(VFD_HIGHLIGHT_MODE mode);
    
    /**
     * test for clock etc...
     */
    void vfd_signal_test();

    /**
     * 
     */
    void vfd_write_line(DISPLAY_LINE line, const char* string);
    
    
    char* get_vfd_display_lines(void);
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _VFD_H */

/* *****************************************************************************
 End of File
 */
