/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    volume.h

  @Summary
 Volume control management

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _VOLUME_H    /* Guard against multiple inclusion */
#define _VOLUME_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdint.h>
#include <stdbool.h>
#include <string.h>
#include <app_config.h>

//#include <stdbool.h>
//#include <string.h>
//#include <stdlib.h>
//#include <stddef.h>

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // Move into app_config.h BUG 3513
//#define VOLUME_MAX                                  99
//#define VOLUME_FIXED_MAX                            97 // BUG 3381
//#define VOLUME_STREAMING_MAX                        99 // BUG 3373
//#define VOLUME_DEFAULT_VOLUME						99 // BUG 3373
//#define VOLUME_DEFAULT_BALANCE						50
    
    typedef enum
    {
        VOLUME_INPUT_DEFAULT,
        VOLUME_INPUT_STREAMER,
    } VOLUME_INPUT;

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Volume mapping 
     * 
     * We needed to cut 4dB in software from the output to match with other amplifiers
     * The 5dB steps at the bottom end have therefore been cut to 4dB (step increment of 8 x 0.5dB)
     * 
     * Copied from A29 audio_settings.c
     */
    static uint8_t volume_mapping[] =
    {
        1,	// 0
        26,	// 1
        34,	// 2
        42,	// 3
        50,	// 4
        58,	// 5
        66,	// 6
        74,	// 7
        82,	// 8
        90,	// 9
        98,	// 10
        102,	// 11
        106,	// 12
        110,	// 13
        114,	// 14
        118,	// 15
        122,	// 16
        126,	// 17
        130,	// 18
        134,	// 19
        138,	// 20
        140,	// 21
        142,	// 22
        144,	// 23
        146,	// 24
        148,	// 25
        150,	// 26
        152,	// 27
        154,	// 28
        156,	// 29
        158,	// 30
        160,	// 31
        162,	// 32
        164,	// 33
        166,	// 34
        168,	// 35
        170,	// 36
        172,	// 37
        174,	// 38
        176,	// 39
        178,	// 40
        180,	// 41
        182,	// 42
        184,	// 43
        186,	// 44
        188,	// 45
        190,	// 46
        192,	// 47
        194,	// 48
        196,	// 49
        198,	// 50
        199,	// 51
        200,	// 52
        201,	// 53
        202,	// 54
        203,	// 55
        204,	// 56
        205,	// 57
        206,	// 58
        207,	// 59
        208,	// 60
        209,	// 61
        210,	// 62
        211,	// 63
        212,	// 64
        213,	// 65
        214,	// 66
        215,	// 67
        216,	// 68
        217,	// 69
        218,	// 70
        219,	// 71
        220,	// 72
        221,	// 73
        222,	// 74
        223,	// 75
        224,	// 76
        225,	// 77
        226,	// 78
        227,	// 79
        228,	// 80
        229,	// 81
        230,	// 82
        231,	// 83
        232,	// 84
        233,	// 85
        234,	// 86
        235,	// 87
        236,	// 88
        237,	// 89
        238,	// 90
        239,	// 91
        240,	// 92
        241,	// 93
        242,	// 94
        243,	// 95
        244,	// 96
        245,	// 97
        246,	// 98
        247,	// 99
    };
    
    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Sets a number of 0.5dB steps by which the volume is automatically attenuated by
     * No no change is made until the next volume_set
     */
    void volume_attenuate(int8_t by);

    /**
     * Sets a number of 0.5dB steps by which the volume is automatically boosted by
     * No no change is made until the next volume_set
     */
    void volume_boost(int8_t by);
    
    /**
     * Sets it so that the max on volume is checked when the volume is next set
     */
    void volume_check_max_on_volume();
    
    /**
     * Simulates a fade in from silence
     */
    void volume_fade_in();
    
    /**
     * Gets the set volume level (note if in a ramp this will not be the same as the 
     * current ramp level)
     * @return 
     */
    uint8_t volume_get();
    
    /**
     * Gets the volume level from the settings 
     * @return 
     */
    uint8_t volume_get_stored();

    /**
     * The RS232 commands can adjust balance relative to center e.g. a value 0f -6
     * changes the balance to L06 - this does the calculation to convert the relative 
     * value to  an absolute balance
     * @param relative_balance
     * @return 
     */
    uint8_t volume_get_adjusted_balance(int relative_balance);
    
    /**
     * Gets the current volume level (this changes as the volume is ramped
     * @return 
     */
    uint8_t volume_get_current();

    /**
     * Gets the current balance
     * @return 
     */
    uint8_t volume_get_balance();
    
    /**
     * Returns a string representation of the current balance value
     * @return 
     */
    void volume_get_balance_string(char* buffer, size_t size);

    /**
     * Get whether fixed volume is in force.
     * @return 
     */
    bool volume_get_fixed_volume();
    
    /**
     * Gets the maximum volume after power-up
     * @return 
     */
    uint8_t volume_get_max_on_volume();

    /**
     * Gets the maximum net volume allowed - note this is always < maximum_volume
     * @return 
     */
    uint8_t volume_get_maximum_net_volume();

    /**
     * Gets the maximum volume allowed
     * @return 
     */
    uint8_t volume_get_maximum_volume();
 
    /**
     * The RS232 commands can adjust balance relative to center e.g. a value 0f -6
     * changes the balance to L06 - this does the calculation to convert the absolute 
     * value to  a relative balance
     * @param relative_balance
     * @return 
     */
    int volume_get_relative_balance(uint8_t balance);
    
    /**
     * Initialise volume control management
     */
    void volume_initialise();

    /**
     * Check whether the volume is currently ramping
     * @return 
     */
    bool volume_in_ramp();
    
    /**
     * Check whether the volume is muted
     */
    bool volume_is_muted();
    
    /**
     * Can be used to check whether the volume is within the valid range
     */
    bool volume_is_valid(uint8_t value);

    /**
     * Mute the volume
     */
    void volume_mute();

    /**
     * Process volume for volume ramping
     */
    void volume_process();
    
    /**
     * Sets the current volume level
     * 
     * @param value the new volume (in the range 0-99)
     * @param store - true stores value in settings, false (processor mode) - does not store the volume
     */
    void volume_set(uint8_t value, bool store);
    
    /**
     * Sets the current balance
     * 
     * @param value the new balance (in the range 0-??)
     */
    void volume_set_balance(uint8_t value, bool store);
    
    /**
     * Sets the current input type - this affects which values is returned by volume_get_max_volume
     * @param input
     */
    void volume_set_current_input(VOLUME_INPUT input);
    
    /**
     * Sets the volume to a fixed 99. volume_set_volume is ignored..
     * @param on
     */
    void volume_set_fixed_volume(bool on);
    
    /**
     * Sets the current volume level without ramping up from the current level
     * 
     * @param value the new volume (in the range 0-99)
     * @param store - true stores value in settings, false (processor mode) - does not store the volume
     */
    void volume_set_immediate(uint8_t value, bool store);
    
    /**
     * Sets the maximum value for when the unit is switched on
     * 
     * @param value the new balance (in the range 0-??)
     */
    void volume_set_max_on_volume(uint8_t value);
     
    /**
     * Sets the maximum value that can be set
     * 
     * @param value the new balance (in the range 0-??)
     */
    uint8_t volume_set_maximum_volume(uint8_t value, VOLUME_INPUT input);
    
    /**
     * Starts volumen processing
     */
    void volume_start();
    
    /**
     * Unmutes the volume
     */
    void volume_unmute();
    
    
    /**-----------------------------------------------------------------------------
     * The functions below are for use only with the volume.c and the volume_hw_xxxx.c source files
     */
    
    /**
     * Sets the actual volume control value using volume mapping
     */
    void volume_set_volume(uint8_t value);

    
    /**-------------------------------------------------------------------------
     * The following functions should be implemented in a device specific c file
     * This is to handle the need for differences in the way the SA30 and ST60
     * volume is controlled. And are only used by the volume.c file
     */
    
    /**
     * Used to perform any specific initialisation of the Volume hardware
     */
    void volume_hw_initialise();
    
    /**
     * Used to ensure that the Volume component (On the SA30) stays alive.
     */
    void volume_hw_keep_alive(uint8_t current_volume);
    
    /**
     * Used in volume mute to allow the hardware to do mute specific things
     */
    void volume_hw_mute();
    
    /**
     * Used to set the volume.  The volumes passed are the mapped value from the 
     * volume_mappings and are in the range 1-247 (1 being silence, 247 being max volume)
     * @param left
     * @param right
     */
    void volume_hw_set_volume(uint8_t left, uint8_t right);
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _VOLUME_H */

/* *****************************************************************************
 End of File
 */
