/* ************************************************************************** */
/** Descriptive File Name

  @Company
    ARCAM

  @File Name
    watchdog.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _WATCHDOG_H    /* Guard against multiple inclusion */
#define _WATCHDOG_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

/* This section lists the other files that are included in this file.
 */

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    
    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    typedef enum
    {
        WATCHDOG_STATUS_DISABLED    = 0x00,
        WATCHDOG_STATUS_ENABLED     = 0x01,
    } WATCHDOG_STATUS;
    
    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Disables the watchdog timer
     */
    void watchdog_disable();
    
    /**
     * Enables the watchdog timer
     */
    void watchdog_enable();
    
    /**
     * Gets the current status of the watchdog timer
     */
    WATCHDOG_STATUS watchdog_get_status();
    
    /**
     * Initialises the watchdog service
     */
    void watchdog_initialise();
    
    /**
     * Resets the watchdog timer
     */
    void watchdog_reset();
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _WATCHDOG_H */

/* *****************************************************************************
 End of File
 */
