/* ************************************************************************** */
/** Descriptive File Name

  @Company
    ARCAM

  @File Name
    adc_data.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/adc/private/adc_data.h"
#include "utilities/adc/adc_defs.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

static ADC_DATA adc_data = {
//    .functions.initialise_function = NULL,
};

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */


/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Used to find an ADC_INPUT_DEFINITION.  If not found it returns the next unused 
 * ADC_INPUT_DEFINITION
 * @param input
 * @return 
 */
ADC_INPUT_DEFINITION* adc_data_allocate_input(ADC_INPUT_ID input)
{
    ADC_INPUT_DEFINITION* result = NULL;
    ADC_INPUT_DEFINITION* next_free_result = NULL;
    
    int x;
    for (x = 0; x < ADC_INPUT_COUNT; x++)
    {
        if (adc_data.inputs[x].input == input)
        {
            result = &adc_data.inputs[x];
            break;
        }
        else if (adc_data.inputs[x].input == ADC_INPUT_ID_INVALID)
        {
            if (next_free_result == NULL)
            {
                next_free_result = &adc_data.inputs[x];
            }
        }
    }
    
    // Allocate the next free result
    if (result == NULL)
    {
        result = next_free_result;
        result->input = input;
    }
    
    return result;
}

/**-----------------------------------------------------------------------------
 * Returns a reference to the ADC_DATA structure
 * @return 
 */    
ADC_DATA* adc_data_get()
{
    return &adc_data;
}

/**-----------------------------------------------------------------------------
 * Gets a reference to the supplied input  definition
 * @param input
 * @return 
 */
ADC_INPUT_DEFINITION* adc_data_get_input_definition(ADC_INPUT_ID input)
{
    ADC_INPUT_DEFINITION* result = NULL;
    
    int x;
    for (x = 0; x < ADC_INPUT_COUNT; x++)
    {
        if (adc_data.inputs[x].input == input)
        {
            result = &adc_data.inputs[x];
            break;
        }
    }
    
    return result;
}

/**-----------------------------------------------------------------------------
 * Gets a reference to the supplied input  definition basd on its ordinal position
 * @param input
 * @return 
 */
ADC_INPUT_DEFINITION* adc_data_get_input_definition_by_index(uint8_t index)
{
    ADC_INPUT_DEFINITION* result = NULL;
    
    if (index < ADC_INPUT_COUNT)
    {
        result = &adc_data.inputs[index];
    }
    
    return result;
}

/**-----------------------------------------------------------------------------
 * Initialises the adc data module
 */
void adc_data_initialise()
{
    int x;
    for (x = 0; x < ADC_INPUT_COUNT; x++)
    {
        adc_data.inputs[x].input = ADC_INPUT_ID_INVALID;
        adc_data.inputs[x].active = false;
        adc_data.inputs[x].data_available_callback = NULL;
    }
}

/* *****************************************************************************
 End of File
 */
