/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    GPIO Hardware implementation for the analogue_direct_hw interface

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/analogue_direct_hw.h"
#include "app_config.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

//! This can be overriden in app_config.h
#ifndef ANALOGUE_DIRECT_PINS_ENABLED
#define ANALOGUE_DIRECT_PINS_ENABLED    1
#endif

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */


/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Gets current analogue direct state
 * @param on
 */
bool analogue_direct_hw_get_active()
{
    bool result = false;
    
#if ANALOGUE_DIRECT_PINS_ENABLED    
    result = !PLIB_PORTS_PinGet(ANALOGUE_DIRECT_BYPASS_PORT, 
                ANALOGUE_DIRECT_BYPASS_CHANNEL, 
                ANALOGUE_DIRECT_BYPASS_PIN);
        
    //logger_log_debugf(0, "ANALOG DIRECT HW GET  %d ", result);

#endif
        
    return result;
}   

/**-----------------------------------------------------------------------------
 * Sets pins for analogue direct on/off
 * @param on
 */
void analogue_direct_hw_set_active(bool on)
{
    logger_log_debugf(LOGGER_CODE_INPUT_CONTROL, "ANALOGUE DIRECT HW Setting %s ", on == true ? "On" : "Off");

    // Check whether Analogue direct is in use
#if ANALOGUE_DIRECT_PINS_ENABLED    
    PLIB_PORTS_PinWrite(ANALOGUE_DIRECT_BYPASS_PORT, 
                    ANALOGUE_DIRECT_BYPASS_CHANNEL, 
                    ANALOGUE_DIRECT_BYPASS_PIN,
                    !on);    
#endif
}

/* *****************************************************************************
 End of File
 */
