/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/avatar_control.h"
#include "utilities/avatar_message.h"
#include "utilities/avatar_packet.h"
#include "utilities/dirac.h"
#include "utilities/input_control_defs.h"
#include "utilities/timers.h"
#include "utilities/volume.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Requests a factory reset in the avatar
 */
void avatar_factory_reset()
{
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_FACTORY_RESET,
                AVATAR_ENCODING_NIL,
                NULL,
                0);
        
        // Queue the packet for sending
        avatar_send_packet(packet);

        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending FACTORY RESET ");
    }

    avatar_wakeup();
}

/**-----------------------------------------------------------------------------
 * Send a mute/unmute to the avatar
 */
void avatar_mute(AVATAR_MUTE_STATE state)
{
    if (!avatar_is_processing_mute())
    {
        switch (state)
        {
            case AVATAR_MUTE_STATE_UNMUTE:
            {
                input_control_set_avatar_volume(volume_get());
                break;
            }

           case AVATAR_MUTE_STATE_MUTE:
            {
                input_control_set_avatar_volume(0);
                break;
            }
        }
    
        AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
        if (packet != NULL)
        {
            uint8_t data;
            data = state;

            avatar_packet_add_data(packet,
                    AVATAR_STATE_ID_MUTE,
                    AVATAR_ENCODING_UINT8,
                    &data,
                    1);

            // Queue the packet for sending
            avatar_send_packet(packet);

            // Set a timer so that the messages received from the avatar will be ignored
            timers_reset(TIMER_INDEX_AVATAR_VOLUME_CHANGE);

            logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending %s ", state == AVATAR_MUTE_STATE_MUTE ? "Mute" : "Unmute");
        }
    }
}

/**-----------------------------------------------------------------------------
 * Send a pause message to the avatar
 */
void avatar_pause_playback()
{
    avatar_send_player_control(AVATAR_PLAYER_CONTROL_PAUSE);
}

///**-----------------------------------------------------------------------------
// * Send a Chromecast App name request to the Avatar
// */
//void avatar_request_chromecast_app_name()
//{
//    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_GET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
//    if (packet != NULL)
//    {
//        avatar_packet_add_data(packet,
//                AVATAR_STATE_ID_CHROMECAST_APP_NAME,
//                AVATAR_ENCODING_NIL,
//                NULL,
//                0);
//        
//        // Queue the packet for sending
//        avatar_send_packet(packet);
//        
//        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending CHROMECAST APP NAME REQUEST ");
//    }
//}
 
/**-----------------------------------------------------------------------------
 * Makes a request to the Avatar to retrieve the album art for the supplied URL
 */
void avatar_request_album_art(char* album_art_url)
{
    if (avatar_is_running())
    {
        AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, 60);

        uint16_t len = AVATAR_ALBUM_ART_URL_MAX_LENGTH;
        uint8_t data[len + sizeof(uint16_t)];
        memcpy(data + sizeof(uint16_t), album_art_url, len);
        data[0] = len % 256;
        data[1] = len / 256;

        if (packet != NULL)
        {
            avatar_packet_add_data(packet,
                    AVATAR_STATE_ID_ALBUM_ART,
                    AVATAR_ENCODING_BIN16,
                    data,
                    len + 2);

            // Queue the packet for sending
            avatar_send_packet(packet);

//            logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending ALBUM ART REQUEST to the Avatar %d --", len);
            logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending ALBUM ART REQUEST %02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X%02X --", 
                    (uint8_t)album_art_url[0],
                    (uint8_t)album_art_url[1],
                    (uint8_t)album_art_url[2],
                    (uint8_t)album_art_url[3],
                    (uint8_t)album_art_url[4],
                    (uint8_t)album_art_url[5],
                    (uint8_t)album_art_url[6],
                    (uint8_t)album_art_url[7],
                    (uint8_t)album_art_url[8],
                    (uint8_t)album_art_url[9],
                    (uint8_t)album_art_url[10],
                    (uint8_t)album_art_url[11],
                    (uint8_t)album_art_url[12],
                    (uint8_t)album_art_url[13],
                    (uint8_t)album_art_url[14],
                    (uint8_t)album_art_url[15],
                    (uint8_t)album_art_url[16],
                    (uint8_t)album_art_url[17],
                    (uint8_t)album_art_url[18],
                    (uint8_t)album_art_url[19],
                    (uint8_t)album_art_url[20],
                    (uint8_t)album_art_url[21],
                    (uint8_t)album_art_url[22],
                    (uint8_t)album_art_url[23],
                    (uint8_t)album_art_url[24],
                    (uint8_t)album_art_url[25],
                    (uint8_t)album_art_url[26],
                    (uint8_t)album_art_url[27],
                    (uint8_t)album_art_url[28],
                    (uint8_t)album_art_url[29],
                    (uint8_t)album_art_url[30],
                    (uint8_t)album_art_url[31]);
        }
    }
}
   
/**-----------------------------------------------------------------------------
 * Send a Dirac curve request to the Avatar
 */
void avatar_request_dirac_curve_names()
{
#if APP_FEATURE_DIRAC == APP_FEATURE_ON
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_GET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_DIRAC_CURVE_NAMES,
                AVATAR_ENCODING_NIL,
                NULL,
                0);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
        
        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending DIRAC CURVENAME REQUEST ");
    }
#endif    
}
    
/**-----------------------------------------------------------------------------
 * Send a Firm ware version request to the Avatar
 */
void avatar_request_firmware_version()
{
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_GET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_FIRMWARE_VERSION,
                AVATAR_ENCODING_NIL,
                NULL,
                0);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
        
        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending FIRMWARE VERSION REQUEST ");
    }
}
    
/**-----------------------------------------------------------------------------
 * Send a Firm ware version request to the Avatar
 */
void avatar_request_mime_type()
{
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_GET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_MIME_TYPE,
                AVATAR_ENCODING_NIL,
                NULL,
                0);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
        
        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Requesting MIME Type --");
    }
}
    
/**-----------------------------------------------------------------------------
 * Request the region from the Avatar
 */
void avatar_request_region()
{
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_GET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_REGION,
                AVATAR_ENCODING_NIL,
                NULL,
                0);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
        
        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Requesting Region --");
    }
}
    
/**-----------------------------------------------------------------------------
 * Send a Avatar System Info request to the Avatar
 */
void avatar_request_serial_number()
{
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_GET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_SERIAL_NUMBER,
                AVATAR_ENCODING_NIL,
                NULL,
                0);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
        
        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending SERIAL NUMBER REQUEST --");
    }
}
    
/**-----------------------------------------------------------------------------
 * Send a Avatar System Info request to the Avatar
 */
void avatar_request_system_info()
{
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_GET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_FIRMWARE_VERSION,
                AVATAR_ENCODING_NIL,
                NULL,
                0);
        
        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_ETHERNET_MAC_ADDRESS,
                AVATAR_ENCODING_NIL,
                NULL,
                0);
        
        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_WIFI_MAC_ADDRESS,
                AVATAR_ENCODING_NIL,
                NULL,
                0);
        
        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_HOST_NAME,
                AVATAR_ENCODING_NIL,
                NULL,
                0);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
        
        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending SYSTEM INFO REQUEST ");
    }
}
    
/**-----------------------------------------------------------------------------
 * Send the message infoming the avatar of the current balance value (-128-127)
 */
void avatar_send_balance(int8_t balance)
{
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        int8_t data;
        data = balance;

        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_BALANCE,
                AVATAR_ENCODING_INT8,
                &data,
                1);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
        
        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending Balance ");
    }
}

/**-----------------------------------------------------------------------------
 * Sets the Control4 Discovery Active state
 * @param 
 */
void avatar_send_control4_discovery(AVATAR_CONTROL4_DISCOVERY index)
{
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        uint8_t data;
        data = (uint8_t)index;

        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_CONTROL4_DISCOVERY,
                AVATAR_ENCODING_UINT8,
                &data,
                1);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
        
        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending CONTROL4 Discovery ");
    }
}

/**-----------------------------------------------------------------------------
 * Send the message instructing the Avatar to send a Control4 Identity message
 */
void avatar_send_control4_identity()
{
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        uint8_t data;
        data = 0x01;

        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_CONTROL4_IDENTIFY,
                AVATAR_ENCODING_UINT8,
                &data,
                1);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
        
        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending CONTROL4 Identity ");
    }
}

/**-----------------------------------------------------------------------------
 * Sends the response to the Crestron configure message
 */
void avatar_send_crestron_configure_response()
{
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES_RESPONSE, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        uint8_t data;
        data = 0x01;

        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_CRESTRON_CONFIGURE,
                AVATAR_ENCODING_UINT8,
                &data,
                1);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
        
        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending CRESTRON Configure response ");
    }
}

/**-----------------------------------------------------------------------------
 * Sets the Crestron Discovery Active state
 * @param 
 */
void avatar_send_crestron_discovery(AVATAR_CRESTRON_DISCOVERY index)
{
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        uint8_t data;
        data = (uint8_t)index;

        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_CRESTRON_DISCOVERY,
                AVATAR_ENCODING_UINT8,
                &data,
                1);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
        
        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending CRESTRON Discovery ");
    }
}
    
/**-----------------------------------------------------------------------------
 * Send a message to the avatar indicating the devices upgrade status
 * @param status
 */
void avatar_send_device_upgrade_status(DEVICE_UPGRADE_STATUS status)
{
    logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Send Device Upgrade Status %s -- ", DEVICE_UPGRADE_STATUS_STRINGS[status]);
    //logger_process();
    
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        uint8_t data;
        data = status;

        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_DEVICE_UPGRADE_STATUS,
                AVATAR_ENCODING_UINT8,
                &data,
                1);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
    }
}
   
/**-----------------------------------------------------------------------------
 * Send a message to the avatar indicating the devices headphone status
 * @param status
 */
void avatar_send_headphone_status(AVATAR_HEADPHONE_STATUS status)
{
    logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Send Headphone Status %s -- ", AVATAR_HEADPHONE_STATUS_STRINGS[status]);
    
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        uint8_t data;
        data = status;

        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_HEADPHONE_STATUS,
                AVATAR_ENCODING_UINT8,
                &data,
                1);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
    } 
}
   
/**-----------------------------------------------------------------------------
 * Send a message to the avatar indicating the lip sync delay to be used
 * @param status
 */
void avatar_send_lip_sync_delay(uint16_t delay)
{
    logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Send Lip Sync Delay %dms -- ", delay);
    
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        uint16_t data;
        data = delay;

        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_LIP_SYNC_DELAY,
                AVATAR_ENCODING_UINT16,
                (uint8_t*)&data,
                sizeof(uint16_t));
        
        // Queue the packet for sending
        avatar_send_packet(packet);
    }    
}
   
/**-----------------------------------------------------------------------------
 * Send a message to the avatar indicating the current maximum volume
 * @param status
 */
void avatar_send_maximum_volume(uint8_t max_volume)
{
    logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Send Maximum Volume %d -- ", max_volume);
    
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        uint8_t data;
        data = max_volume;

        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_MAXIMUM_VOLUME,
                AVATAR_ENCODING_UINT8,
                (uint8_t*)&data,
                sizeof(uint8_t));
        
        // Queue the packet for sending
        avatar_send_packet(packet);
    }    
}
 
/**-----------------------------------------------------------------------------
 * Send the message informing the Avatar of the MQA Decoder level
 */
void avatar_send_mqa_decoder_level(AVATAR_MQA_DECODER_LEVEL level)
{
    logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR MQA Decoder Level %s -- ", AVATAR_MQA_DECODER_LEVEL_STRINGS[level]);
    
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        uint8_t data;
        data = level;

        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_MQA_DECODER_LEVEL,
                AVATAR_ENCODING_UINT8,
                &data,
                1);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
        
        settings_set(SETTING_MQA_DECODER_LEVEL, &data);
    }
}

/**-----------------------------------------------------------------------------
 * Send a player control message to the avatar
 */
void avatar_send_player_control(AVATAR_PLAYER_CONTROL command)
{
    //    logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending Player Control AAA %s ", AVATAR_PLAYER_CONTROL_STRINGS[command]);
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        uint8_t data;
        data = command;

        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_PLAYER_CONTROL,
                AVATAR_ENCODING_UINT8,
                &data,
                1);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
        
        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending Player Control %s ", AVATAR_PLAYER_CONTROL_STRINGS[command]);
    }
}

/**-----------------------------------------------------------------------------
 * Send a power state message to the avatar
 */
void avatar_send_power_state(AVATAR_POWER_STATE state)
{
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        uint8_t data;
        data = AVATAR_POWER_STATE_VALUES[state];

        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_POWER_STATE,
                AVATAR_ENCODING_UINT8,
                &data,
                1);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
        
        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending %s ", AVATAR_POWER_STATE_STRINGS[state]);
    }
}

/**-----------------------------------------------------------------------------
 * Send a repeat message to the avatar
 */
void avatar_send_repeat_state(AVATAR_REPEAT_STATE state)
{
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        uint8_t data;
        data = state;

        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_REPEAT,
                AVATAR_ENCODING_UINT8,
                &data,
                1);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
        
        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending Repeat %s ", AVATAR_REPEAT_STATE_STRINGS[state]);
    }
}

/**-----------------------------------------------------------------------------
 * Send a repeat message to the avatar
 */
void avatar_send_shuffle_state(AVATAR_SHUFFLE_STATE state)
{
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        uint8_t data;
        data = state;

        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_SHUFFLE,
                AVATAR_ENCODING_UINT8,
                &data,
                1);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
        
        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending Shuffle %s ", AVATAR_REPEAT_STATE_STRINGS[state]);
    }
}

/**-----------------------------------------------------------------------------
 * Send a seek message to the avatar time in ms
 */
void avatar_send_track_seek(uint64_t seek_pos)
{
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        uint64_t data;
        data = seek_pos;

        avatar_packet_add_data(packet,
                //AVATAR_STATE_ID_SEEK_TRACK_TIME,
                AVATAR_STATE_ID_TRACK_TIME,
                AVATAR_ENCODING_UINT64,
                (uint8_t*)&data,
                sizeof(uint64_t));
        
        // Queue the packet for sending
        avatar_send_packet(packet);
        
        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending Track Position %d ", data);
    }
}

/**-----------------------------------------------------------------------------
 * Sends data out over the Avatars TCP connection
 */
void avatar_send_tcp_data(uint8_t client_id, char* data, size_t len)
{
    // We have agreed with stream that TCP data will only be sent in 512byte chunks
    size_t bytes_left = len;
    uint16_t data_offset = 0;
    uint16_t message_offset = 0;
    
    uint8_t buffer[AVATAR_TCP_PACKET_SIZE];
    
    uint8_t packet_id = 0;
    
    while (bytes_left > 0)
    {
        // Clear down the buffer
        memset(buffer, 0, AVATAR_TCP_PACKET_SIZE);
        
        AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_TCPIP_TRANSFER, AVATAR_TCP_PACKET_SIZE);
        if (packet != NULL)
        {
            // This message repeats the data length (an error but thats the way it is..)  so add that in first
            uint16_t data_len = min(AVATAR_TCP_PACKET_SIZE, bytes_left + 3);
            
            // Add Bin16 data length
            memcpy(buffer, &data_len, sizeof(uint16_t));
            
            if (packet_id == 0)
            {
                // Add total message data length
                //memcpy(&buffer[2], &data_len, sizeof(uint16_t));
            
                // Set the client identifier
                buffer[2] = client_id;
                
                message_offset = 3;
            }
            else
            {
                message_offset = 0;
            }
            
            // Copy the data
            memcpy(&buffer[message_offset], data + data_offset, data_len - 3);

            // increment the offset while data_len is still the length of the data
            data_offset += (data_len - 3);
            
            avatar_packet_add_data(packet,
                    AVATAR_STATE_ID_TCP_MESSAGE,
                    AVATAR_ENCODING_BIN16,
                    buffer,
                    data_len + 2);
            
            // Update the packet_id in the header
            packet->header.packet_id = packet_id;

            // Queue the packet for sending
            avatar_send_packet(packet);
            
            bytes_left -= data_offset;

            //logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending TCP Data offset:%d bytes_left:%d ", offset - data_len + 3, bytes_left);
        }
        
        packet_id++;
    }
}

/**-----------------------------------------------------------------------------
 * Sets the currently selected Dirac curve - DIRAC_NO_CURVE clears the selection
 * @param on
 */
void avatar_set_dirac_curve(uint8_t index)
{
#if APP_FEATURE_DIRAC == APP_FEATURE_ON
    if (avatar_is_running())
    {
        AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
        if (packet != NULL)
        {
            uint8_t data;
            data = index;

            avatar_packet_add_data(packet,
                    AVATAR_STATE_ID_DIRAC_SELECT_CURVE,
                    AVATAR_ENCODING_UINT8,
                    &data,
                    sizeof(uint8_t));

            // Queue the packet for sending
            avatar_send_packet(packet);

            DIRAC_EQ_CURVE* curve = dirac_get_curve(index);
            logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending DIRAC Select Curve %s --", curve != NULL ? curve->name : "No Curve");
        }
    }
#endif    
}
    
/**-----------------------------------------------------------------------------
 * Used to inform the avatar that it should switch its input
 * Note: This is using the Citation mechanism of selecting the external input 
 * The avatar selects its internal input when there is activity on the smartphone
 * It also appears that we send the Key Press but no Key release!!!
 * @param input
 */
void avatar_set_input(AVATAR_INPUT input)
{
    if (input == AVATAR_INPUT_EXTERNAL)
    {
        avatar_send_player_control(AVATAR_PLAYER_CONTROL_STOP); 
    }
    
    // Tell the avatar to switch input (ARCAM Version))
    if (input != AVATAR_INPUT_INTERNAL)
    {
        AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
        if (packet != NULL)
        {
            uint8_t data;
            data = input;

            avatar_packet_add_data(packet,
                    AVATAR_STATE_ID_AUDIO_INPUT,
                    AVATAR_ENCODING_UINT8,
                    (uint8_t*)&data,
                    sizeof(uint8_t));

            // Queue the packet for sending
            avatar_send_packet(packet);
                    
            logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending INPUT EXTERNAL/BYPASS %d --", input);
        }
    }
}
    
/**-----------------------------------------------------------------------------
 * Used to inform the avatar that it should switch its region
 * @param input
 */
void avatar_set_region(AVATAR_REGION region)
{
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        uint8_t data;
        data = region;

        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_REGION,
                AVATAR_ENCODING_UINT8,
                &data,
                sizeof(uint8_t));
        
        // Queue the packet for sending
        avatar_send_packet(packet);
 
        logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending REGION %s --", AVATAR_REGION_STRINGS[region]);
    }
}
   
/**-----------------------------------------------------------------------------
 * Used to inform the Avatar of the current input sample rate
 * @param input
 */
void avatar_set_sample_rate(AVATAR_SAMPLE_RATE sample_rate, bool force)
{
    if (avatar_is_running())
    {
        if (force || (avatar_message_get_sample_rate() != sample_rate))
        {
            logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Set Sample Rate %s -- ", AVATAR_SAMPLE_RATE_STRINGS[sample_rate]);

            // Set the avatar sample rate - this should be updated by the Sample Rate Set State message
            avatar_message_set_sample_rate(sample_rate);

            AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
            if (packet != NULL)
            {
               // Setup the UINT32 datatype
                uint8_t data = sample_rate; 

                avatar_packet_add_data(packet,
                        AVATAR_STATE_ID_SET_SAMPLE_RATE,
                        AVATAR_ENCODING_UINT8,
                        &data,
                        sizeof(uint8_t));

                // Queue the packet for sending
                avatar_send_packet(packet);
            }
        }
        else
        {
            logger_log_warningf(LOGGER_CODE_AVATAR, "AVATAR Set Sample Rate IGNORED %s -- ", AVATAR_SAMPLE_RATE_STRINGS[sample_rate]);
        }
    }
}
     
/**-----------------------------------------------------------------------------
 * Used to inform the Avatar of the current input sample rate converted from
 * an integer value
 * @param input
 */
void avatar_set_sample_rate2(uint32_t sample_rate)
{
    // Inform the Avatar of the new sample rate
    // First convert the sample rate to an avatar sample rate
    AVATAR_SAMPLE_RATE sample_rate_value = AVATAR_SAMPLE_RATE_0;
    int x;
    for (x = 0; x < AVATAR_SAMPLE_RATE_MAX; x++)
    {
        if (AVATAR_SAMPLE_RATE_VALUES[x] == sample_rate)
        {
            sample_rate_value = (AVATAR_SAMPLE_RATE)x;
            break;
        }
    }

    // BUG 3779 - The fix for 3776 meant that we always switch to External when we set the sample rate
    // when waking up from cast the sample rate is still calculated by we dont want to switch to External,
    // so a test has been put in to check input
    if (input_control_get_current_input_type() != INPUT_CONTROL_TYPE_AVATAR)
    {
        // BUG 3776 - With debugging off we need to tell the avavatr that we are on an external input whe we change the sample rate
        // we also force a sample rate message as we had issues where we move from 44.1 Digital to 44.1 Cast and back - in that
        // case the sample rate was not being sent.
        avatar_set_input(AVATAR_INPUT_EXTERNAL);
    }
    avatar_set_sample_rate(sample_rate_value, true);
}
   
/**-----------------------------------------------------------------------------
 * Sets the advisory upgrade option
 * @param option
 */
void avatar_set_upgrade_option(AVATAR_UPGRADE_OPTION option)
{
    logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Set Upgrade Option %s -- ", AVATAR_UPGRADE_OPTION_STRINGS[option]);

    settings_set(SETTING_AVATAR_UPGRADE_OPTION, &option);
    
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        uint8_t data;
        data = option;

        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_UPGRADE_OPTION,
                AVATAR_ENCODING_UINT8,
                &data,
                sizeof(uint8_t));
        
        // Queue the packet for sending
        avatar_send_packet(packet);
    }
}    
    
/**-----------------------------------------------------------------------------
 * Used to inform the avatar that it should display the new volume
 * @param input
 */
void avatar_set_volume(uint8_t volume)
{
    logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Set Volume %d -- ", volume);

    if (!avatar_is_processing_volume())
    {
        AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_SET_STATES, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
        if (packet != NULL)
        {
            uint8_t data;
            data = volume;

            avatar_packet_add_data(packet,
                    AVATAR_STATE_ID_VOLUME,
                    AVATAR_ENCODING_UINT8,
                    &data,
                    sizeof(uint8_t));

            // Queue the packet for sending
            avatar_send_packet(packet);

            // Set a timer so that the messages received from the avatar will be ignored
            timers_reset(TIMER_INDEX_AVATAR_VOLUME_CHANGE);

            logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR Sending VOLUME %d -- ", volume);
        }
    }
}
    
/**-----------------------------------------------------------------------------
 * Invokes the Avatar TCP Client Control message
 * @param client_id
 * @param state
 */
void avatar_tcp_client_control(uint8_t client_id, AVATAR_TCP_CLIENT_CONTROL_STATE state)
{
    logger_log_debugf(LOGGER_CODE_AVATAR, "AVATAR TCP Client Control %d %s -- ", client_id, AVATAR_TCP_CLIENT_CONTROL_STATE_STRINGS[state]);
    
    AVATAR_PACKET* packet = avatar_get_packet(AVATAR_MESSAGE_ID_TCPIP_TRANSFER, AVATAR_ABITARY_SHORT_MESSAGE_LENGTH);
    if (packet != NULL)
    {
        uint8_t data[2];
        data[0] = client_id;
        data[1] = state;

        avatar_packet_add_data(packet,
                AVATAR_STATE_ID_TCP_CLIENT_CONTROL,
                AVATAR_ENCODING_UINT16,
                data,
                2);
        
        // Queue the packet for sending
        avatar_send_packet(packet);
    }
}

/* *****************************************************************************
 End of File
 */
