/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities\cn_detect.h"
#include "app_config.h"
#include "utilities\timers.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

typedef struct
{
    //! Stores whether the module is active or not.
    bool enabled;
    
    //! Determines whether the handler has been invoked.
    bool interrupt_set;
} CN_DETECT_DATA;

static CN_DETECT_DATA cn_detect_data;

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */
    
/**-----------------------------------------------------------------------------
* Disables CN event monitoring
*/
void cn_detect_disable()
{
    cn_detect_data.interrupt_set = false;
    cn_detect_data.enabled = false;
    
    timers_expire(TIMER_INDEX_CN_DETECT);
}

/**-----------------------------------------------------------------------------
* Enables monitoring of CN events.  This can be run multiple times, however 
* only the first time will set the initial pause
* @param pause_ms = The pause in milliseconds after which the cn_detect_get() 
* message will respond positivly
* @return - true if the pause is set, false if not
*/
bool cn_detect_enable(uint16_t pause_ms)
{
    bool result = false;
    
    if ((!cn_detect_data.enabled) && (pause_ms > 0))
    {
        cn_detect_data.interrupt_set = false;
        cn_detect_data.enabled = true;
        
        timers_update_timeout(TIMER_INDEX_CN_DETECT, pause_ms, TIMERS_RESET_IMMEDIATE);
        result = true;
    }
    
    return  result;
}

/**-----------------------------------------------------------------------------
* Used to determine whether a Change Notification has occurred
* @return 
*/
bool cn_detect_get()
{
    bool result = false;
    
    if (timers_has_expired(TIMER_INDEX_CN_DETECT))
    {
        result = cn_detect_data.interrupt_set;
    }
    
    cn_detect_data.interrupt_set = false;
    return result;
}

/**-----------------------------------------------------------------------------
* This should be added to the change notification interrupt handler generated by 
* Harmony
*/
void cn_detect_interrupt_handler()
{
    if (cn_detect_data.enabled)
    {
        cn_detect_data.interrupt_set = true;
    }
    else
    {
        // Ensure it stays at false
        cn_detect_data.interrupt_set = false;
    }
}
    
/* *****************************************************************************
 End of File
 */
