/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/control4_sddp.h"
#include "utilities/logger.h"
#include "utilities/settings.h"
#include "app_config.h"
#include "utilities/tcpip.h"
#include "utilities/timers.h"
#include "utilities/udp.h"
#include "utilities/uuid.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

#define CONTROL4_SDDP_MAX_AGE_SECS   TIMER_TIMEOUT_CONTROL4_SDDP // Shoud be a minumim of 1800 
#define CONTROL4_SDDP_DISCOVERY_URI  "239.255.255.250:1902"    
#define CONTROL4_SDDP_ADDRESS  "239.255.255.250"    
#define CONTROL4_SDDP_PORT  1902   

#define CONTROL4_SDDP_MESSAGE_TYPES \
    X(RESPONSE, Search Response)\
    X(ALIVE, Notify Alive)\
    X(IDENTIFY, Notify Indentify)\
    X(OFFLINE, Notify Offline)\

// Now use the input data to define the enum
#define X(a, b) CONTROL4_SDDP_MESSAGE_TYPE_##a,
	typedef enum {CONTROL4_SDDP_MESSAGE_TYPES} CONTROL4_SDDP_MESSAGE_TYPE;
#undef X
    
// Redefine the macro and use to create an array of strings
#define X(a, b) #b,
	static char *CONTROL4_SDDP_MESSAGE_TYPES_STRINGS[] = {CONTROL4_SDDP_MESSAGE_TYPES};
#undef X

//typedef enum
//{
//    CONTROL4_SDDP_MESSAGE_TYPE_RESPONSE,
//    CONTROL4_SDDP_MESSAGE_TYPE_ALIVE,
//    CONTROL4_SDDP_MESSAGE_TYPE_IDENTIFY,
//    CONTROL4_SDDP_MESSAGE_TYPE_OFFLINE,
//} CONTROL4_SDDP_MESSAGE_TYPE;

typedef struct
{
    bool active;
    
    // Data for response messages
    IP_ADDR  response_address;
    uint16_t response_port;
    //char     response_tran_buffer[15];
    
} CONTROL4_SDDP_DATA;

static CONTROL4_SDDP_DATA control4_sddp_data;

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

//! Forward declaration
static void control4_sddp_generate_notify_alive_message(char* buffer);
static void control4_sddp_generate_notify_identify_message(char* buffer);
static void control4_sddp_generate_notify_offline_message(char* buffer);
static void control4_sddp_generate_search_response_message(char* buffer);
static void control4_sddp_send_message(CONTROL4_SDDP_MESSAGE_TYPE type);

/**-----------------------------------------------------------------------------
 * Callback for udp messages 
 */
static bool control4_sddp_callback(int port, struct sockaddr_in* address, int* address_len, char* message, int len)
{
    bool result = false;

    // Check IPAddress and port number the message is sent from is correct.....
    // This didn't work as they send the search request as an SSDP message on 1900 and 
    // expect a reply on 1902!!
    if (port == 1902)
    {
        bool send_response = false;
        int timeout = 0;

        // Buffer to hold the content of the TRAN parameter.  Will contain the whole
        // TRAN line e.g. TRAN: 1234 without the
        char ip_address_buffer[16];
        char tran_buffer[15];

//        TCPIP_Helper_IPAddressToString((const IPV4_ADDR*)&address->sin_addr, ip_address_buffer, 16);
//        //logger_log_debugf(0, "CONTROL4 SDDP: address %s:%d --", ip_address_buffer, address->sin_port);
//
        // We're assuming we're looking for a SEARCH message and that the check below is the right format
        //if (strncmp(message, "M-SEARCH * HTTP/1.1", 19) == 0)
        if (strncmp(message, "SEARCH * SDDP/1.0", 15) == 0)
        {
//            logger_log_debugf(0, "CONTROL4 SDDP: Search Request Found - Sending response");
            send_response = true;
        }
        else
        {
            send_response = false;
        }
        //logger_log_debugf(0, "CONTROL4 SDDP A %s ", message);

        if (send_response)
        {
            // Store the address/port
            memcpy(&control4_sddp_data.response_address, &address->sin_addr, sizeof(IPV4_ADDR));
            control4_sddp_data.response_port = 1902;//address->sin_port;

            // tran_buffer is blank in an ssdp message...
            //memcpy(control4_sddp_data.response_tran_buffer, tran_buffer, 15);

            if (timeout > 0)
            {
                timers_update_timeout(TIMER_INDEX_CONTROL4_SDDP_RESPONSE, timeout * 1000, TIMERS_RESET_IMMEDIATE); 
                timers_activate(TIMER_INDEX_CONTROL4_SDDP_RESPONSE);
            }
            else
            {
                control4_sddp_send_message(CONTROL4_SDDP_MESSAGE_TYPE_RESPONSE);
            }

            result = true;
        }
    }

    return result;
}

/**-----------------------------------------------------------------------------
 * Generates the Host: address - the bugger should be at least 21 characters long
 * @param address_buffer
 */
static void control4_sddp_generate_host_address(char* address_buffer)
{
    // Get the IP Address
    char ip_address[16]; // Include trailing 0
    tcpip_get_ip_address_string(ip_address, 16);
    
    sprintf(address_buffer, "%s:%d", ip_address, CONTROL4_SDDP_PORT);
}

/**-----------------------------------------------------------------------------
 * Note: 
 * @param type
 * @param buffer - A buffer to contain the message should be at least UDP_MESSAGE_MAX_SIZE
 */
static void control4_sddp_generate_notify_alive_message(char* buffer)
{
    //char buffer[UDP_MESSAGE_MAX_SIZE];
    const char quote = '\"';
    const char cr = '\r';
    const char lf = '\n';
    
    // Get the IP Address
    char ip_address[21]; // Include trailing 0
    control4_sddp_generate_host_address(ip_address);

    char network_name[TCPIP_NETWORK_NAME_MAX_LEN];
    tcpip_get_network_name_string(network_name, TCPIP_NETWORK_NAME_MAX_LEN);
    
    const char response_message[UDP_MESSAGE_MAX_SIZE] = "NOTIFY ALIVE SDDP/1.0%c%c\
From:%c%s%c%c%c\
Host:%c%s%c%c%c\
Max-Age: 1800%c%c\
Type:%c%s%s%c%c%c\
Primary-Proxy:%c%s%c%c%c\
Proxies:%c%s%c%c%c\
Manufacturer:%c%s%c%c%c\
Model:%c%s%c%c%c\
Driver:%c%s%s.c4i%c%c%c%c%c";
    
    sprintf(buffer, response_message, 
            cr, lf, 
            quote, ip_address, quote, cr, lf,                       // From
            quote, network_name, quote, cr, lf,                     // Host
            cr, lf,                                                 // Max_age
            quote, CONTROL4_SDDP_TYPE, SOFTWARE_MODEL_STRING, quote, cr, lf,               // Type
            quote, CONTROL4_SDDP_PRIMARYPROXY, quote, cr, lf,       // Primary-Proxy
            quote, CONTROL4_SDDP_PROXIES, quote, cr, lf,            // Proxies
            quote, CONTROL4_SDDP_MANUFACTURER, quote, cr, lf,       // Manufacturer
            quote, SOFTWARE_MODEL_STRING, quote, cr, lf,                            // Model
            quote, CONTROL4_SDDP_DRIVER, SOFTWARE_MODEL_STRING, quote, cr, lf,             // Driver
            cr, lf,
            cr, lf);    
}

/**-----------------------------------------------------------------------------
 * Note: 
 * @param type
 * @param buffer - A buffer to contain the message should be at least UDP_MESSAGE_MAX_SIZE
 */
static void control4_sddp_generate_notify_identify_message(char* buffer)
{
    //char buffer[UDP_MESSAGE_MAX_SIZE];
    const char quote = '\"';
    const char cr = '\r';
    const char lf = '\n';
    
    // Get the IP Address
    char ip_address[21]; // Include trailing 0
    control4_sddp_generate_host_address(ip_address);

    char network_name[TCPIP_NETWORK_NAME_MAX_LEN];
    tcpip_get_network_name_string(network_name, TCPIP_NETWORK_NAME_MAX_LEN);
    
    const char response_message[UDP_MESSAGE_MAX_SIZE] = "NOTIFY IDENTIFY SDDP/1.0%c%c\
From:%c%s%c%c%c\
Host:%c%s%c%c%c\
Type:%c%s%s%c%c%c\
Primary-Proxy:%c%s%c%c%c\
Proxies:%c%s%c%c%c\
Manufacturer:%c%s%c%c%c\
Model:%c%s%c%c%c\
Driver:%c%s%s.c4i%c%c%c%c%c";
    
    sprintf(buffer, response_message, 
            cr, lf, 
            quote, ip_address, quote, cr, lf,                       // From
            quote, network_name, quote, cr, lf,                     // Host
            quote, CONTROL4_SDDP_TYPE, SOFTWARE_MODEL_STRING, quote, cr, lf,               // Type
            quote, CONTROL4_SDDP_PRIMARYPROXY, quote, cr, lf,       // Primary-Proxy
            quote, CONTROL4_SDDP_PROXIES, quote, cr, lf,            // Proxies
            quote, CONTROL4_SDDP_MANUFACTURER, quote, cr, lf,       // Manufacturer
            quote, SOFTWARE_MODEL_STRING, quote, cr, lf,                            // Model
            quote, CONTROL4_SDDP_DRIVER, SOFTWARE_MODEL_STRING, quote, cr, lf,             // Driver
            cr, lf,
            cr, lf);    
}

/**-----------------------------------------------------------------------------
 * Note: 
 * @param type
 * @param buffer - A buffer to contain the message should be at least UDP_MESSAGE_MAX_SIZE
 */
static void control4_sddp_generate_notify_offline_message(char* buffer)
{
    //char buffer[UDP_MESSAGE_MAX_SIZE];
    const char quote = '\"';
    const char cr = '\r';
    const char lf = '\n';
    
    // Get the IP Address
    char ip_address[21]; // Include trailing 0
    control4_sddp_generate_host_address(ip_address);

    char network_name[TCPIP_NETWORK_NAME_MAX_LEN];
    tcpip_get_network_name_string(network_name, TCPIP_NETWORK_NAME_MAX_LEN);
    
    const char response_message[UDP_MESSAGE_MAX_SIZE] = "NOTIFY OFFLINE SDDP/1.0%c%c\
From:%c%s%c%c%c\
Host:%c%s%c%c%c\
Type:%c%s%s%c%c%c\
%c%c%c%c";
    
    sprintf(buffer, response_message, 
            cr, lf, 
            quote, ip_address, quote, cr, lf,                       // From
            quote, network_name, quote, cr, lf,                     // Host
            quote, CONTROL4_SDDP_TYPE, SOFTWARE_MODEL_STRING, quote, cr, lf,               // Type
            cr, lf,
            cr, lf);    
}

/**-----------------------------------------------------------------------------
 * Note: It expects the control4_sddp_data.response_tran_buffer to be populated with the TRAN
 * parameter from the Search Request.
 * @param buffer - A buffer to contain the message should be at least UDP_MESSAGE_MAX_SIZE
 */
static void control4_sddp_generate_search_response_message(char* buffer)
{
    //char buffer[UDP_MESSAGE_MAX_SIZE];
    const char quote = '\"';
    const char cr = '\r';
    const char lf = '\n';
    
    // Get the IP Address
    char ip_address[21]; // Include trailing 0
    control4_sddp_generate_host_address(ip_address);

    char network_name[TCPIP_NETWORK_NAME_MAX_LEN];
    tcpip_get_network_name_string(network_name, TCPIP_NETWORK_NAME_MAX_LEN);
    
    const char response_message[UDP_MESSAGE_MAX_SIZE] = "SDDP/1.0 200 OK%c%c\
From:%c%s%c%c%c\
Host:%c%s%c%c%c\
Tran:%c%c\
Max-Age: 1800%c%c\
Type:%c%s%s%c%c%c\
Primary-Proxy:%c%s%c%c%c\
Proxies:%c%s%c%c%c\
Manufacturer:%c%s%c%c%c\
Model:%c%s%c%c%c\
Driver:%c%s%s.c4i%c%c%c%c%c";
    
    sprintf(buffer, response_message, 
            cr, lf, 
            quote, ip_address, quote, cr, lf,                       // From
            quote, network_name, quote, cr, lf,                     // Host
            cr, lf,                                                 // Tran
            cr, lf,                                                 // Max_age
            quote, CONTROL4_SDDP_TYPE, SOFTWARE_MODEL_STRING, quote, cr, lf,               // Type
            quote, CONTROL4_SDDP_PRIMARYPROXY, quote, cr, lf,       // Primary-Proxy
            quote, CONTROL4_SDDP_PROXIES, quote, cr, lf,            // Proxies
            quote, CONTROL4_SDDP_MANUFACTURER, quote, cr, lf,       // Manufacturer
            quote, SOFTWARE_MODEL_STRING, quote, cr, lf,                            // Model
            quote, CONTROL4_SDDP_DRIVER, SOFTWARE_MODEL_STRING, quote, cr, lf,             // Driver
            cr, lf,
            cr, lf);    
}

/**-----------------------------------------------------------------------------
 * 
 */
static void control4_sddp_send_message(CONTROL4_SDDP_MESSAGE_TYPE type)
{
    char buffer[UDP_MESSAGE_MAX_SIZE];
    memset(buffer, 0, UDP_MESSAGE_MAX_SIZE);
     
    IP_ADDR address;
    TCPIP_Helper_StringToIPAddress(CONTROL4_SDDP_ADDRESS, &address);
    
    uint16_t port = CONTROL4_SDDP_PORT;
    
    switch (type)
    {
        case CONTROL4_SDDP_MESSAGE_TYPE_RESPONSE:
        {
            control4_sddp_generate_search_response_message(buffer);
            
            // Copy the saved address and port
            address.Val = control4_sddp_data.response_address.Val;
            port = control4_sddp_data.response_port;
            break;
        }
            
        case CONTROL4_SDDP_MESSAGE_TYPE_ALIVE:
            control4_sddp_generate_notify_alive_message(buffer);
            break;
            
        case CONTROL4_SDDP_MESSAGE_TYPE_IDENTIFY:
            control4_sddp_generate_notify_identify_message(buffer);
            break;
            
        case CONTROL4_SDDP_MESSAGE_TYPE_OFFLINE:
            control4_sddp_generate_notify_offline_message(buffer);
            break;
            
        default:
            break;
    }

    logger_log_debugf(0, "CONTROL4 SDDP Sending %s message ", CONTROL4_SDDP_MESSAGE_TYPES_STRINGS[type]);
    
    // Send the buffer via UDP
    if (strlen(buffer) > 0)
    {
        udp_sendto(&address, port, buffer, strlen(buffer));
    }
}

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */
    
/**-----------------------------------------------------------------------------
 * Returns whether Control 4 SDDP messaging is active or not
 * @return 
 */
bool control4_sddp_get_active()
{
    return control4_sddp_data.active;
}

/**-----------------------------------------------------------------------------
 * Initialise UPnP functionality - must be called after TCPIP/UDP initialisation
 */
void control4_sddp_initialise()
{
    settings_get(SETTING_CONTROL4_SDDP_ACTIVE, &control4_sddp_data.active);
    logger_log_debugf(0, "CONTROL4 SDDP Active : %s ", control4_sddp_data.active ? "Yes" : "No");
    
    // Register a callback with UDP
    udp_add_callback(control4_sddp_callback);
}

/**-----------------------------------------------------------------------------
 * Processes UPnP actions
 */
void control4_sddp_process()
{
    if (control4_sddp_get_active())
    {
        if (tcpip_is_connected() && udp_is_connected())
        {
            if (timers_has_expired(TIMER_INDEX_CONTROL4_SDDP_RESPONSE))
            {
                control4_sddp_send_message(CONTROL4_SDDP_MESSAGE_TYPE_RESPONSE);

                timers_update_timeout(TIMER_INDEX_CONTROL4_SDDP_RESPONSE, 1, TIMERS_TIMEOUT_RESET_INACTIVE);    
            }

            if (timers_has_expired(TIMER_INDEX_CONTROL4_SDDP))
            {
                control4_sddp_send_notify_alive_message();
            }
            else if (timers_has_expired(TIMER_INDEX_CONTROL4_SDDP_DELAYED_ALIVE))
            {
                control4_sddp_send_notify_alive_message();
                
                // Reset the timer but in an inactive state.
                timers_update_timeout(TIMER_INDEX_CONTROL4_SDDP_DELAYED_ALIVE, 5000, TIMERS_TIMEOUT_RESET_INACTIVE);   
            }
        }
    }
}
    
/**-----------------------------------------------------------------------------
 * Send a Notify Identify message
 */
void control4_sddp_send_notify_alive_message()
{
    if (control4_sddp_get_active())
    {
        control4_sddp_send_message(CONTROL4_SDDP_MESSAGE_TYPE_ALIVE);

        timers_reset(TIMER_INDEX_CONTROL4_SDDP);  
    }
}

/**-----------------------------------------------------------------------------
 * Send a Notify Identify message
 */
void control4_sddp_send_notify_identify_message()
{
    if (control4_sddp_get_active())
    {
        control4_sddp_send_message(CONTROL4_SDDP_MESSAGE_TYPE_IDENTIFY);
    }
}

/**-----------------------------------------------------------------------------
 * Send a Notify Offline message
 */
void control4_sddp_send_notify_offline_message()
{
    if (control4_sddp_get_active())
    {
        control4_sddp_send_message(CONTROL4_SDDP_MESSAGE_TYPE_OFFLINE);    
    }
}
    
/**-----------------------------------------------------------------------------
 * Sets whether Control 4 messaging is active or not
 * @param active - true - messaging is active, false - messaging is off
 */
void control4_sddp_set_active(bool active)
{
    control4_sddp_data.active = active;
    
    settings_set(SETTING_CONTROL4_SDDP_ACTIVE, &control4_sddp_data.active);
    
    if (control4_sddp_data.active)
    {
        timers_expire(TIMER_INDEX_CONTROL4_SDDP);
    }
}
    
/**-----------------------------------------------------------------------------
 * Callback for when a new TCPIP connection is established
 */
void control4_sddp_tcpip_new_connection(bool connected, struct sockaddr_in* address, int* address_len)
{
    // Note this fails on the initial TCPListening event as UDB is not ready at 
    // that point, however another is sent afterwards so not a problem.
    if (connected && udp_is_connected())
    {
        // Start a 5 second timer to ensure the TCP/UDP stack has sorted itself 
        // out now the cable has been plugged back in.
        timers_update_timeout(TIMER_INDEX_CONTROL4_SDDP_DELAYED_ALIVE, 5000, TIMERS_RESET_IMMEDIATE); 
        timers_activate(TIMER_INDEX_CONTROL4_SDDP_DELAYED_ALIVE);
    }
}

/**-----------------------------------------------------------------------------
 * Generates a dummy search request message
 */
void control4_sddp_test_search()
{
    logger_log_debugf(0, "CONTROL4 SDDP Generating test Search Message");
    
    char buffer[UDP_MESSAGE_MAX_SIZE];
    memset(buffer, 0, UDP_MESSAGE_MAX_SIZE);
    
    const char quote = '\"';
    const char cr = '\r';
    const char lf = '\n';
    
    IP_ADDR ip_address;
    TCPIP_Helper_StringToIPAddress(CONTROL4_SDDP_ADDRESS, &ip_address);
    
      /* The address is IPv4 */
    struct sockaddr_in address;
    int address_len = sizeof(struct sockaddr_in);
    address.sin_family = AF_INET;
    address.sin_addr.s_addr = ip_address.Val;

    /* sockets are unsigned shorts, htons(x) ensures x is in network byte order, set the port to 7654 */
    address.sin_port = CONTROL4_SDDP_PORT;
 
    const char response_message[UDP_MESSAGE_MAX_SIZE] = "SEARCH SDDP/1.0%c%c\
From:%c%s%c%c%c\
Tran:%d%c%c\
Timeout:   %d%c%c%c%c";
    
    sprintf(buffer, response_message, 
            cr, lf, 
            quote, CONTROL4_SDDP_DISCOVERY_URI, quote, cr, lf,      // From
            2000, cr, lf,                                           // Tran
            3, cr, lf,                                              // Timeout
            cr, lf,
            cr, lf);    

    control4_sddp_callback(CONTROL4_SDDP_PORT, &address, &address_len, buffer, sizeof(buffer));
}
    
/* *****************************************************************************
 End of File
 */
