/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/input_analogue_direct_app.h"
#include <stdio.h>
#include "app_config.h"
#include "utilities/analogue_direct.h"
#include "utilities/input_control_app.h"
#include "utilities/input_control_defs.h"
#include "utilities/menu.h"
#include "utilities/menu_app.h"
#include "utilities/timers.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */


/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */


/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Used in menus to review whether a change in analogue-direct for a channel needs 
 * the actual analogue direct setting or not.
 * @param menu
 * @return 
 */
MENU_ID input_analogue_direct_check(MENU_ITEM menu)
{
    MENU_ID result = MENU_USE_SUPPLIED_MENU_ID;
    
    INPUT_CONTROL_DEFINITION* definition = input_control_get_current_definition();
    if (input_control_app_get_edit_index() == definition->index)
    {
        analogue_direct_set_active(input_analogue_direct_get(definition->index));    //definition->analogue_direct);
    }
    
    menu_app_reset_inactivity_timeout(menu.menu_id);
    
    return result;
}

/**-----------------------------------------------------------------------------
 * Gets the analogue direct setting for the current channel as a string
 * Note: This is used for all the analogue direct menus and is dependent on
 * the correct value of input_control_app_data.analogue_direct_edit_index being
 * set before (usually in the menu enter event)
  * @return 
 */
void input_analogue_direct_get_string(char* buffer, size_t size)
{
    INPUT_CONTROL_DEFINITION* definition = input_control_get_definition(input_control_app_get_edit_index());
    
    if (definition != NULL)
    {
        sprintf(buffer, "%s", APP_COMMAND_OFF_ON_STRINGS[(int)input_analogue_direct_get(definition->index)]);   //definition->analogue_direct]);    
    }
}

/**-----------------------------------------------------------------------------
 * Called when entering the Analogue Direct no navigation menu - setup the timeout
 */
void input_analogue_direct_enter_no_navigation(MENU_ID menu)
{
    // Set the edit index to the current input
    input_control_app_set_edit_index(input_control_get_selected_input());
    
    // Ensure the Analogue Direct menu is only visible for a while
    timers_update_timeout(TIMER_INDEX_MENU_INACTIVE, TIMER_MENU_ANALOGUE_DIRECT_INACTIVE_TIMEOUT_MS, TIMERS_RESET_IMMEDIATE);
}

/**-----------------------------------------------------------------------------
 * Called when leaving the the Analogue Direct no navigation menu 
 * @param menu
 */
MENU_ID input_analogue_direct_exit_no_navigation(MENU_ID menu)
{
    // Check to see if analogue direct needs applying to active input
    MENU_ID result = input_analogue_direct_check(*menu_get_item(menu));
    
    // Make sure Inactive Menu timeout is reset
    menu_app_reset_inactivity_timeout(menu);
    
    return result;
}


/* *****************************************************************************
 End of File
 */
