/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/input_analogue_direct_data.h"
#include "app_config.h"
#include "utilities/settings.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

typedef struct
{
    //! Determines whether analogue direct is used for a given input
    uint8_t input_setting[INPUT_CONTROL_NUM_SETTINGS];
    
    //! Determine whether the settings have been read
    bool settings_read;

} ANALOGUE_DIRECT_DATA;

static ANALOGUE_DIRECT_DATA input_analogue_direct_data = {
    .settings_read = false,
};

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

static void input_analogue_direct_data_ensure_settings()
{
    if (!input_analogue_direct_data.settings_read)
    {
        settings_get(SETTING_ANALOGUE_DIRECT, input_analogue_direct_data.input_setting);
        
        input_analogue_direct_data.settings_read = true;
    }
}

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Gets analogue direct setting for a given input
 * @param input
 * @param on
 */
bool input_analogue_direct_data_get(uint8_t index)
{
    input_analogue_direct_data_ensure_settings();
    
    return input_analogue_direct_data.input_setting[index];
}

/**-----------------------------------------------------------------------------
 * Store analogue direct setting for a given input
 * @param input
 * @param on
 */
void input_analogue_direct_data_set(uint8_t index, bool on)
{
    input_analogue_direct_data_ensure_settings();
    
    input_analogue_direct_data.input_setting[index] = on;
    settings_set(SETTING_ANALOGUE_DIRECT, input_analogue_direct_data.input_setting);
}

/* *****************************************************************************
 End of File
 */
