/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "app_config.h"
#include "utilities/i2c.h"
#include "utilities/input_adc/input_adc_feature.h"
#include "utilities/input_control_analogue.h"
#include "utilities/input_phono_input.h"
#include "utilities/analogue_direct.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */
/**
 * Global data
 */
//typedef struct
//{
//    PHONO_INPUT phono_input;
//    
//} INPUT_CONTROL_ANALOGUE_DATA;
//
//static INPUT_CONTROL_ANALOGUE_DATA input_control_analogue_data =
//{
//};

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Initialise the input control module
 */
bool input_control_analogue_initialise()
{
    bool result = true;
    
    i2c_initialise();

//    input_control_analogue_data.adc_setting_count = 0;
//    input_control_analogue_data.adc_settings_applied = false;
    
    // read input_enabled, and analogue direct from settings
    uint8_t input_enabled_settings[INPUT_CONTROL_NUM_SETTINGS];
    
    settings_get(SETTING_INPUT_ENABLED, input_enabled_settings);
    
    int x;
    for (x = 0; x < INPUT_CONTROL_NUM_SETTINGS; x++)
    {
        INPUT_CONTROL_DEFINITION* definition = input_control_get_definition(x);
        definition->index = 0;
        //definition->analogue_direct = input_analogue_direct_get(x);
        definition->enabled = input_enabled_settings[x];
        
        logger_log_debugf(0, "INPUT CONTROL %d %s %s %s -- ", x, AUDIO_INPUT_STRINGS[x], input_analogue_direct_get(x) == 0 ? "DIGITAL" : "ANALOGUE", input_enabled_settings[x] == 0 ? "DISABLED" : "ENABLED");
    }
    
    PHONO_INPUT input;
    settings_get(SETTING_MM_MC, &input);
    input_phono_input_set(input);

    input_adc_feature_initialise();
    
    return result;
}

/**-----------------------------------------------------------------------------
 * Gets the number of enabled inputs
 * @return 
 */
static uint8_t input_control_get_num_enabled()
{
    uint8_t result = 0;
    
    // Note: Start from 1 as we ignore the None input
    int x;
    for (x = 1; x < INPUT_CONTROL_NUM_SETTINGS; x++)
    {
        INPUT_CONTROL_DEFINITION* definition = input_control_get_definition(x);
        if (definition->enabled)
        {
            result++;
        }
    }
    
    return result;
}
    
/**-----------------------------------------------------------------------------
// * Gets the phono input to be MM or MC
// */
//PHONO_INPUT input_control_get_phono_input()
//{
//    return min(input_control_analogue_data.phono_input, PHONO_INPUT_MAX - 1);
//}

/**-----------------------------------------------------------------------------
 * Sets analogue direct setting for a given input channel
 * @param on
 */
void input_control_set_input_analogue_direct(uint8_t index, bool on)
{
    INPUT_CONTROL_DEFINITION* definition = input_control_get_definition(index);
    //definition->analogue_direct = on;
    
    // Settings
    logger_log_debugf(0, "INPUT CONTROL Analogue Direct %d %d %s --", definition->index, on, AUDIO_INPUT_STRINGS[definition->index]);
    input_analogue_direct_set(definition->index, on);
}

/**-----------------------------------------------------------------------------
 * Sets enabled setting for a given input channel
 * @param on
 */
void input_control_set_input_enabled(uint8_t index, bool on)
{
    // Check that were not setting them all off
    if (on || (input_control_get_num_enabled() > 1))
    {
        INPUT_CONTROL_DEFINITION* definition = input_control_get_definition(index);
        definition->enabled = on;

        // Settings
        uint8_t input_enabled_settings[INPUT_CONTROL_NUM_SETTINGS];
        settings_get(SETTING_INPUT_ENABLED, input_enabled_settings);

        logger_log_debugf(0, "INPUT CONTROL input_control_set_input_enabled %d %d %d %d --", definition->index, index, on, input_enabled_settings[definition->index]);
        input_enabled_settings[definition->index] = on;
        settings_set(SETTING_INPUT_ENABLED, input_enabled_settings);
    }
    else
    {
        logger_log_warningf(LOGGER_CODE_INPUT_CONTROL, "INPUT CONTROL Attempt to set available inputs to 0 ");
    }
}   
    
/**-----------------------------------------------------------------------------
 * Sets the phono input to be MM or MC
 * @param type
 */
//void input_control_set_phono_input(PHONO_INPUT type)
//{
//    input_control_analogue_data.phono_input = type;
//    
//    bool switch0 = false;
//    
//    switch(type)
//    {
//        case PHONO_INPUT_MM:
//            switch0 = false;
//            break;
//            
//        case PHONO_INPUT_MC:
//            switch0 = true;
//            break;
//            
//        default:
//            break;
//    }
//
//    PLIB_PORTS_PinWrite(INPUT_CONTROL_MM_MC_SELECT_ID, 
//                INPUT_CONTROL_MM_MC_SELECT_PORT_CHANNEL, 
//                INPUT_CONTROL_MM_MC_SELECT_PIN,
//                switch0);  
//    
//    settings_set(SETTING_MM_MC, &input_control_analogue_data.phono_input);
//}
    
/**-----------------------------------------------------------------------------
 * Switch to a specific input on the analog input control
 * @param definition - the definition of the control to switch to
 */
void input_control_switch_analog(INPUT_CONTROL_DEFINITION* definition)
{
    bool switch_0 = definition->value & 0x01;
    bool switch_1 = definition->value & 0x02;
    bool switch_2 = definition->value & 0x04;
    
    logger_log_infof(LOGGER_CODE_INPUT_CONTROL, "INPUT CONTROL Switching Analogue Mux to %s (%d %d %d) --", AUDIO_INPUT_STRINGS[definition->index], switch_0, switch_1, switch_2);
 
    PLIB_PORTS_PinWrite(INPUT_CONTROL_SELECT_0_PORTS_ID, 
                    INPUT_CONTROL_SELECT_0_PORT_CHANNEL, 
                    INPUT_CONTROL_SELECT_0_PIN,
                    switch_0);
 
    PLIB_PORTS_PinWrite(INPUT_CONTROL_SELECT_1_PORTS_ID, 
                    INPUT_CONTROL_SELECT_1_PORT_CHANNEL, 
                    INPUT_CONTROL_SELECT_1_PIN,
                    switch_1);
 
    PLIB_PORTS_PinWrite(INPUT_CONTROL_SELECT_2_PORTS_ID, 
                    INPUT_CONTROL_SELECT_2_PORT_CHANNEL, 
                    INPUT_CONTROL_SELECT_2_PIN,
                    switch_2);
}

/* *****************************************************************************
 End of File
 */
