/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/input_control_usb.h"
#include "app_config.h"
#include "utilities/i2c.h"
#include "utilities/input_control_defs.h"
#include "utilities/logger.h"
#include "utilities/settings.h"

#include "system_definitions.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/**
 * Global data
 */
typedef struct
{

    //! Determines whether USB record is turned on or not.
    bool usb_record;
    
} INPUT_CONTROL_USB_DATA;

static INPUT_CONTROL_USB_DATA input_control_usb_data;

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Enables / Disables the USB ports for EMC purposes (See Pete email 20/08/2019 13:28)
 * TODO 03/09/2019 - This apparently didn't work, so consider removing
 * @param on
 */
void input_control_usb_enable(bool on)
{
    logger_log_debugf(0, "INPUT CONTROL USB - %s --", on ? "Enabled" : "Disabled");
    
    if (on)
    {
        PLIB_PORTS_PinDirectionOutputSet(INPUT_CONTROL_USB_BCLK_ID,
                                         INPUT_CONTROL_USB_BCLK_PORT_CHANNEL,
                                         INPUT_CONTROL_USB_BCLK_PIN);
        PLIB_PORTS_PinDirectionOutputSet(INPUT_CONTROL_USB_LRCLK_ID,
                                         INPUT_CONTROL_USB_LRCLK_PORT_CHANNEL,
                                         INPUT_CONTROL_USB_LRCLK_PIN);
        PLIB_PORTS_PinDirectionOutputSet(INPUT_CONTROL_USB_SDATA_ID,
                                         INPUT_CONTROL_USB_SDATA_PORT_CHANNEL,
                                         INPUT_CONTROL_USB_SDATA_PIN);
        
        // Always disable by making an input
        PLIB_PORTS_PinDirectionInputSet(INPUT_CONTROL_USB_MLK_ID,
                                         INPUT_CONTROL_USB_MLK_PORT_CHANNEL,
                                         INPUT_CONTROL_USB_MLK_PIN);
    }
    else
    {
        PLIB_PORTS_PinDirectionInputSet(INPUT_CONTROL_USB_BCLK_ID,
                                        INPUT_CONTROL_USB_BCLK_PORT_CHANNEL,
                                        INPUT_CONTROL_USB_BCLK_PIN);
        PLIB_PORTS_PinDirectionInputSet(INPUT_CONTROL_USB_LRCLK_ID,
                                        INPUT_CONTROL_USB_LRCLK_PORT_CHANNEL,
                                        INPUT_CONTROL_USB_LRCLK_PIN);
        PLIB_PORTS_PinDirectionInputSet(INPUT_CONTROL_USB_SDATA_ID,
                                        INPUT_CONTROL_USB_SDATA_PORT_CHANNEL,
                                        INPUT_CONTROL_USB_SDATA_PIN);
        PLIB_PORTS_PinDirectionInputSet(INPUT_CONTROL_USB_MLK_ID,
                                        INPUT_CONTROL_USB_MLK_PORT_CHANNEL,
                                        INPUT_CONTROL_USB_MLK_PIN);
    }
}

/**-----------------------------------------------------------------------------
 * Returns whether USB record is enabled (though not necessarily active
 * depending on input)
 * @return 
 */
bool input_control_usb_get_record()
{
    return input_control_usb_data.usb_record;
}

/**-----------------------------------------------------------------------------
 * Initialise the input control module
 */
bool input_control_usb_initialise()
{
    bool result = true;
    
    settings_get(SETTING_USB_RECORD, &input_control_usb_data.usb_record);     
    
    return result;
}
    
/**-----------------------------------------------------------------------------
 * Switch to a specific input on the analog input control
 * @param definition - the definition of the control to switch to
 */
void input_control_switch_usb(INPUT_CONTROL_DEFINITION* definition)
{
    usb_audio_port_attach();
    
    // Start processing USB messages
    usb_audio_set_active(true);
}
   
/**-----------------------------------------------------------------------------
 * Sets USB Record function On/Off
 */
void input_control_usb_set_record(bool on)
{
    input_control_usb_data.usb_record = on;
    
    settings_set(SETTING_USB_RECORD, &input_control_usb_data.usb_record);
}
    
/* *****************************************************************************
 End of File
 */
