/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/input_phono_input_app.h"
#include "utilities/input_phono_input.h"
#include <stdio.h>
#include "app_config.h"
#include "utilities/menu.h"
#include "utilities/menu_app.h"
#include "utilities/timers.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */


/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */


/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Gets the analogue direct setting for the current channel as a string
 * Note: This is used for all the analogue direct menus and is dependent on
 * the correct value of input_control_app_data.analogue_direct_edit_index being
 * set before (usually in the menu enter event)
  * @return 
 */
void input_phono_input_get_string(char* buffer, size_t size)
{
    sprintf(buffer, "%s", PHONO_INPUT_STRINGS[input_phono_input_get()]);
}

/**-----------------------------------------------------------------------------
 * Called when entering the Analogue Direct no navigation menu - setup the timeout
 */
void input_phono_input_enter_no_navigation(MENU_ID menu)
{
    // Set the edit index to the current input
    input_control_app_set_edit_index(input_control_get_selected_input());
    
    // Ensure the Phono Input menu is only visible for a while
    timers_update_timeout(TIMER_INDEX_MENU_INACTIVE, TIMER_MENU_PHONO_INPUT_INACTIVE_TIMEOUT_MS, TIMERS_RESET_IMMEDIATE);
}

/**-----------------------------------------------------------------------------
 * Called when leaving the the Analogue Direct no navigation menu 
 * @param menu
 */
MENU_ID input_phono_input_exit_no_navigation(MENU_ID menu)
{
    MENU_ID result = MENU_USE_SUPPLIED_MENU_ID;
    
    // Make sure Inactive Menu timeout is reset
    menu_app_reset_inactivity_timeout(menu);
    
    return result;
}

/* *****************************************************************************
 End of File
 */
