/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/machine_uuid.h"
#include "app_config.h"
#include "utilities/logger/logger.h"
#include "utilities/settings.h"
#include "utilities/uuid.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

typedef struct
{
    //! The unique Id of the machine based on the MAC address and Software model
    UUID machine_uuid;
    
} MACHINE_UUID_DATA;

static MACHINE_UUID_DATA machine_uuid_data;

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Clears the machine uuid in settings
 */
void machine_uuid_clear()
{
    uuid_empty(&machine_uuid_data.machine_uuid);
    settings_set(SETTING_MACHINE_UUID, &machine_uuid_data.machine_uuid);    
}

/**-----------------------------------------------------------------------------
 * Generates the machine uuid from the devices mac address supplie as a 16bit buffer
 * (see tcpip_get_mac_address_string)
 */
void machine_uuid_generate(char* mac_addr_buffer)
{
    // Check if machine UUID has been set
    if (!uuid_is_initialised())
    {
        uuid_initialise(mac_addr_buffer, strlen(mac_addr_buffer));

        if (uuid_is_empty(&machine_uuid_data.machine_uuid))
        {
            uuid_generate(&machine_uuid_data.machine_uuid, SOFTWARE_MODEL_STRING, strlen(SOFTWARE_MODEL_STRING));
            settings_set(SETTING_MACHINE_UUID, &machine_uuid_data.machine_uuid);

            char uuid_buffer[38];
            uuid_to_string(&machine_uuid_data.machine_uuid, uuid_buffer);
            logger_log_infof(LOGGER_CODE_MACHINE_UUID, "MACHINE UUID: generated %s", uuid_buffer);
        }
        else
        {
            char uuid_buffer[38];
            uuid_to_string(&machine_uuid_data.machine_uuid, uuid_buffer);
            logger_log_infof(LOGGER_CODE_MACHINE_UUID, "MACHINE UUID: read %s", uuid_buffer);
        }
    }    
}

/**-----------------------------------------------------------------------------
 * Attempts to load the machine UUID from settings
 */
void machine_uuid_load()
{
    uuid_empty(&machine_uuid_data.machine_uuid);
    settings_get(SETTING_MACHINE_UUID, &machine_uuid_data.machine_uuid);

    char uuid_buffer[38];
    machine_uuid_print(uuid_buffer);
}
    
/**-----------------------------------------------------------------------------
 * Loads the buffer (must be at least 38 chars
 */
void machine_uuid_print(char* buffer)
{
    uuid_to_string(&machine_uuid_data.machine_uuid, buffer);
    logger_log_infof(LOGGER_CODE_MACHINE_UUID, "MACHINE UUID: %s", buffer);
}

/* *****************************************************************************
 End of File
 */
