/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    output_control.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/output_control_streamer.h"
#include "app_config.h"
#include "utilities/dirac.h"
#include "utilities/input_control_defs.h"
#include "utilities/mutes.h"
#include "utilities/power.h"
#include "utilities/structured_message.h"
#include "utilities/timers.h"

#include "utilities/ak4104_spdif_tx.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

typedef struct
{
    // A callback for when system faults are detected
    SYSTEM_FAULT_CALLBACK fault_callback;
    
} OUTPUT_CONTROL_DATA;

static OUTPUT_CONTROL_DATA output_control_data;

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */
    
/**
 * Initialise the output control module
 */
void output_control_initialise(SYSTEM_FAULT_CALLBACK fault_callback)
{
}
    
/**-----------------------------------------------------------------------------
 * Mutes Analogue outputs
 */
void output_control_mute_analogue_outputs()
{
    mutes_mute(MUTE_INDEX_SPEAKER);
    //mutes_mute(MUTE_INDEX_PRE_OUT);
    
    mutes_mute(MUTE_INDEX_VOLUME);

// Removed as we get a loud click when muting
//    if (stop_dac)
//    {
//        input_control_dac_reset(INPUT_CONTROL_SWITCH_OFF); //BUG 3603
//    }
}   
    
/**-----------------------------------------------------------------------------
 * Mutes Speaker, Preamp, and Power AMp outputs
 * @param include_power_amp - True for SA10, false SA20
 */
void output_control_mute_digital_outputs()
{
    ak4104_spdif_tx_mute(AK4104_SPDIF_TX_SWITCH_ON);
}   
    
/**-----------------------------------------------------------------------------
 * Mutes Speaker, Preamp, and Power AMp outputs
 * @param include_power_amp - True for SA10, false SA20
 */
void output_control_mute_outputs()
{
    output_control_mute_analogue_outputs();
    output_control_mute_digital_outputs();
}   
    
/**-----------------------------------------------------------------------------
 * Perform regular output control tasks
 */
void output_control_process()
{
    if (power_get_state() == POWER_STATE_ON)
    {
    }
}

/**-----------------------------------------------------------------------------
 * Reviews the state of the digital outputs based on the current state of the MQA 
 * Decoder settings when unmuting
 */
#ifdef MQA_ENABLED

void output_control_review_mqa_outputs()
{
}
#endif
    
/**-----------------------------------------------------------------------------
 * Mutes Speaker, Headphone, Preamp, and Power AMp outputs
 * @param include_power_amp - True for SA10, false SA20
 */
void output_control_unmute_analogue_outputs()
{
    logger_log_debugf(0, "OUTPUT CONTROL unmute_analogue outputs() --");
      
    //input_control_dac_reset(INPUT_CONTROL_SWITCH_ON);   //BUG 3603
    //input_control_apply_dac_settings();                 //BUG 3603
    mutes_unmute(MUTE_INDEX_VOLUME);
    mutes_unmute(MUTE_INDEX_SPEAKER);
}   
    
/**-----------------------------------------------------------------------------
 * Mutes Speaker, Headphone, Preamp, and Power AMp outputs
 * @param include_power_amp - True for SA10, false SA20
 */
void output_control_unmute_digital_outputs()
{
    logger_log_debugf(0, "OUTPUT CONTROL unmute_digital outputs() --");
    ak4104_spdif_tx_mute(AK4104_SPDIF_TX_SWITCH_OFF);
}   
    
/**-----------------------------------------------------------------------------
 * Mutes Speaker, Headphone, Preamp, and Power AMp outputs
 * @param include_power_amp - True for SA10, false SA20
 */
void output_control_unmute_outputs()
{
#ifdef MQA_ENABLED
    logger_log_debugf(0, "OUTPUT CONTROL reviewing Outputs for MQA --");
    
    uint8_t mqa_value;
    settings_get(SETTING_MQA_DECODER_LEVEL, &mqa_value);

    switch (mqa_value)
    {
        case AVATAR_MQA_DECODER_LEVEL_FULL:
        {
            output_control_mute_digital_outputs();
            output_control_unmute_analogue_outputs();
            break;
        }

        default:
        {
            output_control_unmute_digital_outputs();
            output_control_mute_analogue_outputs();
        }
    }
#else    
    output_control_unmute_analogue_outputs();
    output_control_unmute_digital_outputs();
#endif
}   

/* *****************************************************************************
 End of File
 */
