/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/spdif_rx/private/spdif_rx_data.h"
#include "utilities/spdif_rx/spdif_rx_defs.h"
#include "utilities/timers.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

static SPDIF_RX_DATA spdif_rx_data = {
    .sample_rate = SAMPLE_RATE_0,
    .steady_sample_rate = SAMPLE_RATE_0,
    .calculating_sample_rate = false,
    .functions.configure_function = NULL,
    .functions.initialise_function = NULL,
    .functions.process_function = NULL,
    .functions.reset_function = NULL,
    .functions.switch_function = NULL,
};

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */


/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Returns a reference t th SPDIF_RX_DATA structure
 * @return 
 */    
SPDIF_RX_DATA* spdif_rx_data_get()
{
    return &spdif_rx_data;
}

/**-----------------------------------------------------------------------------
 * Gets whether the SPDIF_RX sample rate is currently being calculated
 * @param sample_rate
 */
bool spdif_rx_data_get_calculating_sample_rate()
{
    return spdif_rx_data.calculating_sample_rate;
}

/**-----------------------------------------------------------------------------
 * Returns true if the spdif_rx module is in a switch
 * @return 
 */
bool spdif_rx_data_get_in_switch()
{
    return spdif_rx_data.in_switch;
}

/**-----------------------------------------------------------------------------
 * Gets the SPDIF_RX sample rate e
 * @param sample_rate
 */
SAMPLE_RATE spdif_rx_data_get_sample_rate()
{
    if (timers_has_expired(TIMER_INDEX_SPDIF_RX_SAMPLE_RATE_SMOOTHING))
    {
        // Copy sample_rate to steady_sample_rate
        spdif_rx_data.steady_sample_rate = spdif_rx_data.sample_rate;
    }
    
    return spdif_rx_data.steady_sample_rate; 
}

/**-----------------------------------------------------------------------------
 * Sets whether the SPDIF_RX sample rate is bing calculated (spdif_rx use ONLY)
 * @param sample_rate
 */
void spdif_rx_data_set_calculating_sample_rate(bool calculating)
{
    logger_log_debugf(0, "SAMPLE RATE CALCULATING %d --", calculating);
    spdif_rx_data.calculating_sample_rate = calculating;
}

/**-----------------------------------------------------------------------------
 * Sets the spdif_rx module switch indicator
 * @return 
 */
void spdif_rx_data_set_in_switch(bool in)
{
    spdif_rx_data.in_switch = in;
}

/**-----------------------------------------------------------------------------
 * Sets the SPDIF_RX sample rate either when calculated or when an error occurs
 * @param sample_rate
 */
void spdif_rx_data_set_sample_rate(SAMPLE_RATE sample_rate)
{
    // Check if its changed, if not set the new sample rate and start the timer
    if (spdif_rx_data.sample_rate != sample_rate)
    {
        spdif_rx_data.sample_rate = sample_rate;
    
        timers_reset(TIMER_INDEX_SPDIF_RX_SAMPLE_RATE_SMOOTHING);
    }
}


/* *****************************************************************************
 End of File
 */
