/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/special_switches.h"
#include "app_config.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

#define SPECIAL_SWITCHES_MAX_COMBINATIONS     3

typedef struct
{
    SPECIAL_SWITCH_COMBINATION switch_combinations[SPECIAL_SWITCHES_MAX_COMBINATIONS];
    
    uint8_t switch_combination_count;

} SPECIAL_SWITCHES_DATA;

static SPECIAL_SWITCHES_DATA special_switches_data;

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */
    
/**
 * Returns the number of currently active switches.
 * @param switch_combination
 */
uint8_t special_switches_active_button_count()
{
    uint8_t result = 0;
    ADC_BUTTON* buttons = adc_button_get_current_buttons();
    int x;
     
    uint8_t test[8];
    memset(test, 0, 8);
    
    for (x = 0; x < ADC_BUTTON_INPUT_COUNT; x++)
    {
        if ((buttons[x] != ADC_BUTTON_NONE) &&
            (buttons[x] != ADC_BUTTON_ERROR))
        {
            test[x] = 1;
            result++;
        }
    }
    
    if (result > 1)
    {
//        logger_log_debugf(0, "SWITCH %d %d %d %d %d %d %d %d %d --", result,
//                test[0],
//                test[1],
//                test[2],
//                test[3],
//                test[4],
//                test[5],
//                test[6],
//                test[7]
//                ); 
    }
    
    return result;
}

/**
 * Add a definition for a switch combination where if all the switches are depressed 
 * a function will be invoked
 * @param switch1
 * @param switch2
 * @param switch3
 */
void special_switches_add(SPECIAL_SWITCH_COMBINATION* switch_combination)
{
    if (special_switches_data.switch_combination_count < SPECIAL_SWITCHES_MAX_COMBINATIONS)
    {
        memcpy(&special_switches_data.switch_combinations[special_switches_data.switch_combination_count], switch_combination, sizeof(SPECIAL_SWITCH_COMBINATION));
        special_switches_data.switch_combination_count++;
    }
    else
    {
        logger_log_warningf(0, "ADC: Unable to add switch combination - exceeds maximum number %d --", SPECIAL_SWITCHES_MAX_COMBINATIONS);
    }
}
    
/**
 * Initialises the special switches module - call befor adding any special switches
 */
void special_switches_initialise()
{
    special_switches_data.switch_combination_count = 0;
}
    
/**
 * Tests the current ADC state for the special switch combinations
 */
bool special_switches_test(STRUCTURED_MESSAGE* message)
{
    bool result = false;
    
    // Just before leaving lets see if we have the buttons depressed to launch the 
    // Engineering menu
    int match_count = 0;
    ADC_BUTTON* buttons = adc_button_get_current_buttons();
    int x;
    int y;
    
    for (x = 0; x < special_switches_data.switch_combination_count; x++)
    {
        match_count = 0;
        SPECIAL_SWITCH_COMBINATION* combination = &special_switches_data.switch_combinations[x];
       
        for (y = 0; y < ADC_BUTTON_INPUT_COUNT; y++)
        {
            //logger_log_debugf(0, "SPECIAL SWITCH TEST %d %d %d --", x, y, buttons[y]);
            if (((buttons[y] == combination->switch1) ||
                 (buttons[y] == combination->switch2) || 
                 (buttons[y] == combination->switch3)))
            {
                match_count++;
            }
        }
        
        if (match_count == 3)
        {
            combination->function(message);
            result = true;
            break;
        }
    }

    return result;
}

/* *****************************************************************************
 End of File
 */
