/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/usb_audio_codec.h"
#include "system/clk/sys_clk.h"
#include "system_definitions.h"
#include "utilities/avatar_control.h"
#include "utilities/logger.h"
#include "utilities/usb_audio_buffer.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

typedef struct
{
    // Connection handle
    DRV_HANDLE handle;
    
    //! The index of the buffer that will next be read for send ing data to the codec
    volatile uint8_t read_queue_idx;

    uint16_t test;

} USB_AUDIO_CODEC_DATA;

static USB_AUDIO_CODEC_DATA usb_audio_data =
{
    .read_queue_idx = 0,
    .test = 0,
};

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/**
 * I2S Event handler
 */
void usb_audio_codec_event_handler(DRV_I2S_BUFFER_EVENT event,
        DRV_I2S_BUFFER_HANDLE handle, uintptr_t context)
{
    // The context handle was set to an application specific
    // object. It is now retrievable easily in the event handler.
    //MY_APP_OBJ myAppObj = (MY_APP_OBJ *) context;
//logger_log_debugf(0, "USB AUDIO usb_audio_codec_event_handler BB %d %d  --", event, handle);    

    switch(event)
    {
        case DRV_I2S_BUFFER_EVENT_COMPLETE:
        {
            uint32_t x = 0;
            for (x = 0; x < USB_AUDIO_BUFFER_QUEUE_MAX_COUNT; x++)
            {
                if (usb_audio_buffer_is_handle(x, handle))
                {
//logger_log_debugf(0, "USB AUDIO usb_audio_codec_event_handler %d %d  --", x, handle);    
                    usb_audio_buffer_clear_data_available(x);
                    break;
                }
            }
            break;
        }

        case DRV_I2S_BUFFER_EVENT_ERROR:

            // Error handling here.
//logger_log_debugf(0, "USB AUDIO usb_audio_codec_event_handler ERROR %d  --", handle);    
            break;

        default:
            break;
    }
}

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */
    
/**-----------------------------------------------------------------------------
 * Add an I2S write request
 */
bool usb_audio_codec_add_write(uint32_t index)
{
    bool result = false;
    
    USB_DEVICE_AUDIO_V2_TRANSFER_HANDLE* handle_ptr = usb_audio_buffer_get_read_handle(index);
//logger_log_debugf(0, "USB AUDIO usb_audio_codec_add_write %d %d --", index, usb_audio_buffer_get_data_length(index));    
//        usb_audio_data.test++;
//        if (usb_audio_data.test % 100 == 0)
//         {
//            logger_log_debugf(0, "USB AUDIO usb_audio_codec_add_write %d %d %d %d --", index, *handle_ptr, usb_audio_data.handle, usb_audio_buffer_get_data_length(index));    
//        }
        
//timers_delay_ms(10);        

// Pretend we've processed it...
    //usb_audio_buffer_clear_data_available(index);
    //DRV_CODEC_BufferAddWrite(
    DRV_I2S_BufferAddWrite(
            usb_audio_data.handle, 
            handle_ptr,
            usb_audio_buffer_get_read_buffer(index), 
            usb_audio_buffer_get_data_length(index));

    if(*handle_ptr != DRV_I2S_BUFFER_HANDLE_INVALID)
    {
        //TODO REMOVE usb_audio_data.codec_data.next_buffer++;
        //if (usb_audio_data.test % 1000 == 0)
        // {
        //    logger_log_debugf(0, "USB AUDIO usb_audio_codec_add_write CC %d %d %d --", index, *handle_ptr, *usb_audio_buffer_get_read_handle(index));    
        //}
        
        result = true;
    }
    else
    {
//       if (usb_audio_data.test % 1000 == 0)
//         {
        logger_log_debugf(0, "USB AUDIO usb_audio_codec_add_write FAIL %d %d %d --", index, *handle_ptr, usb_audio_buffer_get_unprocessed_reads());    
//        }
        *handle_ptr = (USB_DEVICE_AUDIO_V2_TRANSFER_HANDLE)(-1);
    }
    
    return result;
}
    
/**-----------------------------------------------------------------------------
 * Gets the combined size of the available buffers
 */
size_t usb_audio_codec_get_buffer_size()
{
    return DRV_I2S_BufferCombinedQueueSizeGet(usb_audio_data.handle);
}

/**-----------------------------------------------------------------------------
 * Gets the index of the buffer from which the next codec write will be performed
 */
uint32_t usb_audio_codec_get_read_index()
{
    return usb_audio_data.read_queue_idx;
}
    
/**-----------------------------------------------------------------------------
 * Initialises the codec
 */
bool usb_audio_codec_initialise()
{
    bool result = true;
    
    usb_audio_codec_reset();
    
    return result;
}

/**-----------------------------------------------------------------------------
 * Opens the Codec I2S connection
 */
bool usb_audio_codec_open()
{
    logger_log_debugf(0, "USB AUDIO usb_audio_codec_open --");
        
    bool result = false;
    
//    usb_audio_data.handle = DRV_CODEC_Open(
//                                DRV_CODEC_INDEX_0, 
//                                DRV_IO_INTENT_EXCLUSIVE);
//    
//   if (usb_audio_data.handle != DRV_HANDLE_INVALID)
//   {
//        DRV_CODEC_BufferEventHandlerSet(usb_audio_data.handle,
//            (DRV_I2S_BUFFER_EVENT_HANDLER) usb_audio_codec_event_handler,
//            NULL);
//
//        logger_log_infof(0, "USB AUDIO Open I2S --");
//
//        result = true;
//    }
//    else
//    {
//        logger_log_errorf(0, "USB AUDIO Open I2S Error --");
//    }

    
    
    // Open the I2S SPI connection
    usb_audio_data.handle = DRV_I2S_Open(DRV_I2S_INDEX_0,
            (DRV_IO_INTENT_WRITE | DRV_IO_INTENT_NONBLOCKING));
    
    if (usb_audio_data.handle != DRV_HANDLE_INVALID)
    {
        DRV_I2S_TransmitErrorIgnore(usb_audio_data.handle, true);
        DRV_I2S_ReceiveErrorIgnore(usb_audio_data.handle, true);

        //TODO Verify
//        DRV_I2S_BaudSet(usb_audio_data.handle,
//                99000000,// (drvObj->samplingRate * drvObj->bclk_divider),
//                USB_AUDIO_SAMPLING_RATE_192KHZ);
        usb_audio_codec_set_sample_rate(USB_AUDIO_SAMPLING_RATE_192KHZ);
        
        // Set the buffer handler
        DRV_I2S_BufferEventHandlerSet(usb_audio_data.handle,
            (DRV_I2S_BUFFER_EVENT_HANDLER) usb_audio_codec_event_handler,
            NULL);

        logger_log_infof(0, "USB AUDIO Open I2S --");

        result = true;
    }
    else
    {
        logger_log_errorf(0, "USB AUDIO Open I2S Error --");
    }

    return result;
}

/**-----------------------------------------------------------------------------
 * Resets the codec for writing 
 */
void usb_audio_codec_reset()
{
    usb_audio_data.read_queue_idx = 0;
}
    
/**-----------------------------------------------------------------------------
 * Sets up the codec sample rate.  In the example this is used to set the master
 * clock and divider etc..
 */
void usb_audio_codec_set_sample_rate(uint32_t sample_rate)
{
    logger_log_debugf(0, "USB AUDIO usb_audio_codec_set_sample_rate %d --", sample_rate);
    uint16_t mclk_multiplier = USB_AUDIO_MCLK_MULTIPLIER_128FS;
    uint16_t bclk_divider = 2;
    
    /* If MCLK multiplier specified during initialization
       is not supported for the sampling rate range
       Normal Speed Mode 8kHz~48kHz, Program a suitable value */
    if (
        (
            sample_rate <= USB_AUDIO_SAMPLING_RATE_48KHZ &&
            sample_rate >= USB_AUDIO_SAMPLING_RATE_8KHZ
        ) 
        && 
        ( 
            mclk_multiplier != USB_AUDIO_MCLK_MULTIPLIER_256FS ||
            mclk_multiplier != USB_AUDIO_MCLK_MULTIPLIER_384FS ||
            mclk_multiplier != USB_AUDIO_MCLK_MULTIPLIER_512FS ||
            mclk_multiplier != USB_AUDIO_MCLK_MULTIPLIER_768FS ||
            mclk_multiplier != USB_AUDIO_MCLK_MULTIPLIER_1152FS)
        )
    {
        mclk_multiplier = USB_AUDIO_MCLK_MULTIPLIER_256FS;
        /* Bit Clock Divider is 256/64 = 4 */               
        bclk_divider = 4;   
    }
    /* If MCLK multiplier specified during initialization
       is not supported for the sampling rate range
       Double Speed Mode 60kHz~96kHz, Program a suitable value */    
    else if(
        (
            sample_rate <= USB_AUDIO_SAMPLING_RATE_96KHZ &&
            sample_rate > USB_AUDIO_SAMPLING_RATE_48KHZ
        ) 
        && 
        ( 
            mclk_multiplier != USB_AUDIO_MCLK_MULTIPLIER_128FS ||
            mclk_multiplier != USB_AUDIO_MCLK_MULTIPLIER_192FS ||            
            mclk_multiplier != USB_AUDIO_MCLK_MULTIPLIER_256FS ||
            mclk_multiplier != USB_AUDIO_MCLK_MULTIPLIER_384FS)
        )
    {           
        mclk_multiplier = USB_AUDIO_MCLK_MULTIPLIER_128FS;
        /* Bit Clock Divider is 128/64 = 2 */               
        bclk_divider = 2;        
    }
    /* If MCLK multiplier specified during initialization
       is not supported for the sampling rate range
       Quad Speed Mode 120kHz~192kHz, Program a suitable value */        
    else if(
        (
            sample_rate <= USB_AUDIO_SAMPLING_RATE_192KHZ &&
            sample_rate > USB_AUDIO_SAMPLING_RATE_96KHZ
        ) 
        && 
        ( 
            mclk_multiplier != USB_AUDIO_MCLK_MULTIPLIER_128FS ||
            mclk_multiplier != USB_AUDIO_MCLK_MULTIPLIER_192FS)
        )
    {
        mclk_multiplier = USB_AUDIO_MCLK_MULTIPLIER_128FS;        
        /* Bit Clock Divider is 128/64 = 2 */               
        bclk_divider = 2;        
    }
    
    // Set the master clock
    // Note this needs the Clock Service enabled in Harmony as DYNAMIC
    uint32_t mclk_in_Hz = mclk_multiplier * sample_rate;
    SYS_CLK_ReferenceFrequencySet(
            CLK_BUS_REFERENCE_1,
            CLK_SOURCE_REF_SYSPLL_OUT,
            mclk_in_Hz, 
            true );   
    
    DRV_I2S_BaudSet(usb_audio_data.handle,
        sample_rate * bclk_divider, //99000000,// (drvObj->samplingRate * drvObj->bclk_divider),
        sample_rate);
    
    avatar_set_sample_rate2(sample_rate);
}

/**-----------------------------------------------------------------------------
 * Attempts to write data to the audio codex from the current write index
 */
bool usb_audio_codec_try_write()
{
    bool result = false;
    if (usb_audio_buffer_data_available(usb_audio_codec_get_read_index()))
    {
        //logger_log_debugf(0, "USB AUDIO usb_audio_codec_try_write %d %d --", usb_audio_codec_get_read_index(), usb_audio_buffer_get_data_length(usb_audio_codec_get_read_index()));    
        
        if (usb_audio_codec_add_write(usb_audio_codec_get_read_index()))
        {
            usb_audio_data.read_queue_idx++;
            usb_audio_data.read_queue_idx %= USB_AUDIO_BUFFER_QUEUE_MAX_COUNT;    

            //usb_audio_data.test++;
            //if (usb_audio_data.test % 1000 == 0)
            // {
            //    logger_log_debugf(0, "USB AUDIO usb_audio_codec_try_write %d --", usb_audio_data.test);
            //}
            result = true;
        }
    }
//    else
//    {
//        logger_log_debugf(0, "USB AUDIO usb_audio_codec_try_write FAIL %d --", usb_audio_codec_get_read_index());    
//    }
    
    return result;
}

/* *****************************************************************************
 End of File
 */
