/* ************************************************************************** */
/** Descriptive File Name

  @Company
    ARCAM

  @File Name
    watchdog.c

  @Summary
    Brief description of the file.

  @Description
    Provides a simple interface for controlling the watchdog and reporting on its status.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "system/wdt/sys_wdt.h"
#include "app_config.h"
#include "utilities/watchdog.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

typedef struct
{
    WATCHDOG_STATUS status;
} WATCHDOG_DATA;

static WATCHDOG_DATA watchdog_data;

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**
 * Disables the watchdog timer
 */
void watchdog_disable()
{
    SYS_WDT_Disable();
    watchdog_data.status = WATCHDOG_STATUS_DISABLED;
}

/**
 * Enables the watchdog timer
 */
void watchdog_enable()
{
#if ARCAM_PIC_DEVICE_FAMILY == ARCAM_PIC_DEVICE_FAMILY_MZ    

    // Note for the PIC32MZ2048EFG100 windowing is set to false, setting it to true
    // caused resets immediately after the watchdog was enabled!
    logger_log_infof(0, "WATCHDOG Enabling Watchdog - Windowing Off --");
    SYS_WDT_Enable(false);
    
#elif ARCAM_PIC_DEVICE_FAMILY == ARCAM_PIC_DEVICE_FAMILY_MX
    
    logger_log_infof(0, "WATCHDOG Enabling Watchdog - Windowing On ");
    SYS_WDT_Enable(true);
#else
    logger_log_warningf(0, "WATCHDOG No enable function defined ");
    
#endif    

    watchdog_data.status = WATCHDOG_STATUS_ENABLED;
}

/**
 * Gets the current status of the watchdog timer
 */
WATCHDOG_STATUS watchdog_get_status()
{
    return watchdog_data.status;
}
    
/**
 * Initialises the watchdog service
 */
void watchdog_initialise()
{
    watchdog_data.status = WATCHDOG_STATUS_DISABLED;
}
    
/**
 * Resets the watchdog timer
 */
void watchdog_reset()
{
    if (watchdog_data.status == WATCHDOG_STATUS_ENABLED)
    {
        SYS_WDT_TimerClear();
    }
}

/* *****************************************************************************
 End of File
 */
