/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _APP_COMMANDS_H    /* Guard against multiple inclusion */
#define _APP_COMMANDS_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/structured_message.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************
    
    /**
     * Processes the current input context to setup a structured message that can then
     * be processed by the app_commands
     * @param - a callback function to handle the generated structured message
     */
    void app_commands_process_input_controls(STRUCTURED_MESSAGE_CALLBACK callback);
    
    /**
     * Process a structured message
     * This command will map the messages command and data to a function
     */
    void app_commands_process_message(STRUCTURED_MESSAGE* message);
    
    /**
     * ends the splash screen
     */
    void app_commands_splash_screen_end();
    
    /**
     * Initiates the splash screen
     */
    void app_commands_splash_screen_start();
    
    /**
     * Handle structured messages received via rs232
     * @param message - A pointer to a structured message - do NOT store this pointer
     */
    void app_commands_structured_message_callback(STRUCTURED_MESSAGE* message);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
