/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _APP_COMMANDS_AUTO_SHUTDOWN_H    /* Guard against multiple inclusion */
#define _APP_COMMANDS_AUTO_SHUTDOWN_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/structured_message.h"
#include "utilities/menu.h"

    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */


    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************
    
    /**
     * Check the Auto Shutdown timer
     * Note: This call should be made AFTER all inputs have updated the input_context
     * as it uses the state of that to determine what to do.
     * @return true - if auto shutdown invoked
     */
    bool app_commands_check_auto_shutdown();
    
    /**
     * Used to edit the auto_shutdown timeout based on the current inputs
     * @param message
     */
    bool app_commands_edit_auto_shutdown_timeout(STRUCTURED_MESSAGE* message);

    /**
     * Returns a string representation of the auto shutdown timeout value
     * @return 
     */
    void app_commands_get_auto_shutdown_timeout_string(char* buffer, size_t size);

    /**
     * Returns a string representation of the auto shutdown timeout value
     * @return 
     */
    void app_commands_get_auto_shutdown_timeout_remaining_string(char* buffer, size_t size);
    
    /**
     * Gets the number of minutes until the auto shutdown timeout occurs
     * @param message
     */
    void app_commands_get_timeout_counter(STRUCTURED_MESSAGE* message);
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
