/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _APP_COMMANDS_AVATAR_H    /* Guard against multiple inclusion */
#define _APP_COMMANDS_AVATAR_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include "utilities/avatar.h"
#include "utilities/menu.h"
#include "utilities/structured_message.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************
   
    /**
     * Draw the bespoke Balance display
     */
    void app_commands_avatar_draw_info(APP_COMMAND_MENU menu, char* top_line);

    /**
     * Draw the avatar reset menu
     */
    void app_commands_avatar_draw_reset(APP_COMMAND_MENU menu);

    /**
     * Draw the avatar reset menu
     */
    void app_commands_avatar_draw_serial_number(APP_COMMAND_MENU menu);
 
    /**
     * Draw the avatar upgrade menu
     */
    void app_commands_avatar_draw_upgrade(APP_COMMAND_MENU menu);
 
    /**
     * Draw the avatar upgrade menu
     */
    void app_commands_avatar_draw_upgrade_available(APP_COMMAND_MENU menu);
 
    /**
     * Draw the avatar upgrade complete menu
     */
    void app_commands_avatar_draw_upgrade_complete(APP_COMMAND_MENU menu);

    /**
     * Used to change the region
     */
    bool app_commands_avatar_edit_region(STRUCTURED_MESSAGE* message);
    
    /**
     * Trigger a refresh of the Avatar playing nfo
     */
    void app_comands_avatar_invalidate_info();

    /**
     * Draws a screen displayed during Dirac calibration
     */
    void app_commands_draw_dirac_calibration(APP_COMMAND_MENU menu);
    
    /**
     * Draws a standard navigation menu with title on top line and navigation buttons 
     * on the bottom
     */
    void app_commands_draw_dirac_no_navigation(APP_COMMAND_MENU menu);
    
    /**
     * Called when the Region menus is exited.  Used to set the Avatar Region
     * @param message
     * @return 
     */
    MENU_ID app_commands_edit_exit_region(MENU_ID menu);
 
    /**
     * Initiates an Avatr USB upgrade
     */
    void app_commands_avatar_arc_upgrade();
 
    /**
     * Initiates an Avatr USB upgrade
     */
    void app_commands_avatar_usb_upgrade();

    /**
     * Used to start the Upgrade process
     */
    void app_commands_avatar_upgrade();

    /**
     * Called when entering the avatar upgrade menu
     */
    void app_commands_enter_avatar_upgrade_continue_menu(MENU_ID menu);
 
    /**
     * Called when entering the INFO menu
     */
    void app_commands_enter_info(MENU_ID menu);

    /**
     * Draw the bespoke Balance display
     */
    void app_commands_avatar_factory_reset();
      
    /**
     * Gets the current status of the region as a string
     * @return 
     */
    void app_commands_avatar_get_region_string(char* buffer, size_t size);
   
    /**
     * Sets the avatar volume
     */
    //void app_commands_avatar_set_volume(uint8_t volume);

    /**
     * Callback for avatar messages
     * @param message
     */
    void app_commands_avatar_message_callback(AVATAR_MESSAGE* message);
    
    /**
     * Command level avatar processing
     */
    void app_commands_avatar_process();

    /**
     * Used to edit the auto_shutdown mode based on the current inputs
     * @param message
     */
    bool app_commands_edit_dirac_selected(STRUCTURED_MESSAGE* message);
    
    /**
     * Used to edit the auto_shutdown mode based on the current inputs
     * @param message
     */
    bool app_commands_edit_upgrade_option(STRUCTURED_MESSAGE* message);

    /**
     * Called when leaving any of the avatar upgrade menus
     */
    void app_commands_enter_avatar_upgrade_menu(MENU_ID menu);
    
    /**
     * Called when leaving the Balance menu
     */
    void app_commands_enter_dirac_calibration(MENU_ID menu);
    
    /**
     * Called when leaving the Balance menu
     */
    void app_commands_enter_dirac_no_navigation(MENU_ID menu);

    /**
     * Used to change the mqa decoder level
     */
    bool app_commands_edit_mqa_decoder_level(STRUCTURED_MESSAGE* message);
    
//    /**
//     * Called when leaving the Balance menu
//     * @param menu
//     */
//    void app_commands_exit_dirac(MENU_ID menu);

    /**
     * Used to simulate player status stopped when entering standby
     */
    void app_commands_force_player_status_stopped();

    /**
     * Returns a string representation of the currently playing album
     * @param buffer
     * @param size
     */
    void app_commands_get_album_name_string(char* buffer, size_t size);
    
    /**
     * Returns a string representation of the currently playing artist
     * @param buffer
     * @param size
     */
    void app_commands_get_artist_name_string(char* buffer, size_t size);
  
    /**
     * Returns a string representation of the avatars firmware version
     * @return 
     */
    void app_commands_get_audio_source_string(char* buffer, size_t len);
  
    /**
     * Returns the avatars firmware version
     * @return 
     */
    uint16_t app_commands_get_avatar_firmware_version();
  
    /**
     * Returns a string representation of the avatars firmware version
     * @return 
     */
    void app_commands_get_avatar_firmware_version_string(char* buffer, size_t len);

    /**
     * Returns a string representation of whether dirac is enabled or not
     * @param buffer
     * @param size
     */
    void app_commands_get_dirac_selected_string(char* buffer, size_t size);

    /**
     * Gets the current value of the repeat state
     * @return 
     */
    AVATAR_REPEAT_STATE app_commands_avatar_get_repeat_state();
    
    /**
     * Gets the current value of the shuffle state
     * @return 
     */
    AVATAR_SHUFFLE_STATE app_commands_avatar_get_shuffle_state();
    
    /**
     * Returns the current track length
      */
    uint64_t app_commands_get_track_length();

    /**
     * Returns a string representation of the currently playing track
     * @param buffer
     * @param size
     */
    void app_commands_get_track_name_string(char* buffer, size_t size);

    /**
     * Returns the current track time (position)
     */
    uint64_t app_commands_get_track_time();

    /**
     * Gets the last known player status.
     */
    AVATAR_PLAYER_STATUS app_commands_get_player_status();

    /**
     * Can be used to reset the inactivity timeout in cases where its been adjusted 
     * by displaying the Avatar Upgrade Complete menu
     * @param menu
     */
    MENU_ID app_commands_menu_reset_upgrade_complete(MENU_ID menu);

    /**
     * Used to report on IPAddress, MAC Address, Friendly Name
     * @param message
     */
    bool app_commands_network(STRUCTURED_MESSAGE* message);

    /**
     * Used to report on the current player status
     * @param message
     */
    bool app_commands_now_playing(STRUCTURED_MESSAGE* message);

    /**
     * Used to report on the current player status
     * @param message
     */
    bool app_commands_player_status(STRUCTURED_MESSAGE* message);
    
    /**
     * Used to process avatar messages triggered by the remote, or as a simulated RC5 command
     * @param message
     */
    void app_commands_remote_avatar_message(STRUCTURED_MESSAGE* message);

    /**
     * Used to report on the names of the Dirac EQ settings
     * @param message
     */
    bool app_commands_room_eq_names(STRUCTURED_MESSAGE* message);
 
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _APP_COMMANDS_AVATAR_H */

/* *****************************************************************************
 End of File
 */
