/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _APP_COMMANDS_BROWSING_H    /* Guard against multiple inclusion */
#define _APP_COMMANDS_BROWSING_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/menu.h"
#include "utilities/structured_message.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */


    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************


    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Gets the current value of the repeat state
     * @return 
     */
    AVATAR_REPEAT_STATE app_commands_browsing_get_repeat_state();

    /**
     * Gets the current value of the shuffle state
     * @return 
     */
    AVATAR_SHUFFLE_STATE app_commands_browsing_get_shuffle_state();

    /**
     * Handles a repeat commands from a remote
     */
    void app_commands_browsing_handle_repeat();

    /**
     * Handles a shuffle or random commands from a remote
     */
    void app_commands_browsing_handle_shuffle();
    
    /**
     * Resets browsing shuffl/repeat state
     */
    void app_commands_browsing_reset();

    /**
     * Draws a screen displayed while browsing
     */
    void app_commands_draw_browsing(APP_COMMAND_MENU menu);

    /**
     * Called when entering the Browsing menu
     */
    void app_commands_enter_browsing(MENU_ID menu);

    /**
     * Called when leaving the Browsing menu
     * @param menu
     */
    MENU_ID app_commands_exit_browsing(MENU_ID menu);
    
    /**
     * Handles internal browsing messages
     * @param message
     */
    void app_commands_handle_browsing(STRUCTURED_MESSAGE* message);


    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
