/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _APP_COMMANDS_BRIGHTNESS_H    /* Guard against multiple inclusion */
#define _APP_COMMANDS_BRIGHTNESS_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include "app_config.h"
#include "utilities\structured_message.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /** 
     * Changes the display brightness dependent on both the setting and the currently 
     * selected value
     */
    void app_commands_adjust_brightness(APP_COMMAND_BRIGHTNESS brightness);
    
    /**
     * Draws a standard navigation menu with title on top line and navigation buttons 
     * on the bottom
     */
    void app_commands_draw_display_no_navigation(APP_COMMAND_MENU menu);
   
    /**
     * Used to edit the balance setting based on the current inputs
     * @param message
     */
    bool app_commands_edit_display_brightness(STRUCTURED_MESSAGE* message);
   
    /**
     * Used to edit the balance setting based on the current inputs
     * @param message
     */
    bool app_commands_edit_dark_mode(STRUCTURED_MESSAGE* message);

    /**
     * Display a blank screen
     */
    void app_commands_display_clear();

    /**
     * Display the current system fault on the screen
     */
    void app_commands_display_system_fault();
    
    /**
     * 
     * @return 
     */
    APP_COMMAND_BRIGHTNESS app_commands_get_brightness();
    
    /**
     * Returns a string representation of the current balance value
     * @return 
     */
    void app_commands_get_brightness_string(char* buffer, size_t size);
    
    /**
     * 
     * @return 
     */
    APP_COMMAND_DARK_MODE app_commands_get_dark_mode();
    
    /**
     * Returns a string representation of the current dark_mode value
     * @return 
     */
    void app_commands_get_dark_mode_string(char* buffer, size_t size);

    /**
     * 
     * @return 
     */
    bool app_commands_brightness_get_remote_change();

    /**
     * Sets the current full brightness setting
     */
    void app_commands_set_brightness(APP_COMMAND_BRIGHTNESS brightness);

    /**
     * Sets the dark mode setting
     */
    void app_commands_set_dark_mode(APP_COMMAND_DARK_MODE dark_mode);
    
    /**
     * 
     * @param change
     */
    void app_commands_brightness_set_remote_change(bool change);
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif
//   
//    /**
//     * Used to edit the balance setting based on the current inputs
//     * @param message
//     */
//    bool app_commands_edit_display_brightness(STRUCTURED_MESSAGE* message);

    /**
     * Sets the current full brightness setting
     * @return 
     */
    void app_commands_set_brightness(APP_COMMAND_BRIGHTNESS brightness);

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
