/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
 Manages commands related to the engineering menu

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _APP_COMMANDS_ENGINEERING_H    /* Guard against multiple inclusion */
#define _APP_COMMANDS_ENGINEERING_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/menu.h"
#include "app_context.h"
#include "utilities/menu.h"
#include "utilities/structured_message.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Used to enter/exit the BIAS test
     */
    bool app_commands_bias_test(STRUCTURED_MESSAGE* message);
    
    /**
     * Draw the bias check menu
     * @param menu
     */
    void app_commands_draw_bias_check(APP_COMMAND_MENU menu);
    
    /**
     * Draw the IR debug menu
     * @param menu
     */
    void app_commands_draw_ir_debug(APP_COMMAND_MENU menu);

    /**
     * Used to edit the check channel status setting
     */
    bool app_commands_edit_check_channel_status(STRUCTURED_MESSAGE* message);

    /**
     * Used to change the logger level
     */
    bool app_commands_edit_logger_level(STRUCTURED_MESSAGE* message);
    
    /**
     * Called when entering the Bias check menu
     * @param menu
     */
    void app_commands_enter_bias_check(MENU_ID menu);
    
    /**
     * Called when leaving the Bias check menu
     * @param menu
     */
    MENU_ID app_commands_exit_bias_check(MENU_ID menu);
    
    /**
     * Called when entering the IR Debug menu
     * @param menu
     */
    void app_commands_enter_ir_debug(MENU_ID menu);
    
    /**
     * Called when leaving the IR Debug menu
     * @param menu
     */
    MENU_ID app_commands_exit_ir_debug(MENU_ID menu);
      
    /**
     * Called when entering the Switch voltages menu
     * @param menu
     */
    void app_commands_enter_switch_voltages(MENU_ID menu);
    
    /**
     * Called when leaving the Switch voltages menu
     * @param menu
     */
    MENU_ID app_commands_exit_switch_voltages(MENU_ID menu);

    /**
     * Returns a string representation of the on/off check channel status value
     * @return 
     */
    void app_commands_get_check_channel_status_string(char* buffer, size_t size);

    /**
     * Returns a string representation of the logger level
     * @return 
     */
    void app_commands_get_logger_level_string(char* buffer, size_t size);

    /**
     * Used to simulate fault conditions via external commands
     * @param message
     */
     void app_commands_inject_fault(STRUCTURED_MESSAGE* message);

    /**
     * Launch the engineering menu
     * @param message
     */
    void app_commands_launch_engineering_menu(STRUCTURED_MESSAGE* message);

     /**
     * Launch the reset menu
     */
    void app_commands_launch_reset_menu(STRUCTURED_MESSAGE* message);

    /**
     * Launch the software_version menu
     * @param message
     */
    void app_commands_launch_software_version_menu(STRUCTURED_MESSAGE* message);

    /**
     * Performs the BIAS check as part of the main message loop
     */
    void app_commands_perform_bias_check();

    /**
     * Returns the current watchdog status
     */
    void app_commands_watchdog_status(STRUCTURED_MESSAGE* message);
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
