/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _APP_COMMANDS_HDMI_ARC_H    /* Guard against multiple inclusion */
#define _APP_COMMANDS_HDMI_ARC_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities\hdmi_arc.h"
#include "utilities/menu.h" 
#include "utilities\structured_message.h"
#include "utilities/upgrade_metadata.h" 

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Overrides the version check for the upgrade one time only
     */
    void app_commands_hdmi_arc_override_upgrade_version_check();
    
    /**
     * Version check used to decide whether the upgrade can continue based on the 
     * version number read from the packed upgrade image.
     * @param major_version
     * @param minor_version
     * @return 
     */
    bool app_commands_hdmi_arc_check_upgrade_version(uint8_t major_version, uint8_t minor_version);

    /**
     * Callback for RS232 connection
     * @param message
     */
    void app_commands_hdmi_arc_callback(HDMI_ARC_MESSAGE* message);

    /**
     * Edits the HDMI ARC Audio control setting
     */
    bool app_commands_hdmi_arc_edit_audio_control(STRUCTURED_MESSAGE* message);

    /**
     * Edits the HDMI ARC Power control setting
     */
    bool app_commands_hdmi_arc_edit_power_control(STRUCTURED_MESSAGE* message);

    /**
     * Returns a string representation of the HDMI ARC Audio Control
     * @return 
     */
    void app_commands_hdmi_arc_get_hdmi_audio_control_string(char* buffer, size_t size);

    /**
     * Returns a string representation of the HDMI ARC Audio Control
     * @return 
     */
    void app_commands_hdmi_arc_get_hdmi_power_control_string(char* buffer, size_t size);

    /**
     * Sends a request for the HDMI ARC Transmitter version when entering the version number menu
     * @param 
     */
    void app_commands_enter_hdmi_arc_rx_version_menu(MENU_ID menu_id);
    
    /**
     * Determines whether the HDMI ARC Input is currently active
     * @return 
     */
    bool app_commands_hdmi_arc_is_active();
    
    /**
     * Regular HDMI ARC processing to ensure that handshaking has occurred
     */
    void app_commands_hdmi_arc_process();
    
    /**
     * Used to suppress/unsuppress handshaking (during update where it can mess up the
     * SA30 update
     */
    void app_commands_hdmi_arc_suppress_handshake(bool suppress);
    
    /**
     * Indicates to the ARC MCU that the system is ready to receive ARC commands
     */
    void app_commands_hdmi_arc_system_ready();
       
    /**
     * Prepare the HDMI Arc upgrade
     */
    UPGRADE_METADATA_LOAD_ID app_commands_hdmi_arc_upload_prepare(uint8_t major_version, uint8_t minor_version, size_t size, uint8_t* checksum, bool override);
    
    /**
     * Upload HDMI Arc upgrade data
     */
    UPGRADE_METADATA_UPLOAD_STATUS app_commands_hdmi_arc_upload_data(UPGRADE_METADATA_LOAD_ID load_id, 
            uint8_t* data, size_t 
            data_length,
            UPGRADE_METADATA_UPLOAD_LOCAL_CALLBACK local_callback);
    
    /**
     * Complete the upload of HDMI Arc upgrade data
     */
    UPGRADE_METADATA_UPLOAD_STATUS app_commands_hdmi_arc_upload_end(UPGRADE_METADATA_LOAD_ID load_id,
            UPGRADE_METADATA_UPLOAD_LOCAL_CALLBACK local_callback, UPGRADE_METADATA_UPGRADE_STATUS status);         

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
