/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _APP_COMMANDS_INPUT_H    /* Guard against multiple inclusion */
#define _APP_COMMANDS_INPUT_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/structured_message.h"
#include "utilities/input_control.h"
#include "utilities/menu.h"

// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************


// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************
//
///**
// * Used in menus to review whether a change in analogue-direct for a channel needs 
// * the actual analogue direct setting or not.
// * @param menu
// * @return 
// */
//MENU_ID app_commands_check_analogue_direct(MENU_ITEM menu);

/**
 * Performs an input change
 */
void app_commands_current_source(STRUCTURED_MESSAGE* message);

/**
 * Draws the Analogue Direct menu when initiated from a button press.
 */
void app_commands_draw_analogue_direct_no_navigation(APP_COMMAND_MENU menu);

/**
 * Used for editing analogue direct settings on individual channels
 * Note: This is used for all the analogue direct menus and is dependent on
 * the correct value of input_control_app_data.input_edit_index being
 * set before (usually in the menu enter event)
 */
bool app_commands_edit_analogue_direct(STRUCTURED_MESSAGE* message);

/**
 * Called when one of the Dirac Curve menus is exited.  Used to select the curve
 * @param message
 * @return 
 */
MENU_ID app_commands_edit_exit_dirac_curve(MENU_ID menu);

/**
 * Used to change the logger level
 */
bool app_commands_edit_filter(STRUCTURED_MESSAGE* message);

/**
 * Used for editing input enabled settings on individual channels
 * Note: This is used for all the analogue direct menus and is dependent on
 * the correct value of input_control_app_data.input_edit_index being
 * set before (usually in the menu enter event)
 */
bool app_commands_edit_input_enabled(STRUCTURED_MESSAGE* message);

/**
 * Sets the maximum volume at switch on
 */
bool app_commands_edit_mm_mc(STRUCTURED_MESSAGE* message);

/**
 * Used to change the USB Record setting
 */
bool app_commands_edit_usb_record(STRUCTURED_MESSAGE* message);
//
///**
// * Called when entering the Analogue Direct no navigation menu - setup the timeout
// */
//void app_commands_enter_analogue_direct_no_navigation(MENU_ID menu);
//
///**
// * Called when leaving the Balance menu
// * @param menu
// */
//MENU_ID app_commands_exit_analogue_direct_no_navigation(MENU_ID menu);

///**
// * Gets analogue direct as a string
// * Note: This is used for all the analogue direct menus and is dependent on
// * the correct value of input_control_app_data.input_edit_index being
// * set before (usually in the menu enter event)
// * @return 
// */
//void app_commands_get_analogue_direct_string(char* buffer, size_t size);

/**
 * 
 * @param message
 * @return 
 */
void app_commands_get_input_detection_status(STRUCTURED_MESSAGE* message);

///**
// * Gets the index of the Audio Input currently being edited.
// * @return 
// */
//AUDIO_INPUT app_commands_get_input_edit_index();

/**
 * Gets the input enabled setting as a string
 * Note: This is used for all the analogue direct menus and is dependent on
 * the correct value of input_control_app_data.input_edit_index being
 * set before (usually in the menu enter event)
 * @return 
 */
void app_commands_get_input_enabled_string(char* buffer, size_t size);

/**
 * Gets the max on volume as a string
 * @return 
 */
//void app_commands_get_mm_mc_string(char* buffer, size_t size);

/** 
 * Used to return the sample rate.  This is invoked by an RS232/TCPIP call and will currently
 * only handle the GET_STATUS message
 */
void app_commands_get_sample_rate(STRUCTURED_MESSAGE* message);

/**
 * Returns a string representation of the USB Record setting
 * @return 
 */
void app_commands_get_usb_record_string(char* buffer, size_t size);

/**
 * Configures the input control module
 */
void app_commands_input_configure();

/**
 * Callback for when inputs are switching
 */
void app_commands_input_switch_callback(INPUT_CONTROL_SWITCH_STATE state);

/**
 * Prepares editing of the analogue direct and input enabled setting for an input 
 * based on the current menu
 * @param menu
 */
void app_commands_set_input_for_edit(MENU_ITEM menu);

/**
 * Simulate remote control commands via RS232
 * @param 
 */
void app_commands_simulate_remote(STRUCTURED_MESSAGE* message);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _APP_COMMANDS_INPUT_H */

/* *****************************************************************************
 End of File
 */
