/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _APP_COMMANDS_LIP_SYNC_H    /* Guard against multiple inclusion */
#define _APP_COMMANDS_LIP_SYNC_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities\menu.h"
#include "utilities\structured_message.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Used in menus to review whether a change in lip sync for a channel needs 
     * the actual lip sync value setting on the avatar or not.
     * @param menu
     * @return 
     */
    MENU_ID app_commands_check_lip_sync(MENU_ITEM menu);

    /**
     * Prepares editing of the lip_sync setting for an input 
     * based on the current menu
     * @param menu
     */
    void app_commands_enter_lip_sync_menu(MENU_ITEM menu);

    /**
     * gets a string representing the Lip Sync setting for the currently selected 
     * audio channel
     * @return 
     */
    void app_commands_get_lip_sync_string(char* buffer, size_t size);

    /**
     * Used for editing lip sync settings on individual channels
     * Note: This is used for all the analogue direct menus and is dependent on
     * the correct value of input_control_app_data.input_edit_index being
     * set before (usually in the menu enter event)
     */
    bool app_commands_edit_lip_sync(STRUCTURED_MESSAGE* message);
    
    /**
     * Makes sure that the Avatar Lip Sync message is up-to-date
     */
    void app_commands_process_lip_sync();

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
