/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _APP_COMMANDS_OUTPUT_H    /* Guard against multiple inclusion */
#define _APP_COMMANDS_OUTPUT_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include "utilities/structured_message.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Used to edit the headphone override status
     * @param message
     * @return 
     */
    bool app_commands_edit_headphone_override(STRUCTURED_MESSAGE* message);

    /**
     * Used to change the short_circuit_sense setting
     */
    bool app_commands_edit_short_circuit_sense(STRUCTURED_MESSAGE* message);
    
    /**
     * Used to get the DC Offset for an RS232 GET_STATUS call
     */
    void app_commands_get_dc_offset_status(STRUCTURED_MESSAGE* message);

    /**
     * Returns a string representation of the headphone override status
     * @return 
     */
    void app_commands_get_headphone_override_string(char* buffer, size_t size);

    /**
     * Returns a string representation of the Short Circuit Sensor setting
     * @return 
     */
    void app_commands_get_short_circuit_sense_string(char* buffer, size_t size);

    /**
     * Used to get the DC Offset for an RS232 GET_STATUS call
     */
    void app_commands_get_short_circuit_status(STRUCTURED_MESSAGE* message);

    /**
     * Determines whether headphones are connected
     */
    void app_commands_headphones(STRUCTURED_MESSAGE* message);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
