/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _APP_COMMANDS_POWER_H    /* Guard against multiple inclusion */
#define _APP_COMMANDS_POWER_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include "utilities/structured_message.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    //--------------------------------------------------------------------------
    // Application Commands - Enum of available Power change types
    //--------------------------------------------------------------------------
    typedef enum
    {
        APP_COMMAND_POWER_OFF               = 0x00,
        APP_COMMAND_POWER_ON                = 0x01,
        APP_COMMAND_POWER_TOGGLE            = 0x02,
        APP_COMMAND_POWER_REQUEST_STATE     = APP_COMMAND_VALUE_REQUEST_STATUS,
    } APP_COMMAND_POWER;

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************
    
    /**
     * Checks the power state during normal processing
     */
    void app_commands_check_power_state();
 
    /**
     * Used to notify the app that we have stopped moving between Power states
     */
    void app_commands_clear_power_state_transition();
    
    /**
     * Gets the state of the ac_not_present_triggered_powerdown flag
     */
    bool app_commands_get_ac_not_present_triggered_powerdown();
 
    /**
     * Used to check whether we are currently moving between Power states
     */
    bool app_commands_in_power_state_transition();

    /**
     * Switches power on/off
     */
    void app_commands_power(STRUCTURED_MESSAGE* message);
   
    /**
     * Handles the power down operation
     */
    void app_commands_power_down();

    /**
     * Sets the state of the ac_not_present_triggered_powerdown flag
     * @param on
     */
    void app_commands_set_ac_not_present_triggered_powerdown(bool on);
 
    /**
     * Used to notify the app that we have started moving between Power states
     */
    void app_commands_set_power_state_transition();

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _APP_COMMANDS_POWER_H */

/* *****************************************************************************
 End of File
 */
