/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _APP_COMMANDS_SYSTEM_H    /* Guard against multiple inclusion */
#define _APP_COMMANDS_SYSTEM_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdio.h>
#include "utilities\structured_message.h"
#include "utilities\menu.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Draw the bespoke Balance display
     */
    void app_commands_draw_reboot(APP_COMMAND_MENU menu);

    /**
     * Used to change the logger level
     */
    bool app_commands_edit_logger_level(STRUCTURED_MESSAGE* message);

    /**
     * Used to change the remote system code
     */
    bool app_commands_edit_system_code(STRUCTURED_MESSAGE* message);

    /**
    * Initiates a factory reset from an external command
    * Includes checking a code before proceeding
    */
   void app_commands_factory_reset(STRUCTURED_MESSAGE* message);

    /**
     * Returns a string representation of the currently pressed button voltage
     * @param buffer
     * @param size
     */
    void app_commands_get_active_switch_voltage_string(char* buffer, size_t size);

    /**
     * Returns a string representation of the current amplifier temperature
     * @param buffer
     * @param size
     */
    void app_commands_get_amplifier_temperature_string(char* buffer, size_t size);

    /**
     * Returns the last shutdown reason.
     * @param buffer
     * @param size
     */
    void app_commands_get_last_shutdown_code_string(char* buffer, size_t size);

    /**
     * Returns a string representation of the current amplifier temperature
     * @param buffer
     * @param size
     */
    void app_commands_get_lifter_temperature(STRUCTURED_MESSAGE* message);

    /**
     * Returns a string representation of the current lifter temperature
     * @param buffer
     * @param size
     */
    void app_commands_get_lifter_temperature_string(char* buffer, size_t size);

    /**
     * Returns a string representation of the logger level
     * @return 
     */
    void app_commands_get_logger_level_string(char* buffer, size_t size);

    /**
     * Returns a string representation of the current model number e.g. SA20
     * @param buffer
     * @param size
     */
    void app_commands_get_model_number(STRUCTURED_MESSAGE* message);
    
    /**
     * Returns a string representation of the current amplifier temperature
     * @param buffer
     * @param size
     */
    void app_commands_get_output_temperature(STRUCTURED_MESSAGE* message);
    
    /**
     * Returns a string representation of the current balance value
     * @return 
     */
    void app_commands_get_software_version(char* buffer);

    /**
     * Heartbeat command to check unit is still connected and communicating
     * also resets the shutdown timer
     * @param message
     */
    void app_commands_heartbeat(STRUCTURED_MESSAGE* message);

    /**
     * Performs a factory reset (includes a reboot)
     */
    void app_commands_process_factory_reset(MENU_ID menu_id);
    
    /**
     * Initiates a reboot from an external command
     * Includes checking a code before proceeding
     */
    void app_commands_reboot(STRUCTURED_MESSAGE* message);

    /**
    * Gets the software version
    */
   void app_commands_software_version(STRUCTURED_MESSAGE* message);

    /**
     * System Status triggers multiple status requests to give an overall picture of the
     * system state
     * @param message
     */
    void app_commands_system_status(STRUCTURED_MESSAGE* message);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
