/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _APP_COMMANDS_UPGRADE_H    /* Guard against multiple inclusion */
#define _APP_COMMANDS_UPGRADE_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/flash_memory.h"
#include "utilities//structured_message.h"
#include "utilities/upgrade_metadata.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Version check used to decide whether the upgrade can continue based on the 
     * version number read from the packed upgrade image.
     * @param major_version
     * @param minor_version
     * @return 
     */
    bool app_commands_check_upgrade_version(uint8_t major_version, uint8_t minor_version);

    /**
     * Performs a 1-time suppression of the upgrade in the version check
     */
    void app_commands_suppress_upgrade();
    
    /**
     * Prints Upgrade Metadata
     */
    void app_commands_upgrade_print_info(STRUCTURED_MESSAGE* message);

    /**
     * Completes an upgrade upload
     * @param message
     */
    void app_commands_upgrade_upload_end(STRUCTURED_MESSAGE* message);

    /**
     * Starts an upgrade upload
     * @param message
     */
    void app_commands_upgrade_upload_data(STRUCTURED_MESSAGE* message);
    
    /**
     * Starts an upgrade 
     * @param message
     */
    void app_commands_upgrade_upload_start(STRUCTURED_MESSAGE* message);
    
    /**
     * Callback invoked when a packet of data has been uploaded into the image data 
     * @param request
     */
    void app_commands_upgrade_write_callback(UPGRADE_METADATA_LOAD_ID load_id, FLASH_MEMORY_CLIENT_REQUEST* request);
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
