/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _APP_COMMANDS_VOLUME_H    /* Guard against multiple inclusion */
#define _APP_COMMANDS_VOLUME_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include <stdbool.h>
#include "app_config.h"
#include "utilities/menu.h"
#include "utilities/structured_message.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    //--------------------------------------------------------------------------
    // Application Commands - Enum of available MUTE options
    //--------------------------------------------------------------------------
    typedef enum
    {
        APP_COMMAND_MUTE_ON         = 0x00, // Mutes the output
        APP_COMMAND_MUTE_OFF        = 0x01, // Unmutes the output
        APP_COMMAND_MUTE_TOGGLE     = 0x02, // Toggles the mute state
        APP_COMMAND_MUTE_GETSTATUS  = APP_COMMAND_VALUE_REQUEST_STATUS,
    } APP_COMMAND_MUTE;

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************

    /**
     * Draws a standard navigation menu with title on top line and navigation buttons 
     * on the bottom
     */
    void app_commands_draw_balance(APP_COMMAND_MENU menu);

    /**
     * Draws main Volume screen
     * @param menu
     */
    void app_commands_draw_volume(APP_COMMAND_MENU menu);
 
    /**
     * Used to edit the balance setting based on the current inputs
     * @param message
     */
    bool app_commands_edit_balance(STRUCTURED_MESSAGE* message);

    /**
     * Sets the maximum volume at switch on
     */
    bool app_commands_edit_max_on_volume(STRUCTURED_MESSAGE* message);

    /**
     * Sets the maximum volume for streaming
     */
    bool app_commands_edit_maximum_streaming_volume(STRUCTURED_MESSAGE* message);

    /**
     * Sets the maximum volume
     */
    bool app_commands_edit_maximum_volume(STRUCTURED_MESSAGE* message);

    /**
     * Performs a volume change based upon a delta value
     */
    bool app_commands_edit_volume(STRUCTURED_MESSAGE* message);

    /**
     * Called when leaving the Balance menu
     */
    void app_commands_enter_balance_no_navigation(MENU_ID menu);

    /**
     * Called when leaving the Balance menu
     * @param menu
     */
    MENU_ID app_commands_exit_balance(MENU_ID menu);

    /**
     * Gets the max on volume as a string
     * @return 
     */
    void app_commands_get_max_on_volume_string(char* buffer, size_t size);

    /**
     * Gets the max volume as a string
     * @return 
     */
    void app_commands_get_maximum_streaming_volume_string(char* buffer, size_t size);

    /**
     * Gets the max volume as a string
     * @return 
     */
    void app_commands_get_maximum_volume_string(char* buffer, size_t size);
    
    /**
     * Performs a user mute
     */
    void app_commands_mute_user(STRUCTURED_MESSAGE* message);

    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
