/* 
 * File:   app_config.h
 * Author: chrism
 *
 * Created on 26 October 2017, 12:04
 */

#ifndef APP_CONFIG_H
#define	APP_CONFIG_H

#ifdef	__cplusplus
extern "C" {
#endif
         
#include "system/ports/sys_ports.h"
#include "utilities/app_data.h"
#include "device_configuration.h"
#include "utilities/adc/adc_defs.h"
#include "utilities/avatar_data.h"
#include "utilities/input_control_data.h"
    
//------------------------------------------------------------------------------
// Application launch address
//------------------------------------------------------------------------------    
    
//! The address of the main program in memory (see the main apps linker file _RESET_ADDR)    
#define APP_LAUNCH_ADDRESS          0x9D020000   
    
//------------------------------------------------------------------------------
// PIUC Utilities Constants
//------------------------------------------------------------------------------    

//! This is used in the PICUtilities code to provide PIC Family specific functionality
#define ARCAM_PIC_DEVICE_FAMILY         ARCAM_PIC_DEVICE_FAMILY_MZ
    
//------------------------------------------------------------------------------avatar
// Software Version / Model 
//------------------------------------------------------------------------------    

#define SOFTWARE_VERSION_MAJOR 1
#define SOFTWARE_VERSION_MINOR 41
#define SOFTWARE_VERSION_BUILD 11
#define SOFTWARE_BUILD_TYPE SOFTWARE_BUILD_TYPE_BETA
    
//------------------------------------------------------------------------------
// Software Version SA30
//------------------------------------------------------------------------------    
//
//#define MAKES \
//    X(ARCAM, =1)\
//    X(JBL, =2)\
//
//// Now use the input data to define the enum
//#define X(a, b) MAKE_##a b,
//	typedef enum {MAKES} MAKE_ENUM;
//#undef X
//    
//// Redefine the macro and use to create an array of strings
//#define X(a, b) #a,
//	static char *MAKE_STRINGS[] = {MAKES};
//#undef X
//     
//#define PRODUCT_MAKE                    MAKE_ARCAM

//#define SOFTWARE \
//    X(SA30, SA30\0)
//
//// Now use the input data to define the enum
//#define X(a, b) SOFTWARE_MODEL_##a,
//	typedef enum {SOFTWARE} SOFTWARE_MODEL_ENUM;
//#undef X
//    
//// Redefine the macro and use to create an array of strings
//#define X(a, b) #b,
//	static char *SOFTWARE_VERSION_STRINGS[] = {SOFTWARE};
//#undef X
//    
//SOFTWARE_MODEL_ENUM SOFTWARE_MODEL;
//char* SOFTWARE_MODEL_STRING;   

//! Used for checking data loaded from eeproms (see settings/upgrade_metadata)
#define SOFTWARE_MODEL_STRING_LENGTH    5
    
//--------------------------------------------------------------------------
// Application Strings
//--------------------------------------------------------------------------

#define STRINGS \
    X(BALANCE, Balance)\
    X(NAME, Name)\
    X(CHARACTERS, ABCDEFGHIJKLMNOPQRSTUVWXYZ- 0123456789)\
    X(MUTE, Mute)\
    X(HEADPHONE, Headphone)\
    X(PROCESSOR_MODE, Processor)\
    X(ANALOG, Analogue)\
    X(ACTIVE, Active)\
    X(OFF, Off)\
    X(ON, On)\
    X(SYSTEM_FAULT, SYSTEM FAULT)\
    X(NETWORK_NAME, Name)\
    X(IP_ADDRESS, IP)\
    X(SWITCHING, Switching)\
    X(ARCAM, ARCAM)\
    X(JBL, JBL)\

// Now use the input data to define the enum
#define X(a, b) APP_STRINGS_ENUM_##a,
	typedef enum {STRINGS} APP_STRINGS_ENUM;
#undef X
    
// Redefine the macro and use to create an array of strings
#define X(a, b) #b,
	static char *APP_STRINGS[] = {STRINGS};
#undef X

//------------------------------------------------------------------------------
// Application Commands
// Commands used by the application.  These are used to map to RS232 commands and IR
// These are used by the STRUCTURED MESSAGE command    
//------------------------------------------------------------------------------    

//--------------------------------------------------------------------------
// Application Commands - Enum of available Input Mode change types
//--------------------------------------------------------------------------

/**
 * Enum of available commands
 */
typedef enum
{
    APP_COMMANDS_POWER                      = 0x00,
    APP_COMMANDS_BRIGHTNESS                 = 0x01,
    APP_COMMANDS_HEADPHONES                 = 0x02,
    APP_COMMANDS_SOFTWARE_VERSION           = 0x04,
    APP_COMMANDS_FACTORY_RESET              = 0x05,
    APP_COMMANDS_SIMULATE_RC5               = 0x08,

    APP_COMMANDS_VOLUME                     = 0x0D,
    APP_COMMANDS_MUTE                       = 0x0E,
    APP_COMMANDS_ANALOGUE_DIRECT            = 0x0F,

    APP_COMMANDS_PLAYBACK_STATUS            = 0x1C,
    APP_COMMANDS_CURRENT_SOURCE             = 0x1D,
    APP_COMMANDS_HEADPHONE_OVERRIDE         = 0x1F,

    APP_COMMANDS_HEARTBEAT                  = 0x25,
    APP_COMMANDS_REBOOT                     = 0x26,

    APP_COMMANDS_NETWORK                    = 0x30,
    APP_COMMANDS_ROOM_EQ_NAMES              = 0x34,
    APP_COMMANDS_ROOM_EQUALISATION          = 0x37,
    APP_COMMANDS_BALANCE                    = 0x3B,

    APP_COMMANDS_SAMPLE_RATE                = 0x44,
            
    APP_COMMANDS_DC_OFFSET_STATUS           = 0x51,
    APP_COMMANDS_SHORT_CIRCUIT_STATUS       = 0x52,
    APP_COMMANDS_TIMEOUT_COUNTER            = 0x55,
    APP_COMMANDS_LIFTER_TEMP                = 0x56,
    APP_COMMANDS_OUTPUT_TEMP                = 0x57,
    APP_COMMANDS_AUTO_SHUTDOWN              = 0x58,
    APP_COMMANDS_PHONO_INPUT_MM_MC          = 0x59,
    APP_COMMANDS_INPUT_DETECT               = 0x5A,
    APP_COMMANDS_PROCESSOR_MODE_INPUT       = 0x5B,
    APP_COMMANDS_PROCESSOR_MODE_VOLUME      = 0x5C,
    APP_COMMANDS_GET_STATUS_SUMMARY         = 0x5D,
    APP_COMMANDS_MODEL_NUMBER               = 0x5E,
           
    APP_COMMANDS_BIAS_CHECK                 = 0x60,
    APP_COMMANDS_SET_FILTER                 = 0x61,   
    APP_COMMANDS_NOW_PLAYING                = 0x64,
    APP_COMMANDS_MAX_ON_VOLUME              = 0x65,
    APP_COMMANDS_MAXIMUM_VOLUME             = 0x66,  
    APP_COMMANDS_MAXIMUM_STREAMING_VOLUME   = 0x67,
    APP_COMMANDS_REMOTE_AVATAR_MESSAGE      = 0x68,
    APP_COMMANDS_AVATAR_MQA_DECODER_LEVEL   = 0x69,

    APP_COMMANDS_MENU                       = 0xB0,
    APP_COMMANDS_MENU_NAVIGATION            = 0xB1,
    APP_COMMANDS_MENU_VALUE_CHANGE          = 0xB3,
    APP_COMMANDS_PREVIEW_SOURCE             = 0xB4,
    APP_COMMANDS_BROWSING                   = 0xB5,
            
    APP_COMMANDS_TEST_RGB_COLOURS           = 0xB8,
    APP_COMMANDS_UPGRADE_LOAD_START         = 0xB9,
    APP_COMMANDS_UPGRADE_LOAD_DATA          = 0xBA,
    APP_COMMANDS_UPGRADE_LOAD_END           = 0xBB,
    APP_COMMANDS_ERASE_FLASH                = 0xBC,
    APP_COMMANDS_PRINT_UPGRADE_INFO         = 0xBD,
    APP_COMMANDS_AVATAR_UPGRADE_OPTION      = 0xBE,
    APP_COMMANDS_AVATAR_UPGRADE_DATA        = 0xBF, //TODO Testing Only
            
    APP_COMMANDS_AMX_DUET                   = 0xC0,

    APP_COMMANDS_ADC_RAW_VALUES             = 0xF0,
    APP_COMMANDS_INJECT_FAULT               = 0xF1,
    APP_COMMANDS_WATCHDOG_STATUS            = 0xF2,
    //! Command used for logger messages - outgoing messages only
    APP_COMMANDS_LOGGER_MESSAGE             = 0xF3,   
    APP_COMMANDS_SET_LOGGER_LEVEL           = 0xF4,   
} APP_COMMANDS;
 
//--------------------------------------------------------------------------
// Application Commands - Enum of special command values used in processing of IR/Button messages
//--------------------------------------------------------------------------
typedef enum
{
    APP_COMMAND_VALUE_REQUEST_STATUS = 0xF0,
    APP_COMMAND_VALUE_INCREMENT = 0xF1,
    APP_COMMAND_VALUE_DECREMENT = 0xF2,
} APP_COMMAND_VALUES;

//--------------------------------------------------------------------------
// Application Commands - Auto Shutdown
//--------------------------------------------------------------------------

#define AUTO_SHUTDOWN_TIMEOUTS \
    X(OFF, 0)\
    X(20MIN, 20)\
    X(30MIN, 30)\
    X(60MIN, 60)\
    X(120MIN, 120)\
    X(240MIN, 240)\
    X(MAX, 300)

// Now use the input data to define the enum
#define X(a, b) APP_COMMAND_AUTO_SHUTDOWN_TIMEOUT_##a,
	typedef enum {AUTO_SHUTDOWN_TIMEOUTS} APP_COMMAND_AUTO_SHUTDOWN_TIMEOUT;
#undef X
    
// Redefine the macro and use to create an array of strings
#define X(a, b) b,
	static int APP_COMMAND_AUTO_SHUTDOWN_TIMEOUT_VALUES[] = {AUTO_SHUTDOWN_TIMEOUTS};
#undef X

//--------------------------------------------------------------------------
// Application Commands - Enum of available Bias Check states
//--------------------------------------------------------------------------
typedef enum
{
    APP_COMMAND_BIAS_CHECK_START            = 0x00,
    APP_COMMAND_BIAS_CHECK_STOP             = 0x01,
    APP_COMMAND_BIAS_CHECK_DATA             = 0x02,
    APP_COMMAND_BIAS_CHECK_REQUEST_STATE    = APP_COMMAND_VALUE_REQUEST_STATUS,
} APP_COMMAND_BIAS_TEST;

//--------------------------------------------------------------------------
// Application Commands - Enum of Display Brightness options
//--------------------------------------------------------------------------
typedef enum
{
    APP_COMMAND_BRIGHTNESS_OFF          = 0x00,
    APP_COMMAND_BRIGHTNESS_DIMMED       = 0x01,
    APP_COMMAND_BRIGHTNESS_FULL         = 0x02,
    APP_COMMAND_BRIGHTNESS_TOGGLE       = 0x03,
    APP_COMMAND_BRIGHTNESS_GETSTATUS    = APP_COMMAND_VALUE_REQUEST_STATUS,
} APP_COMMAND_BRIGHTNESS;

static char* APP_COMMAND_BRIGHTNESS_STRINGS[] = { "Off", "Dim", "Full" };

//--------------------------------------------------------------------------
// Application Commands - Enum of Dark Mode options
//--------------------------------------------------------------------------
typedef enum
{
    APP_COMMAND_DARK_MODE_OFF           = 0x00,
    APP_COMMAND_DARK_MODE_ON            = 0x01,
    APP_COMMAND_DARK_MODE_GETSTATUS     = APP_COMMAND_VALUE_REQUEST_STATUS,
} APP_COMMAND_DARK_MODE;

static char* APP_COMMAND_DARK_MODE_STRINGS[] = { "Off", "On" };

//--------------------------------------------------------------------------
// Application Commands - Enum of Network information options
//--------------------------------------------------------------------------
typedef enum
{
    APP_COMMAND_NETWORK_IP_ADDRESS          = 0xF0,
    APP_COMMAND_NETWORK_MAC_ADDRESS         = 0xF1,
    APP_COMMAND_NETWORK_WIFI_MAC_ADDRESS    = 0xF2, 
    APP_COMMAND_NETWORK_FRIENDLY_NAME       = 0xF3,
    APP_COMMAND_NETWORK_HOST_NAME           = 0xF4,
    APP_COMMAND_NETWORK_SSID                = 0xF5,
} APP_COMMAND_NETWORK;

//--------------------------------------------------------------------------
// Application Commands - Enum of Now Playing information options
//--------------------------------------------------------------------------
typedef enum
{
    APP_COMMAND_NOW_PLAYING_TRACK       = 0xF0,
    APP_COMMAND_NOW_PLAYING_ARTIST      = 0xF1,
    APP_COMMAND_NOW_PLAYING_ALBUM       = 0xF2,
    APP_COMMAND_NOW_PLAYING_APPLICATION = 0xF3,
    APP_COMMAND_NOW_PLAYING_SAMPLE_RATE = 0xF4,
    APP_COMMAND_NOW_PLAYING_ENCODER     = 0xF5,
} APP_COMMAND_NOW_PLAYING;

//--------------------------------------------------------------------------
// Application Commands - Generic strings for displaying Off/On
//--------------------------------------------------------------------------

static char* APP_COMMAND_OFF_ON_STRINGS[] = { "Off", "On" };
static char* APP_COMMAND_LOW_HIGH_STRINGS[] = { "Low", "High" };
 
//--------------------------------------------------------------------------
// Application Modes
//--------------------------------------------------------------------------

typedef enum
{
    //! Default running mode - generally displays the Splash screen 
    APP_CONTEXT_MODE_DEFAULT,

    //! The user has taken an action that displays the menu
    APP_CONTEXT_MODE_MENU,

    //! The user is editing a specific menu option
    APP_CONTEXT_MODE_MENU_EDIT,

    //! A process is being invoked  as a response to selecting a menu item
    APP_CONTEXT_MODE_PROCESS,

    //! The app is in a standby state
    APP_CONTEXT_MODE_STANDBY,

    //! The app is performing a BIAS test
    APP_CONTEXT_MODE_BIAS_TEST,
} APP_CONTEXT_MODE;
 
//--------------------------------------------------------------------------
// Application Modules
//--------------------------------------------------------------------------

/**
 * APP_MODULES are used by app_context_module_running to indicate that the module 
 * is now fully up and running.
 * This only needs to contain modules that the startup depends on or whose state 
 * needs checking
 */

#define APP_MODULES \
    X(POWER, true)\
    X(INPUT_CONTROL, true)\
    X(TCP_IP, false)\
    X(AVATAR, true)\

// Now use the input data to define the enum
#define X(a, b) APP_MODULE_##a,
	typedef enum {APP_MODULES APP_MODULE_MAX} APP_MODULE;
#undef X
    
// Used to determine whether the module needs to be running (app_context_module_running)
// to indicate that the while system is now ready for use.    
#define X(a, b) b,
	static bool APP_MODULE_STARTUP_DEPENDENCY[] = {APP_MODULES};
#undef X

//--------------------------------------------------------------------------
// Menus - This needs to be above Inputs
//--------------------------------------------------------------------------

#define MENU_ITEMS \
    X(SYSTEM_SETTINGS, "System Settings")\
    X(AUDIO_SETTINGS, "Audio Settings")\
    X(NETWORK_SETTINGS, "Network Settings")\
    X(SYSTEM_INFO, "System Info")\
    X(ENGINEERING_SETTINGS, "Eng Settings")\
    X(VOLUME, "Volume")\
    X(BALANCE, "Balance")\
    X(BALANCE_NO_NAVIGATION, "Balance")\
    X(PROCESSOR_MODE_INPUT, "PM Input")\
    X(PROCESSOR_MODE_VOLUME, "PM Volume")\
    X(MAX_ON_VOLUME, "Max On Vol")\
    X(MAXIMUM_VOLUME, "Maximum Vol")\
    X(MAXIMUM_STREAMING_VOLUME, "Max Net Vol")\
    X(SYSTEM_CODE, "System Code")\
    X(DISPLAY, "Display")\
    X(DISPLAY_NO_NAVIGATION, "Display")\
    X(DISPLAY_TEST, "Display Test")\
    X(DISPLAY_TESTING, "Display Testing")\
    X(BIAS_CHECK, "Bias Check")\
    X(SERIAL_NUMBER, "Serial Number")\
    X(SYSTEM_INFO_HOST_VERSION, "Version")\
    X(SYSTEM_INFO_AVATAR_VERSION, "Net Ver")\
    X(SYSTEM_INFO_HDMI_ARC_VERSION, "ARC Ver")\
    X(SYSTEM_INFO_HDMI_ARC_RX_VERSION, "ARC Rx")\
    X(SYSTEM_INFO_AVATAR_FACTORY_RESET, "Net Reset")\
    X(SYSTEM_INFO_AVATAR_FACTORY_RESET_CONFIRM, "Continue?")\
    X(SYSTEM_INFO_AVATAR_FACTORY_RESETTING, "Net Resetting")\
    X(SYSTEM_INFO_NETWORK_IP_ADDRESS, "IP")\
    X(SYSTEM_INFO_NETWORK_NAME_INFO, "Name")\
    X(SYSTEM_INFO_NETWORK_FRIENDLY_NAME_INFO, "")\
    X(SYSTEM_RESET, "System Reset")\
    X(SYSTEM_RESET_CONFIRM, "Continue?")\
    X(SYSTEM_RESET_RESETTING, "Resetting")\
    X(SYSTEM_RESET_REBOOT, "Reboot")\
    X(USB_UPGRADE, "USB Upgrade")\
    X(USB_UPGRADE_CONFIRM, "Continue?")\
    X(STANDBY_TIMER, "Stdby")\
    X(SHUTDOWN_CODE, "LSDC")\
    X(IR_DEBUG, "IR Debug")\
    X(SWITCH_VOLTAGES, "Switch Voltages")\
    X(LOG_LEVEL, "Logger")\
    X(HEADPHONE_OVERRIDE, "Phones Ovr")\
    X(AUTO_STANDBY_TIMEOUT, "Timeout")\
    X(NETWORK_INFO, "Network Info")\
    X(NETWORK_DHCP_INFO, "DHCP")\
    X(NETWORK_IP_ADDRESS_INFO, "IP")\
    X(NETWORK_INFO_MAC_ADDRESS, "MAC")\
    X(NETWORK_NAME_INFO, "Name")\
    X(NETWORK_FRIENDLY_NAME_INFO, "")\
    X(SSID, "SSID")\
    X(ANALOGUE_DIRECT, "Analogue Direct")\
    X(ANALOGUE_DIRECT_NO_NAVIGATION, "Analogue Direct")\
    X(ANALOGUE_DIRECT_PHONO, "Phono")\
    X(ANALOGUE_DIRECT_AUX, "AUX")\
    X(ANALOGUE_DIRECT_CD, "CD")\
    X(ANALOGUE_DIRECT_PVR, "PVR")\
    X(ANALOGUE_DIRECT_STB, "STB")\
    X(INPUT_ENABLED, "Enable Input")\
    X(INPUT_ENABLED_PHONO, "Phono")\
    X(INPUT_ENABLED_AUX, "AUX")\
    X(INPUT_ENABLED_CD, "CD")\
    X(INPUT_ENABLED_PVR, "PVR")\
    X(INPUT_ENABLED_STB, "STB")\
    X(INPUT_ENABLED_AV, "AV")\
    X(INPUT_ENABLED_BD, "BD")\
    X(INPUT_ENABLED_SAT, "SAT")\
    X(INPUT_ENABLED_GAME, "GAME")\
    X(INPUT_ENABLED_PCUSB, "PC USB")\
    X(INPUT_ENABLED_NET, "NET")\
    X(INPUT_ENABLED_USB, "USB")\
    X(INPUT_ENABLED_ARC, "ARC")\
    X(PHONO_INPUT, "Phono Input")\
    X(PHONO_INPUT_NO_NAVIGATION, "Phono Input")\
    X(FILTER, "Filter")\
    X(AMP_TEMP, "Amp Temp")\
    X(LIFTER_TEMP, "Lifter")\
    X(RS232_STANDBY, "RS232 Stdby")\
    X(NETWORK_STANDBY, "Net Standby")\
    X(INFO, "Info")\
    X(CONTROL4_ACTIVE, "C4 SDDP")\
    X(CONTROL4_IDENTITY, "Send C4 Identity")\
    X(CONTROL4_IDENTITY_SENT, "C4 Identity Sent")\
    X(CRESTRON_ACTIVE, "CTron")\
    X(CRESTRON_CONFIG, "Crestron Config")\
    X(CRESTRON_CONFIG_COMPLETING, "Crestron Config")\
    X(DIRAC_NO_NAVIGATION, "Dirac")\
    X(DIRAC_CALIBRATION, "Dirac")\
    X(DIRAC_CURVE, "Dirac Curve")\
    X(DIRAC_CURVE_PHONO, "Phono")\
    X(DIRAC_CURVE_AUX, "AUX")\
    X(DIRAC_CURVE_CD, "CD")\
    X(DIRAC_CURVE_PVR, "PVR")\
    X(DIRAC_CURVE_STB, "STB")\
    X(DIRAC_CURVE_AV, "AV")\
    X(DIRAC_CURVE_BD, "BD")\
    X(DIRAC_CURVE_SAT, "SAT")\
    X(DIRAC_CURVE_GAME, "GAME")\
    X(DIRAC_CURVE_PCUSB, "PC USB")\
    X(DIRAC_CURVE_NET, "NET")\
    X(DIRAC_CURVE_USB, "USB")\
    X(AVATAR_UPGRADE_OPTION, "OTA Upg")\
    X(AVATAR_UPGRADE, "Update")\
    X(AVATAR_UPGRADE_CONFIRM, "Continue?")\
    X(AVATAR_UPGRADE_IN_PROGRESS, "Net Updating")\
    X(AVATAR_UPGRADE_COMPLETE, "Update Complete")\
    X(AVATAR_UPGRADE_FAIL, "Update Fail")\
    X(STANDBY_SETTINGS, "Required For Menu")\
    X(DARK_MODE, "Dark Mode")\
    X(REGION, "Region")\
    X(ARC_UPGRADE, "ARC Recovery")\
    X(ARC_UPGRADE_CONFIRM, "Continue?")\
    X(CHECK_CHANNEL_STATUS, "Ch Status")\
    X(HDMI_AUDIO_CONTROL, "TV Audio")\
    X(HDMI_POWER_CONTROL, "TV Power")\
    X(SHORT_CIRCUIT_SENSE, "SC Sense")\
	X(MQA_DECODER_LEVEL, "MQA Decoder Level")\
	X(LIP_SYNC, "Lip Sync")\
    X(LIP_SYNC_PHONO, "Phono")\
    X(LIP_SYNC_AUX, "AUX")\
    X(LIP_SYNC_CD, "CD")\
    X(LIP_SYNC_PVR, "PVR")\
    X(LIP_SYNC_STB, "STB")\
    X(LIP_SYNC_AV, "AV")\
    X(LIP_SYNC_BD, "BD")\
    X(LIP_SYNC_SAT, "SAT")\
    X(LIP_SYNC_GAME, "GAME")\
    X(LIP_SYNC_PCUSB, "PC USB")\
    X(LIP_SYNC_NET, "NET")\
    X(LIP_SYNC_USB, "USB")\
    X(LIP_SYNC_ARC, "ARC")\
    X(MEDIA_BROWSING, "Media Browser")\

    //! Define a menu types enumeration
    typedef enum 
    {
#define X(menu, title) APP_COMMAND_MENU_##menu,
	MENU_ITEMS
#undef X
      APP_COMMAND_MENU_SIZE,      
    } APP_COMMAND_MENU;
    
    //! Declare a menu names array - This is passed to menu_initialise()
#define X(menu, title) title,
	static char *app_menu_titles[] = { MENU_ITEMS };
#undef X

#define APP_COMMAND_MENU_DEFAULT  APP_COMMAND_MENU_VOLUME

//------------------------------------------------------------------------------
// ADC / Buttons
//------------------------------------------------------------------------------    

//! Defines the number of ADC inputs that will be monitored by the ADC module    
#define ADC_INPUT_COUNT     ADC_BUTTON_INPUT_COUNT + ADC_VOLTAGE_INPUT_COUNT + ADC_TEMPERATURE_INPUT_COUNT + ADC_POT_INPUT_COUNT
    
//! Defines the maximum number of buttons that can be associated with a single ADC Input    
#define ADC_MAX_BUTTON_COUNT            4

//! The data resolution form comparison e,f 12bit - 2^12 - 1 = 4095
#define ADC_DATA_RESOLUTION     4095

/*
 * Switch voltages (from SA30)
 */        
#define ADC_BUTTON_1_LO_MV              260
#define ADC_BUTTON_1_HI_MV              500
#define ADC_BUTTON_2_LO_MV              560
#define ADC_BUTTON_2_HI_MV              800
#define ADC_BUTTON_3_LO_MV              930
#define ADC_BUTTON_3_HI_MV              1150
#define ADC_BUTTON_4_LO_MV              1310
#define ADC_BUTTON_4_HI_MV              1550
#define ADC_BUTTON_ERROR_MV             3100

static ADC_BUTTON_DEFINITION ADC_BUTTON_DEFAULTS[ADC_MAX_BUTTON_COUNT] = {
    { ADC_BUTTON_1_LO_MV, ADC_BUTTON_1_HI_MV, ADC_BUTTON_NONE },
    { ADC_BUTTON_2_LO_MV, ADC_BUTTON_2_HI_MV, ADC_BUTTON_NONE },
    { ADC_BUTTON_3_LO_MV, ADC_BUTTON_3_HI_MV, ADC_BUTTON_NONE },
    { ADC_BUTTON_4_LO_MV, ADC_BUTTON_4_HI_MV, ADC_BUTTON_NONE },
};

//------------------------------------------------------------------------------
// Avatar 
//------------------------------------------------------------------------------    
        
/**
 * MUST be set to 1 to enable I2C functionality 
 */
#define AVATAR_I2C_ENABLED                  1 

/**
 * The module (defined in Harmony Drivers->I2C Configuration) 
 */
#define AVATAR_I2C_MODULE                   0

/**
 * IRQ Port for requesting communication with the Avatar
 */
#define AVATAR_COMMUNICATION_PORTS_ID       PORTS_ID_0
#define AVATAR_COMMUNICATION_PORT_CHANNEL   PORT_CHANNEL_E
#define AVATAR_COMMUNICATION_PIN            PORTS_BIT_POS_9

/**
 * Avatar Reset pin
 */
#define AVATAR_RESET_PORTS_ID               PORTS_ID_0
#define AVATAR_RESET_PORT_CHANNEL           PORT_CHANNEL_A
#define AVATAR_RESET_PIN                    PORTS_BIT_POS_4

//TODO Is BOOTMODE THE Enable Pin?
#define AVATAR_BOOTMODE_PORTS_ID            PORTS_ID_0
#define AVATAR_BOOTMODE_PORT_CHANNEL        PORT_CHANNEL_A
#define AVATAR_BOOTMODE_PIN                 PORTS_BIT_POS_5

/**
 * Used to identify that tha Avatar is ready
 */
#define AVATAR_READY_PORTS_ID               PORTS_ID_0
#define AVATAR_READY_PORT_CHANNEL           PORT_CHANNEL_E
#define AVATAR_READY_PIN                    PORTS_BIT_POS_4

/**
 * List of Avatar State Ids that we need to subscribe to 
 * Take note of AVATAR_MESSAGE_MAX_STATES when allocation more notify states
 */
static AVATAR_STATE_ID AVATAR_STATE_SUBSCRIPTIONS[] = {
    AVATAR_STATE_ID_PLAYER_STATUS,
    AVATAR_STATE_ID_REPEAT,
    AVATAR_STATE_ID_SHUFFLE,
    AVATAR_STATE_ID_NETWORK_STATE,
    AVATAR_STATE_ID_ARTIST_NAME,
    AVATAR_STATE_ID_ALBUM_NAME,
    AVATAR_STATE_ID_TRACK_NAME,
    AVATAR_STATE_ID_DISPLAY_SAMPLE_RATE,    
    AVATAR_STATE_ID_SAMPLE_RATE,    
    AVATAR_STATE_ID_MIME_TYPE,   
    AVATAR_STATE_ID_AUDIO_SOURCE,    
    AVATAR_STATE_ID_VOLUME,    
    AVATAR_STATE_ID_AUDIO_INPUT,    
    AVATAR_STATE_ID_IP_ADDRESS,    
    AVATAR_STATE_ID_TRACK_LENGTH, 
    AVATAR_STATE_ID_TRACK_TIME,  
    AVATAR_STATE_ID_SSID,  
    AVATAR_STATE_ID_MUTE,
    AVATAR_STATE_ID_CHROMECAST_APP_NAME,
    //AVATAR_STATE_ID_ALBUM_ART_URL,
    AVATAR_STATE_ID_FRIENDLY_NAME,
    AVATAR_STATE_ID_MQA_STATE,
    AVATAR_STATE_ID_DIRAC_CALIBRATION,
    AVATAR_STATE_ID_DIRAC_CURVE_NAMES,
    AVATAR_STATE_ID_DIRAC_CLIENT_STATUS,
};

//! Defines the number of upgrade images handled by the avatar_upgrade
#define AVATAR_UPGRADE_IMAGE_NUM_IMAGES     3   //TODO Change back to 1

//! Defines the maximum number of results to be returned by an Avatar browsing request
#define AVATAR_BROWSE_RESULTS_MAX   1

//! maximum number of folder levels supported by browsing
#define AVATAR_BROWSE_LEVELS        10

//------------------------------------------------------------------------------
// Dirac 
//------------------------------------------------------------------------------    

//! The maximum number of Dirac curves available  
// Note this includes the No Dirac option (DIRAC_NO_CURVE))
#define DIRAC_MAX_EQ_CURVES                 7    

//---------------app_commands_hdmi_arc_check_upgrade_version---------------------------------------------------------------
// Encoder 
//------------------------------------------------------------------------------    

#define ENCODER_ITEMS \
    X(MAIN, PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_10, PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_12)\
    X(INPUT_SELECT, PORTS_ID_0, PORT_CHANNEL_G, PORTS_BIT_POS_0, PORTS_ID_0, PORT_CHANNEL_F, PORTS_BIT_POS_1)\

//------------------------------------------------------------------------------
// Events 
//------------------------------------------------------------------------------    

/**
 * a: Enum name b: Event Module Description
 */
#define EVENT_MODULE_ITEMS \
    X(SPDIF_RX_HW, SPDIF Rx Hardware)\
    X(SPDIF_RX, SPDIF Rx)\
    X(INPUT_ADC_HW, Input ADC Hardware)\
    X(ADC_POT, ADC Potentiometer)\
 
//------------------------------------------------------------------------------
// Filter 
//------------------------------------------------------------------------------    

//! Note these should be in the same order as the filter types are added to the
// input_control_dac filter_type_register

static char* DAC_FILTER_STRINGS[] = { "LinP Fast", 
                                      "LinP Slow", 
                                      "MinP Fast", 
                                      "MinP Slow", 
                                      "B Wall", 
                                      "Corr MinP", 
                                      "Apodizing" };

//------------------------------------------------------------------------------
// Flash Memory
//------------------------------------------------------------------------------  

#define FLASH_MEMORY_MAX_CLIENTS            2
    
//! The device name to be supplied to the SYS_FS_Mount function
//  This should match the device name supplied in the Harmony configuration    
#define FLASH_MEMORY_DEVICE_NAME            "/dev/mtda1"    

//! The mount name to be suppled to the SYS_FS_Mount function
//  This should match the mount name supplied in the Harmony configuration    
#define FLASH_MEMORY_MOUNT_NAME             "/mnt/Drive1"
    
//! The number of requests that can be queued at any point
#define FLASH_MEMORY_REQUEST_QUEUE_SIZE     5
    
//! Define client ids for the modules using the flash memory
//  These should be ascending from 0    
#define FLASH_MEMORY_CLIENT_SETTINGS        0    
#define FLASH_MEMORY_CLIENT_UPGRADE         1  

//! The maximum number of files that can be associated with a single client
#define FLASH_MEMORY_MAX_CLIENT_FILES       3
 
//------------------------------------------------------------------------------
// HDMI ARC
//------------------------------------------------------------------------------    

//! Used to identify to modules such as input control that HDMI_ARC is in use
#define HDMI_ARC_ACTIVE

#define HDMI_ARC_ENABLED   1

#define HDMI_ARC_RESET_PORTS_ID                PORTS_ID_0
#define HDMI_ARC_RESET_PORT_CHANNEL            PORT_CHANNEL_B
#define HDMI_ARC_RESET_PIN                     PORTS_BIT_POS_14

//------------------------------------------------------------------------------
// I2C
//------------------------------------------------------------------------------    
    
// I2C Ports
//! Port definitions of I2C bit bash functions
#define I2C_BIT_BASH_SCL_MODULE_0_PORT_ID               PORTS_ID_0
#define I2C_BIT_BASH_SCL_MODULE_0_CHANNEL               PORT_CHANNEL_A
#define I2C_BIT_BASH_SCL_MODULE_0_PIN                   PORTS_BIT_POS_14
#define I2C_BIT_BASH_SDA_MODULE_0_PORT_ID               PORTS_ID_0
#define I2C_BIT_BASH_SDA_MODULE_0_CHANNEL               PORT_CHANNEL_A
#define I2C_BIT_BASH_SDA_MODULE_0_PIN                   PORTS_BIT_POS_15

/**
 * MUST be set to 1 to enable I2C functionality 
 */
#define I2C_ENABLED             1 

//! Default to allow max of 2 I2C busses
#define I2C_MAX_MODULES         2

//! Defines the maximum number of I2C peripherals (over all I2c busses)
#define I2C_MAX_CLIENTS         4   // For SA30 we need 4 (SPDIF Rx/ADC/Avatar/DAC)
 
#define I2C_REQUEST_QUEUE_SIZE  50  // MUST be larger than the max number of requests at any one time
  
#define I2C_SLAVE_NUM_MODULES   2

//--------------------------------------------------------------------------
// Inputs
//--------------------------------------------------------------------------
    
//! Define the types of input - note if these are added to/changed then the device
//  needs resetting to clear the settings.
#define AUDIO_INPUTS \
	X(NONE, None, 0, APP_COMMAND_MENU_DEFAULT)\
	X(PHONO, PHONO, 1, APP_COMMAND_MENU_DEFAULT)\
    X(AUX, AUX, 2, APP_COMMAND_MENU_DEFAULT)\
	X(CD, CD, 6, APP_COMMAND_MENU_DEFAULT)\
    X(PVR, PVR, 3, APP_COMMAND_MENU_DEFAULT)\
	X(STB, STB, 5, APP_COMMAND_MENU_DEFAULT)\
   	X(AV, AV, 4, APP_COMMAND_MENU_DEFAULT)\
    X(BD, BD, 7, APP_COMMAND_MENU_DEFAULT)\
	X(SAT, SAT, 8, APP_COMMAND_MENU_DEFAULT)\
	X(GAME, GAME, 9, APP_COMMAND_MENU_DEFAULT)\
	X(NET, NET, 11, APP_COMMAND_MENU_MEDIA_BROWSING)\
	X(AVATAR, AVATAR, 11, APP_COMMAND_MENU_DEFAULT)\
	X(HDMI_ARC, ARC, 103, APP_COMMAND_MENU_DEFAULT)\
	X(DIGITAL_MUTE, DIG_MUTE, 104, APP_COMMAND_MENU_DEFAULT)\
    //X(PROCESSOR, Processor, 0)

// Now use the input data to define the enum
#define X(a, b, c, d) AUDIO_INPUT_##a,
	typedef enum {AUDIO_INPUTS AUDIO_INPUT_MAX} AUDIO_INPUT;
#undef X
   
// Now use the input data to define the enum
#define X(a, b, c, d) c,
    static int AUDIO_INPUT_VALUES[] = { AUDIO_INPUTS };
#undef X
    
// Redefine the macro and use to create an array of strings
#define X(a, b, c, d) #b,
	static char *AUDIO_INPUT_STRINGS[] = { AUDIO_INPUTS };
#undef X
   
// Now use the input data to define the enum
#define X(a, b, c, d) d,
    static APP_COMMAND_MENU AUDIO_INPUT_MENUS[] = { AUDIO_INPUTS };
#undef X
     
//! Used by input_control to switchto / change inputs    
#define AUDIO_INPUT_DEFAULT     AUDIO_INPUT_CD
#define AUDIO_INPUT_ALTERNATIVE AUDIO_INPUT_PHONO    
   
/**
 * Define the number of input controls to be managed - this is required for use with 
 * input_control.h/c - Include 1 for NO INPUT
 */
#define INPUT_CONTROL_NUM_SETTINGS                      AUDIO_INPUT_MAX 
    
/**
 * MUST be set to 1 to enable Pins for setting/reading input_control
 */
#define INPUT_CONTROL_PINS_ENABLED                      1
    
// Input Control
#define INPUT_CONTROL_AUDIO_PRESENT_PIN_STATE           INPUT_CONTROL_PIN_STATE_LOW
#define INPUT_CONTROL_AUDIO_PRESENT_ID                  PORTS_ID_0
#define INPUT_CONTROL_AUDIO_PRESENT_PORT_CHANNEL        PORT_CHANNEL_A
#define INPUT_CONTROL_AUDIO_PRESENT_PIN                 PORTS_BIT_POS_9    

//! Used in INPUT_CONTROL_STATE_SWITCH_INPUTS 
#define INPUT_CONTROL_VOLUME_BOOST_ANALOGUE             5
#define INPUT_CONTROL_VOLUME_BOOST_ANALOGUE_DIRECT      -7
#define INPUT_CONTROL_VOLUME_BOOST_DIGITAL              -8
#define INPUT_CONTROL_VOLUME_BOOST_AVATAR               -8
    
// Analogue
//---------------------------
#define INPUT_CONTROL_SELECT_0_PORTS_ID                 PORTS_ID_0
#define INPUT_CONTROL_SELECT_0_PORT_CHANNEL             PORT_CHANNEL_C
#define INPUT_CONTROL_SELECT_0_PIN                      PORTS_BIT_POS_13
#define INPUT_CONTROL_SELECT_1_PORTS_ID                 PORTS_ID_0
#define INPUT_CONTROL_SELECT_1_PORT_CHANNEL             PORT_CHANNEL_C
#define INPUT_CONTROL_SELECT_1_PIN                      PORTS_BIT_POS_14
#define INPUT_CONTROL_SELECT_2_PORTS_ID                 PORTS_ID_0
#define INPUT_CONTROL_SELECT_2_PORT_CHANNEL             PORT_CHANNEL_D
#define INPUT_CONTROL_SELECT_2_PIN                      PORTS_BIT_POS_3
#define INPUT_CONTROL_MM_MC_SELECT_ID                   PORTS_ID_0
#define INPUT_CONTROL_MM_MC_SELECT_PORT_CHANNEL         PORT_CHANNEL_A
#define INPUT_CONTROL_MM_MC_SELECT_PIN                  PORTS_BIT_POS_7  
         
//! MUST be set to 1 to enable Pins for setting/reading input_control
#define INPUT_CONTROL_I2C_ADC_ENABLED               1 

////! The module (defined in Harmony Drivers->I2C Configuration) used for the DAC
//#define INPUT_CONTROL_ADC_I2C_MODULE                DRV_I2C_INDEX_1
//
////! Read/Write addresses for the I2C ADC device
//#define INPUT_CONTROL_ADC_I2C_WRITE_ADDRESS         0x22 
//#define INPUT_CONTROL_ADC_I2C_READ_ADDRESS          0x23  

//! The max number of DAC registers that require initialising - setup using input_control_add_dac_setting()
#define INPUT_CONTROL_NUM_ADC_SETTING               8 
    
// Digital    
//---------------------------
//#define INPUT_CONTROL_SPDIF_RX_RESET_PORTS_ID           PORTS_ID_0
//#define INPUT_CONTROL_SPDIF_RX_RESET_PORT_CHANNEL       PORT_CHANNEL_F
//#define INPUT_CONTROL_SPDIF_RX_RESET_PIN                PORTS_BIT_POS_12
//#define INPUT_CONTROL_SPDIF_RX_INT0_ID                  PORTS_ID_0
//#define INPUT_CONTROL_SPDIF_RX_INT0_PORT_CHANNEL        PORT_CHANNEL_F
//#define INPUT_CONTROL_SPDIF_RX_INT0_PIN                 PORTS_BIT_POS_0  
//#define INPUT_CONTROL_SPDIF_RX_INT1_ID                  PORTS_ID_0
//#define INPUT_CONTROL_SPDIF_RX_INT1_PORT_CHANNEL        PORT_CHANNEL_F
//#define INPUT_CONTROL_SPDIF_RX_INT1_PIN                 PORTS_BIT_POS_1  
         
//! MUST be set to 1 to enable Pins for setting/reading input_control
//#define INPUT_CONTROL_I2C_DIGITAL_ENABLED               1  
//
////! The module (defined in Harmony Drivers->I2C Configuration) used for the DAC
//#define INPUT_CONTROL_DIGITAL_I2C_MODULE                DRV_I2C_INDEX_1
//
////! Read/Write addresses for the I2C Mux device
//#define INPUT_CONTROL_DIGITAL_I2C_WRITE_ADDRESS         0x20 
//#define INPUT_CONTROL_DIGITAL_I2C_READ_ADDRESS          0x21  

//! The max number of DAC registers that require initialising - setup using input_control_add_dac_setting()
//#define INPUT_CONTROL_NUM_DIGITAL_SETTING               9 
    
// MUX
//---------------------------
#define INPUT_CONTROL_I2S_SWITCH_SELECT_0_ID            PORTS_ID_0
#define INPUT_CONTROL_I2S_SWITCH_SELECT_0_PORT_CHANNEL  PORT_CHANNEL_B
#define INPUT_CONTROL_I2S_SWITCH_SELECT_0_PIN           PORTS_BIT_POS_10    
#define INPUT_CONTROL_I2S_SWITCH_SELECT_1_ID            PORTS_ID_0
#define INPUT_CONTROL_I2S_SWITCH_SELECT_1_PORT_CHANNEL  PORT_CHANNEL_A
#define INPUT_CONTROL_I2S_SWITCH_SELECT_1_PIN           PORTS_BIT_POS_10  
    
// DAC    
//---------------------------
#define INPUT_CONTROL_DAC_RESET_PORTS_ID                PORTS_ID_0
#define INPUT_CONTROL_DAC_RESET_PORT_CHANNEL            PORT_CHANNEL_A
#define INPUT_CONTROL_DAC_RESET_PIN                     PORTS_BIT_POS_0   
#define INPUT_CONTROL_DAC_GPIO_PORTS_ID                 PORTS_ID_0
#define INPUT_CONTROL_DAC_GPIO_PORT_CHANNEL             PORT_CHANNEL_B
#define INPUT_CONTROL_DAC_GPIO_PIN                      PORTS_BIT_POS_12
         
//! MUST be set to 1 to enable Pins for setting/reading input_control
#define INPUT_CONTROL_I2C_DAC_ENABLED                   1

//! The module (defined in Harmony Drivers->I2C Configuration) used for the DAC
#define INPUT_CONTROL_DAC_I2C_MODULE                    DRV_I2C_INDEX_1

//! Read/Write addresses for the I2C DAC device
#define INPUT_CONTROL_DAC_I2C_WRITE_ADDRESS             0x90
#define INPUT_CONTROL_DAC_I2C_READ_ADDRESS              0x91  

//! The max number of DAC registers that require initialising - setup using input_control_add_dac_setting()
#define INPUT_CONTROL_NUM_DAC_SETTING                   20 //???
     
//! This value is the default for the 0x2C register in the ESS9038 DAC    
#define INPUT_CONTROL_CUSTOM_FILTER_ENABLE_REGISTER_DEFAULT     0x04    // Cosine symmetric filter
   
// USB    
//---------------------------
#define INPUT_CONTROL_USB_RECORD_ID                     PORTS_ID_0
#define INPUT_CONTROL_USB_RECORD_PORT_CHANNEL           PORT_CHANNEL_D
#define INPUT_CONTROL_USB_RECORD_PIN                    PORTS_BIT_POS_14  
    
// Enable / Disable USB    
//TODO REmove#define INPUT_CONTROL_USB_BCLK_ID                       PORTS_ID_0
//#define INPUT_CONTROL_USB_BCLK_PORT_CHANNEL             PORT_CHANNEL_D
//#define INPUT_CONTROL_USB_BCLK_PIN                      PORTS_BIT_POS_1  
//#define INPUT_CONTROL_USB_LRCLK_ID                      PORTS_ID_0
//#define INPUT_CONTROL_USB_LRCLK_PORT_CHANNEL            PORT_CHANNEL_D
//#define INPUT_CONTROL_USB_LRCLK_PIN                     PORTS_BIT_POS_9  
//#define INPUT_CONTROL_USB_SDATA_ID                      PORTS_ID_0
//#define INPUT_CONTROL_USB_SDATA_PORT_CHANNEL            PORT_CHANNEL_F
//#define INPUT_CONTROL_USB_SDATA_PIN                     PORTS_BIT_POS_5  
//#define INPUT_CONTROL_USB_MLK_ID                        PORTS_ID_0
//#define INPUT_CONTROL_USB_MLK_PORT_CHANNEL              PORT_CHANNEL_D
//#define INPUT_CONTROL_USB_MLK_PIN                       PORTS_BIT_POS_15  

//------------------------------------------------------------------------------
// Analogue Direct
//------------------------------------------------------------------------------    

#define ANALOGUE_DIRECT_BYPASS_PORT                 PORTS_ID_0
#define ANALOGUE_DIRECT_BYPASS_CHANNEL              PORT_CHANNEL_E
#define ANALOGUE_DIRECT_BYPASS_PIN                  PORTS_BIT_POS_3  

//------------------------------------------------------------------------------
// Input ADC Settings 
//------------------------------------------------------------------------------    

//------------------------------------------------------------------------------
// AK5552 Input ADC Settings 
//------------------------------------------------------------------------------    
    
#define INPUT_ADC_AK5552_RESET_PORT                 PORTS_ID_0
#define INPUT_ADC_AK5552_RESET_CHANNEL              PORT_CHANNEL_E
#define INPUT_ADC_AK5552_RESET_PIN                  PORTS_BIT_POS_8

//! The module (defined in Harmony Drivers->I2C Configuration) used for the DAC
#define INPUT_ADC_AK5552_I2C_MODULE                 DRV_I2C_INDEX_1

//! Read/Write addresses for the I2C ADC device
#define INPUT_ADC_AK5552_I2C_WRITE_ADDRESS          0x22 
#define INPUT_ADC_AK5552_I2C_READ_ADDRESS           0x23  

//------------------------------------------------------------------------------
// AK5552 Input ADC Settings 
//------------------------------------------------------------------------------    
    
#define INPUT_ADC_TIPCM4202_RESET_PORT              PORTS_ID_0
#define INPUT_ADC_TIPCM4202_RESET_CHANNEL           PORT_CHANNEL_E
#define INPUT_ADC_TIPCM4202_RESET_PIN               PORTS_BIT_POS_8

//! The module (defined in Harmony Drivers->I2C Configuration) used for the DAC
#define INPUT_ADC_TIPCM4202_I2C_MODULE              DRV_I2C_INDEX_1

//! Read/Write addresses for the I2C ADC device
#define INPUT_ADC_TIPCM4202_I2C_WRITE_ADDRESS       0x22 
#define INPUT_ADC_TIPCM4202_I2C_READ_ADDRESS        0x23  

//------------------------------------------------------------------------------
// Spdif Rx Settings 
//------------------------------------------------------------------------------    

#define SPDIF_RX_MODEL_SELECT_PORT                      PORTS_ID_0
#define SPDIF_RX_MODEL_SELECT_CHANNEL                   PORT_CHANNEL_B
#define SPDIF_RX_MODEL_SELECT_PIN                       PORTS_BIT_POS_11

//------------------------------------------------------------------------------
// AK4113 Spdif Rx Settings 
//------------------------------------------------------------------------------    

    //! The module (defined in Harmony Drivers->I2C Configuration) used for the DAC
#define SPDIF_RX_AK4113_I2C_MODULE                      DRV_I2C_INDEX_1

//! Read/Write addresses for the I2C Mux device
#define SPDIF_RX_AK4113_I2C_WRITE_ADDRESS               0x20 
#define SPDIF_RX_AK4113_I2C_READ_ADDRESS                0x21  

#define SPDIF_RX_AK4113_RESET_PORTS_ID                  PORTS_ID_0
#define SPDIF_RX_AK4113_RESET_PORT_CHANNEL              PORT_CHANNEL_F
#define SPDIF_RX_AK4113_RESET_PIN                       PORTS_BIT_POS_12
#define SPDIF_RX_AK4113_INT0_ID                         PORTS_ID_0
#define SPDIF_RX_AK4113_INT0_PORT_CHANNEL               PORT_CHANNEL_F
#define SPDIF_RX_AK4113_INT0_PIN                        PORTS_BIT_POS_0  
#define SPDIF_RX_AK4113_INT1_ID                         PORTS_ID_0
#define SPDIF_RX_AK4113_INT1_PORT_CHANNEL               PORT_CHANNEL_F
#define SPDIF_RX_AK4113_INT1_PIN                        PORTS_BIT_POS_4  

// Used for enabling / disabling change notice ports 
#define SPDIF_RX_AK4113_CHANGE_NOTICE_PORT              PORTS_ID_0
#define SPDIF_RX_AK4113_CHANGE_NOTICE_CHANNEL           PORT_CHANNEL_F
#define SPDIF_RX_AK4113_CHANGE_NOTICE_MASK              SYS_PORT_F_CNEN || 0x11

//------------------------------------------------------------------------------
// TI PCM9211 Spdif Rx Settings 
//------------------------------------------------------------------------------    

    //! The module (defined in Harmony Drivers->I2C Configuration) used for the DAC
#define SPDIF_RX_TIPCM9211_I2C_MODULE                      DRV_I2C_INDEX_1

//! Read/Write addresses for the I2C Mux device
#define SPDIF_RX_TIPCM9211_I2C_WRITE_ADDRESS               0x80 
#define SPDIF_RX_TIPCM9211_I2C_READ_ADDRESS                0x81  

#define SPDIF_RX_TIPCM9211_RESET_PORTS_ID                  PORTS_ID_0
#define SPDIF_RX_TIPCM9211_RESET_PORT_CHANNEL              PORT_CHANNEL_F
#define SPDIF_RX_TIPCM9211_RESET_PIN                       PORTS_BIT_POS_12
#define SPDIF_RX_TIPCM9211_INT0_ID                         PORTS_ID_0
#define SPDIF_RX_TIPCM9211_INT0_PORT_CHANNEL               PORT_CHANNEL_F
#define SPDIF_RX_TIPCM9211_INT0_PIN                        PORTS_BIT_POS_0  
#define SPDIF_RX_TIPCM9211_INT1_ID                         PORTS_ID_0
#define SPDIF_RX_TIPCM9211_INT1_PORT_CHANNEL               PORT_CHANNEL_F
#define SPDIF_RX_TIPCM9211_INT1_PIN                        PORTS_BIT_POS_4  

// Used for enabling / disabling change notice ports 
#define SPDIF_RX_TIPCM9211_CHANGE_NOTICE_PORT              PORTS_ID_0
#define SPDIF_RX_TIPCM9211_CHANGE_NOTICE_CHANNEL           PORT_CHANNEL_F
#define SPDIF_RX_TIPCM9211_CHANGE_NOTICE_MASK              SYS_PORT_F_CNEN || 0x11

//------------------------------------------------------------------------------
// Analogue Direct 
//------------------------------------------------------------------------------    
    
/**
 * MUST be set to 1 to enable Pins for setting/reading analogue_direct HW
 */
#define ANALOGUE_DIRECT_PINS_ENABLED    1
    

//------------------------------------------------------------------------------
// INPUT_PHONO_INPUT
//------------------------------------------------------------------------------    
    
#define INPUT_PHONO_INPUT_SELECT_ID                     PORTS_ID_0
#define INPUT_PHONO_INPUT_SELECT_PORT_CHANNEL           PORT_CHANNEL_A
#define INPUT_PHONO_INPUT_SELECT_PIN                    PORTS_BIT_POS_7  

//------------------------------------------------------------------------------
// Logger
//------------------------------------------------------------------------------    

/**
 * Define logger enabled to enable logging, comment out if logging not required
 */
#define LOGGER_ENABLED 1

//! Asynchronous logging of messages (good when logging in interrupts))
#define LOGGER_PROCESS_ASYNC
    
/**
 * Logger message codes
 */
typedef enum
{
    LOGGER_CODE_DEBUG,
    LOGGER_CODE_SET_LOGGER_LEVEL,
    LOGGER_CODE_INITIALISATION,
    LOGGER_CODE_POWERUP,
    LOGGER_CODE_POWERDOWN,
    LOGGER_CODE_STATE_CHANGE,
    LOGGER_CODE_FACTORY_RESET,
    LOGGER_CODE_SYSTEM_FAULT,
    LOGGER_CODE_MENU,
    LOGGER_CODE_STANDBY,
    LOGGER_CODE_EVENTS,            
            
    LOGGER_CODE_ADC,

    LOGGER_CODE_BRIGHTNESS,
    LOGGER_CODE_DARK_MODE,
            
    LOGGER_CODE_SOFTWARE_VERSION,
    LOGGER_CODE_LAST_SHUTDOWN_CODE,
           
    LOGGER_CODE_FLASH_INITIALISATION,
    LOGGER_CODE_FLASH_REQUEST_OVERRUN,

    LOGGER_CODE_I2C,
    LOGGER_CODE_I2C_CALLBACK,
    LOGGER_CODE_I2C_CALLBACK_INVALID_STATE,
    LOGGER_CODE_I2C_CALLBACK_ERROR_STATE,
    LOGGER_CODE_I2C_CALLBACK_INVALID_I2C_STATE,
    LOGGER_CODE_I2C_INVALID_HANDLE,
    LOGGER_CODE_I2C_OPEN_ERROR,
    LOGGER_CODE_I2C_READ_SEND_ADDRESS_ERROR,
    LOGGER_CODE_I2C_READ_DATA_ERROR,        
    LOGGER_CODE_I2C_WRITE_DATA_ERROR,        
    LOGGER_CODE_I2C_REQUEST_QUEUE_ERROR,        
            
    LOGGER_CODE_USART_OPEN_ERROR,
    LOGGER_CODE_USART_READ_ERROR,
    LOGGER_CODE_USART_WRITE_ERROR,
            
    LOGGER_CODE_UPGRADE,
            
    LOGGER_CODE_PINS,
            
    LOGGER_CODE_SETTINGS_CALLBACK,
    LOGGER_CODE_SETTINGS_CALLBACK_INVALID_STATE,
    LOGGER_CODE_SETTINGS_CALLBACK_ERROR_STATE,
    LOGGER_CODE_SETTINGS_CALLBACK_INVALID_I2C_STATE,
    LOGGER_CODE_SETTINGS_INVALID_HANDLE,
    LOGGER_CODE_SETTINGS_OPEN_ERROR,
    LOGGER_CODE_SETTINGS_READ_SEND_ADDRESS_ERROR,
    LOGGER_CODE_SETTINGS_READ_DATA_ERROR,        
    LOGGER_CODE_SETTINGS_READ_DATA_AVAILABLE,
    LOGGER_CODE_SETTINGS_SAVING_SETTINGS,        
    LOGGER_CODE_SETTINGS_SAVE_SETTINGS,        
    LOGGER_CODE_SETTINGS_SETTINGS_LOADED,        
    LOGGER_CODE_SETTINGS_SETTINGS_SAVED,
    LOGGER_CODE_SETTINGS_SETTINGS_SET,
    LOGGER_CODE_SETTINGS_CALLBACK_WARNING,
            
    LOGGER_CODE_POWER,
    LOGGER_CODE_NO_AC_DETECT,

    LOGGER_CODE_VOLUME,
    LOGGER_CODE_VOLUME_MUTE,
    LOGGER_CODE_VOLUME_UNMUTE,
    LOGGER_CODE_VOLUME_CHANGE,
    LOGGER_CODE_BALANCE_CHANGE,
            
    LOGGER_CODE_MUTES_ADDED,
    LOGGER_CODE_MUTES_MUTE,
    LOGGER_CODE_MUTES_UNMUTE,
            
    LOGGER_CODE_INPUT_CONTROL,
    LOGGER_CODE_SPDIF_RX,
    LOGGER_CODE_INPUT_ADC,
    LOGGER_CODE_HDMI_ARC,
    LOGGER_CODE_RS232_HDMI_ARC,
            
    LOGGER_CODE_APP_CONTEXT_REMOTE_MESSAGE,            
            
    LOGGER_CODE_REMOTE_SYSTEM_CODE,

    LOGGER_CODE_TEST_HEADPHONES_CONNECTED,
    LOGGER_CODE_TEST_HEADPHONES_DISCONNECTED,
            
    LOGGER_CODE_TCPIP,
    LOGGER_CODE_TCPIP_IP_ADDRESS,
    LOGGER_CODE_TCPIP_SOCKET_INITILISATION,
    LOGGER_CODE_TCPIP_SOCKET_LISTENING,
    LOGGER_CODE_TCPIP_SOCKET_ERROR,
    LOGGER_CODE_TCPIP_READ_ERROR,
    LOGGER_CODE_TCPIP_SET_STATIC_IP,

    LOGGER_CODE_MACHINE_UUID,
            
    LOGGER_CODE_RGB_LED_TEST,
            
    LOGGER_CODE_AVATAR = 0xC8,              //200
    LOGGER_CODE_DIRAC = 0xC9,               //201
    LOGGER_CODE_AVATAR_MESSAGE = 0xCA,      //202
    LOGGER_CODE_AVATAR_MESSAGE_DATA = 0xCB, //203
            
    LOGGER_CODE_TEST_REMOTE_INCREMENT = 0xD0,
    LOGGER_CODE_TEST_REMOTE_DECREMENT = 0xD1,
    LOGGER_CODE_TEST_BUTTON = 0xD2,
            
    LOGGER_CODE_TEST_RGB_OFF = 0xDE,
    LOGGER_CODE_TEST_RGB_RED = 0xDF,
    LOGGER_CODE_TEST_RGB_GREEN = 0xE0,
    LOGGER_CODE_TEST_RGB_BLUE = 0xE1,
    LOGGER_CODE_TEST_RGB_ORANGE = 0xE2,
    LOGGER_CODE_TEST_RGB_PURPLE = 0xE3,
    LOGGER_CODE_TEST_RGB_WHITE = 0xE4,
            
    LOGGER_CODE_TEST_DISPLAY_TEST = 0xE5,
            
} LOGGER_CODES;

//--------------------------------------------------------------------------
// Lip Sync
//--------------------------------------------------------------------------

//! Define the types of input - note if these are added to/changed then the device
//  needs resetting to clear the settings.
#define LIP_SYNC_OPTIONS \
	X(0MS, 0ms, 0, 0)\
	X(5MS, 5ms, 5, 1)\
	X(10MS, 10ms, 10, 2)\
	X(15MS, 15ms, 15, 3)\
	X(20MS, 20ms, 20, 4)\
	X(25MS, 25ms, 25, 5)\
	X(30MS, 30ms, 30, 6)\
	X(35MS, 35ms, 35, 7)\
	X(40MS, 40ms, 40, 8)\
	X(45MS, 45ms, 45, 9)\
	X(50MS, 50ms, 50, 10)\
	X(55MS, 55ms, 55, 11)\
	X(60MS, 60ms, 60, 12)\
	X(65MS, 65ms, 65, 13)\
	X(70MS, 70ms, 70, 14)\
	X(75MS, 75ms, 75, 15)\
	X(80MS, 80ms, 80, 16)\
	X(85MS, 85ms, 85, 17)\
	X(90MS, 90ms, 90, 18)\
	X(95MS, 95ms, 95, 19)\
	X(100MS, 100ms, 100, 20)\
	X(105MS, 105ms, 105, 21)\
	X(110MS, 110ms, 110, 22)\
	X(115MS, 115ms, 115, 23)\
	X(120MS, 120ms, 120, 24)\
	X(125MS, 125ms, 125, 25)\
	X(130MS, 130ms, 130, 26)\
	X(135MS, 135ms, 135, 27)\
	X(140MS, 140ms, 140, 28)\
	X(145MS, 145ms, 145, 29)\
	X(150MS, 150ms, 150, 30)\
	X(155MS, 155ms, 155, 31)\
	X(160MS, 160ms, 160, 32)\
	X(165MS, 165ms, 165, 33)\
	X(170MS, 170ms, 170, 34)\
	X(175MS, 175ms, 175, 35)\
	X(180MS, 180ms, 180, 36)\
	X(185MS, 185ms, 185, 37)\
	X(190MS, 190ms, 190, 38)\
	X(195MS, 195ms, 195, 39)\
	X(200MS, 200ms, 200, 40)\

// Now use the input data to define the enum
#define X(a, b, c, d) LIP_SYNC_OPTION_##a,
	typedef enum {LIP_SYNC_OPTIONS LIP_SYNC_OPTIONS_MAX} LIP_SYNC_OPTION;
#undef X
   
// Now use the input data to define the enum
#define X(a, b, c, d) d,
    static int LIP_SYNC_IDX[] = { LIP_SYNC_OPTIONS };
#undef X
   
// Now use the input data to define the enum
#define X(a, b, c, d) c,
    static int LIP_SYNC_VALUES[] = { LIP_SYNC_OPTIONS };
#undef X
    
// Redefine the macro and use to create an array of strings
#define X(a, b, c, d) #b,
	static char *LIP_SYNC_STRINGS[] = { LIP_SYNC_OPTIONS };
#undef X
    
//------------------------------------------------------------------------------
// Mutes
//------------------------------------------------------------------------------    

/**
 * Define PSU's in the system (SA30 settings)
 * Note: Enabling/Disable will use the order that these are specified in.
 * a: Enum name b: Port c: Channel d: pin e: active high/low f: description
 */
#define MUTE_ITEMS \
    X(PRE_OUT, PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_6, false, Pre Out)\
    X(HEADPHONE, PORTS_ID_0, PORT_CHANNEL_E, PORTS_BIT_POS_1, false, Headphones)\
    X(SPEAKER, PORTS_ID_0, PORT_CHANNEL_G, PORTS_BIT_POS_1, false, Speaker)\
    X(VOLUME, PORTS_ID_0, PORT_CHANNEL_G, PORTS_BIT_POS_13, false, Volume)\
        
/**
 * MUST be set to 1 to enable Pins for setting volume
 */
#define MUTES_PINS_ENABLED                      1 

//------------------------------------------------------------------------------
// Output Control
//------------------------------------------------------------------------------    

// Output Control
// SA30 Settings
#define OUTPUT_HEADPHONES_PORTS_ID              PORTS_ID_0
#define OUTPUT_HEADPHONES_PORT_CHANNEL          PORT_CHANNEL_D       
#define OUTPUT_HEADPHONES_PIN                   PORTS_BIT_POS_13
#define OUTPUT_DC_OFFSET_PORTS_ID               PORTS_ID_0
#define OUTPUT_DC_OFFSET_PORT_CHANNEL           PORT_CHANNEL_C       
#define OUTPUT_DC_OFFSET_PIN                    PORTS_BIT_POS_2
#define OUTPUT_SHORT_CIRCUIT_PORTS_ID           PORTS_ID_0
#define OUTPUT_SHORT_CIRCUIT_PORT_CHANNEL       PORT_CHANNEL_C       
#define OUTPUT_SHORT_CIRCUIT_PIN                PORTS_BIT_POS_1
         
/**
 * MUST be set to 1 to enable Pins for setting/reading output_control
 */
#define OUTPUT_CONTROL_PINS_ENABLED             1 

//------------------------------------------------------------------------------
// Power
//------------------------------------------------------------------------------    
  
/**
 * Power flags - used to denote the type of PSU and its functionality
 */
typedef enum
{
    //! Treated as a normal power item, no special processig
    POWER_FLAGS_NONE = 0x00,
            
    //! Only one item should have this - used to determine when Power state == On
    POWER_FLAGS_MAIN = 0x01,
            
    //! Determines whether power is disabled during standby based on network settings
    POWER_FLAGS_NETWORK = 0x02,
            
    //! Not used at the moment
    POWER_FLAGS_DIGITAL = 0x04,
            
    //! Any item with this flag will NOT be enabled by power_enable, and must be done
    // Manually using power_enable_by_index
    POWER_FLAGS_DELAYED = 0x08,
            
    //! Any item with this flag will NOT be disabled by power_disable, and must be done
    // Manually using power_disable_by_index
    POWER_FLAGS_ALWAYS_ON = 0x10,
} POWER_FLAGS_ENUM;

/**
 * Define PSU's in the system (SA30 settings)
 * Note: Enabling/Disable will use the order that these are specified in.
 * a: Enum b:Port c: Channel d:Pin d: Type
 */
#define POWER_ITEMS \
    X(PSU, PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_13, POWER_FLAGS_MAIN)\
    X(STREAMER_PSU, PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_1, POWER_FLAGS_NETWORK)\
    X(ARC_IC_PSU, PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_15, POWER_FLAGS_NETWORK)\
    X(DIGITAL_PSU, PORTS_ID_0, PORT_CHANNEL_E, PORTS_BIT_POS_7, POWER_FLAGS_DIGITAL | POWER_FLAGS_NETWORK)\

/**
 * AC Detect Pins (SA30 settings)
 */
#define POWER_AC_DETECT_PORTS_ID        PORTS_ID_0      
#define POWER_AC_DETECT_PORT_CHANNEL    PORT_CHANNEL_E
#define POWER_AC_DETECT_PIN             PORTS_BIT_POS_0
        
/**
 * MUST be set to 1 to enable Pins for setting/reading power
 */
#define POWER_PINS_ENABLED                  1 
   
//------------------------------------------------------------------------------
// RC5 Remote
//------------------------------------------------------------------------------    
//--------------------------------------------------------------------------
// Remote command mapping 
// This will need changing dependent upon the remote
// Then update the code in app_context.process_default_remote etc..
// Currently CR108    
//--------------------------------------------------------------------------

// SA30 Remote codes
#define REMOTE_COMMAND \
	X(POWER, =12, Power)\
    X(POWER_ON, =123, Pwr On)\
    X(POWER_OFF, =124, Pwr Off)\
	X(ZERO, =0, 0)\
	X(ONE, =1, 1)\
	X(TWO, =2, 3)\
	X(THREE, =3, 3)\
	X(FOUR, =4, 4)\
	X(FIVE, =5, 5)\
	X(SIX, =6, 6)\
	X(SEVEN, =7, 7)\
	X(EIGHT, =8, 8)\
	X(NINE, =9, 9)\
    X(BALANCE, =37, Balance)\
    X(BALANCE_LEFT, =38, Bal L)\
    X(BALANCE_RIGHT, =40, Bal R)\
    X(INFO, =55, Info)\
    X(REWIND, =121, Rwnd)\
    X(FAST_FORWARD, =52, FF)\
    X(SKIP_BACK, =33, Back)\
    X(SKIP_FORWARD, =11, Fwd)\
    X(STOP, =54, Stop)\
    X(PLAY, =53, Play)\
    X(PAUSE, =48, Pause)\
    X(MENU, =82, Menu)\
    X(UP, =81, Up)\
    X(LEFT, =86, Left)\
    X(OK, =87, OK)\
    X(RIGHT, =85, Right)\
    X(DOWN, =80, Down)\
    X(BACK, =51, Back)\
    X(HOME, =43, Home)\
    X(DIRAC, =30, EQ)\
    X(MUTE, =13, Mute)\
    X(MUTE_ON, =26, Mute On)\
    X(MUTE_OFF, =120, Mute Off)\
    X(VOLUME_UP, =16, Vol Up)\
    X(VOLUME_DOWN, =17, Vol Dn)\
    X(DISPLAY, =59, Disp)\
    X(DISPLAY_OFF, =31, Disp On)\
    X(DISPLAY_L1, =34, Disp L1)\
    X(DISPLAY_L2, =35, Disp L2)\
    X(ANALOG_DIRECT, =10, An Dir)\
    X(ANALOG_DIRECT_ON, =78, AD On)\
    X(ANALOG_DIRECT_OFF, =79, AD Off)\
 	X(PHONO, =117, Phono)\
	X(AUX, =99, AUX)\
	X(NET, =92, NET)\
	X(USB, =93, USB)\
	X(AV, =94, AV)\
   	X(SAT, =27, SAT)\
	X(PVR, =96, PVR)\
	X(GAME, =97, GAME)\
    X(BD, =98, BD)\
    X(CD, =118, CD)\
    X(STB, =100, STB)\
    X(PCUSB, =125, PCUSB)\
    X(RANDOM, =76, Random)\
    X(REPEAT, =49, Repeat)\
    X(SHUFFLE, =76, Shuffle)\
    X(RANDOM_REPEAT, =47, Random Repeat)\
    X(RANDOM_REPEAT_MODE, =20, Random Repeat)\

// Now use the input data to define the enum
#define X(a, b, c) REMOTE_CMD_INPUT_##a b,
	typedef enum {REMOTE_COMMAND} REMOTE_CMD;
#undef X
   
// Now use the input data to define the enum
#define X(a, b, c) REMOTE_CMD_INPUT_##a,
    static int REMOTE_CMD_VALUES[] = { REMOTE_COMMAND };
#undef X
    
// Redefine the macro and use to create an array of strings
#define X(a, b, c) #c,
	static char *REMOTE_CMD_INPUT_STRINGS[] = { REMOTE_COMMAND };
#undef X

// Enable Change Notification on ports, then use the Harmony Pin Manager to
// Set it up as a CN port
#define IR_PORTS_ID                     PORTS_ID_0
#define IR_PORT_CHANNEL                 PORT_CHANNEL_D
#define IR_PIN                          PORTS_BIT_POS_11

//------------------------------------------------------------------------------
// RGB LED
//------------------------------------------------------------------------------  

// LED Ports all defined as Out/Low/Digital 
// Note settings are for AMP    
#define RGB_LED_BLUE_PORTS_ID           PORTS_ID_0
#define RGB_LED_BLUE_PORT_CHANNEL       PORT_CHANNEL_D
#define RGB_LED_BLUE_PIN                PORTS_BIT_POS_4
#define RGB_LED_RED_PORTS_ID            PORTS_ID_0
#define RGB_LED_RED_PORT_CHANNEL        PORT_CHANNEL_D 
#define RGB_LED_RED_PIN                 PORTS_BIT_POS_0 
#define RGB_LED_GREEN_PORTS_ID          PORTS_ID_0
#define RGB_LED_GREEN_PORT_CHANNEL      PORT_CHANNEL_D
#define RGB_LED_GREEN_PIN               PORTS_BIT_POS_2
        
//------------------------------------------------------------------------------
// RS232
//------------------------------------------------------------------------------    
 
//! This is used for CN detection during standby mode.   It needs the Port 
//  defining as a CN port in Harmony, and cn_detect_interrupt_handler() adding to
//  the interrupt handler.    
//  Note this is an extra port in addition to the RX/TX.  We believe this is 
//  neccessary because the RX/TX ports are associated with the UART peripheral in
//  harmony, and the time it takes RS232 to wind up from standby means that they 
//  cannot be used....   
// Check this wakes up once the PIN is connected on the board    
#define RS232_CHANGE_NOTICE_PORTS_ID        PORTS_ID_0
#define RS232_CHANGE_NOTICE_PORT_CHANNEL    PORT_CHANNEL_G  
#define RS232_CHANGE_NOTICE_PIN             PORTS_BIT_POS_15 
       
//------------------------------------------------------------------------------
// RS232 HDMI ARC
//------------------------------------------------------------------------------    

#if APP_FEATURE_HDMI_ARC == APP_FEATURE_ON  
    
// This uses the asynchronous method of transferring the data as we were having 
// trouble sending album art - it was keeling over after it send a few images 
// Note the number of buffer items needs to match the number of Transmit buffers
// defined in Harmony    
#define RS232_HDMI_ARC_RX_QUEUE_SIZE     DRV_USART_RCV_QUEUE_SIZE_IDX1 //- This should be around the maximum sixze of a message that can be sent (15))
#define RS232_HDMI_ARC_RX_BUFFER_SIZE    1  // This has to be 1 otherwise Harmony queues up tha data recieved and only generates the callback when the buffer is fuill..

#define RS232_HDMI_ARC_TX_QUEUE_SIZE     DRV_USART_XMIT_QUEUE_SIZE_IDX1
#define RS232_HDMI_ARC_TX_BUFFER_SIZE    512 //100 //TODO This is the maximum size a display message can be..
    
#endif    

//------------------------------------------------------------------------------
// Serial Numbers
//------------------------------------------------------------------------------    

#define SERIAL_NUMBER_PREFIX_US         "HU0015"    
#define SERIAL_NUMBER_PREFIX_EU         "HU0016"
#define SERIAL_NUMBER_PREFIX_CHINA      "HU0017"    
    
//------------------------------------------------------------------------------
// Shutdown code (part of System Fault)
//------------------------------------------------------------------------------    

#define SHUTDOWN_CODE_HISTORY_LENGTH    10   
    
//------------------------------------------------------------------------------
// Settings
//------------------------------------------------------------------------------    

//! Define the length required to hold the model name - this is used to identify 
//  whether the settings being read are for the correct model
//  Note: This should include a null at the end. e.g. SA20\0
#define SETTINGS_MODEL_STRING_LENGTH    SOFTWARE_MODEL_STRING_LENGTH

#define SETTINGS_STORAGE_NAME           "SETTINGS"

/**
 * Set the client Id to use the Flash Memory client Id
 */
#define SETTINGS_CLIENT_ID              FLASH_MEMORY_CLIENT_SETTINGS

/**
 * An enum of settings
 */
#define SETTINGS \
	X(SOFTWARE_VERSION_MAJOR, sizeof(uint8_t))\
	X(SOFTWARE_VERSION_MINOR, sizeof(uint8_t))\
	X(CURRENT_VOLUME, sizeof(uint8_t))\
	X(CURRENT_BALANCE, sizeof(uint8_t))\
	X(MAX_ON_VOLUME, sizeof(uint8_t))\
	X(MAXIMUM_VOLUME, sizeof(uint8_t))\
	X(MAXIMUM_STREAMING_VOLUME, sizeof(uint8_t))\
	X(ACTIVE_INPUT, sizeof(uint8_t))\
	X(PREVIOUS_INPUT, sizeof(uint8_t))\
	X(PROCESSOR_MODE_INPUT, sizeof(uint8_t))\
	X(PROCESSOR_MODE_VOLUME, sizeof(uint8_t))\
	X(FILTER_TYPE, sizeof(uint32_t))\
	X(AUTOSHUTDOWN_TIMEOUT, sizeof(uint16_t))\
	X(NETWORK_STANDBY, sizeof(bool))\
	X(RS232_STANDBY, sizeof(bool))\
	X(LOGGER_TYPE, sizeof(uint32_t))\
	X(SHUTDOWN_CODE, sizeof(uint32_t))\
	X(SHUTDOWN_CODE_HISTORY, SHUTDOWN_CODE_HISTORY_LENGTH * sizeof(uint32_t))\
	X(BRIGHTNESS, sizeof(uint8_t))\
	X(REMOTE_SYSTEM_CODE, sizeof(uint8_t))\
	X(MACHINE_UUID, 16)\
	X(HEADPHONE_OVERRIDE, sizeof(bool))\
	X(ANALOGUE_DIRECT, sizeof(uint8_t) * INPUT_CONTROL_NUM_SETTINGS)\
	X(INPUT_ENABLED, sizeof(uint8_t) * INPUT_CONTROL_NUM_SETTINGS)\
	X(MM_MC, sizeof(uint32_t))\
	X(DIRAC_CURVE_ENABLED, sizeof(uint8_t) * DIRAC_MAX_EQ_CURVES)\
	X(DIRAC_CURVE, sizeof(uint8_t) * INPUT_CONTROL_NUM_SETTINGS)\
	X(USB_RECORD, sizeof(bool))\
	X(CONTROL4_SDDP_ACTIVE, sizeof(bool))\
    X(CRESTRON_DISCOVERY_ACTIVE, sizeof(bool))\
    X(AVATAR_UPGRADE_OPTION, sizeof(uint32_t))\
    X(UPGRADE_STATUS, sizeof(bool))\
    X(UPGRADE_SLEEP_ON_RESTART, sizeof(bool))\
    X(DARK_MODE, sizeof(bool))\
	X(UPGRADE_SOFTWARE_VERSION_MAJOR, sizeof(uint8_t))\
	X(UPGRADE_SOFTWARE_VERSION_MINOR, sizeof(uint8_t))\
	X(CHECK_CHANNEL_STATUS, sizeof(bool))\
	X(HDMI_AUDIO_CONTROL, sizeof(bool))\
	X(HDMI_POWER_CONTROL, sizeof(bool))\
	X(SHORT_CIRCUIT_SENSE, sizeof(bool))\
	X(MQA_DECODER_LEVEL, sizeof(uint8_t))\
	X(LIP_SYNC, sizeof(uint8_t) * INPUT_CONTROL_NUM_SETTINGS)\
	X(MUTE_ON_STARTUP, sizeof(bool))\

// Now use the setting data to define the enum
#define X(a, b) SETTING_##a,
	typedef enum {SETTINGS SETTING_MAX} SETTINGS_INDEX;
#undef X
    
// Redefine the macro and use to create an array of strings
#define X(a, b) b,
	static size_t SETTINGS_LENGTH[] = { SETTINGS };
#undef X
    
// Redefine the macro and use to create an array of strings
#define X(a, b) #a,
	static char *SETTING_STRINGS[] = { SETTINGS };
#undef X

/**
 * Define the number of settings to be managed - this is required for use with 
 * settings.h/c
 */
#define SETTINGS_NUM_SETTINGS           SETTING_MAX
    
//------------------------------------------------------------------------------
// Short Circuit detection
//------------------------------------------------------------------------------    

// Used in output_control.c to see if we're checking for shortcircuit detection
#define USE_SHORT_CIRCUIT_SENSE_SETTING    
    
//------------------------------------------------------------------------------
// System Faults
//------------------------------------------------------------------------------    

//! The temperature at which the amp will shut down.    
#define APP_MAX_WORKING_TEMPERATURE     65//70   //TODO Put put back to 80 after testing
   
#define FAULT_CODES \
    X(NO_FAULT, "No faults")\
    X(DC_OFFSET, "DC Offset")\
    X(SHORT_CIRCUIT, "Short Circuit")\
    X(OUTPUT_OVER_TEMP, "Output Over Temp")\
    X(LIFTER_OVER_TEMP, "Lifter Over Temp")\
  
    //! Define a menu types enumeration
    typedef enum 
    {
#define X(a, b) SYSTEM_FAULT_CODE_##a,
	FAULT_CODES SYSTEM_FAULT_CODE_MAX
#undef X
    } SYSTEM_FAULT_CODE;
    
    //! Declare a menu names array - This is passed to menu_initialise()
#define X(a, b) b,
	static char *system_fault_codes[] = { FAULT_CODES };
#undef X    
    
//------------------------------------------------------------------------------
// Timers
//------------------------------------------------------------------------------    

/** Timer reset type
 */
typedef enum
{
    TIMERS_RESET_NONE = 0,              
    TIMERS_RESET_INACTIVE = 1,
    TIMERS_RESET_IMMEDIATE = 2,
 } TIMERS_RESET;

 /**
  * Types of timer duration 
  */
 typedef enum
 {
    TIMERS_HANDLE_1MS,
    TIMERS_HANDLE_100US,
 } TIMERS_HANDLE;

//! Defines the time that the menus are displayed after the user has stopped editing
//  unless they specifically return using the HOME button 
#define TIMER_MENU_INACTIVE_TIMEOUT_MS                  30000 // ms 
 
//! Determines the length of time the BALANCE (BALANCE_NO_NAVIGATION) menu is 
//  displayed (after any activity) when invoked by the front panel button 
#define TIMER_MENU_BALANCE_INACTIVE_TIMEOUT_MS          5000 // ms 
 
//! Determines the length of time the AVATAR Upgrade Complete menu is 
//  displayed (after any activity) when invoked by the front panel button 
#define TIMER_MENU_AVATAR_UPGRADE_INACTIVE_TIMEOUT_MS   3000 // ms 
 
//! Determines the length of time the DIRAC (DIRAC_NO_NAVIGATION) menu is 
//  displayed (after any activity) when invoked by the front panel button 
#define TIMER_MENU_DIRAC_INACTIVE_TIMEOUT_MS            3000 // ms 
 
//! Determines the length of time the Analogue Direct (ANALOGUE_DIRECT_NO_NAVIGATION) menu is 
//  displayed (after any activity) when invoked by the front panel button 
#define TIMER_MENU_ANALOGUE_DIRECT_INACTIVE_TIMEOUT_MS  3000 // ms 
 
//! Determines the length of time the Phono Input (PHONO_INPUT_NO_NAVIGATION) menu is 
//  displayed (after any activity) when invoked by the front panel button 
#define TIMER_MENU_PHONO_INPUT_INACTIVE_TIMEOUT_MS      3000 // ms 

 //! Defines the time between the initial remote key and its first repeat, so if
 //  its released within this it is not considered a repeat.
#define TIMER_REMOTE_REPEAT_INITIAL_TIMEOUT_MS          600 // ms - 
 
 //! Timeout for when a PlayerStatus STop message is received and when we actually switch channels
#define TIMER_AVATAR_STOPPED_CHECK_TIMEOUT_MS           5000
/**
 * Timer X code 1: Name, 2: Timeout, 3: Reset type, 4: Base duration
 */
#define TIMER_ITEMS \
    X(TEST, 5000, TIMERS_RESET_IMMEDIATE, TIMERS_HANDLE_1MS)\
    X(ADC, 50, TIMERS_RESET_IMMEDIATE, TIMERS_HANDLE_1MS)\
    X(ENCODER, 5, TIMERS_RESET_IMMEDIATE, TIMERS_HANDLE_100US)\
    X(SETTINGS, 5000, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(RGB_LED, 500, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(RGB_PWM, 1, TIMERS_RESET_IMMEDIATE, TIMERS_HANDLE_100US)\
    X(RGB_LED_TEST, 2000, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(AUTO_SHUTDOWN, 30000, TIMERS_RESET_IMMEDIATE, TIMERS_HANDLE_1MS)\
    X(POWER_CYCLE, 4000, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(VOLUME_RAMP, 20, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(VOLUME_KEEP_ALIVE, 2000, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(BRIGHTNESS, 5000, TIMERS_RESET_IMMEDIATE, TIMERS_HANDLE_1MS)\
    X(SYSTEM_FAULT, 2000, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(VFD_FLASH, 500, TIMERS_RESET_IMMEDIATE, TIMERS_HANDLE_1MS)\
    X(DISPLAY_TEST, 10, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(DISPLAY_TEST_TIMEOUT, 90000, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(BIAS_TEST, 1000, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(MENU_SELECT, 2000, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(MENU_INACTIVE, TIMER_MENU_INACTIVE_TIMEOUT_MS, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(REMOTE_CLOCK, 20000, TIMERS_RESET_NONE, TIMERS_HANDLE_100US)\
    X(REMOTE_REPEAT, TIMER_REMOTE_REPEAT_INITIAL_TIMEOUT_MS, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(REMOTE_RESET_REPEAT, 800, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(CN_DETECT, 100, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(DC_OFFSET, 20, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(SHORT_CIRCUIT, 20, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(DPLL_LOCK , 500, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(INPUT_SWITCH, 500, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(INPUT_PREVIEW, 1000, TIMERS_RESET_INACTIVE, TIMERS_HANDLE_1MS)\
    X(INPUT_CONTROL_WAIT, 500, TIMERS_RESET_INACTIVE, TIMERS_HANDLE_1MS)\
    X(DISPLAY_SCROLL, 100, TIMERS_RESET_INACTIVE, TIMERS_HANDLE_1MS)\
    X(AVATAR_VOLUME_CHANGE, 1000, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(SPDIF_RX_POLL, 1000, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(RESTART_PAUSE, 5000, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(SPDIF_RX_SAMPLE_RATE_SMOOTHING, 500, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(DIRAC_CURVE_NAME_CHECK, 1000, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(AVATAR_SHUTDOWN, 1000, TIMERS_RESET_NONE, TIMERS_HANDLE_1MS)\
    X(HDMI_ARC_HANDSHAKE, 60000, TIMERS_RESET_INACTIVE, TIMERS_HANDLE_1MS)\
    X(AVATAR_STOPPED_CHECK, TIMER_AVATAR_STOPPED_CHECK_TIMEOUT_MS, TIMERS_RESET_INACTIVE, TIMERS_HANDLE_1MS)\
    X(LIP_SYNC_SEND, 1000, TIMERS_RESET_INACTIVE, TIMERS_HANDLE_1MS)\
    X(BROWSING_SOURCE_CHECK, 10000, TIMERS_RESET_INACTIVE, TIMERS_HANDLE_1MS)\
 
///**
// * Defines the number of timers to be managed
// */
//#define TIMERS_NUM_TIMERS TIMER_INDEX_MAX

//! Used by the remote to work out how many cycles have passed in-between receiving data
#define TIMER_TIMEOUT_REMOTE_CLOCK              TIMER_TIMEOUT_VALUES[TIMER_INDEX_REMOTE_CLOCK]  
    
//------------------------------------------------------------------------------
// Upgrade
//------------------------------------------------------------------------------    

#define USE_PAGE_ERASE          1
#define USE_QUAD_WORD_WRITE     1

    //! Calculated from Linker file / memory map
#define PROGRAM_FLASH_END_ADDRESS APP_LAUNCH_ADDRESS + 0x200000 - (0x20000 + 0x480) - 1 // (0x9D000000+BMXPFMSZ-1)

/* APP_FLASH_BASE_ADDRESS and APP_FLASH_END_ADDRESS reserves program Flash for the application*/ 
/* Rule: 
 		1)The memory regions kseg0_program_mem, kseg0_boot_mem, exception_mem and 
 		kseg1_boot_mem of the application linker script must fall with in APP_FLASH_BASE_ADDRESS 
 		and APP_FLASH_END_ADDRESS
 		 
 		2)The base address and end address must align on  4K address boundary */
 		
#define APP_FLASH_BASE_ADDRESS          APP_LAUNCH_ADDRESS 
#define APP_FLASH_END_ADDRESS           PROGRAM_FLASH_END_ADDRESS    
    
//! A unique identifier for the module
#define UPGRADE_METADATA_CLIENT_ID      FLASH_MEMORY_CLIENT_UPGRADE   

//! Name associated with storage
#define UPGRADE_STORAGE_NAME            "UPGRADE_INFO"
     
//------------------------------------------------------------------------------
// VFD
//------------------------------------------------------------------------------    
                
/**
 * Set to 1 if VFD functionality is to be used
 */
#define VFD_ENABLED                         1

#if VFD_ENABLED
// Vfd
// Note settings are for SA30
// Note - when rebuilding the SA30 project I needed to NOT map any of the channels 
// (other than making them Out/Low) - so no SS5(out) etc...)
#define VFD_CS_PORTS_ID             PORTS_ID_0      // Output Low 
#define VFD_CS_PORT_CHANNEL         PORT_CHANNEL_B  
#define VFD_CS_PIN                  PORTS_BIT_POS_8
#define VFD_MOSI_PORTS_ID           PORTS_ID_0      // Output Low
#define VFD_MOSI_PORT_CHANNEL       PORT_CHANNEL_B
#define VFD_MOSI_PIN                PORTS_BIT_POS_9
#define VFD_SCLK_PORTS_ID           PORTS_ID_0      // Output Low 
#define VFD_SCLK_PORT_CHANNEL       PORT_CHANNEL_F
#define VFD_SCLK_PIN                PORTS_BIT_POS_13
#define VFD_RESET_PORTS_ID          PORTS_ID_0      // Output Low
#define VFD_RESET_PORT_CHANNEL      PORT_CHANNEL_D
#define VFD_RESET_PIN               PORTS_BIT_POS_5

/**
 * MUST be set to 1 to enable Pins for setting volume
 */
#define VFD_PINS_ENABLED                1 

#endif
    
//------------------------------------------------------------------------------
// Volume
//------------------------------------------------------------------------------    

#define VOLUME_MAX                                  99
#define VOLUME_FIXED_MAX                            97 // BUG 3381
#define VOLUME_STREAMING_MAX                        99 // BUG 3373
#define VOLUME_DEFAULT_VOLUME						30 // BUG 3373
#define VOLUME_DEFAULT_BALANCE						50
 
//#define VOLUME_HAS_SETTING_FIXED_VOLUME 
 
// Volume
// Note settings are for SA30
#define VOLUME_CS_PORTS_ID              PORTS_ID_0      
#define VOLUME_CS_PORT_CHANNEL          PORT_CHANNEL_E
#define VOLUME_CS_PIN                   PORTS_BIT_POS_2
#define VOLUME_MOSI_PORTS_ID            PORTS_ID_0      
#define VOLUME_MOSI_PORT_CHANNEL        PORT_CHANNEL_G
#define VOLUME_MOSI_PIN                 PORTS_BIT_POS_14
#define VOLUME_SCLK_PORTS_ID            PORTS_ID_0      
#define VOLUME_SCLK_PORT_CHANNEL        PORT_CHANNEL_G
#define VOLUME_SCLK_PIN                 PORTS_BIT_POS_12
#ifndef VOLUME_MUTE_PORTS_ID // This may be define in Mutes
#define VOLUME_MUTE_PORTS_ID            PORTS_ID_0      
#define VOLUME_MUTE_PORT_CHANNEL        PORT_CHANNEL_G
#define VOLUME_MUTE_PIN                 PORTS_BIT_POS_13
#endif
        
/**
 * MUST be set to 1 to enable Pins for setting volume
 */
#define VOLUME_PINS_ENABLED             1 
    
//------------------------------------------------------------------------------
// 
//------------------------------------------------------------------------------    

#ifdef	__cplusplus
}
#endif

#endif	/* APP_CONFIG_H */

