/* 
 * File:   app_context.h
 * Author: chrism
 *
 * Created on 26 October 2017, 11:45
 */

#ifndef APP_CONTEXT_H
#define	APP_CONTEXT_H

#include "utilities/menu.h"
#include "utilities/rgb_led.h"
#include "utilities/structured_message.h"

#ifdef	__cplusplus
extern "C" {
#endif

//------------------------------------------------------------------------------
// Constants
//-------------------------------------------------------------------------------

    typedef enum
    {
        APP_STATE_INIT = 0,
        APP_STATE_LOADING_SETTINGS,
        APP_STATE_READ_SETTINGS,
        APP_STATE_WAIT_FOR_POWER,
        APP_STATE_POWERUP,
        APP_STATE_COMPLETE_INITIALISE,
//        APP_STATE_WAIT_FOR_INPUT_CONTROL,
//        APP_STATE_WAIT_FOR_AVATAR,
//        APP_STATE_HANDSHAKE_AVATAR,
        APP_STATE_WAIT_READY,
        APP_STATE_READY,
        APP_STATE_RUNNING,
        APP_STATE_POWERDOWN,
        APP_STATE_WAIT_FOR_AVATAR_POWERDDOWN,
        APP_STATE_WAIT_FOR_AVATAR_ROON_POWERDDOWN,
        APP_STATE_PRE_POWEREDDOWN,
        APP_STATE_POWEREDDOWN,
        APP_STATE_WAKEUP,
        APP_STATE_WOKENUP,
        APP_STATE_ENTERING_FAULT,
        APP_STATE_FAULT,
        APP_STATE_FAULT_WAIT,
        APP_STATE_ENTER_BIAS_CHECK,
        APP_STATE_BIAS_CHECK,
        APP_STATE_EXIT_BIAS_CHECK,
        APP_STATE_PRE_RESTART,
        APP_STATE_RESTART,
    } APP_STATE;

//------------------------------------------------------------------------------
// Interface
//-------------------------------------------------------------------------------

    /**
     * Resets the running state on Wake-Up
     */
    void app_context_clear_running();
    
    /**
     * Returns the current application state
     * @return 
     */
    APP_STATE app_context_get_app_state();
    
    /**
     * Gets the current mode
     */
    APP_CONTEXT_MODE app_context_get_mode();
    
    /**
     * Initialise the application context
     */
    void app_context_initialise();
    
    /**
     * Can be used to determine whether all the initialisation is complete.  This
     * is different to checking the runtime state as its possible for the state 
     * to be RUNNING while initialisation is still finishing off (INPUT_CONTROL)
     * @return 
     */
    bool app_context_is_running();
    
    /**
     * Used to identify whether the code is entering the running state for the first time
     * Note: Can only be called once as this will set the state to NOT_LOADING
     * @return 
     */
    bool app_context_loading();
    
    /**
     * Used to set the given module into a stopped state - used by app_context_is_running()
     * @param module
     */
    void app_context_module_reset(APP_MODULE module);
    
    /**
     * Used to determine that the given module is running - used by app_context_is_running()
     * @param module
     */
    void app_context_module_running(APP_MODULE module);
    
    /**
     * Called prior to message processing
     * @param messaage - The message to be processed
     */
    void app_context_pre_process_message(STRUCTURED_MESSAGE* message);
    
    /**
     * Sets the current application state
     * @param state - The application state to change to 
     */
    APP_STATE app_context_set_app_state(APP_STATE state);
    
    /**
     * This should be used to directly jump to a menu when not using the standard 
     * menu navigation functions (left/right/up/down)
     * @param menu_id
     */
    void app_context_set_menu(MENU_ID menu_id);

    /**
     * Sets the current mode
     */
    void app_context_set_mode(APP_CONTEXT_MODE mode);
     
#ifdef	__cplusplus
}
#endif

#endif	/* APP_CONTEXT_H */

