/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    app_initialise.h

  @Summary
    Brief description of the file.

  @Description
    Contains application level initialisation functions called in app.c
 */
/* ************************************************************************** */

#ifndef _APP_INITIALISE_H    /* Guard against multiple inclusion */
#define _APP_INITIALISE_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/logger/logger.h"
#include "utilities/encoder_multi.h"
#include "utilities/hdmi_arc.h"
#include "utilities/pins.h"
#include "utilities/rs232.h"
#include "utilities/settings.h"
#include "utilities/structured_message.h"
#include "utilities/timers.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************
 
    /**
     * 
     */
//    void configure_input_control_digital();
    
    /**
     * Initialises ADC
     */
    void initialise_adc();
 
    /**
     * Initialises Avatar
     */
    void initialise_avatar();
    
    /**
     * Sets up the display for the type of display used
     */
    void initialise_display();
 
    /**
     * Sets up the encoder
     */
    void initialise_encoder();
    
    /**
     * Initialise flash memory and allocate space for clients
     */
    void initialise_flash_memory();
    
    /** 
     * Set up i2c
     */
    void initialise_i2c();    
    
    /** 
     * Set up input controls
     */
    void initialise_input_control();    
    
    /**
     * Initialise logger functions
     * @param message
     */
    void initialise_logger(LOGGER_CALLBACK callback);

    /**
     * Setup and enable Power
     */
    void initialise_mutes();

    /**
     * Sets up TCPIP/UDP/UPNP/Control4 etc
     */
    void initialise_network(STRUCTURED_MESSAGE_CALLBACK callback);
    
    /**
     * Setup pins for standby
     */
    void initialise_pins();
    
    /**
     * Setup and enable Power
     */
    void initialise_power();
    
    /**
     * Setup RGB LED 
     */
    void initialise_rgb_led();
    
    /**
     * Initialise USART
     * @param callback
     */
    void initialise_rs232();

    /**
     * Initialise settings module and define individual settings
     */
    void initialise_settings(SETTINGS_INITIALISE_CALLBACK callback);
    
    /**
     * This method is where default application settings should be set.  This is called 
     * the first time the eeprom is accessed for the settings in  order to set any 
     * default values so that they are stored.
     */
    void initialise_settings_defaults();

    /**
     * Checks that the stored software version matches this one and performs any 
     * upgrade specific work
     */
    void initialise_software_version();

    /**
     * sets up special switch data
     */
    void initialise_special_switches();

    /**
     * Sets up TCP/IP
     */
    void initialise_tcpip(STRUCTURED_MESSAGE_CALLBACK callback);
    
    /**
     * Initialise timers
     */
    void initialise_timers();

    /**
     * Initialise USB and USB Audio
     */
//TODO REMOVE     void initialise_usb();
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
