/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.h

  @Summary
    Brief description of the file.

  @Description
 Note: This needs ../include/config/SA30 adding to the includes
 */
/* ************************************************************************** */

#ifndef _DEVICE_CONFIGURATION_H    /* Guard against multiple inclusion */
#define _DEVICE_CONFIGURATION_H


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "utilities/app_data.h"
#include "utilities/adc/adc_defs.h"
#include "utilities/display/display_defs.h"

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Constants                                                         */
/* ************************************************************************** */
/* ************************************************************************** */
        
//------------------------------------------------------------------------------
// APP_FEATURES
//------------------------------------------------------------------------------    

#define APP_FEATURE_ANALOGUE_INPUTS     APP_FEATURE_ON
#define APP_FEATURE_ANALOGUE_DIRECT     APP_FEATURE_ON
#define APP_FEATURE_DIRAC               APP_FEATURE_ON
#define APP_FEATURE_HDMI_ARC            APP_FEATURE_ON
#define APP_FEATURE_LIP_SYNC            APP_FEATURE_ON
#define APP_FEATURE_BALANCE_POT         APP_FEATURE_OFF    
#define APP_FEATURE_SPECIAL_SWITCHES    APP_FEATURE_ON    
        
//------------------------------------------------------------------------------
// Software Model
//------------------------------------------------------------------------------    
    
static char* MANUFACTURER_STRING = "ARCAM";   
static char* SOFTWARE_MODEL_STRING = "SA30";   
        
//------------------------------------------------------------------------------
// Display Test
//------------------------------------------------------------------------------    
    
static DISPLAY_SPECIAL_CHARACTERS DISPLAY_TEST_SPLASH_CHARACTER = DISPLAY_SPECIAL_CHARACTERS_COLON;
        
//------------------------------------------------------------------------------
// ADC Input Mapping
//------------------------------------------------------------------------------    
  
/**
 * Indexing for ADC inputs must match the order that the ADC Definitions are passed into 
 * ADCInitialise()
 */
typedef enum
{
    APP_ADC_INPUTS_BUTTONS_1            = ADCHS_AN0,            //AN00
    APP_ADC_INPUTS_BUTTONS_2            = ADCHS_AN1,            //AN01
    APP_ADC_INPUTS_BUTTONS_3            = ADCHS_AN2,            //AN02
    APP_ADC_INPUTS_BIAS_TEST_L          = ADCHS_AN3,            //AN03
    APP_ADC_INPUTS_BIAS_TEST_R          = ADCHS_AN4,            //AN04
    APP_ADC_INPUTS_OUTPUT_TEMPERATURE   = ADCHS_AN16,           //AN16
    APP_ADC_INPUTS_LIFTER_TEMPERATURE   = ADCHS_AN17,           //AN17
} APP_ADC_INPUTS;

    //! These should be defined in config_settings...
//! Defines the number of ADC button inputs that will be monitored by the ADC module    
#define ADC_BUTTON_INPUT_COUNT          3

//! Defines the number of ADC voltage inputs that will be monitored by the ADC module    
#define ADC_VOLTAGE_INPUT_COUNT         2

//! Defines the number of ADC temperature inputs that will be monitored by the ADC module    
#define ADC_TEMPERATURE_INPUT_COUNT     2

//! Defines the number of ADC button inputs that will be monitored by the ADC module    
#define ADC_POT_INPUT_COUNT             0

/**
 * Application button mappings
 * Note these are loosely mapped to the adc.h ADC_BUTTON enum and must never exceed ADC_BUTTON_ERROR
 * as that has special meaning
 */
typedef enum
{
    APP_BUTTONS_NONE            = ADC_BUTTON_NONE,

    //Buttons
    APP_BUTTONS_MENU            = ADC_BUTTON_1,
    APP_BUTTONS_INPUT_DOWN      = ADC_BUTTON_2,
    APP_BUTTONS_INPUT_UP        = ADC_BUTTON_3, 
    APP_BUTTONS_DIRAC           = ADC_BUTTON_4,
    APP_BUTTONS_MUTE            = ADC_BUTTON_5,
    APP_BUTTONS_INFO            = ADC_BUTTON_6,
    APP_BUTTONS_DIRECT          = ADC_BUTTON_7,
    APP_BUTTONS_DISPLAY         = ADC_BUTTON_8,
    APP_BUTTONS_BALANCE         = ADC_BUTTON_9,
            
    // Mappings for menu navigation
    APP_BUTTONS_HOME            = APP_BUTTONS_DIRAC,
    APP_BUTTONS_UP_LEFT         = APP_BUTTONS_MUTE,
    APP_BUTTONS_DOWN_RIGHT      = APP_BUTTONS_INFO,
    APP_BUTTONS_OK              = APP_BUTTONS_DIRECT,
    APP_BUTTONS_BACK            = APP_BUTTONS_DISPLAY,
} APP_BUTTONS;

// *****************************************************************************
// *****************************************************************************
// Section: Data Types
// *****************************************************************************
// *****************************************************************************


// *****************************************************************************
// *****************************************************************************
// Section: Interface Functions
// *****************************************************************************
// *****************************************************************************


    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
