/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    output_control.h

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

#ifndef _OUTPUT_CONTROL_H    /* Guard against multiple inclusion */
#define _OUTPUT_CONTROL_H

#include <stdbool.h>
#include "utilities/system_fault.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

/* Provide C++ Compatibility */
#ifdef __cplusplus
extern "C" {
#endif


    /* ************************************************************************** */
    /* ************************************************************************** */
    /* Section: Constants                                                         */
    /* ************************************************************************** */
    /* ************************************************************************** */

    // *****************************************************************************
    // *****************************************************************************
    // Section: Data Types
    // *****************************************************************************
    // *****************************************************************************

    // *****************************************************************************
    // *****************************************************************************
    // Section: Interface Functions
    // *****************************************************************************
    // *****************************************************************************
    
    /**
     * Use to determine whether there is a DC offset fault
     * @return 
     */
    bool output_control_dc_offset_detect();
    
    /**
     * Gets the headphone override status
     * @return 
     */
    bool output_control_get_headphone_override();
    
    /**
     * Determines whether the headphones are currently connected or not
     */
    bool output_control_headphones_connected();
    
    /**
     * Initialise the output control module
     */
    void output_control_initialise(SYSTEM_FAULT_CALLBACK fault_callback);    
    
    /**
     */
    void output_control_mute_analogue_outputs();
    
    /**
      */
    void output_control_mute_digital_outputs();
    
    /**
     * Mutes Speaker, Headphone, Preamp, and Power AMp outputs
     * @param include_power_amp - True for SA10, false SA20
     */
    void output_control_mute_outputs(bool include_headphones);
    
    /**
     * Perform regular output control tasks
     */
    void output_control_process();
    
    /**
     * Sets the headphone override status
     */
    void output_control_set_headphone_override(bool override);

    /**
     * Use to determine whether there is a short circuit fault
     * @return 
     */
    bool output_control_short_circuit_detect();    
    
    /**
     * Unmutes 
     */
    void output_control_unmute_analogue_outputs();
    
    /**
     * Unmutes 
     */
    void output_control_unmute_digital_outputs();
    
    /**
     * Unmutes Speaker, Headphone, Preamp, and Power AMp outputs
     * @param include_power_amp - True for SA10, false SA20
     */
    void output_control_unmute_outputs();
    
    /* Provide C++ Compatibility */
#ifdef __cplusplus
}
#endif

#endif /* _EXAMPLE_FILE_NAME_H */

/* *****************************************************************************
 End of File
 */
