/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    An Avatar version of the Control4 app_commands
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "app_commands_control4.h"
#include <stdint.h>
#include <stdio.h>
#include "utilities/app_commands_utilities.h"
#include "utilities/avatar_control.h"
#include "utilities/menu_display.h"

#include "utilities/menu.h"
#include "utilities/timers.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Returns a string representation of control4 SDDP active state
 * @return 
 */
void app_commands_get_control4_active_string(char* buffer, size_t size)
{
    sprintf(buffer, "%s", app_commands_control4_get_active() ? "Active" : "Inactive");
}

/**-----------------------------------------------------------------------------
 * Used to edit the auto_shutdown mode based on the current inputs
 * @param message
 */
bool app_commands_edit_control4_active(STRUCTURED_MESSAGE* message)
{
    bool result = false;
    
    app_commands_review_message_action(message);

    if (!message->repeat)
    {
        int8_t value = (int8_t)message->data[0];
        int8_t current_value = (int8_t)app_commands_control4_get_active();
        
        // Set changed state
        if (message->action != STRUCTURED_MESSAGE_ACTION_GET_STATUS)
        {
            menu_try_initialise_change_state(&current_value, sizeof(int8_t));
        }

        if (app_commands_process_message_value_bool(message, &current_value, &value, true))
        {
            app_commands_control4_set_active(current_value);

            menu_update_change_state(&current_value, sizeof(int8_t));
        }             
    }   
     
    return result;
}

/**-----------------------------------------------------------------------------
 * Called when the Control4 Active menu is exited
 * @param menu
 */
bool app_commands_edit_exit_control4_active(MENU_ID menu)
{
    // if active it will go to send Identify menu, if not it will continue to the next menu
    // in edit mode
    if (app_commands_control4_get_active())
    {
        app_context_set_mode(APP_CONTEXT_MODE_MENU);
        app_context_set_menu(APP_COMMAND_MENU_CONTROL4_IDENTITY);
    }
    else
    {
        // Jump straight to editing the network name
        //menu_move_right(menu_get_current_item());
        //app_context_set_mode(APP_CONTEXT_MODE_MENU);
        //app_context_set_menu(APP_COMMAND_MENU_CRESTRON_ACTIVE); // Next menu item
    }
    
    return true;
}

/**-----------------------------------------------------------------------------
 * Called when the Control4 Active menu is exited
 * @param menu
 */
MENU_ID app_commands_edit_exit_control4_send_identity(MENU_ID menu)
{
    MENU_ID result = MENU_USE_SUPPLIED_MENU_ID;
    
    // Send an Identity message
    avatar_send_control4_identity();
    
    // Goto Identity menu
    app_context_set_mode(APP_CONTEXT_MODE_MENU);
    
    app_context_set_menu(APP_COMMAND_MENU_CONTROL4_IDENTITY_SENT); // Next menu item
    
    return result;
}

/**-----------------------------------------------------------------------------
 * Returns whether Control 4 is active (from Settings)
 * @return 
 */
bool app_commands_control4_get_active()
{
    bool result = false;
    
    settings_get(SETTING_CONTROL4_SDDP_ACTIVE, &result);
    
    return result;
}

/**-----------------------------------------------------------------------------
 * Sets Control 4 is active (in Settings)
 */
void app_commands_control4_set_active(bool active)
{
    settings_set(SETTING_CONTROL4_SDDP_ACTIVE, &active);
    
    avatar_send_control4_discovery((AVATAR_CONTROL4_DISCOVERY)active);
}

/* *****************************************************************************
 End of File
 */
