/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    This handles Crestron functionality for the Avatar
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "app_commands_crestron.h"
#include <stdio.h>
#include <stdint.h>
#include "utilities/avatar_control.h"
#include "utilities/display.h"
#include "utilities/app_commands_utilities.h"
#include "utilities/menu_display.h"

#include "utilities/menu.h"
#include "utilities/timers.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Used during crestron_discovery configuration
 */
//void app_commands_crestron_discovery_callback(CRESTRON_DISCOVERY_CONFIGURATION_STATE state, char* hostname)
//{
//    switch(state)
//    {
//        case CRESTRON_DISCOVERY_CONFIGURATION_STATE_START:
//        {
//            // Display the crestron config menu
//            app_context_set_mode(APP_CONTEXT_MODE_MENU);
//            app_context_set_menu(APP_COMMAND_MENU_CRESTRON_CONFIG); // Next menu item
//            break;
//        }
//
//        case CRESTRON_DISCOVERY_CONFIGURATION_STATE_STOP:
//        {
//            // Return to default display
//            app_context_set_mode(APP_CONTEXT_MODE_DEFAULT);
//            app_context_set_menu(APP_COMMAND_MENU_VOLUME);
//            break;
//        }
//    }
//}

/**-----------------------------------------------------------------------------
 * Draws a screen displayed during Crestron discovery prompting the user to acknowledge the input
 */
void app_commands_draw_crestron_discovery_config(APP_COMMAND_MENU menu)
{
    MENU_ITEM* item = menu_get_current_item();

    // Draw the top line
    // Clear the display
    display_clear(false);
    
    // Draw the top line
    display_clear_line(DISPLAY_LINE_1);
    display_start_text(DISPLAY_LINE_1, menu_get_title(item->menu_id));
    display_write_line(DISPLAY_LINE_1, false);

    display_clear_line(DISPLAY_LINE_2);
    
    if (item->menu_id == APP_COMMAND_MENU_CRESTRON_CONFIG)
    {
        display_start_text(DISPLAY_LINE_2, "Confirm");
        display_special_character(DISPLAY_LINE_2, 11, 
            DISPLAY_SPECIAL_CHARACTERS_RIGHT);  
    }
    else if (item->menu_id == APP_COMMAND_MENU_CRESTRON_CONFIG_COMPLETING)
    {
        display_start_text(DISPLAY_LINE_2, "Completing");
    }
    
    display_write_line(DISPLAY_LINE_2, false);
}

/**-----------------------------------------------------------------------------
 * Used to edit the auto_shutdown mode based on the current inputs
 * @param message
 */
bool app_commands_edit_crestron_discovery_active(STRUCTURED_MESSAGE* message)
{
    bool result = false;
    
    app_commands_review_message_action(message);

    if (!message->repeat)
    {
        int8_t value = (int8_t)message->data[0];
        int8_t current_value = (int8_t)app_commands_crestron_get_active();
        
        // Set changed state
        if (message->action != STRUCTURED_MESSAGE_ACTION_GET_STATUS)
        {
            menu_try_initialise_change_state(&current_value, sizeof(int8_t));
        }

        if (app_commands_process_message_value_bool(message, &current_value, &value, true))
        {
            app_commands_crestron_set_active(current_value);

            menu_update_change_state(&current_value, sizeof(int8_t));
        }             
    }   
     
    return result;
}

/**-----------------------------------------------------------------------------
 * Returns a string representation of control4 SDDP active state
 * @return 
 */
void app_commands_get_crestron_discovery_active_string(char* buffer, size_t size)
{
    sprintf(buffer, "%s", app_commands_crestron_get_active() ? "Active" : "Inactive");
}

/**-----------------------------------------------------------------------------
 * Returns whether Control 4 is active (from Settings)
 * @return 
 */
bool app_commands_crestron_get_active()
{
    bool result = false;
    
    settings_get(SETTING_CRESTRON_DISCOVERY_ACTIVE, &result);
    
    return result;
}

/**-----------------------------------------------------------------------------
 * Called when the Crestron configure menu is exited
 * @param menu
 */
void app_commands_send_crestron_configure()
{
    // Send an Identity message
    avatar_send_crestron_configure_response();
    
    // Goto Identity menu
    app_context_set_mode(APP_CONTEXT_MODE_MENU);
    app_context_set_menu(APP_COMMAND_MENU_CRESTRON_CONFIG_COMPLETING); // Next menu item
}

/**-----------------------------------------------------------------------------
 * Sets Control 4 is active (in Settings)
 */
void app_commands_crestron_set_active(bool active)
{
    settings_set(SETTING_CRESTRON_DISCOVERY_ACTIVE, &active);
   
    avatar_send_crestron_discovery((AVATAR_CRESTRON_DISCOVERY)active);
}

/* *****************************************************************************
 End of File
 */
