/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "app_commands_network.h"
#include <stdio.h>
#include "utilities/power.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Used to edit the auto_shutdown mode based on the current inputs
 * @param message
 */
bool app_commands_edit_network_standby(STRUCTURED_MESSAGE* message)
{
    bool result = false;
    
    app_commands_review_message_action(message);

    if (!message->repeat)
    {
        int8_t value = (int8_t)message->data[0];
        int8_t current_value = (int8_t)power_get_network_active_in_standby();
        
        // Set changed state
        if (message->action != STRUCTURED_MESSAGE_ACTION_GET_STATUS)
        {
            menu_try_initialise_change_state(&current_value, sizeof(int8_t));
        }

        if (app_commands_process_message_value_bool(message, &current_value, &value, true))
        {
            power_set_network_active_in_standby(current_value);
            // Endure that RS232 is active is network is active
            if ((bool)current_value)
            {
                power_set_rs232_active_in_standby(true);
            }

            menu_update_change_state(&current_value, sizeof(int8_t));
        }             
    }    
     
    return result;
}

/**-----------------------------------------------------------------------------
 * Used to edit the auto_shutdown mode based on the current inputs
 * @param message
 */
bool app_commands_edit_rs232_standby(STRUCTURED_MESSAGE* message)
{
    bool result = false;
    
    app_commands_review_message_action(message);

    if (!message->repeat)
    {
        int8_t value = (int8_t)message->data[0];
        int8_t current_value = (int8_t)power_get_rs232_active_in_standby();
        
        // Set changed state
        if (message->action != STRUCTURED_MESSAGE_ACTION_GET_STATUS)
        {
            menu_try_initialise_change_state(&current_value, sizeof(int8_t));
        }

        if (app_commands_process_message_value_bool(message, &current_value, &value, true))
        {
            power_set_rs232_active_in_standby(current_value);
            // Endure that RS232 is active is network is active
            if (!(bool)current_value)
            {
                power_set_network_active_in_standby(false);
            }

            menu_update_change_state(&current_value, sizeof(int8_t));
        }             
    }    
     
    return result;
}

/**-----------------------------------------------------------------------------
 * Returns a string representation of the network standby value
 * @return 
 */
void app_commands_get_network_standby_string(char* buffer, size_t size)
{
    sprintf(buffer, "%s", APP_COMMAND_OFF_ON_STRINGS[(int)power_get_network_active_in_standby()]);
}

/**-----------------------------------------------------------------------------
 * Returns a string representation of the rs232 standby value
 * @return 
 */
void app_commands_get_rs232_standby_string(char* buffer, size_t size)
{
    sprintf(buffer, "%s", APP_COMMAND_OFF_ON_STRINGS[(int)power_get_rs232_active_in_standby()]);
}

/* *****************************************************************************
 End of File
 */
