/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "app_commands_output.h"
#include <stdio.h>
#include "app_config.h"
#include "utilities/output_control.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Used to edit the headphone override status
 * @param message
 * @return 
 */
bool app_commands_edit_headphone_override(STRUCTURED_MESSAGE* message)
{
    bool result = false;
    
    app_commands_review_message_action(message);

    if (!message->repeat)
    {
        int8_t value = (int8_t)message->data[0];
        int8_t current_value = (int8_t)output_control_get_headphone_override();
        
        // Set changed state
        if (message->action != STRUCTURED_MESSAGE_ACTION_GET_STATUS)
        {
            menu_try_initialise_change_state(&current_value, sizeof(int8_t));
        }

        if (app_commands_process_message_value_bool(message, &current_value, &value, true))
        {
            output_control_set_headphone_override(current_value);
 
            menu_update_change_state(&current_value, sizeof(int8_t));
        }
    }    
     
    return result;
}

/**-----------------------------------------------------------------------------
 * Used to change the short_circuit_sense setting
 */
bool app_commands_edit_short_circuit_sense(STRUCTURED_MESSAGE* message)
{
     bool result = false;
    
    app_commands_review_message_action(message);

    if (!message->repeat)
    {
        int8_t value = (int8_t)message->data[0];
        
        bool short_circuit_sense = false;
        settings_get(SETTING_SHORT_CIRCUIT_SENSE, &short_circuit_sense);
        
        int8_t current_value = (int8_t)short_circuit_sense;
        
        // Set changed state
        if (message->action != STRUCTURED_MESSAGE_ACTION_GET_STATUS)
        {
            menu_try_initialise_change_state(&current_value, sizeof(int8_t));
        }

        if (app_commands_process_message_value_bool(message, &current_value, &value, true))
        {
            short_circuit_sense = (bool)current_value;
            settings_set(SETTING_SHORT_CIRCUIT_SENSE, &short_circuit_sense);
            settings_save();

            menu_update_change_state(&current_value, sizeof(int8_t));
        }             
    }    
     
    return result;
}

/**-----------------------------------------------------------------------------
 * Used to get the DC Offset for an RS232 GET_STATUS call
 */
void app_commands_get_dc_offset_status(STRUCTURED_MESSAGE* message)
{
    if (message->data[0] == APP_COMMAND_VALUE_REQUEST_STATUS)
    {
        message->data[0] = output_control_dc_offset_detect();
        message->data_length = 1;
        message->response = STATUS_UPDATE;
    }
    else
    {
        message->response = COMMAND_INVALID_NOW;
    }
    
    app_commands_send_message(message);
}

/**-----------------------------------------------------------------------------
 * Returns a string representation of the headphone override status
 * @return 
 */
void app_commands_get_headphone_override_string(char* buffer, size_t size)
{
    sprintf(buffer, "%s", APP_COMMAND_OFF_ON_STRINGS[(int)output_control_get_headphone_override()]);
}

/**-----------------------------------------------------------------------------
 * Returns a string representation of the Short Circuit Sensor setting
 * @return 
 */
void app_commands_get_short_circuit_sense_string(char* buffer, size_t size)
{
    bool short_circuit_sense;
    settings_get(SETTING_SHORT_CIRCUIT_SENSE, &short_circuit_sense);
    
    sprintf(buffer, "%s", APP_COMMAND_LOW_HIGH_STRINGS[short_circuit_sense ? 1 : 0]);
}

/**-----------------------------------------------------------------------------
 * Used to get the DC Offset for an RS232 GET_STATUS call
 */
void app_commands_get_short_circuit_status(STRUCTURED_MESSAGE* message)
{
    message->command = APP_COMMANDS_SHORT_CIRCUIT_STATUS;
    
    if (message->data[0] == APP_COMMAND_VALUE_REQUEST_STATUS)
    {
        message->data[0] = output_control_short_circuit_detect();
        message->data_length = 1;
        message->response = STATUS_UPDATE;
    }
    else
    {
        message->response = PARAM_NOT_RECOGNISED;
    }
    
    app_commands_send_message(message);
}

/**-----------------------------------------------------------------------------
 * Determines whether headphones are connected
 */
void app_commands_headphones(STRUCTURED_MESSAGE* message)
{
    if (message->data[0] == APP_COMMAND_VALUE_REQUEST_STATUS)
    {
        // Update the message
        message->command = APP_COMMANDS_HEADPHONES;
        message->response = STATUS_UPDATE;
        message->data_length = 1;
        message->data[0] = output_control_headphones_connected();
    }
    else
    {
        message->response = PARAM_NOT_RECOGNISED;
    }
    
    app_commands_send_message(message);
}

/* *****************************************************************************
 End of File
 */
