/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "app_commands_system_fault.h"
#include <stdbool.h>
#include "app_config.h"
#include "app_context.h"
#include "utilities/logger/logger.h"
#include "utilities/system_fault.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

typedef struct
{
    //! True if we are waiting to the SHUTDOWN_CODE is being saved before proceeding.
    bool fault_settings_save;
 
} APP_COMMANDS_SYSTEM_FAULT_DATA;

static APP_COMMANDS_SYSTEM_FAULT_DATA app_commands_system_fault_data = 
{
    .fault_settings_save = false,
};

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Check to see if a fault has been raised anywhere in the system
 */
void app_commands_check_for_fault()
{
    SYSTEM_FAULT fault = system_fault_get();
    if (fault.code != SYSTEM_FAULT_CODE_NO_FAULT)
    {
        if (system_fault_save_to_settings())
        {
            app_context_set_app_state(APP_STATE_FAULT);
        }
        else
        {
            app_context_set_app_state(APP_STATE_ENTERING_FAULT);
        }
    }
}

/**-----------------------------------------------------------------------------
 * Returns the last shutdown reason.
 * @param buffer
 * @param size
 */
void app_commands_get_last_shutdown_code_string(char* buffer, size_t size)
{
    SYSTEM_FAULT_CODE history[SHUTDOWN_CODE_HISTORY_LENGTH];
    settings_get(SETTING_SHUTDOWN_CODE_HISTORY, &history);
    
    // Note: This is assuming that SHUTDOWN_CODE_HISTORY_LENGTH >= 10, note it
    // also assumes the first entry in the history is the current last shutdown 
    // code as this is copied up-front once settings are initialised.
    sprintf(buffer, "%d %d%d%d%d%d%d%d%d%d", 
            system_fault_get_last_shutdown_code(), 
            history[1],
            history[2],
            history[3],
            history[4],
            history[5],
            history[6],
            history[7],
            history[8],
            history[9]);
}

/* *****************************************************************************
 End of File
 */
