/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "app_commands_volume.h"
#include <stdio.h>
#include "app_config.h"
#include "app_commands_avatar.h"
#include "utilities/dirac.h"
#include "utilities\display.h"
#include "utilities\hdmi_arc_processing.h"
#include "utilities\menu.h"
#include "utilities/menu_app.h"
#include "utilities\menu_display.h"
#include "utilities\input_control.h"
#include "utilities\output_control.h"
#include "utilities/timers.h"
#include "utilities/volume.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Draws a standard navigation menu with title on top line and navigation buttons 
 * on the bottom
 */
void app_commands_draw_balance(APP_COMMAND_MENU menu)
{
    MENU_ITEM* item = menu_get_current_item();

    // Draw the top line
    // Clear the display
    display_clear(false);
    
    // Draw the top line
    display_clear_line(DISPLAY_LINE_1);
    
    if (!input_control_in_processor_mode())
    {
        display_start_text(DISPLAY_LINE_1, menu_get_title(menu_get_current_item()->menu_id));
    }
    else
    {
        display_start_text(DISPLAY_LINE_1, "Balance (PM)");
    }
    
    // Check if there is a value to be displayed
    size_t len = display_get_line_length();
    char value[len];

    volume_get_balance_string((char*)&value[0], len);

    // Check whether we're displaying the edited value or not
    // MENU_ITEM* current_item = menu_get_current_item(); 
    if (menu_is_editable_and_original(item))
    {
        display_append_text_right(DISPLAY_LINE_1, "*%s", value);
    }
    else
    {
        display_append_text_right(DISPLAY_LINE_1, "%s", value);
    }
     
    display_write_line(DISPLAY_LINE_1, false);
    
    if (menu == APP_COMMAND_MENU_BALANCE)
    {
        // Draw the navigation buttons 
        menu_display_draw_generic_navigation(item, MENU_SOFTKEYS_UP_DOWN, true);
    }
    else
    {
        display_clear_line(DISPLAY_LINE_2);
        display_write_line(DISPLAY_LINE_2, false);
    }
}

/**-----------------------------------------------------------------------------
 * Draw the bespoke volume display
 */
void app_commands_draw_volume(APP_COMMAND_MENU menu)
{
    AUDIO_INPUT selected_input = input_control_get_selected_input();
    uint8_t volume = volume_get();
    bool analogue_direct = analogue_direct_get_active();
    INPUT_CONTROL_TYPE type = input_control_get_type();
    uint8_t input_start_pos = 0;
    bool app_running = app_context_is_running();
    
    if (selected_input == INPUT_CONTROL_NUM_SETTINGS)
    {
        selected_input = AUDIO_INPUT_NONE;
    }
    
    // Clear the display
    display_clear(false);

    // Check for a preview
    INPUT_CONTROL_PREVIEW_INFO* preview_info = input_control_get_preview_info();
    if (preview_info->active)
    {
        selected_input = (AUDIO_INPUT)preview_info->input;
        analogue_direct = input_analogue_direct_get(selected_input);//input_control_get_input_analogue_direct(selected_input);
        type = preview_info->type;
        
        if (selected_input == input_control_get_processor_mode_input())
        {
            volume = input_control_get_processor_mode_volume();
        }
        else
        {
            // Get stored volume rather than active as we may be in processor mode.
            volume = volume_get_stored();
        }
        
        // Indicate that we are previewing the track
        display_special_character(DISPLAY_LINE_1, 0, DISPLAY_SPECIAL_CHARACTERS_RIGHT);
        input_start_pos = 1;
    }
    
    // Sort out Input name if we are in the Avatar
    char input_name[14];
    memset(input_name, 0, 14);
    if (selected_input == AUDIO_INPUT_AVATAR)
    {
        app_commands_get_audio_source_string(input_name, 14);
        input_name[13] = 0;
    }
#if APP_FEATURE_HDMI_ARC == APP_FEATURE_ON  
    else if (selected_input == AUDIO_INPUT_HDMI_ARC)
    {
        hdmi_arc_get_arc_format_string(input_name, 16);
    }
#endif
    else
    {
        memcpy(input_name, AUDIO_INPUT_STRINGS[selected_input], strlen(AUDIO_INPUT_STRINGS[selected_input]));
    }

    if (app_running)
    {
        // Display input name
        if ((selected_input != input_control_get_processor_mode_input()) || (selected_input == AUDIO_INPUT_NONE))
        {
            // MM/MC string if Phono
            char mm_mc_string[5];
            memset(mm_mc_string, 0, 5);

            if (selected_input == AUDIO_INPUT_PHONO)
            {
                mm_mc_string[0] = '(';
                input_phono_input_get_string(&mm_mc_string[1], 2);
                mm_mc_string[3] = ')';
            }

            display_append_text(DISPLAY_LINE_1, input_start_pos, "%s %s", input_name, mm_mc_string);
        }
        else
        {
            display_append_text(DISPLAY_LINE_1, input_start_pos, "%s (PM)", input_name);
        }

        // display volume or Mute
        if (input_control_in_mute_state())
        {
            if (!input_control_in_switch())
            {
                display_append_text_right(DISPLAY_LINE_1, "%s", APP_STRINGS[APP_STRINGS_ENUM_MUTE]);
            }
            else
            {
                if (input_control_mute_state_pre_switch())
                {
                    display_append_text_right(DISPLAY_LINE_1, "%s", APP_STRINGS[APP_STRINGS_ENUM_MUTE]);
                }
                else
                {
                    //display_append_text_right(DISPLAY_LINE_1, "%s", APP_STRINGS[APP_STRINGS_ENUM_SWITCHING]);
                    display_append_text_right(DISPLAY_LINE_1, "%02d", volume);
                }
            }
        }
        else if (selected_input != AUDIO_INPUT_NONE)
        {
            if (!input_control_in_switch())
            {
                display_append_text_right(DISPLAY_LINE_1, "%02d", volume);
            }
            else
            {
                if (input_control_mute_state_pre_switch())
                {
                    display_append_text_right(DISPLAY_LINE_1, "%s", APP_STRINGS[APP_STRINGS_ENUM_MUTE]);
                }
                else
                {
                    display_append_text_right(DISPLAY_LINE_1, "%02d", input_control_volume_in_switch());
                }
            }
        }
    }
    else
    {
        display_append_text(DISPLAY_LINE_1, 0, "Press MENU for", input_name);
    }
  
    display_write_line(DISPLAY_LINE_1, false);

    // Bottom Line
    display_clear_line(DISPLAY_LINE_2);
 
    if (app_running)
    {
        // Build Icon strings
        char icon_buffer[DISPLAY_LINE_LEN];
        memset(icon_buffer, 0, DISPLAY_LINE_LEN);

        // Right align icons
        uint8_t icon_pos = 0;

        // MQA
        if ((input_control_get_type() == INPUT_CONTROL_TYPE_AVATAR) && (avatar_message_get_mqa_state() != AVATAR_MQA_STATE_NO_MQA))
        {
            sprintf(&icon_buffer[icon_pos], "%s ", AVATAR_MQA_STATE_STRINGS[avatar_message_get_mqa_state()]);
            icon_pos += strlen(AVATAR_MQA_STATE_STRINGS[avatar_message_get_mqa_state()]) + 1;
        }

        // display headphones if present
        if (output_control_headphones_connected())
        {
            icon_buffer[icon_pos] = DISPLAY_SPECIAL_CHARACTERS_HEADPHONES;
            icon_pos++;
        }

        // Analogue Direct
        if (analogue_direct)
        {
            icon_buffer[icon_pos] = DISPLAY_SPECIAL_CHARACTERS_ANALOGUE;
            icon_pos++;
        }
        else
        {
            // Removed during the discussions over MQA to save space
            //icon_buffer[icon_pos] = DISPLAY_SPECIAL_CHARACTERS_DIGITAL;
            //icon_pos++;

            // BUG 3556 - Dirac icon shown in DIRECT mode - moved into analogue_direct check
            // Display Dirac
            // Show that a Dirac curve is active - they can press the Dirac button to find out which one
            if ((dirac_get_curve_for_input(selected_input) != DIRAC_NO_CURVE) && !output_control_headphones_connected())
            {
                // Perhaps display EQ1/EQ2?
                icon_buffer[icon_pos] = DISPLAY_SPECIAL_CHARACTERS_DIRAC_LOGO;
                icon_pos++;
            }
        }

        // Display Upgrade Available
        if (avatar_upgrade_available())
        {
            icon_buffer[icon_pos] = DISPLAY_SPECIAL_CHARACTERS_UPGRADE_AVAILABLE;
            icon_pos++;
        }

        // We have a scrolling display for the Avatar so treat it differently
        if (selected_input != AUDIO_INPUT_AVATAR)
        {
            // Display the ICONS
            int x;
            for (x = 0; x < icon_pos; x++)
            {
                display_special_character(DISPLAY_LINE_2, x, icon_buffer[x]);
            }

            // Display Sample Rate
            // display sampling frequency if present
            switch(type)
            {
                case INPUT_CONTROL_TYPE_ANALOGUE:
                {
                    display_append_text_right(DISPLAY_LINE_2, "%s", INPUT_CONTROL_TYPE_STRINGS[type]);
                    break;
                }

                case INPUT_CONTROL_TYPE_DIGITAL:
                {
                    display_append_text_right(DISPLAY_LINE_2, "%s", SAMPLE_RATE_STRINGS[input_control_get_sample_rate()]);
                    break;
                }

                case INPUT_CONTROL_TYPE_AVATAR:
                {
                    // Don't display Sample rate for NET Input display_append_text_right(DISPLAY_LINE_2, "%s", SAMPLE_RATE_STRINGS[input_control_get_sample_rate()]); //INPUT_CONTROL_TYPE_STRINGS[type]);
                    break;
                }

                default:
                {
                    break;
                }
            }

            display_write_line(DISPLAY_LINE_2, false);
        }
        else
        {
            // Avatar bottom line scrolling
            app_commands_avatar_draw_info(menu, icon_buffer);
            //display_write_line(DISPLAY_LINE_2, false);
        }
    }
    else
    {
        display_append_text(DISPLAY_LINE_2, 0, "Options", input_name);
        display_write_line(DISPLAY_LINE_2, false);
    }
}
 
/**-----------------------------------------------------------------------------
 * Used to edit the balance setting based on the current inputs
 * @param message
 */
bool app_commands_edit_balance(STRUCTURED_MESSAGE* message)
{
    bool result = false;
    
    app_commands_review_message_action(message);

    int16_t value = (int16_t)message->data[0];
    int16_t current_value = (int16_t)volume_get_balance();
        
    // Set changed state
    if (message->action != STRUCTURED_MESSAGE_ACTION_GET_STATUS)
    {
        menu_try_initialise_change_state(&current_value, sizeof(int16_t));
    }

    // If value < 0xFO then is an absolute value
    switch (message->action)
    {
        case STRUCTURED_MESSAGE_ACTION_SET_ABSOLUTE:
        {
            if (value >= 0x80)
            {
                // Subtract 128 and make negative
                value -= 0x80;
                value*= -1;
            }

            value = volume_get_adjusted_balance(value);
            break;
        }

        case STRUCTURED_MESSAGE_ACTION_GET_STATUS:
        {
            current_value = volume_get_relative_balance(current_value);
            if (current_value < 0)
            {
                current_value *= -1;
                current_value += 0x80;
            }
            break;
        }
    }

    app_commands_process_message_value(message, &current_value, &value, 1);

    if (message->action != STRUCTURED_MESSAGE_ACTION_GET_STATUS)
    {
        if (!input_control_in_processor_mode())
        {
            volume_set_balance(current_value, true);
            input_control_set_avatar_balance(volume_get_balance());
            
            menu_update_change_state(&current_value, sizeof(int16_t));

            current_value = volume_get_relative_balance(current_value);
            if (current_value < 0)
            {
                current_value *= -1;
                current_value += 0x80;
            }

            uint8_t log_value = current_value;
            logger_log_command(APP_COMMANDS_BALANCE, &log_value, 1);
        }
        else
        {
            current_value = VOLUME_DEFAULT_BALANCE;
            logger_log_infof(APP_COMMANDS_BALANCE, "VOLUME Balance cannot be changed in Processor Mode --");
        }
    }
        
    message->data[0] = current_value;
    message->data_length = 1;
    message->command = APP_COMMANDS_BALANCE;
    message->response = STATUS_UPDATE;

    app_commands_send_message(message);
    
    return result;
}

/**-----------------------------------------------------------------------------
 * Performs a volume change based upon a delta value
 */
bool app_commands_edit_max_on_volume(STRUCTURED_MESSAGE* message)
{
    bool result = true;
    
    app_commands_review_message_action(message);

    int16_t value = (int16_t)message->data[0];
    int16_t current_value = volume_get_max_on_volume();

    // Set changed state
    if (message->action != STRUCTURED_MESSAGE_ACTION_GET_STATUS)
    {
        menu_try_initialise_change_state(&current_value, sizeof(int16_t));
    }

    app_commands_process_message_value(message, &current_value, &value, 1);

    message->command = APP_COMMANDS_MAX_ON_VOLUME;
    message->response = STATUS_UPDATE;

    if (message->action != STRUCTURED_MESSAGE_ACTION_GET_STATUS)
    {
        if (volume_is_valid(current_value))
        {
            volume_set_max_on_volume(current_value);

            menu_update_change_state(&current_value, sizeof(int16_t));
               
            message->data[0] = current_value;
        }
        else
        {
            message->response = COMMAND_INVALID_NOW;

            message->data[0] = volume_get_max_on_volume();
        }
 
        message->data_length = 1;
    }
        
    // Send the response
    app_commands_send_message(message);
    
    return result;
}

/**-----------------------------------------------------------------------------
 * Performs a volume change based upon a delta value
 */
bool app_commands_edit_maximum_streaming_volume(STRUCTURED_MESSAGE* message)
{
    bool result = true;
    
    app_commands_review_message_action(message);

    int16_t value = (int16_t)message->data[0];
    int16_t current_value = 0;
    
    uint8_t curr_vol;
    settings_get(SETTING_MAXIMUM_STREAMING_VOLUME, &curr_vol);
    current_value = curr_vol;
    
    // Set changed state
    if (message->action != STRUCTURED_MESSAGE_ACTION_GET_STATUS)
    {
        menu_try_initialise_change_state(&current_value, sizeof(int16_t));
    }

    app_commands_process_message_value(message, &current_value, &value, 1);

    message->command = APP_COMMANDS_MAXIMUM_STREAMING_VOLUME;
    message->response = STATUS_UPDATE;

    if (message->action != STRUCTURED_MESSAGE_ACTION_GET_STATUS)
    {
        if (volume_is_valid(current_value))
        {
            current_value = volume_set_maximum_volume(current_value, VOLUME_INPUT_STREAMER);
            
            if (input_control_get_type() == INPUT_CONTROL_TYPE_AVATAR)
            {
                avatar_send_maximum_volume(current_value);
            }

#if APP_FEATURE_HDMI_ARC == APP_FEATURE_ON  
            // Remap the HDMI ARC volume to stop jump to 99
            hdmi_arc_send_volume_message(volume_get());
#endif

            menu_update_change_state(&current_value, sizeof(int16_t));
               
            message->data[0] = current_value;
        }
        else
        {
            message->response = COMMAND_INVALID_NOW;

            message->data[0] = curr_vol;
        }
 
        message->data_length = 1;
    }
        
    // Send the response
    app_commands_send_message(message);
    
    return result;
}

/**-----------------------------------------------------------------------------
 * Performs a volume change based upon a delta value
 */
bool app_commands_edit_maximum_volume(STRUCTURED_MESSAGE* message)
{
    bool result = true;
    
    app_commands_review_message_action(message);

    int16_t value = (int16_t)message->data[0];
    int16_t current_value = 0;
    
    uint8_t curr_vol;
    settings_get(SETTING_MAXIMUM_VOLUME, &curr_vol);
    current_value = curr_vol;

    // Set changed state
    if (message->action != STRUCTURED_MESSAGE_ACTION_GET_STATUS)
    {
        menu_try_initialise_change_state(&current_value, sizeof(int16_t));
    }

    app_commands_process_message_value(message, &current_value, &value, 1);

    message->command = APP_COMMANDS_MAXIMUM_VOLUME;
    message->response = STATUS_UPDATE;

    if (message->action != STRUCTURED_MESSAGE_ACTION_GET_STATUS)
    {
        if (volume_is_valid(current_value))
        {
            current_value = volume_set_maximum_volume(current_value, VOLUME_INPUT_DEFAULT);
            
            if (input_control_get_type() != INPUT_CONTROL_TYPE_AVATAR)
            {
                avatar_send_maximum_volume(volume_get_maximum_volume());
            }
            else
            {
                avatar_send_maximum_volume(volume_get_maximum_net_volume());
            }
            
            // Remap the HDMI ARC volume to stop jump to 99
#if APP_FEATURE_HDMI_ARC == APP_FEATURE_ON  
            hdmi_arc_send_volume_message(volume_get());
#endif

            menu_update_change_state(&current_value, sizeof(int16_t));
               
            message->data[0] = current_value;
        }
        else
        {
            message->response = COMMAND_INVALID_NOW;

            message->data[0] = curr_vol;
        }
 
        message->data_length = 1;
    }
        
    // Send the response
    app_commands_send_message(message);
    
    return result;
}

/**-----------------------------------------------------------------------------
 * Performs a volume change based upon a delta value
 */
bool app_commands_edit_volume(STRUCTURED_MESSAGE* message)
{
    bool result = true;

    app_commands_review_message_action(message);

    int16_t value = (int16_t)message->data[0];
    int16_t current_value = volume_get();

    // Set changed state
    if (message->action != STRUCTURED_MESSAGE_ACTION_GET_STATUS)
    {
        menu_try_initialise_change_state(&current_value, sizeof(int16_t));
    }

    app_commands_process_message_value(message, &current_value, &value, 1);

    message->command = APP_COMMANDS_VOLUME;
    message->response = STATUS_UPDATE;

    if (message->action != STRUCTURED_MESSAGE_ACTION_GET_STATUS)
    {
        if (volume_is_valid(current_value))
        {
            // Don't allow an edit when in processor mode
            if (!input_control_in_processor_mode())
            {
                if ((volume_get() > 0) && volume_is_muted())
                {
                    output_control_unmute_outputs();
                }
                
                volume_set(current_value, true);
                
                input_control_set_avatar_volume(current_value);
#if APP_FEATURE_HDMI_ARC == APP_FEATURE_ON  
                input_control_set_hdmi_arc_volume(current_value);
#endif                

                menu_update_change_state(&current_value, sizeof(int16_t));
               
                message->data[0] = current_value;
            }
            else
            {
                message->response = COMMAND_INVALID_NOW;
               
                message->data[0] = volume_get();
                
                // BUG-20171206-1
                timers_reset(TIMER_INDEX_BRIGHTNESS);
            }
 
            message->data_length = 1;
        }
        else
        {
            message->response = PARAM_NOT_RECOGNISED;
        }
    }
        
    // Send the response
    app_commands_send_message(message);
    
    return result;
}

/**-----------------------------------------------------------------------------
 * Called when leaving the Balance menu
 */
void app_commands_enter_balance_no_navigation(MENU_ID menu)
{
    // Ensure the Balance menu is only visible for a while
    timers_update_timeout(TIMER_INDEX_MENU_INACTIVE, TIMER_MENU_BALANCE_INACTIVE_TIMEOUT_MS, TIMERS_RESET_IMMEDIATE);
}

/**-----------------------------------------------------------------------------
 * Called when leaving the Balance menu
 * @param menu
 */
MENU_ID app_commands_exit_balance(MENU_ID menu)
{
    MENU_ID result = MENU_USE_SUPPLIED_MENU_ID;

    // Make sure Inactive Menu timeout is reset
    menu_app_reset_inactivity_timeout(menu);
    
    return result;
}

/**-----------------------------------------------------------------------------
 * Returns a string representation of the maximum on value
 * @return 
 */
void app_commands_get_max_on_volume_string(char* buffer, size_t size)
{
    sprintf(buffer, "%d", volume_get_max_on_volume());
}

/**-----------------------------------------------------------------------------
 * Returns a string representation of the maximum volume
 * @return 
 */
void app_commands_get_maximum_streaming_volume_string(char* buffer, size_t size)
{
    uint8_t volume;
    settings_get(SETTING_MAXIMUM_STREAMING_VOLUME, &volume);
    
    sprintf(buffer, "%d", volume);
}

/**-----------------------------------------------------------------------------
 * Returns a string representation of the maximum volume
 * @return 
 */
void app_commands_get_maximum_volume_string(char* buffer, size_t size)
{
    uint8_t volume;
    settings_get(SETTING_MAXIMUM_VOLUME, &volume);
    
    sprintf(buffer, "%d", volume);
}

/**-----------------------------------------------------------------------------
 * Performs a user mute
 */
void app_commands_mute_user(STRUCTURED_MESSAGE* message)
{
    APP_COMMAND_MUTE action = message->data[0];

    // Sort out toggle
    if (action == APP_COMMAND_MUTE_TOGGLE)
    {
        action = APP_COMMAND_MUTE_ON;
        if (volume_is_muted())
        {
            action = APP_COMMAND_MUTE_OFF;
        }
        
        message->data[0] = action;
    }
    
    message->command = APP_COMMANDS_MUTE;
    message->response = STATUS_UPDATE;
    
    switch(action)
    {
        case APP_COMMAND_MUTE_ON:
        {
            output_control_mute_outputs(true);
            volume_mute();
            
            avatar_mute(AVATAR_MUTE_STATE_MUTE);
#if APP_FEATURE_HDMI_ARC == APP_FEATURE_ON  
            hdmi_arc_send_mute_message(HDMI_ARC_MUTE_STATE_MUTED);
#endif
           
            break;
        }
        
        case APP_COMMAND_MUTE_OFF:
        {
            output_control_unmute_outputs();
            volume_unmute();            
            
            avatar_mute(AVATAR_MUTE_STATE_UNMUTE);
#if APP_FEATURE_HDMI_ARC == APP_FEATURE_ON  
            hdmi_arc_send_mute_message(HDMI_ARC_MUTE_STATE_UNMUTED);
#endif
            break;
        }
        
        case APP_COMMAND_MUTE_GETSTATUS:
        {
            // Check source of message
            // if rs232 or ethernet then get state and send message back
            message->data[0] = APP_COMMAND_MUTE_OFF;
            
            if (volume_is_muted())
            {
                message->data[0] = APP_COMMAND_MUTE_ON;
            }
            
            break;
        }
            
        default:
        {
            message->response = COMMAND_NOT_RECOGNISED;
            break;
        }
    }

    // Just incase its not been defined
    message->data_length = 1;
    
    app_commands_send_message(message);
}

/* *****************************************************************************
 End of File
 */
