/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    filename.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "app_initialise.h"
#include "app_config.h"
#include "app_commands.h"
#include "app_commands_avatar.h"
#include "app_device_configuration.h"
#include "app_commands_display.h"
#include "app_commands_hdmi_arc.h"
#include "app_commands_input.h"
#include "app_commands_engineering.h"
#include "app_commands_power.h"
#include "app_commands_standby.h"
#include "app_commands_upgrade.h"
#include "app_commands_volume.h"
#include "system_definitions.h"
#include "utilities/adc/adc_module.h"
#include "utilities/avatar.h"
#include "utilities/avatar_message_processing.h"
#include "utilities/dirac.h"
#include "utilities/flash_memory.h"
#include "utilities/hdmi_arc_processing.h"
#include "utilities/i2c.h"
#include "utilities/i2c_slave.h"
#include "utilities/input_control.h"
#include "utilities/input_control_app.h"
#include "utilities/input_phono_input.h"
#include "utilities/power.h"
#include "utilities/remote.h"
#include "utilities/rgb_led.h"
#include "utilities/settings.h"
#include "utilities/spdif_rx/spdif_rx.h"
#include "utilities/spdif_rx/spdif_rx_hw_ak4113.h"
#include "utilities/spdif_rx/spdif_rx_hw_tipcm9211.h"
#include "utilities/special_switches.h"
#include "utilities/volume.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */
  
/**-----------------------------------------------------------------------------
 * 
 */
//void configure_input_control_analogue()
//{
//    logger_log_info(LOGGER_CODE_INITIALISATION, "INPUT CONTROL Analogue Configuration --");
//    
//    input_control_add_adc_setting(0X00, 0xFF);  // 00000000
//    input_control_add_adc_setting(0X01, 0x01);  // 00000000
//    input_control_add_adc_setting(0X02, 0x03);  // 00000000
//    input_control_add_adc_setting(0X03, 0x00);  // 00000000
//    input_control_add_adc_setting(0X04, 0x00);  // 00000000
//    input_control_add_adc_setting(0X05, 0x00);  // 00000000
//    input_control_add_adc_setting(0X06, 0x00);  // 00000000
//    input_control_add_adc_setting(0X07, 0x00);  // 00000000
//    input_control_apply_adc_settings();
//}
  
/**-----------------------------------------------------------------------------
 * 
 */
void configure_input_control_dac()
{
    logger_log_info(LOGGER_CODE_INITIALISATION, "INPUT CONTROL DAC Configuration --");
    
    input_control_add_dac_setting(0X00, 0x00);  // 00000000
    input_control_add_dac_setting(0X01, 0xC0);//0xCC);  // 11001100 - 32bit/I2S/Auto Select/default
    input_control_add_dac_setting(0X02, 0x34);  // 
    input_control_add_dac_setting(0X03, 0x40);  // 
    input_control_add_dac_setting(0X04, 0x00);  // 00000000 - Disables Automute
    input_control_add_dac_setting(0X05, 0x68);  // 
    input_control_add_dac_setting(0X06, 0x42);  // 
    input_control_add_dac_setting(0X07, 0x80);  // Filter - set default filter
    input_control_add_dac_setting(0X08, 0x71);  // GPIO Monitoring of DPLL_LOCK
    input_control_add_dac_setting(0X09, 0x22);  // 
    input_control_add_dac_setting(0X0A, 0x02);  // 
    input_control_add_dac_setting(0X0B, 0x00);  // 
    input_control_add_dac_setting(0X0C, 0xFA);  // HiFi PLL
    input_control_add_dac_setting(0X0D, 0x00);  // 00000000
    input_control_add_dac_setting(0X0E, 0x8a);  // 
    input_control_add_dac_setting(0X0F, 0x00);  // Volume 1  BUG3506 was 0x08
    input_control_add_dac_setting(0X10, 0x00);  // Volume 2  BUG3506 was 0x08

    input_control_apply_dac_settings();      

    // Having moved to the 100MHz crystal we can no longer just read register 0x44
    // So have to add in 0x43 as well, though 0x45 is still not needed as its 0 
    // for all the sample rates were interested in
    input_control_define_dac_sample_rate_address((REGISTER_BITS_DEFINITION[]) {
        (REGISTER_BITS_DEFINITION){ 0x42, 0, 7 },
        (REGISTER_BITS_DEFINITION){ 0x43, 0, 7 },
        (REGISTER_BITS_DEFINITION){ 0x44, 0, 7 },  
        //(REGISTER_BITS_DEFINITION){ 0x45, 0, 7 } 
        },
        3 // 4
    );

    input_control_define_custom_filter_address((REGISTER_BITS_DEFINITION[]) {
        (REGISTER_BITS_DEFINITION){ 0x28, 0, 7 },
        (REGISTER_BITS_DEFINITION){ 0x29, 0, 7 },
        (REGISTER_BITS_DEFINITION){ 0x2A, 0, 7 },  
        (REGISTER_BITS_DEFINITION){ 0x2B, 0, 7 }});
        
    // Define the address used for enabling custom filters - Note bit pos not used..
    input_control_define_custom_filter_enable_address((REGISTER_BITS_DEFINITION){ 0x2C, 0, 7 });
    
    // Setup the custom filter
    input_control_setup_custom_filter();
    
    // Define the address to check to see if the DAC has reset itself.
    // Currently we are using register 15 (Volume L) which is set to 0 on reset
    // but which we change for both the SA10 and SA20)
    input_control_define_dac_reset_check_address((REGISTER_BITS_DEFINITION){ 0x0F, 0, 7 });

    input_control_define_source_type_address((REGISTER_BITS_DEFINITION){ 0x46, 1, 1 });

    input_control_define_filter_address((REGISTER_BITS_DEFINITION){ 0x07, 5, 7 });    
}

/**-----------------------------------------------------------------------------
 * 
 */
//void configure_input_control_digital()
//{
//    logger_log_info(LOGGER_CODE_INITIALISATION, "INPUT CONTROL Digital Configuration --");
//
//    input_control_add_digital_setting(0X00, 0x23);  // 00000000
//    input_control_add_digital_setting(0X01, 0x5A);  // 00000000
//    input_control_add_digital_setting(0X02, 0x40);  // 00000000
//    input_control_add_digital_setting(0X03, 0x50);  // IP Select
//    input_control_add_digital_setting(0X04, 0xFB);  // Sample Rate Interrupt Mask
//    input_control_add_digital_setting(0X05, 0xAD);  // BUG 3558 - See Petes Email 29/07/2020 13:27 0xA4);  // Error Interrupt Mask
//    input_control_add_digital_setting(0X06, 0x1F);  // 00000000
//
//    input_control_define_digital_error_address((REGISTER_BITS_DEFINITION){ 0x07, 0, 7 }); 
//    input_control_define_digital_switch_address((REGISTER_BITS_DEFINITION){ 0x03, 0, 2 }); 
//    input_control_define_digital_sample_rate_address((REGISTER_BITS_DEFINITION){ 0x08, 4, 7 });
//
//    input_control_apply_digital_settings();    
//}

/**-----------------------------------------------------------------------------
 * Initialise the ADC
 */
void initialise_adc()
{
    logger_log_info(LOGGER_CODE_INITIALISATION, "Initialising ADC --");
    
    adc_initialise();
    adc_button_initialise();
    adc_voltage_initialise();
    adc_temperature_initialise();
    adc_pot_initialise();
    
    app_device_initialise_adc();
//    ADC_BUTTON input_buttons[ADC_MAX_BUTTON_COUNT];
//    input_buttons[0] = APP_BUTTONS_MENU;
//    input_buttons[1] = APP_BUTTONS_NONE;
//    input_buttons[2] = APP_BUTTONS_NONE;
//    input_buttons[3] = APP_BUTTONS_NONE;
//    adc_button_add_input(APP_ADC_INPUTS_BUTTONS_1, input_buttons, ADC_INPUT_STATE_ACTIVE);
//
//    adc_voltage_add_input(APP_ADC_INPUTS_BALANCE, ADC_INPUT_STATE_ACTIVE);
//
//    input_buttons[0] = APP_BUTTONS_INPUT_DOWN;
//    input_buttons[1] = APP_BUTTONS_INPUT_UP;
//    input_buttons[2] = APP_BUTTONS_DIRAC;
//    input_buttons[3] = APP_BUTTONS_MUTE;
//    adc_button_add_input(APP_ADC_INPUTS_BUTTONS_2, input_buttons, ADC_INPUT_STATE_ACTIVE);
//
//    input_buttons[0] = APP_BUTTONS_INFO;
//    input_buttons[1] = APP_BUTTONS_DIRECT;
//    input_buttons[2] = APP_BUTTONS_DISPLAY;
//    input_buttons[3] = APP_BUTTONS_BALANCE;
//    adc_button_add_input(APP_ADC_INPUTS_BUTTONS_3, input_buttons, ADC_INPUT_STATE_ACTIVE);
//    
//    adc_voltage_add_input(APP_ADC_INPUTS_BIAS_TEST_L, ADC_INPUT_STATE_INACTIVE);
//    adc_voltage_add_input(APP_ADC_INPUTS_BIAS_TEST_R, ADC_INPUT_STATE_INACTIVE);
//    adc_temperature_add_input(APP_ADC_INPUTS_OUTPUT_TEMPERATURE, ADC_INPUT_STATE_ACTIVE);
//    adc_temperature_add_input(APP_ADC_INPUTS_LIFTER_TEMPERATURE, ADC_INPUT_STATE_ACTIVE);
    
//    // Setup the scan input definitions
//    // Note: These should be specified in ascending scan order number e.g AN1, AN2, AN3
//    // otherwise the outputs will be confused because the ADC module considers each 
//    // input in channel ascending order.
//    //APP_ADC_INPUTS_BUTTONS,
//    //APP_ADC_INPUTS_OUTPUT_TEMPERATURE
//    //APP_ADC_INPUTS_LIFTER_TEMPERATURE
//
//    ADC_SCAN_INPUT_DEFINITION adc_input_defs[7];
//    adc_input_defs[0].active = ADC_INPUT_ACTIVE;
//    adc_input_defs[0].input = ADCHS_AN0;
//    adc_input_defs[0].input_upper = ADC_SW_OC_MIN;
//    adc_input_defs[0].mode = ADC_SCAN_INPUT_MODE_BUTTONS;
//    adc_input_defs[0].input_data.input_switch.switches[0].range_low = ADC_SW1_LO;
//    adc_input_defs[0].input_data.input_switch.switches[0].range_high = ADC_SW1_HI;
//    adc_input_defs[0].input_data.input_switch.switches[0].adc_switch = APP_BUTTONS_NONE;
//    adc_input_defs[0].input_data.input_switch.switches[1].range_low = ADC_SW2_LO;
//    adc_input_defs[0].input_data.input_switch.switches[1].range_high = ADC_SW2_HI;
//    adc_input_defs[0].input_data.input_switch.switches[1].adc_switch = APP_BUTTONS_NONE;
//    adc_input_defs[0].input_data.input_switch.switches[2].range_low = ADC_SW3_LO;
//    adc_input_defs[0].input_data.input_switch.switches[2].range_high = ADC_SW3_HI;
//    adc_input_defs[0].input_data.input_switch.switches[2].adc_switch = APP_BUTTONS_NONE;
//    adc_input_defs[0].input_data.input_switch.switches[3].range_low = ADC_SW4_LO;
//    adc_input_defs[0].input_data.input_switch.switches[3].range_high = ADC_SW4_HI;
//    adc_input_defs[0].input_data.input_switch.switches[3].adc_switch = APP_BUTTONS_MENU;
//
//    adc_input_defs[1].active = ADC_INPUT_ACTIVE;
//    adc_input_defs[1].input = ADCHS_AN1;
//    adc_input_defs[1].input_upper = ADC_SW_OC_MIN;
//    adc_input_defs[1].mode = ADC_SCAN_INPUT_MODE_BUTTONS;
//    adc_input_defs[1].input_data.input_switch.switches[0].range_low = ADC_SW1_LO;
//    adc_input_defs[1].input_data.input_switch.switches[0].range_high = ADC_SW1_HI;
//    adc_input_defs[1].input_data.input_switch.switches[0].adc_switch = APP_BUTTONS_MUTE;
//    adc_input_defs[1].input_data.input_switch.switches[1].range_low = ADC_SW2_LO;
//    adc_input_defs[1].input_data.input_switch.switches[1].range_high = ADC_SW2_HI;
//    adc_input_defs[1].input_data.input_switch.switches[1].adc_switch = APP_BUTTONS_DIRAC;
//    adc_input_defs[1].input_data.input_switch.switches[2].range_low = ADC_SW3_LO;
//    adc_input_defs[1].input_data.input_switch.switches[2].range_high = ADC_SW3_HI;
//    adc_input_defs[1].input_data.input_switch.switches[2].adc_switch = APP_BUTTONS_INPUT_UP;
//    adc_input_defs[1].input_data.input_switch.switches[3].range_low = ADC_SW4_LO;
//    adc_input_defs[1].input_data.input_switch.switches[3].range_high = ADC_SW4_HI;
//    adc_input_defs[1].input_data.input_switch.switches[3].adc_switch = APP_BUTTONS_INPUT_DOWN;
//
//    adc_input_defs[2].active = ADC_INPUT_ACTIVE;
//    adc_input_defs[2].input = ADCHS_AN2;
//    adc_input_defs[2].input_upper = ADC_SW_OC_MIN;
//    adc_input_defs[2].mode = ADC_SCAN_INPUT_MODE_BUTTONS;
//    adc_input_defs[2].input_data.input_switch.switches[0].range_low = ADC_SW1_LO;
//    adc_input_defs[2].input_data.input_switch.switches[0].range_high = ADC_SW1_HI;
//    adc_input_defs[2].input_data.input_switch.switches[0].adc_switch = APP_BUTTONS_BALANCE;
//    adc_input_defs[2].input_data.input_switch.switches[1].range_low = ADC_SW2_LO;
//    adc_input_defs[2].input_data.input_switch.switches[1].range_high = ADC_SW2_HI;
//    adc_input_defs[2].input_data.input_switch.switches[1].adc_switch = APP_BUTTONS_DISPLAY;
//    adc_input_defs[2].input_data.input_switch.switches[2].range_low = ADC_SW3_LO;
//    adc_input_defs[2].input_data.input_switch.switches[2].range_high = ADC_SW3_HI;
//    adc_input_defs[2].input_data.input_switch.switches[2].adc_switch = APP_BUTTONS_DIRECT;
//    adc_input_defs[2].input_data.input_switch.switches[3].range_low = ADC_SW4_LO;
//    adc_input_defs[2].input_data.input_switch.switches[3].range_high = ADC_SW4_HI;
//    adc_input_defs[2].input_data.input_switch.switches[3].adc_switch = APP_BUTTONS_INFO;
//
//    adc_input_defs[3].active = ADC_INPUT_INACTIVE;
//    adc_input_defs[3].input = ADCHS_AN3;
//    adc_input_defs[3].mode = ADC_SCAN_INPUT_MODE_VOLTAGE;
//
//    adc_input_defs[4].active = ADC_INPUT_INACTIVE;
//    adc_input_defs[4].input = ADCHS_AN4;
//    adc_input_defs[4].mode = ADC_SCAN_INPUT_MODE_VOLTAGE;
//
//    adc_input_defs[4].active = ADC_INPUT_ACTIVE;
//    adc_input_defs[5].input = ADCHS_AN16;
//    adc_input_defs[5].mode = ADC_SCAN_INPUT_MODE_TEMPERATURE;
//
//    adc_input_defs[4].active = ADC_INPUT_ACTIVE;
//    adc_input_defs[5].input = ADCHS_AN17;
//    adc_input_defs[5].mode = ADC_SCAN_INPUT_MODE_TEMPERATURE;
//
//    // Initialise the ADC
//    adc_initialise(adc_input_defs, 7);
}
 
/**-----------------------------------------------------------------------------
 * Initialises Avatar
 */
void initialise_avatar()
{
    logger_log_info(LOGGER_CODE_INITIALISATION, "Initialising Avatar ");
    
    avatar_initialise(AVATAR_DEVICE_ARCAM_SA30, 
            app_commands_avatar_message_callback,
            AVATAR_STATE_SUBSCRIPTIONS,
            (uint8_t)(sizeof(AVATAR_STATE_SUBSCRIPTIONS)/sizeof(AVATAR_STATE_ID)));

#if APP_FEATURE_HDMI_ARC == APP_FEATURE_ON  
    // Note the number of images here should be <= (ideally =) AVATAR_UPGRADE_IMAGE_NUM_IMAGES 
    AVATAR_UPGRADE_IMAGE_ACTIONS actions[] = {
        {   
            "SA30", 
            app_commands_check_upgrade_version,
            upgrade_metadata_prepare_upload, 
            upgrade_metadata_upload_data, 
            upgrade_metadata_upload_end
        },
        { 
            "HDMI_ARC", 
            app_commands_hdmi_arc_check_upgrade_version,
            app_commands_hdmi_arc_upload_prepare, 
            app_commands_hdmi_arc_upload_data, 
            app_commands_hdmi_arc_upload_end
        }
    }; 
    
    avatar_message_initialise(actions, 2);
#else    
    // Note the number of images here should be <= (ideally =) AVATAR_UPGRADE_IMAGE_NUM_IMAGES 
    AVATAR_UPGRADE_IMAGE_ACTIONS actions[] = {
        {   
            "SA30", 
            app_commands_check_upgrade_version,
            upgrade_metadata_prepare_upload, 
            upgrade_metadata_upload_data, 
            upgrade_metadata_upload_end
        },
    }; 
    
    avatar_message_initialise(actions, 1);
#endif
}

/**-----------------------------------------------------------------------------
 * Sets up the display for the type of display used
 */
void initialise_display()
{
    logger_log_info(LOGGER_CODE_INITIALISATION, "Initialising Display --");
    
    settings_suppress_save(true);
    
    // Set the brightness
    //APP_COMMAND_BRIGHTNESS brightness;
    uint8_t brightness;
    settings_get(SETTING_BRIGHTNESS, &brightness);
    app_commands_set_brightness((APP_COMMAND_BRIGHTNESS)brightness);
    
    // Setup the display
    display_initialise();
            
    settings_suppress_save(false);
    
    timers_reset(TIMER_INDEX_BRIGHTNESS);
}
 
/**-----------------------------------------------------------------------------
 * Sets up the encoder
 */
void initialise_encoder()
{
    logger_log_info(LOGGER_CODE_INITIALISATION, "Initialising Encoder --");
    
    encoder_initialise();
 
    // Set the main encoder to display the volume
    // This will involve setting its default value
    ENCODER_STATE state;
    encoder_get_state(&state, ENCODER_INDEX_MAIN);
    
    state.current_value = volume_get();
    encoder_set(state, ENCODER_INDEX_MAIN, encoder_get_wrap(ENCODER_INDEX_MAIN));
    input_context_set_encoder(&state, ENCODER_INDEX_MAIN);
    
    // Set the input encoder to display the input source
    // This will involve setting its default value
    encoder_get_state(&state, ENCODER_INDEX_INPUT_SELECT);
    
    state.current_value = input_control_app_get_edit_index();
    encoder_set(state, ENCODER_INDEX_INPUT_SELECT, encoder_get_wrap(ENCODER_INDEX_INPUT_SELECT));
    input_context_set_encoder(&state, ENCODER_INDEX_INPUT_SELECT); 
}

/**-----------------------------------------------------------------------------
 * Initialise flash memory and allocate space for clients
 */
void initialise_flash_memory()
{
    flash_memory_initialise();
    //flash_memory_add_client(FLASH_MEMORY_CLIENT_SETTINGS, SETTINGS_STORAGE_SIZE, &flash_memory_dummy_callback);
    //flash_memory_add_client(FLASH_MEMORY_CLIENT_UPGRADE, UPGRADE_STORAGE_NAME, UPGRADE_STORAGE_SIZE, &flash_memory_dummy_callback);
}
    
/**-----------------------------------------------------------------------------
 * Set up i2c
 */
void initialise_i2c()
{
    logger_log_info(LOGGER_CODE_INITIALISATION, "Initialising I2C --");
        
    i2c_bit_bash_initialise();
    
    I2C_BIT_BASH_MODULE input_control_module = 
    {
        .index = INPUT_CONTROL_DAC_I2C_MODULE,
        .scl_pin.port = I2C_BIT_BASH_SCL_MODULE_0_PORT_ID,
        .scl_pin.channel = I2C_BIT_BASH_SCL_MODULE_0_CHANNEL,
        .scl_pin.pin = I2C_BIT_BASH_SCL_MODULE_0_PIN,
        .sda_pin.port = I2C_BIT_BASH_SDA_MODULE_0_PORT_ID,
        .sda_pin.channel = I2C_BIT_BASH_SDA_MODULE_0_CHANNEL,
        .sda_pin.pin = I2C_BIT_BASH_SDA_MODULE_0_PIN,
    };

    i2c_bit_bash_add_module(&input_control_module);

    i2c_initialise();
    
    // This could be the ADC/DAC or SPDIF module theyre all the same
    i2c_set_module_type(INPUT_CONTROL_DAC_I2C_MODULE, I2C_MODULE_TYPE_BIT_BASH);
    //i2c_set_module_type(AVATAR_I2C_MODULE, I2C_MODULE_TYPE_HARMONY);
    
    // Initialise I2C Slave
    i2c_slave_initialise();
    
    I2C_SLAVE_MODULE module = 
    {
      .address_7bit = 0x38,
      .callback = avatar_i2c_slave_callback,
      .module_id = I2C_ID_2,
    };
    
    i2c_slave_add_module(AVATAR_I2C_MODULE, &module);
}

/** ----------------------------------------------------------------------------
 * Set up input controls
 */
void initialise_input_control()
{
    logger_log_info(LOGGER_CODE_INITIALISATION, "Initialising Input Control --");

    input_control_initialise();
    
//    input_control_analogue_set_callbacks(configure_input_control_analogue);
    input_control_dac_set_callbacks(configure_input_control_dac);
//    input_control_digital_set_callbacks(configure_input_control_digital);
    input_control_set_switch_callback(app_commands_input_switch_callback);

    // Set the dac into reset mode during power up
    //input_control_adc_reset(INPUT_CONTROL_SWITCH_ON);  
    input_control_dac_reset(INPUT_CONTROL_SWITCH_OFF);  
    spdif_rx_reset(INPUT_CONTROL_SWITCH_OFF);
    
#if APP_FEATURE_HDMI_ARC == APP_FEATURE_ON  
    hdmi_arc_initialise(app_commands_hdmi_arc_callback);
#endif
}

/**-----------------------------------------------------------------------------
 * Sets up the logger 
 */
void initialise_logger(LOGGER_CALLBACK callback)
{
    logger_initialise();
    logger_set_callback(callback);
}

/**-----------------------------------------------------------------------------
 * Sets up the mutes 
 */
void initialise_mutes()
{
    logger_log_info(LOGGER_CODE_INITIALISATION, "Initialising Mutes --");
    
    mutes_initialise();

    output_control_mute_outputs(true);
    
    volume_mute();  
}

/**-----------------------------------------------------------------------------
 * Sets up TCPIP/UDP/UPNP/Control4 etc
 */
void initialise_network(STRUCTURED_MESSAGE_CALLBACK callback)
{
    // Initialise TCP/IP
    initialise_tcpip(callback);
}

/**-----------------------------------------------------------------------------
 * Setup pins for standby
 */
void initialise_pins()
{
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_0, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_1, STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_2, STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_3, STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_4, STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_5, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_6, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_7, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_9, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_10, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_14, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_15, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    

    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_0, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_1, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_2, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_3, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_4, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_5, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_8, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_9, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_10, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_11, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_12, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_13, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_14, STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_15, STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    

    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_C, PORTS_BIT_POS_1, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_C, PORTS_BIT_POS_2, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_C, PORTS_BIT_POS_3, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_C, PORTS_BIT_POS_4, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_C, PORTS_BIT_POS_12, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_C, PORTS_BIT_POS_13, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_C, PORTS_BIT_POS_14, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_C, PORTS_BIT_POS_15, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
   
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_0, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_1, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_2, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_3, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_4, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_5, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_9, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_10, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_11, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_12, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_13, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_14, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_15, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    

    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_E, PORTS_BIT_POS_0, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_E, PORTS_BIT_POS_1, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_E, PORTS_BIT_POS_2, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_E, PORTS_BIT_POS_3, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_E, PORTS_BIT_POS_4, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_E, PORTS_BIT_POS_5, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_E, PORTS_BIT_POS_6, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_E, PORTS_BIT_POS_7, STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_E, PORTS_BIT_POS_8, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_E, PORTS_BIT_POS_9, STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, true);    

    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_F, PORTS_BIT_POS_0, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_F, PORTS_BIT_POS_1, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_F, PORTS_BIT_POS_4, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_F, PORTS_BIT_POS_5, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_F, PORTS_BIT_POS_12, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_F, PORTS_BIT_POS_13, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    

    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_G, PORTS_BIT_POS_0, STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, true);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_G, PORTS_BIT_POS_1, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_G, PORTS_BIT_POS_6, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //TODO Put in for release version ---RS232 Debugger pins_add_pin(PORTS_ID_0, PORT_CHANNEL_G, PORTS_BIT_POS_7, STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, true);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_G, PORTS_BIT_POS_8, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_G, PORTS_BIT_POS_9, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_G, PORTS_BIT_POS_12, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_G, PORTS_BIT_POS_13, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_G, PORTS_BIT_POS_14, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
    //pins_add_pin(PORTS_ID_0, PORT_CHANNEL_G, PORTS_BIT_POS_15, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    

//    // GPIO - Input Select
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_C, PORTS_BIT_POS_13, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_C, PORTS_BIT_POS_14, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_3, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
//    
//    // VFD
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_8, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_F, PORTS_BIT_POS_13, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
//    
//    // VOL MUTE
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_G, PORTS_BIT_POS_13, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
//    
//    // VOL
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_G, PORTS_BIT_POS_12, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_E, PORTS_BIT_POS_2, STANDBY_SLEEP_STATE_ACTIVE | STANDBY_SLEEP_STATE_IDLE | STANDBY_SLEEP_STATE_SLEEP, false);    
//    
//    // ARCTX
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_G, PORTS_BIT_POS_0, STANDBY_SLEEP_STATE_SLEEP, true);     // Force set to input on SLEEP
//    
//    // SUE I2C
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_2, STANDBY_SLEEP_STATE_SLEEP, false);    
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_3, STANDBY_SLEEP_STATE_SLEEP, false);    
//    
//    // HOST IRQ
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_E, PORTS_BIT_POS_9, STANDBY_SLEEP_STATE_SLEEP, false);    
//    
//    // SCL/SDA
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_14, STANDBY_SLEEP_STATE_SLEEP, false);    
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_15, STANDBY_SLEEP_STATE_SLEEP, false);    
//    
//    // DIG IP SEL
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_B, PORTS_BIT_POS_12, STANDBY_SLEEP_STATE_SLEEP, false);    
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_10, STANDBY_SLEEP_STATE_SLEEP, false);    
//    
//    // Avatar RST
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_A, PORTS_BIT_POS_4, STANDBY_SLEEP_STATE_SLEEP, false);   
//    
//    // USB
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_9, STANDBY_SLEEP_STATE_SLEEP, true);    
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_15, STANDBY_SLEEP_STATE_SLEEP, true);    
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_F, PORTS_BIT_POS_5, STANDBY_SLEEP_STATE_SLEEP, true);    
//    pins_add_pin(PORTS_ID_0, PORT_CHANNEL_D, PORTS_BIT_POS_1, STANDBY_SLEEP_STATE_SLEEP, true);    
}

/**-----------------------------------------------------------------------------
 * Setup and enable Power
 */
void initialise_power()
{
    logger_log_info(LOGGER_CODE_INITIALISATION, "Initialising Power --");
    
    power_initialise(true);

    // Power up                                             
    power_enable();
            
    // Send unsolicited power message
    app_commands_request_status(APP_COMMANDS_POWER, app_commands_power);

    // Start wait timer
    timers_reset(TIMER_INDEX_POWER_CYCLE);
}
    
/**-----------------------------------------------------------------------------
 * Setup RGB LED 
 */
void initialise_rgb_led()
{
    rgb_led_initialise();
    
    // Adjust red and white colours for CDS50
    // Removed 04/10/2019 - Pete will adjust voltage instead as we can't PWN in standby
    //rgb_led_adjust_colour(RGB_LED_COLOUR_RED, 26, 0, 0);
    rgb_led_adjust_colour(RGB_LED_COLOUR_WHITE, 0, 0, 6);

    // Set the power LED
    rgb_led_set(RGB_LED_COLOUR_RED);    
}

/**-----------------------------------------------------------------------------
 * Initialise rs232 functions
 */
void initialise_rs232()
{
    logger_log_info(LOGGER_CODE_INITIALISATION, "Initialising RS232 --");
    
    // Initialise RS232 port
    rs232_initialise(DRV_USART_INDEX_0, app_commands_structured_message_callback);
    
#if APP_FEATURE_HDMI_ARC == APP_FEATURE_ON  
    // Initialise RS232 port
    rs232_hdmi_arc_initialise(DRV_USART_INDEX_1, hdmi_arc_processing_callback);
#endif
}

/**-----------------------------------------------------------------------------
 * Sets up the settings 
 */
void initialise_settings(SETTINGS_INITIALISE_CALLBACK callback)
{
    logger_log_info(LOGGER_CODE_INITIALISATION, "Initialising Settings --");
    
    settings_initialise(callback);
}

/**-----------------------------------------------------------------------------
 * This method is where default application settings should be set.  This is called 
 * the firinitialise_settings_defaust time the eeprom is accessed for the settings in  order to set any 
 * default values so that they are stored.
 */
void initialise_settings_defaults()
{
    logger_log_info(LOGGER_CODE_INITIALISATION, "SETUP Initialising Defaults ------------- ");
    
    // Software version
    uint8_t version_major = SOFTWARE_VERSION_MAJOR;
    uint8_t version_minor = SOFTWARE_VERSION_MINOR;
    settings_set(SETTING_SOFTWARE_VERSION_MAJOR, &version_major);
    settings_set(SETTING_SOFTWARE_VERSION_MINOR, &version_minor);
    settings_set(SETTING_UPGRADE_SOFTWARE_VERSION_MAJOR, &version_major);
    settings_set(SETTING_UPGRADE_SOFTWARE_VERSION_MINOR, &version_minor);
    
    volume_set(VOLUME_DEFAULT_VOLUME, true);
    volume_set_balance(VOLUME_DEFAULT_BALANCE, true);
    volume_set_max_on_volume(VOLUME_DEFAULT_VOLUME);
    volume_set_maximum_volume(VOLUME_MAX, VOLUME_INPUT_DEFAULT);
    volume_set_maximum_volume(VOLUME_STREAMING_MAX, VOLUME_INPUT_STREAMER);
    
    app_commands_set_brightness(APP_COMMAND_BRIGHTNESS_FULL);
    power_set_auto_shutdown_timeout(APP_COMMAND_AUTO_SHUTDOWN_TIMEOUT_VALUES[APP_COMMAND_AUTO_SHUTDOWN_TIMEOUT_20MIN]);//POWER_AUTO_TIMEOUT_INCREMENT); // 30 mins
    remote_set_system_code(REMOTE_SYSTEM_CODE_DEFAULT);
    
    bool dark_mode = false;
    settings_set(SETTING_DARK_MODE, &dark_mode);
    
    AUDIO_INPUT input = AUDIO_INPUT_CD;
    settings_set(SETTING_ACTIVE_INPUT, &input);
    settings_set(SETTING_PREVIOUS_INPUT, &input);
    input_control_set_processor_mode_input(AUDIO_INPUT_NONE);
    input_control_set_processor_mode_volume(VOLUME_DEFAULT_VOLUME);
    
//TODO REMOVE     input_control_usb_set_record(false);
    
    // Set standby modes to go into SLEEP by default
    power_set_network_active_in_standby(false);
    power_set_rs232_active_in_standby(false);
    
    machine_uuid_clear();
    
    output_control_set_headphone_override(false);
    
    // Set Shutdown code
    SYSTEM_FAULT_CODE fault_code = SYSTEM_FAULT_CODE_NO_FAULT;
    settings_set(SETTING_SHUTDOWN_CODE, &fault_code);
    
    SYSTEM_FAULT_CODE fault_code_history[SHUTDOWN_CODE_HISTORY_LENGTH];
    memset(fault_code_history, (uint8_t)SYSTEM_FAULT_CODE_NO_FAULT, sizeof(SYSTEM_FAULT_CODE) * SHUTDOWN_CODE_HISTORY_LENGTH);
    settings_set(SETTING_SHUTDOWN_CODE_HISTORY, &fault_code_history);

    LOGGER_TYPE logger_type = LOGGER_TYPE_NONE;
    settings_set(SETTING_LOGGER_TYPE, &logger_type);
    
    // Filter defaults see input_control.initialise())
    FILTER_TYPE filter_type = (FILTER_TYPE)6; // BUG3436 - Set to aphodizing --;   //MinP Slow;
    settings_set(SETTING_FILTER_TYPE, &filter_type);
    
    // Analogue Direct
    input_analogue_direct_reset();
    
    // Input Enabled
    uint8_t input_enabled_settings[INPUT_CONTROL_NUM_SETTINGS];
    memset(input_enabled_settings, 1, sizeof(uint8_t) * INPUT_CONTROL_NUM_SETTINGS);
    settings_set(SETTING_INPUT_ENABLED, input_enabled_settings);
    
    // Control 4 SDDP active
    bool control4 = false;
    settings_set(SETTING_CONTROL4_SDDP_ACTIVE, &control4);
    
    //TODO REMOVE - Crestron 29/01/2020
    // Crestron active
    //bool crestron = false;
    //settings_set(SETTING_CRESTRON_DISCOVERY_ACTIVE, &crestron);
  
    // Dirac
    uint8_t dirac_eq_curve_enabled_settings[DIRAC_MAX_EQ_CURVES];
    memset(dirac_eq_curve_enabled_settings, 0, sizeof(uint8_t) * DIRAC_MAX_EQ_CURVES);
    dirac_eq_curve_enabled_settings[0] = true;  
    settings_set(SETTING_DIRAC_CURVE_ENABLED, dirac_eq_curve_enabled_settings);
    
    uint8_t dirac_eq_curve_settings[INPUT_CONTROL_NUM_SETTINGS];
    memset(dirac_eq_curve_settings, 0, sizeof(uint8_t) * INPUT_CONTROL_NUM_SETTINGS);
    settings_set(SETTING_DIRAC_CURVE, dirac_eq_curve_settings);
    
    avatar_set_upgrade_option(AVATAR_UPGRADE_OPTION_OTA_ENABLED);
    
    PHONO_INPUT phono_input = PHONO_INPUT_MM;
    settings_set(SETTING_MM_MC, &phono_input);

    bool upgrade_status = false; 
    settings_set(SETTING_UPGRADE_STATUS, &upgrade_status);
    
    bool sleep = false;
    settings_set(SETTING_UPGRADE_SLEEP_ON_RESTART, &sleep);
    
    bool check_channel_status = true;
    settings_set(SETTING_CHECK_CHANNEL_STATUS, &check_channel_status);
    
#if APP_FEATURE_HDMI_ARC == APP_FEATURE_ON  
    hdmi_arc_set_audio_control_setting(HDMI_ARC_CONTROL_STATE_OFF);
    hdmi_arc_set_power_control_setting(HDMI_ARC_CONTROL_STATE_AUTO);
#endif
    
    bool short_circuit_sense = true;
    settings_set(SETTING_SHORT_CIRCUIT_SENSE, &short_circuit_sense);

    uint8_t mqa_decoder_level = AVATAR_MQA_DECODER_LEVEL_FULL;
    settings_set(SETTING_MQA_DECODER_LEVEL, &mqa_decoder_level);
    
    uint8_t lip_sync_settings[INPUT_CONTROL_NUM_SETTINGS];
    memset(lip_sync_settings, 0, sizeof(uint8_t) * INPUT_CONTROL_NUM_SETTINGS);
    settings_set(SETTING_LIP_SYNC, lip_sync_settings);

    logger_log_info(LOGGER_CODE_INITIALISATION, "SETUP Initialising Defaults End ---------- ");
}

/**-----------------------------------------------------------------------------
 * Checks that the stored software version matches this one and performs any 
 * upgrade specific work
 */
void initialise_software_version()
{
    // Do any work that involves checking the previous software version stored in the settings
    uint8_t version_major;
    uint8_t version_minor;
    settings_get(SETTING_SOFTWARE_VERSION_MAJOR, &version_major);
    settings_get(SETTING_SOFTWARE_VERSION_MINOR, &version_minor);

    logger_log_infof(LOGGER_CODE_SOFTWARE_VERSION, "-------------------------------------------------- --");
    logger_log_infof(LOGGER_CODE_SOFTWARE_VERSION, "Settings Software Version - %d.%d --", version_major, version_minor);
    logger_log_infof(LOGGER_CODE_SOFTWARE_VERSION, "Current Software Version - %d.%d --", SOFTWARE_VERSION_MAJOR, SOFTWARE_VERSION_MINOR);
    logger_log_infof(LOGGER_CODE_SOFTWARE_VERSION, "-------------------------------------------------- --");
    
    // Do the conversion work here    
    if ((version_major <= 1) && (version_minor < 15))
    {
        // Setup HDMI ARC settings
        logger_log_debugf(LOGGER_CODE_SOFTWARE_VERSION, "VERSION Setting HDMI ARC CONTROL Defaults --");
#if APP_FEATURE_HDMI_ARC == APP_FEATURE_ON  
        hdmi_arc_set_audio_control_setting(HDMI_ARC_CONTROL_STATE_OFF);
        hdmi_arc_set_power_control_setting(HDMI_ARC_CONTROL_STATE_AUTO);
#endif
    }
    else if ((version_major <= 1) && (version_minor < 23))
    {
        logger_log_debugf(LOGGER_CODE_SOFTWARE_VERSION, "VERSION Setting SHORT_CIRCUIT_SENSE Defaults (Low) --");
        // Set SHORT_CIRCUIT_SENSE
        bool short_circuit_sense = false;
        settings_set(SETTING_SHORT_CIRCUIT_SENSE, &short_circuit_sense);
    }
    else if ((version_major <= 1) && (version_minor <= 25))
    {
        logger_log_info(LOGGER_CODE_INITIALISATION, "APP: Initialising MQA DECODER LEVEL --");
        // Setup thje default MQA Decoder level
        uint8_t mqa_decoder_level = AVATAR_MQA_DECODER_LEVEL_FULL;
        settings_get(SETTING_MQA_DECODER_LEVEL, &mqa_decoder_level);
        
        if (mqa_decoder_level >= AVATAR_MQA_DECODER_LEVEL_MAX)
        {
            mqa_decoder_level = AVATAR_MQA_DECODER_LEVEL_FULL;
            settings_set(SETTING_MQA_DECODER_LEVEL, &mqa_decoder_level);
        }
    }
    else if ((version_major <= 1) && (version_minor <= 26))
    {
        uint8_t lip_sync_settings[INPUT_CONTROL_NUM_SETTINGS];
        memset(lip_sync_settings, 0, sizeof(uint8_t) * INPUT_CONTROL_NUM_SETTINGS);
        settings_set(SETTING_LIP_SYNC, lip_sync_settings);
    }
    else if ((version_major == 1) && (version_minor == 33))
    {
        
    }
    
    if ((version_major == 1) && (version_minor <= 40))
    {
        // Check we're updating
        if ((version_major != SOFTWARE_VERSION_MAJOR) || (version_minor != SOFTWARE_VERSION_MINOR))
        {
            // Set the MUTE_ON_STARTUP setting
            bool mute_on_startup = false;
            logger_log_infof(LOGGER_CODE_INITIALISATION, "APP: Initialising MUTE_ON_STARTUP %d --", mute_on_startup);
            settings_set(SETTING_MUTE_ON_STARTUP, &mute_on_startup);
        }
    }

    // Finally update the software version
    version_major = SOFTWARE_VERSION_MAJOR;
    version_minor = SOFTWARE_VERSION_MINOR;
    settings_suppress_save(true);
    settings_set(SETTING_SOFTWARE_VERSION_MAJOR, &version_major);
    settings_set(SETTING_SOFTWARE_VERSION_MINOR, &version_minor);
    settings_suppress_save(false);
}

/**-----------------------------------------------------------------------------
 * sets up special switch data
 */
#if APP_FEATURE_SPECIAL_SWITCHES == APP_FEATURE_ON    

void initialise_special_switches()
{
    special_switches_initialise();
    
    SPECIAL_SWITCH_COMBINATION combination;
    
    combination.switch1 = APP_BUTTONS_MENU;
    combination.switch2 = APP_BUTTONS_INPUT_DOWN;
    combination.switch3 = APP_BUTTONS_DIRECT;
    combination.function = app_commands_launch_engineering_menu;
            
    special_switches_add(&combination);    
    
    combination.switch1 = APP_BUTTONS_MENU;
    combination.switch2 = APP_BUTTONS_INPUT_DOWN;
    combination.switch3 = APP_BUTTONS_INFO;
    combination.function = app_commands_launch_software_version_menu;
    
    special_switches_add(&combination);
   
    combination.switch1 = APP_BUTTONS_MUTE;
    combination.switch2 = APP_BUTTONS_INPUT_DOWN;
    combination.switch3 = APP_BUTTONS_DISPLAY;
    combination.function = app_commands_launch_reset_menu;
    
    special_switches_add(&combination);
}

#endif
/**-----------------------------------------------------------------------------
 * Sets up TCP/IP
 */
void initialise_tcpip(STRUCTURED_MESSAGE_CALLBACK callback)
{
    logger_log_info(LOGGER_CODE_INITIALISATION, "Initialising TCP/IP --");
    
    tcpip_initialise(0, callback, NULL);//control4_sddp_tcpip_new_connection);
    
    tcpip_connect();
}  

/**-----------------------------------------------------------------------------
 * Initialise timers
 */
void initialise_timers()
{
    logger_log_info(LOGGER_CODE_INITIALISATION, "Initialising Timers --");
    
    // Initialise the timers module
    // Look in timer1msCallback for the timers_process() method
    timers_initialise();
    
    // Add timer modules
    // note DRV_TMR_INDEX_N - Timer Instance N
    // We have allocated Timer 0 to the System Timers Service MEANING that we need
    // 3 timer instances setting up in Harmony
    short timer_1ms_handle = timers_add_timer(DRV_TMR_INDEX_1, 1000);
    short timer_100us_handle = timers_add_timer(DRV_TMR_INDEX_2, 10000);
    
    int x;
    for (x = 0; x < TIMER_INDEX_MAX; x++)
    {
        switch(TIMER_TIMEOUT_HANDLES[x])
        {
            case TIMERS_HANDLE_100US:
            {
                timers_set_timeout(timer_100us_handle, x, TIMER_TIMEOUT_VALUES[x], TIMER_TIMEOUT_MODES[x]);
                break;
            }
            
            default:
            {
                timers_set_timeout(timer_1ms_handle, x, TIMER_TIMEOUT_VALUES[x], TIMER_TIMEOUT_MODES[x]);
                break;
            }
        }
    }

    timers_add_expired_event(TIMER_INDEX_RGB_PWM, rgb_led_process_interrupt);
    timers_reset(TIMER_INDEX_RGB_PWM);      // Start the PWM timer now the interrupt has been set
}

/* *****************************************************************************
 End of File
 */
