/* ************************************************************************** */
/** Descriptive File Name

  @Company
    Company Name

  @File Name
    app_menu.c

  @Summary
    Brief description of the file.

  @Description
    Describe the purpose of this file.
 */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: Included Files                                                    */
/* ************************************************************************** */
/* ************************************************************************** */

#include "app_menu.h"
#include <string.h>
#include <stdio.h>
#include <stdbool.h>
#include "app_context.h"
#include "app_commands.h"
#include "app_commands_auto_shutdown.h"
#include "app_commands_avatar.h"
#include "app_commands_browsing.h"
#include "app_commands_control4.h"
//TODO REMOVE - Creatron 29/01/2020#include "app_commands_crestron.h"
#include "app_commands_display.h"
#include "app_commands_engineering.h"
#include "app_commands_hdmi_arc.h"
#include "app_commands_input.h"
#include "app_commands_lip_sync.h"
//#include "app_commands_menu.h"
#include "app_commands_network.h"
#include "app_commands_output.h"
#include "app_commands_processor_mode.h"
#include "app_commands_system.h"
#include "app_commands_volume.h"
#include "utilities/avatar_message_processing.h"
#include "utilities/display.h"
#include "utilities/display_test.h"
#include "utilities/hdmi_arc_processing.h"
#include "utilities/input_analogue_direct_app.h"
#include "utilities/input_control.h"
#include "utilities/input_control.h"
#include "utilities/input_control_app.h"
#include "utilities/input_phono_input_app.h"
#include "utilities/logger/logger.h"
#include "utilities/menu.h"
#include "utilities/menu_app.h"
#include "utilities/mutes.h"
#include "utilities/menu_display.h"
#include "utilities/output_control.h"
#include "utilities/power.h"
#include "utilities/remote.h"
#include "utilities/stateful_process.h"
#include "utilities/tcpip.h"
#include "utilities/volume.h"

/* ************************************************************************** */
/* ************************************************************************** */
/* Section: File Scope or Global Data                                         */
/* ************************************************************************** */
/* ************************************************************************** */

//! Declare an array of MENU_ITEMS, one for each menu.  This is passed to menu_initialise())
MENU_ITEM app_menu_items[APP_COMMAND_MENU_SIZE];

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Local Functions                                                   */
/* ************************************************************************** */
/* ************************************************************************** */

/* ************************************************************************** */
/* ************************************************************************** */
// Section: Interface Functions                                               */
/* ************************************************************************** */
/* ************************************************************************** */

/**-----------------------------------------------------------------------------
 * Initialises the menu - sets up the menu items
 */
void app_menu_initialise()
{
    MENU_ITEM* item;
    
    menu_initialise(&app_menu_items[0], app_menu_titles, APP_COMMAND_MENU_SIZE);
    menu_display_initialise(APP_COMMAND_MENU_SYSTEM_SETTINGS);
    
    item = menu_get_item(APP_COMMAND_MENU_VOLUME);
    menu_initialise_item(item, APP_COMMAND_MENU_VOLUME, true);
    menu_add_draw_function(item, (void*)app_commands_draw_volume);
    
    //BROWSE
    // Add on_enter to initiate browsing
    item = menu_get_item(APP_COMMAND_MENU_MEDIA_BROWSING);
    menu_initialise_item(item, APP_COMMAND_MENU_MEDIA_BROWSING, true);
    menu_add_draw_function(item, (void*)app_commands_draw_browsing); // Needs changing for bespoke NET menu for browsing
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_browsing, (void*)app_commands_exit_browsing);
    
    // Setup SYSTEM_SETTINGS MENU-----------------------------------------------
    //--------------------------------------------------------------------------
    MENU_ITEM* system_settings_item = menu_get_item(APP_COMMAND_MENU_SYSTEM_SETTINGS);
    menu_initialise_item(system_settings_item, APP_COMMAND_MENU_SYSTEM_SETTINGS, true);
    menu_add_child(NULL, system_settings_item, true);
   
    item = menu_get_item(APP_COMMAND_MENU_DISPLAY);
    menu_initialise_item(item, APP_COMMAND_MENU_DISPLAY, true);
    menu_add_display_function(item, (void*)app_commands_get_brightness_string);
    menu_add_edit_function(item, (void*)app_commands_edit_display_brightness, true, false);
    menu_add_child(system_settings_item, item, true);
    
    // Version of the Display menu without navigation keys used when editing via buttons, remote or RS232/TCPIP
    item = menu_get_item(APP_COMMAND_MENU_DISPLAY_NO_NAVIGATION);
    menu_initialise_item(item, APP_COMMAND_MENU_DISPLAY_NO_NAVIGATION, true);
    menu_add_draw_function(item, (void*)app_commands_draw_display_no_navigation);
    menu_add_edit_function(item, (void*)app_commands_edit_display_brightness, true, false);
   
    item = menu_get_item(APP_COMMAND_MENU_DARK_MODE);
    menu_initialise_item(item, APP_COMMAND_MENU_DARK_MODE, true);
    menu_add_display_function(item, (void*)app_commands_get_dark_mode_string);
    menu_add_edit_function(item, (void*)app_commands_edit_dark_mode, true, false);
    menu_add_child(system_settings_item, item, false);

    // Enabled Inputs
    MENU_ITEM* input_enabled_item = menu_get_item(APP_COMMAND_MENU_INPUT_ENABLED);
    menu_initialise_item(input_enabled_item, APP_COMMAND_MENU_INPUT_ENABLED, false);
    menu_add_child(system_settings_item, input_enabled_item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_INPUT_ENABLED_PHONO);
    menu_initialise_item(item, APP_COMMAND_MENU_INPUT_ENABLED_PHONO, true);
    menu_add_display_function(item, (void*)app_commands_get_input_enabled_string);
    menu_add_edit_function(item, (void*)app_commands_edit_input_enabled, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, NULL);
    menu_add_child(input_enabled_item, item, true);
    
    item = menu_get_item(APP_COMMAND_MENU_INPUT_ENABLED_AUX);
    menu_initialise_item(item, APP_COMMAND_MENU_INPUT_ENABLED_AUX, true);
    menu_add_display_function(item, (void*)app_commands_get_input_enabled_string);
    menu_add_edit_function(item, (void*)app_commands_edit_input_enabled, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, NULL);
    menu_add_child(input_enabled_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_INPUT_ENABLED_CD);
    menu_initialise_item(item, APP_COMMAND_MENU_INPUT_ENABLED_CD, true);
    menu_add_display_function(item, (void*)app_commands_get_input_enabled_string);
    menu_add_edit_function(item, (void*)app_commands_edit_input_enabled, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, NULL);
    menu_add_child(input_enabled_item, item, false);
     
    item = menu_get_item(APP_COMMAND_MENU_INPUT_ENABLED_PVR);
    menu_initialise_item(item, APP_COMMAND_MENU_INPUT_ENABLED_PVR, true);
    menu_add_display_function(item, (void*)app_commands_get_input_enabled_string);
    menu_add_edit_function(item, (void*)app_commands_edit_input_enabled, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, NULL);
    menu_add_child(input_enabled_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_INPUT_ENABLED_STB);
    menu_initialise_item(item, APP_COMMAND_MENU_INPUT_ENABLED_STB, true);
    menu_add_display_function(item, (void*)app_commands_get_input_enabled_string);
    menu_add_edit_function(item, (void*)app_commands_edit_input_enabled, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, NULL);
    menu_add_child(input_enabled_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_INPUT_ENABLED_AV);
    menu_initialise_item(item, APP_COMMAND_MENU_INPUT_ENABLED_AV, true);
    menu_add_display_function(item, (void*)app_commands_get_input_enabled_string);
    menu_add_edit_function(item, (void*)app_commands_edit_input_enabled, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, NULL);
    menu_add_child(input_enabled_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_INPUT_ENABLED_BD);
    menu_initialise_item(item, APP_COMMAND_MENU_INPUT_ENABLED_BD, true);
    menu_add_display_function(item, (void*)app_commands_get_input_enabled_string);
    menu_add_edit_function(item, (void*)app_commands_edit_input_enabled, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, NULL);
    menu_add_child(input_enabled_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_INPUT_ENABLED_SAT);
    menu_initialise_item(item, APP_COMMAND_MENU_INPUT_ENABLED_SAT, true);
    menu_add_display_function(item, (void*)app_commands_get_input_enabled_string);
    menu_add_edit_function(item, (void*)app_commands_edit_input_enabled, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, NULL);
    menu_add_child(input_enabled_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_INPUT_ENABLED_GAME);
    menu_initialise_item(item, APP_COMMAND_MENU_INPUT_ENABLED_GAME, true);
    menu_add_display_function(item, (void*)app_commands_get_input_enabled_string);
    menu_add_edit_function(item, (void*)app_commands_edit_input_enabled, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, NULL);
    menu_add_child(input_enabled_item, item, false);

    item = menu_get_item(APP_COMMAND_MENU_INPUT_ENABLED_NET);
    menu_initialise_item(item, APP_COMMAND_MENU_INPUT_ENABLED_NET, true);
    menu_add_display_function(item, (void*)app_commands_get_input_enabled_string);
    menu_add_edit_function(item, (void*)app_commands_edit_input_enabled, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, NULL);
    menu_add_child(input_enabled_item, item, false);

    item = menu_get_item(APP_COMMAND_MENU_INPUT_ENABLED_ARC);
    menu_initialise_item(item, APP_COMMAND_MENU_INPUT_ENABLED_ARC, true);
    menu_add_display_function(item, (void*)app_commands_get_input_enabled_string);
    menu_add_edit_function(item, (void*)app_commands_edit_input_enabled, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, NULL);
    menu_add_child(input_enabled_item, item, false);

    item = menu_get_item(APP_COMMAND_MENU_AUTO_STANDBY_TIMEOUT);
    menu_initialise_item(item, APP_COMMAND_MENU_AUTO_STANDBY_TIMEOUT, true);
    menu_add_display_function(item, (void*)app_commands_get_auto_shutdown_timeout_string);
    menu_add_edit_function(item, (void*)app_commands_edit_auto_shutdown_timeout, true, false);
    menu_add_child(system_settings_item, item, false);
  
    item = menu_get_item(APP_COMMAND_MENU_SYSTEM_CODE);
    menu_initialise_item(item, APP_COMMAND_MENU_SYSTEM_CODE, true);
    menu_add_display_function(item, (void*)remote_get_system_code_string);
    menu_add_edit_function(item, (void*)app_commands_edit_system_code, true, false);
    menu_add_child(system_settings_item, item, false);
     
//     
//    item = menu_get_item(APP_COMMAND_MENU_SERIAL_NUMBER);
//    menu_initialise_item(item, APP_COMMAND_MENU_SERIAL_NUMBER, true);
////    menu_add_display_function(item, (void*)avatar_message_get_serial_number_string);
//    menu_add_draw_function(item, (void*)app_commands_avatar_draw_serial_number);
//    menu_add_child(system_settings_item, item, false);
     
    MENU_ITEM* avatar_reset_item  = menu_get_item(APP_COMMAND_MENU_SYSTEM_INFO_AVATAR_FACTORY_RESET);
    menu_initialise_item(avatar_reset_item, APP_COMMAND_MENU_SYSTEM_INFO_AVATAR_FACTORY_RESET, true);
    menu_add_child(system_settings_item, avatar_reset_item, false);
    
    MENU_ITEM* avatar_reset_confirm_item  = menu_get_item(APP_COMMAND_MENU_SYSTEM_INFO_AVATAR_FACTORY_RESET_CONFIRM);
    menu_initialise_item(avatar_reset_confirm_item, APP_COMMAND_MENU_SYSTEM_INFO_AVATAR_FACTORY_RESET_CONFIRM, true);
    menu_add_process_function(avatar_reset_confirm_item, (void*)app_commands_avatar_factory_reset);
    menu_add_child(avatar_reset_item, avatar_reset_confirm_item, true);
    
    // Display a resetting display - not available from the menu 
    item  = menu_get_item(APP_COMMAND_MENU_SYSTEM_INFO_AVATAR_FACTORY_RESETTING);
    menu_initialise_item(item, APP_COMMAND_MENU_SYSTEM_INFO_AVATAR_FACTORY_RESETTING, true);
    menu_add_process_function(avatar_reset_confirm_item, (void*)app_commands_avatar_factory_reset);
    menu_add_draw_function(item, (void*)app_commands_avatar_draw_reset);
    
//    MENU_ITEM* system_update_item = menu_get_item(APP_COMMAND_MENU_SYSTEM_UPDATE);
//    menu_initialise_item(system_update_item, APP_COMMAND_MENU_SYSTEM_UPDATE, true);
//    menu_add_child(system_settings_item, system_update_item, false);
        
//    item  = menu_get_item(APP_COMMAND_MENU_SYSTEM_UPDATE_CONFIRM);
//    menu_initialise_item(item, APP_COMMAND_MENU_SYSTEM_UPDATE_CONFIRM, true);
//    menu_add_process_function(item, (void*)app_commands_process_system_update);
//    menu_add_child(system_update_item, item, true);

    MENU_ITEM* system_reset_item = menu_get_item(APP_COMMAND_MENU_SYSTEM_RESET);
    menu_initialise_item(system_reset_item, APP_COMMAND_MENU_SYSTEM_RESET, true);
    menu_add_child(system_settings_item, system_reset_item, false);
    
    MENU_ITEM* system_reset_confirm_item  = menu_get_item(APP_COMMAND_MENU_SYSTEM_RESET_CONFIRM);
    menu_initialise_item(system_reset_confirm_item, APP_COMMAND_MENU_SYSTEM_RESET_CONFIRM, true);
    menu_add_process_function(system_reset_confirm_item, (void*)app_commands_process_factory_reset);
    menu_add_child(system_reset_item, system_reset_confirm_item, true);

    // Display a resetting display - not available from the menu 
    item  = menu_get_item(APP_COMMAND_MENU_SYSTEM_RESET_REBOOT);
    menu_initialise_item(item, APP_COMMAND_MENU_SYSTEM_RESET_REBOOT, true);
    menu_add_draw_function(item, (void*)app_commands_draw_reboot);
    
    //TODO USB Output
//    item = menu_get_item(APP_COMMAND_MENU_USB_IN_OUT);
//    menu_initialise_item(item, APP_COMMAND_MENU_USB_IN_OUT, true);
//    menu_add_display_function(item, (void*)app_commands_get_usb_in_out_string);
//    menu_add_edit_function(item, (void*)app_commands_edit_usb_in_out, true, false);
//    menu_add_child(input_enabled_item, item, false);
     
    item = menu_get_item(APP_COMMAND_MENU_AVATAR_UPGRADE_OPTION);
    menu_initialise_item(item, APP_COMMAND_MENU_AVATAR_UPGRADE_OPTION, true);
    menu_add_display_function(item, (void*)avatar_message_get_upgrade_option_string);
    menu_add_edit_function(item, (void*)app_commands_edit_upgrade_option, true, false);
    menu_add_child(system_settings_item, item, false);
//
//    MENU_ITEM* avatar_upgrade_item  = menu_get_item(APP_COMMAND_MENU_AVATAR_UPGRADE);
//    menu_initialise_item(avatar_upgrade_item, APP_COMMAND_MENU_AVATAR_UPGRADE, true);
//    menu_add_draw_function(avatar_upgrade_item, (void*)app_commands_avatar_draw_upgrade_available);
//    menu_add_child(system_settings_item, avatar_upgrade_item, false);
//    
//    MENU_ITEM* avatar_upgrade_confirm_item  = menu_get_item(APP_COMMAND_MENU_AVATAR_UPGRADE_CONFIRM);
//    menu_initialise_item(avatar_upgrade_confirm_item, APP_COMMAND_MENU_AVATAR_UPGRADE_CONFIRM, true);
//    menu_add_enter_exit_functions(avatar_upgrade_confirm_item, (void*)app_commands_enter_avatar_upgrade_continue_menu, NULL);
//    menu_add_process_function(avatar_upgrade_confirm_item, (void*)app_commands_avatar_upgrade);
//    menu_add_child(avatar_upgrade_item, avatar_upgrade_confirm_item, true);

    MENU_ITEM* usb_upgrade_item = menu_get_item(APP_COMMAND_MENU_USB_UPGRADE);
    menu_initialise_item(usb_upgrade_item, APP_COMMAND_MENU_USB_UPGRADE, true);
    menu_add_child(system_settings_item, usb_upgrade_item, false);
    
    MENU_ITEM* usb_upgrade_confirm_item  = menu_get_item(APP_COMMAND_MENU_USB_UPGRADE_CONFIRM);
    menu_initialise_item(usb_upgrade_confirm_item, APP_COMMAND_MENU_USB_UPGRADE_CONFIRM, true);
    menu_add_process_function(usb_upgrade_confirm_item, (void*)app_commands_avatar_usb_upgrade);
    menu_add_child(usb_upgrade_item, usb_upgrade_confirm_item, true);

    item = menu_get_item(APP_COMMAND_MENU_AVATAR_UPGRADE_IN_PROGRESS);
    menu_initialise_item(item, APP_COMMAND_MENU_AVATAR_UPGRADE_IN_PROGRESS, true);
    menu_add_draw_function(item, (void*)app_commands_avatar_draw_upgrade);
        
    item = menu_get_item(APP_COMMAND_MENU_AVATAR_UPGRADE_COMPLETE);
    menu_initialise_item(item, APP_COMMAND_MENU_AVATAR_UPGRADE_COMPLETE, true);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_avatar_upgrade_menu, (void*)app_commands_menu_reset_upgrade_complete);
    menu_add_draw_function(item, (void*)app_commands_avatar_draw_upgrade_complete);
        
    item = menu_get_item(APP_COMMAND_MENU_AVATAR_UPGRADE_FAIL);
    menu_initialise_item(item, APP_COMMAND_MENU_AVATAR_UPGRADE_FAIL, true);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_avatar_upgrade_menu, (void*)menu_app_reset_inactivity_timeout);
    menu_add_draw_function(item, (void*)app_commands_avatar_draw_upgrade_complete);

    // Setup INFO_SETTINGS MENU------------------------------------------------
    //--------------------------------------------------------------------------

    MENU_ITEM* info_settings_item = menu_get_item(APP_COMMAND_MENU_SYSTEM_INFO);
    menu_initialise_item(info_settings_item, APP_COMMAND_MENU_SYSTEM_INFO, true);
    menu_add_child(NULL, info_settings_item, true);
    
    item = menu_get_item(APP_COMMAND_MENU_SYSTEM_INFO_HOST_VERSION);
    menu_initialise_item(item, APP_COMMAND_MENU_SYSTEM_INFO_HOST_VERSION, true);
    menu_add_display_function(item, (void*)app_commands_get_software_version);
    menu_add_child(info_settings_item, item, false);
     
    item = menu_get_item(APP_COMMAND_MENU_SYSTEM_INFO_AVATAR_VERSION);
    menu_initialise_item(item, APP_COMMAND_MENU_SYSTEM_INFO_AVATAR_VERSION, true);
    menu_add_display_function(item, (void*)app_commands_get_avatar_firmware_version_string);
    menu_add_child(info_settings_item, item, false);
     
#if APP_FEATURE_HDMI_ARC == APP_FEATURE_ON  
    item = menu_get_item(APP_COMMAND_MENU_SYSTEM_INFO_HDMI_ARC_VERSION);
    menu_initialise_item(item, APP_COMMAND_MENU_SYSTEM_INFO_HDMI_ARC_VERSION, true);
    menu_add_display_function(item, (void*)hdmi_arc_get_firmware_version_string);
    menu_add_child(info_settings_item, item, false);
     
    item = menu_get_item(APP_COMMAND_MENU_SYSTEM_INFO_HDMI_ARC_RX_VERSION);
    menu_initialise_item(item, APP_COMMAND_MENU_SYSTEM_INFO_HDMI_ARC_RX_VERSION, true);
    menu_add_display_function(item, (void*)hdmi_arc_get_rx_firmware_version_string);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_hdmi_arc_rx_version_menu, NULL);
    menu_add_child(info_settings_item, item, false);
#endif
    
    item = menu_get_item(APP_COMMAND_MENU_SYSTEM_INFO_NETWORK_IP_ADDRESS);
    menu_initialise_item(item, APP_COMMAND_MENU_SYSTEM_INFO_NETWORK_IP_ADDRESS, false);
    menu_add_display_function(item, (void*)tcpip_get_ip_address_string); 
    menu_add_child(info_settings_item, item, false);
   
    item = menu_get_item(APP_COMMAND_MENU_SERIAL_NUMBER);
    menu_initialise_item(item, APP_COMMAND_MENU_SERIAL_NUMBER, true);
//    menu_add_display_function(item, (void*)avatar_message_get_serial_number_string);
    menu_add_draw_function(item, (void*)app_commands_avatar_draw_serial_number);
    menu_add_child(info_settings_item, item, false);
     
    item = menu_get_item(APP_COMMAND_MENU_SYSTEM_INFO_NETWORK_NAME_INFO);
    menu_initialise_item(item, APP_COMMAND_MENU_SYSTEM_INFO_NETWORK_NAME_INFO, false);
    menu_add_display_function(item, (void*)tcpip_get_network_name_string); 
    menu_add_child(info_settings_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_SYSTEM_INFO_NETWORK_FRIENDLY_NAME_INFO);
    menu_initialise_item(item, APP_COMMAND_MENU_SYSTEM_INFO_NETWORK_FRIENDLY_NAME_INFO, false);
    menu_add_display_function(item, (void*)avatar_message_get_friendly_name_string); 
    menu_add_child(info_settings_item, item, false);

    // Setup AUDIO_SETTINGS MENU------------------------------------------------
    //--------------------------------------------------------------------------
    MENU_ITEM* audio_settings_item = menu_get_item(APP_COMMAND_MENU_AUDIO_SETTINGS);
    menu_initialise_item(audio_settings_item, APP_COMMAND_MENU_AUDIO_SETTINGS, true);
    menu_add_child(NULL, audio_settings_item, false);
    
    // Dirac On/Off
    // Version of the Dirac menu without navigation keys used when editing via buttons, remote or RS232/TCPIP
    item = menu_get_item(APP_COMMAND_MENU_DIRAC_NO_NAVIGATION);
    menu_initialise_item(item, APP_COMMAND_MENU_DIRAC_NO_NAVIGATION, true);
    menu_add_draw_function(item, (void*)app_commands_draw_dirac_no_navigation);
    menu_add_edit_function(item, (void*)app_commands_edit_dirac_selected, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_dirac_no_navigation, (void*)app_commands_edit_exit_dirac_curve);  
        
    // Dirac Calibration menu
    item = menu_get_item(APP_COMMAND_MENU_DIRAC_CALIBRATION);
    menu_initialise_item(item, APP_COMMAND_MENU_DIRAC_CALIBRATION, true);
    menu_add_draw_function(item, (void*)app_commands_draw_dirac_calibration);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_dirac_calibration, (void*)menu_app_reset_inactivity_timeout);  

    // Dirac Curve
    MENU_ITEM* dirac_curve_item = menu_get_item(APP_COMMAND_MENU_DIRAC_CURVE);
    menu_initialise_item(dirac_curve_item, APP_COMMAND_MENU_DIRAC_CURVE, false);
    menu_add_child(audio_settings_item, dirac_curve_item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_DIRAC_CURVE_PHONO);
    menu_initialise_item(item, APP_COMMAND_MENU_DIRAC_CURVE_PHONO, true);
    menu_add_display_function(item, (void*)app_commands_get_dirac_selected_string);
    menu_add_edit_function(item, (void*)app_commands_edit_dirac_selected, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, app_commands_edit_exit_dirac_curve);
    menu_add_child(dirac_curve_item, item, true);
    
    item = menu_get_item(APP_COMMAND_MENU_DIRAC_CURVE_AUX);
    menu_initialise_item(item, APP_COMMAND_MENU_DIRAC_CURVE_AUX, true);
    menu_add_display_function(item, (void*)app_commands_get_dirac_selected_string);
    menu_add_edit_function(item, (void*)app_commands_edit_dirac_selected, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, app_commands_edit_exit_dirac_curve);
    menu_add_child(dirac_curve_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_DIRAC_CURVE_CD);
    menu_initialise_item(item, APP_COMMAND_MENU_DIRAC_CURVE_CD, true);
    menu_add_display_function(item, (void*)app_commands_get_dirac_selected_string);
    menu_add_edit_function(item, (void*)app_commands_edit_dirac_selected, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, app_commands_edit_exit_dirac_curve);
    menu_add_child(dirac_curve_item, item, false);
     
    item = menu_get_item(APP_COMMAND_MENU_DIRAC_CURVE_PVR);
    menu_initialise_item(item, APP_COMMAND_MENU_DIRAC_CURVE_PVR, true);
    menu_add_display_function(item, (void*)app_commands_get_dirac_selected_string);
    menu_add_edit_function(item, (void*)app_commands_edit_dirac_selected, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, app_commands_edit_exit_dirac_curve);
    menu_add_child(dirac_curve_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_DIRAC_CURVE_STB);
    menu_initialise_item(item, APP_COMMAND_MENU_DIRAC_CURVE_STB, true);
    menu_add_display_function(item, (void*)app_commands_get_dirac_selected_string);
    menu_add_edit_function(item, (void*)app_commands_edit_dirac_selected, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, app_commands_edit_exit_dirac_curve);
    menu_add_child(dirac_curve_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_DIRAC_CURVE_AV);
    menu_initialise_item(item, APP_COMMAND_MENU_DIRAC_CURVE_AV, true);
    menu_add_display_function(item, (void*)app_commands_get_dirac_selected_string);
    menu_add_edit_function(item, (void*)app_commands_edit_dirac_selected, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, app_commands_edit_exit_dirac_curve);
    menu_add_child(dirac_curve_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_DIRAC_CURVE_BD);
    menu_initialise_item(item, APP_COMMAND_MENU_DIRAC_CURVE_BD, true);
    menu_add_display_function(item, (void*)app_commands_get_dirac_selected_string);
    menu_add_edit_function(item, (void*)app_commands_edit_dirac_selected, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, app_commands_edit_exit_dirac_curve);
    menu_add_child(dirac_curve_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_DIRAC_CURVE_SAT);
    menu_initialise_item(item, APP_COMMAND_MENU_DIRAC_CURVE_SAT, true);
    menu_add_display_function(item, (void*)app_commands_get_dirac_selected_string);
    menu_add_edit_function(item, (void*)app_commands_edit_dirac_selected, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, app_commands_edit_exit_dirac_curve);
    menu_add_child(dirac_curve_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_DIRAC_CURVE_GAME);
    menu_initialise_item(item, APP_COMMAND_MENU_DIRAC_CURVE_GAME, true);
    menu_add_display_function(item, (void*)app_commands_get_dirac_selected_string);
    menu_add_edit_function(item, (void*)app_commands_edit_dirac_selected, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, app_commands_edit_exit_dirac_curve);
    menu_add_child(dirac_curve_item, item, false);
    
//TODO REMOVE    item = menu_get_item(APP_COMMAND_MENU_DIRAC_CURVE_PCUSB);
//    menu_initialise_item(item, APP_COMMAND_MENU_DIRAC_CURVE_PCUSB, true);
//    menu_add_display_function(item, (void*)app_commands_get_dirac_selected_string);
//    menu_add_edit_function(item, (void*)app_commands_edit_dirac_selected, true, false);
//    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, app_commands_edit_exit_dirac_curve);
//    menu_add_child(dirac_curve_item, item, false);

    item = menu_get_item(APP_COMMAND_MENU_DIRAC_CURVE_NET);
    menu_initialise_item(item, APP_COMMAND_MENU_DIRAC_CURVE_NET, true);
    menu_add_display_function(item, (void*)app_commands_get_dirac_selected_string);
    menu_add_edit_function(item, (void*)app_commands_edit_dirac_selected, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, app_commands_edit_exit_dirac_curve);
    menu_add_child(dirac_curve_item, item, false);
    
//TODO REMOVE    item = menu_get_item(APP_COMMAND_MENU_DIRAC_CURVE_USB);
//    menu_initialise_item(item, APP_COMMAND_MENU_DIRAC_CURVE_USB, true);
//    menu_add_display_function(item, (void*)app_commands_get_dirac_selected_string);
//    menu_add_edit_function(item, (void*)app_commands_edit_dirac_selected, true, false);
//    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, app_commands_edit_exit_dirac_curve);
//    menu_add_child(dirac_curve_item, item, false);
 
    // Balance    
    item = menu_get_item(APP_COMMAND_MENU_BALANCE);
    menu_initialise_item(item, APP_COMMAND_MENU_BALANCE, true);
    //menu_add_display_function(item, (void*)volume_get_balance_string);
    menu_add_draw_function(item, (void*)app_commands_draw_balance);
    menu_add_edit_function(item, (void*)app_commands_edit_balance, true, false);
    menu_add_enter_exit_functions(item, NULL, (void*)app_commands_exit_balance);
    menu_add_child(audio_settings_item, item, false);
    
    // Version of the balance menu without navigation keys used when editing via remote or RS232/TCPIP
    item = menu_get_item(APP_COMMAND_MENU_BALANCE_NO_NAVIGATION);
    menu_initialise_item(item, APP_COMMAND_MENU_BALANCE_NO_NAVIGATION, true);
    menu_add_draw_function(item, (void*)app_commands_draw_balance);
    menu_add_edit_function(item, (void*)app_commands_edit_balance, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_balance_no_navigation, (void*)app_commands_exit_balance);
    
    item = menu_get_item(APP_COMMAND_MENU_HEADPHONE_OVERRIDE);
    menu_initialise_item(item, APP_COMMAND_MENU_HEADPHONE_OVERRIDE, true);
    menu_add_display_function(item, (void*)app_commands_get_headphone_override_string);
    menu_add_edit_function(item, (void*)app_commands_edit_headphone_override, true, false);
    menu_add_child(audio_settings_item, item, false);
  
    item = menu_get_item(APP_COMMAND_MENU_PROCESSOR_MODE_INPUT);
    menu_initialise_item(item, APP_COMMAND_MENU_PROCESSOR_MODE_INPUT, true);
    menu_add_display_function(item, (void*)app_commands_get_processor_mode_input_string);
    menu_add_edit_function(item, (void*)app_commands_edit_processor_mode_input, true, false);
    menu_add_child(audio_settings_item, item, false);
   
    item = menu_get_item(APP_COMMAND_MENU_PROCESSOR_MODE_VOLUME);
    menu_initialise_item(item, APP_COMMAND_MENU_PROCESSOR_MODE_VOLUME, true);
    menu_add_display_function(item, (void*)app_commands_get_processor_mode_volume_string);
    menu_add_edit_function(item, (void*)app_commands_edit_processor_mode_volume, true, false);
    menu_add_child(audio_settings_item, item, false);   
    
    item = menu_get_item(APP_COMMAND_MENU_MAX_ON_VOLUME);
    menu_initialise_item(item, APP_COMMAND_MENU_MAX_ON_VOLUME, true);
    menu_add_display_function(item, (void*)app_commands_get_max_on_volume_string);
    menu_add_edit_function(item, (void*)app_commands_edit_max_on_volume, true, false);
    menu_add_child(audio_settings_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_MAXIMUM_VOLUME);
    menu_initialise_item(item, APP_COMMAND_MENU_MAXIMUM_VOLUME, true);
    menu_add_display_function(item, (void*)app_commands_get_maximum_volume_string);
    menu_add_edit_function(item, (void*)app_commands_edit_maximum_volume, true, false);
    menu_add_child(audio_settings_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_MAXIMUM_STREAMING_VOLUME);
    menu_initialise_item(item, APP_COMMAND_MENU_MAXIMUM_STREAMING_VOLUME, true);
    menu_add_display_function(item, (void*)app_commands_get_maximum_streaming_volume_string);
    menu_add_edit_function(item, (void*)app_commands_edit_maximum_streaming_volume, true, false);
    menu_add_child(audio_settings_item, item, false);
    
#if APP_FEATURE_HDMI_ARC == APP_FEATURE_ON  
    item = menu_get_item(APP_COMMAND_MENU_HDMI_AUDIO_CONTROL);
    menu_initialise_item(item, APP_COMMAND_MENU_HDMI_AUDIO_CONTROL, true);
    menu_add_display_function(item, (void*)app_commands_hdmi_arc_get_hdmi_audio_control_string);
    menu_add_edit_function(item, (void*)app_commands_hdmi_arc_edit_audio_control, true, false);
    menu_add_child(audio_settings_item, item, false);
#endif
 
#if APP_FEATURE_HDMI_ARC == APP_FEATURE_ON  
    item = menu_get_item(APP_COMMAND_MENU_HDMI_POWER_CONTROL);
    menu_initialise_item(item, APP_COMMAND_MENU_HDMI_POWER_CONTROL, true);
    menu_add_display_function(item, (void*)app_commands_hdmi_arc_get_hdmi_power_control_string);
    menu_add_edit_function(item, (void*)app_commands_hdmi_arc_edit_power_control, true, false);
    menu_add_child(audio_settings_item, item, false);
#endif

    //-----------------------------------------------------------------------------------------
    // Lip Sync
    MENU_ITEM* lip_sync_item = menu_get_item(APP_COMMAND_MENU_LIP_SYNC);
    menu_initialise_item(lip_sync_item, APP_COMMAND_MENU_LIP_SYNC, false);
    menu_add_child(audio_settings_item, lip_sync_item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_LIP_SYNC_PHONO);
    menu_initialise_item(item, APP_COMMAND_MENU_LIP_SYNC_PHONO, true);
    menu_add_display_function(item, (void*)app_commands_get_lip_sync_string);
    menu_add_edit_function(item, (void*)app_commands_edit_lip_sync, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_lip_sync_menu, (void*)app_commands_check_lip_sync);
    menu_add_child(lip_sync_item, item, true);
    
    item = menu_get_item(APP_COMMAND_MENU_LIP_SYNC_AUX);
    menu_initialise_item(item, APP_COMMAND_MENU_LIP_SYNC_AUX, true);
    menu_add_display_function(item, (void*)app_commands_get_lip_sync_string);
    menu_add_edit_function(item, (void*)app_commands_edit_lip_sync, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_lip_sync_menu, (void*)app_commands_check_lip_sync);
    menu_add_child(lip_sync_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_LIP_SYNC_CD);
    menu_initialise_item(item, APP_COMMAND_MENU_LIP_SYNC_CD, true);
    menu_add_display_function(item, (void*)app_commands_get_lip_sync_string);
    menu_add_edit_function(item, (void*)app_commands_edit_lip_sync, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_lip_sync_menu, (void*)app_commands_check_lip_sync);
    menu_add_child(lip_sync_item, item, false);
     
    item = menu_get_item(APP_COMMAND_MENU_LIP_SYNC_PVR);
    menu_initialise_item(item, APP_COMMAND_MENU_LIP_SYNC_PVR, true);
    menu_add_display_function(item, (void*)app_commands_get_lip_sync_string);
    menu_add_edit_function(item, (void*)app_commands_edit_lip_sync, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_lip_sync_menu, (void*)app_commands_check_lip_sync);
    menu_add_child(lip_sync_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_LIP_SYNC_STB);
    menu_initialise_item(item, APP_COMMAND_MENU_LIP_SYNC_STB, true);
    menu_add_display_function(item, (void*)app_commands_get_lip_sync_string);
    menu_add_edit_function(item, (void*)app_commands_edit_lip_sync, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_lip_sync_menu, (void*)app_commands_check_lip_sync);
    menu_add_child(lip_sync_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_LIP_SYNC_AV);
    menu_initialise_item(item, APP_COMMAND_MENU_LIP_SYNC_AV, true);
    menu_add_display_function(item, (void*)app_commands_get_lip_sync_string);
    menu_add_edit_function(item, (void*)app_commands_edit_lip_sync, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_lip_sync_menu, (void*)app_commands_check_lip_sync);
    menu_add_child(lip_sync_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_LIP_SYNC_BD);
    menu_initialise_item(item, APP_COMMAND_MENU_LIP_SYNC_BD, true);
    menu_add_display_function(item, (void*)app_commands_get_lip_sync_string);
    menu_add_edit_function(item, (void*)app_commands_edit_lip_sync, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_lip_sync_menu, (void*)app_commands_check_lip_sync);
    menu_add_child(lip_sync_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_LIP_SYNC_SAT);
    menu_initialise_item(item, APP_COMMAND_MENU_LIP_SYNC_SAT, true);
    menu_add_display_function(item, (void*)app_commands_get_lip_sync_string);
    menu_add_edit_function(item, (void*)app_commands_edit_lip_sync, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_lip_sync_menu, (void*)app_commands_check_lip_sync);
    menu_add_child(lip_sync_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_LIP_SYNC_GAME);
    menu_initialise_item(item, APP_COMMAND_MENU_LIP_SYNC_GAME, true);
    menu_add_display_function(item, (void*)app_commands_get_lip_sync_string);
    menu_add_edit_function(item, (void*)app_commands_edit_lip_sync, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_lip_sync_menu, (void*)app_commands_check_lip_sync);
    menu_add_child(lip_sync_item, item, false);

    item = menu_get_item(APP_COMMAND_MENU_LIP_SYNC_NET);
    menu_initialise_item(item, APP_COMMAND_MENU_LIP_SYNC_NET, true);
    menu_add_display_function(item, (void*)app_commands_get_lip_sync_string);
    menu_add_edit_function(item, (void*)app_commands_edit_lip_sync, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_lip_sync_menu, (void*)app_commands_check_lip_sync);
    menu_add_child(lip_sync_item, item, false);

    item = menu_get_item(APP_COMMAND_MENU_LIP_SYNC_ARC);
    menu_initialise_item(item, APP_COMMAND_MENU_LIP_SYNC_ARC, true);
    menu_add_display_function(item, (void*)app_commands_get_lip_sync_string);
    menu_add_edit_function(item, (void*)app_commands_edit_lip_sync, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_lip_sync_menu, (void*)app_commands_check_lip_sync);
    menu_add_child(lip_sync_item, item, false);
    
    //------------------------------------------------------------------------------------------

    // Analog Direct
    MENU_ITEM* analogue_direct_item = menu_get_item(APP_COMMAND_MENU_ANALOGUE_DIRECT);
    menu_initialise_item(analogue_direct_item, APP_COMMAND_MENU_ANALOGUE_DIRECT, false);
    menu_add_child(audio_settings_item, analogue_direct_item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_ANALOGUE_DIRECT_PHONO);
    menu_initialise_item(item, APP_COMMAND_MENU_ANALOGUE_DIRECT_PHONO, true);
    menu_add_display_function(item, (void*)input_analogue_direct_get_string);
    menu_add_edit_function(item, (void*)app_commands_edit_analogue_direct, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, (void*)input_analogue_direct_check);
    menu_add_child(analogue_direct_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_ANALOGUE_DIRECT_AUX);
    menu_initialise_item(item, APP_COMMAND_MENU_ANALOGUE_DIRECT_AUX, true);
    menu_add_display_function(item, (void*)input_analogue_direct_get_string);
    menu_add_edit_function(item, (void*)app_commands_edit_analogue_direct, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, (void*)input_analogue_direct_check);
    menu_add_child(analogue_direct_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_ANALOGUE_DIRECT_CD);
    menu_initialise_item(item, APP_COMMAND_MENU_ANALOGUE_DIRECT_CD, true);
    menu_add_display_function(item, (void*)input_analogue_direct_get_string);
    menu_add_edit_function(item, (void*)app_commands_edit_analogue_direct, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, (void*)input_analogue_direct_check);
    menu_add_child(analogue_direct_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_ANALOGUE_DIRECT_PVR);
    menu_initialise_item(item, APP_COMMAND_MENU_ANALOGUE_DIRECT_PVR, true);
    menu_add_display_function(item, (void*)input_analogue_direct_get_string);
    menu_add_edit_function(item, (void*)app_commands_edit_analogue_direct, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, (void*)input_analogue_direct_check);
    menu_add_child(analogue_direct_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_ANALOGUE_DIRECT_STB);
    menu_initialise_item(item, APP_COMMAND_MENU_ANALOGUE_DIRECT_STB, true);
    menu_add_display_function(item, (void*)input_analogue_direct_get_string);
    menu_add_edit_function(item, (void*)app_commands_edit_analogue_direct, true, false);
    menu_add_enter_exit_functions(item, (void*)app_commands_set_input_for_edit, (void*)input_analogue_direct_check);
    menu_add_child(analogue_direct_item, item, false);
    
    // Version of the Analog Direct menu without navigation keys used when editing via buttons, remote or RS232/TCPIP
    item = menu_get_item(APP_COMMAND_MENU_ANALOGUE_DIRECT_NO_NAVIGATION);
    menu_initialise_item(item, APP_COMMAND_MENU_ANALOGUE_DIRECT_NO_NAVIGATION, true);
    menu_add_draw_function(item, (void*)app_commands_draw_analogue_direct_no_navigation);
    menu_add_edit_function(item, (void*)app_commands_edit_analogue_direct, true, false);
    menu_add_enter_exit_functions(item, (void*)input_analogue_direct_enter_no_navigation, (void*)input_analogue_direct_exit_no_navigation);

    // Phono Input Type MM/MC
    item = menu_get_item(APP_COMMAND_MENU_PHONO_INPUT);
    menu_initialise_item(item, APP_COMMAND_MENU_PHONO_INPUT, true);
    menu_add_display_function(item, (void*)input_phono_input_get_string);
    menu_add_edit_function(item, (void*)app_commands_edit_mm_mc, true, false);
    menu_add_child(audio_settings_item, item, false);
   
    // Version of the Phono Input menu without navigation keys used when editing via buttons, remote or RS232/TCPIP
    item = menu_get_item(APP_COMMAND_MENU_PHONO_INPUT_NO_NAVIGATION);
    menu_initialise_item(item, APP_COMMAND_MENU_PHONO_INPUT_NO_NAVIGATION, true);
    menu_add_display_function(item, (void*)input_phono_input_get_string);
    menu_add_edit_function(item, (void*)app_commands_edit_mm_mc, true, false);
    menu_add_enter_exit_functions(item, (void*)input_phono_input_enter_no_navigation, (void*)input_phono_input_exit_no_navigation);
   
    item = menu_get_item(APP_COMMAND_MENU_FILTER);
    menu_initialise_item(item, APP_COMMAND_MENU_FILTER, true);
    menu_add_display_function(item, (void*)input_control_get_filter_string);
    menu_add_edit_function(item, (void*)app_commands_edit_filter, true, false);
    menu_add_child(audio_settings_item, item, false);

//REMOVED 20190906 - Petes email 15:26    
//    item = menu_get_item(APP_COMMAND_MENU_USB_RECORD);
//    menu_initialise_item(item, APP_COMMAND_MENU_USB_RECORD, true);
//    menu_add_display_function(item, (void*)app_commands_get_usb_record_string);
//    menu_add_edit_function(item, (void*)app_commands_edit_usb_record, true, false);
//    menu_add_child(audio_settings_item, item, false);

    // Setup Network Menu-------------------------------------------------------
    //--------------------------------------------------------------------------
    MENU_ITEM* network_settings_item = menu_get_item(APP_COMMAND_MENU_NETWORK_SETTINGS);
    menu_initialise_item(network_settings_item, APP_COMMAND_MENU_NETWORK_SETTINGS, true);
    menu_add_child(NULL, network_settings_item, false);
    
    MENU_ITEM* network_info_item = menu_get_item(APP_COMMAND_MENU_NETWORK_INFO);
    menu_initialise_item(network_info_item, APP_COMMAND_MENU_NETWORK_INFO, false);
    menu_add_child(network_settings_item, network_info_item, true);
    
//TODO    item = menu_get_item(APP_COMMAND_MENU_NETWORK_DHCP_INFO);
//    menu_initialise_item(item, APP_COMMAND_MENU_NETWORK_DHCP_INFO, true);
//    menu_add_display_function(item, (void*)app_commands_get_network_dhcp_string); 
//    menu_add_child(network_info_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_NETWORK_IP_ADDRESS_INFO);
    menu_initialise_item(item, APP_COMMAND_MENU_NETWORK_IP_ADDRESS_INFO, false);
    menu_add_display_function(item, (void*)tcpip_get_ip_address_string); 
    menu_add_child(network_info_item, item, false);
   
    item = menu_get_item(APP_COMMAND_MENU_NETWORK_INFO_MAC_ADDRESS);
    menu_initialise_item(item, APP_COMMAND_MENU_NETWORK_INFO_MAC_ADDRESS, true);
    menu_add_display_function(item, (void*)tcpip_get_mac_address_string);
    menu_add_child(network_info_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_NETWORK_NAME_INFO);
    menu_initialise_item(item, APP_COMMAND_MENU_NETWORK_NAME_INFO, false);
    menu_add_display_function(item, (void*)tcpip_get_network_name_string); 
    menu_add_child(network_info_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_NETWORK_FRIENDLY_NAME_INFO);
    menu_initialise_item(item, APP_COMMAND_MENU_NETWORK_FRIENDLY_NAME_INFO, false);
    menu_add_display_function(item, (void*)avatar_message_get_friendly_name_string); 
    menu_add_child(network_info_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_SSID);
    menu_initialise_item(item, APP_COMMAND_MENU_SSID, false);
    menu_add_display_function(item, (void*)tcpip_get_ssid_string); 
    menu_add_child(network_info_item, item, false);  
    
    item = menu_get_item(APP_COMMAND_MENU_NETWORK_STANDBY);
    menu_initialise_item(item, APP_COMMAND_MENU_NETWORK_STANDBY, true);
    menu_add_display_function(item, (void*)app_commands_get_network_standby_string);
    menu_add_edit_function(item, (void*)app_commands_edit_network_standby, true, false);
    menu_add_child(network_settings_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_RS232_STANDBY);
    menu_initialise_item(item, APP_COMMAND_MENU_RS232_STANDBY, false);
    menu_add_display_function(item, (void*)app_commands_get_rs232_standby_string);
    menu_add_edit_function(item, (void*)app_commands_edit_rs232_standby, true, false);
    menu_add_child(network_settings_item, item, false);
      
    MENU_ITEM* control4_item = menu_get_item(APP_COMMAND_MENU_CONTROL4_ACTIVE);
    menu_initialise_item(control4_item, APP_COMMAND_MENU_CONTROL4_ACTIVE, true);
    menu_add_display_function(control4_item, (void*)app_commands_get_control4_active_string);
    menu_add_edit_function(control4_item, (void*)app_commands_edit_control4_active, true, false);
    menu_add_edit_exit_function(control4_item, app_commands_edit_exit_control4_active);
    menu_add_child(network_settings_item, control4_item, false);
    
    MENU_ITEM* control4_ident_item = menu_get_item(APP_COMMAND_MENU_CONTROL4_IDENTITY);
    menu_initialise_item(control4_ident_item, APP_COMMAND_MENU_CONTROL4_IDENTITY, true);
    menu_add_enter_exit_functions(control4_ident_item, NULL, app_commands_edit_exit_control4_send_identity);
    menu_add_child(control4_item, control4_ident_item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_CONTROL4_IDENTITY_SENT);
    menu_initialise_item(item, APP_COMMAND_MENU_CONTROL4_IDENTITY_SENT, true);
    menu_add_child(control4_ident_item, item, false);
    
    //TODO REMOVE - Creatron 29/01/2020 - Crestron Config removed
    //item = menu_get_item(APP_COMMAND_MENU_CRESTRON_ACTIVE);
    //menu_initialise_item(item, APP_COMMAND_MENU_CRESTRON_ACTIVE, true);
    //menu_add_display_function(item, (void*)app_commands_get_crestron_discovery_active_string);
    //menu_add_edit_function(item, (void*)app_commands_edit_crestron_discovery_active, true, false);
    //menu_add_child(network_settings_item, item, false);

    // These 2 are displayed during the crestron configuration process
    // Note Config Complete is a child of Creastron Confirm
    //MENU_ITEM* crestron_config_item = menu_get_item(APP_COMMAND_MENU_CRESTRON_CONFIG);
    //menu_initialise_item(crestron_config_item, APP_COMMAND_MENU_CRESTRON_CONFIG, true);
    //menu_add_draw_function(crestron_config_item, (void*)app_commands_draw_crestron_discovery_config);
    //menu_add_process_function(crestron_config_item, (void*)app_commands_send_crestron_configure);
          
    //item = menu_get_item(APP_COMMAND_MENU_CRESTRON_CONFIG_COMPLETING);
    //menu_initialise_item(item, APP_COMMAND_MENU_CRESTRON_CONFIG_COMPLETING, true);
    //menu_add_draw_function(item, (void*)app_commands_draw_crestron_discovery_config);
   
    // Setup Engineering menu
    //------------------------
    MENU_ITEM* engineering_item = menu_get_item(APP_COMMAND_MENU_ENGINEERING_SETTINGS);
    menu_initialise_item(engineering_item, APP_COMMAND_MENU_ENGINEERING_SETTINGS, true);
   
    item = menu_get_item(APP_COMMAND_MENU_REGION);
    menu_initialise_item(item, APP_COMMAND_MENU_REGION, true);
    menu_add_display_function(item, (void*)app_commands_avatar_get_region_string);
    menu_add_edit_function(item, (void*)app_commands_avatar_edit_region, true, false);
    menu_add_enter_exit_functions(item, NULL, (void*)app_commands_edit_exit_region);  
    menu_add_child(engineering_item, item, true);

    MENU_ITEM* arc_upgrade_item = menu_get_item(APP_COMMAND_MENU_ARC_UPGRADE);
    menu_initialise_item(arc_upgrade_item, APP_COMMAND_MENU_ARC_UPGRADE, true);
    menu_add_child(engineering_item, arc_upgrade_item, false);
    
    MENU_ITEM* arc_upgrade_confirm_item  = menu_get_item(APP_COMMAND_MENU_ARC_UPGRADE_CONFIRM);
    menu_initialise_item(arc_upgrade_confirm_item, APP_COMMAND_MENU_ARC_UPGRADE_CONFIRM, true);
    menu_add_process_function(arc_upgrade_confirm_item, (void*)app_commands_avatar_arc_upgrade);
    menu_add_child(arc_upgrade_item, arc_upgrade_confirm_item, true);
    
    item = menu_get_item(APP_COMMAND_MENU_AMP_TEMP);
    menu_initialise_item(item, APP_COMMAND_MENU_AMP_TEMP, true);
    menu_add_display_function(item, (void*)app_commands_get_amplifier_temperature_string);
    menu_add_child(engineering_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_LIFTER_TEMP);
    menu_initialise_item(item, APP_COMMAND_MENU_LIFTER_TEMP, true);
    menu_add_display_function(item, (void*)app_commands_get_lifter_temperature_string);
    menu_add_child(engineering_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_STANDBY_TIMER);
    menu_initialise_item(item, APP_COMMAND_MENU_STANDBY_TIMER, true);
    menu_add_display_function(item, (void*)app_commands_get_auto_shutdown_timeout_remaining_string);
    menu_add_child(engineering_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_SHUTDOWN_CODE);
    menu_initialise_item(item, APP_COMMAND_MENU_SHUTDOWN_CODE, true);
    menu_add_display_function(item, (void*)app_commands_get_last_shutdown_code_string);
    menu_add_child(engineering_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_CHECK_CHANNEL_STATUS);
    menu_initialise_item(item, APP_COMMAND_MENU_CHECK_CHANNEL_STATUS, true);
    menu_add_display_function(item, (void*)app_commands_get_check_channel_status_string);
    menu_add_edit_function(item, (void*)app_commands_edit_check_channel_status, true, false);
    menu_add_child(engineering_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_IR_DEBUG);
    menu_initialise_item(item, APP_COMMAND_MENU_IR_DEBUG, true);
    menu_add_draw_function(item, (void*)app_commands_draw_ir_debug);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_ir_debug, (void*)app_commands_exit_ir_debug);
    menu_add_child(engineering_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_SWITCH_VOLTAGES);
    menu_initialise_item(item, APP_COMMAND_MENU_SWITCH_VOLTAGES, true);
    menu_add_display_function(item, (void*)app_commands_get_active_switch_voltage_string);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_switch_voltages, (void*)app_commands_exit_switch_voltages);
    menu_add_child(engineering_item, item, false);
    
    MENU_ITEM* display_test_item = menu_get_item(APP_COMMAND_MENU_DISPLAY_TEST);
    menu_initialise_item(display_test_item, APP_COMMAND_MENU_DISPLAY_TEST, true);
    menu_add_process_function(display_test_item, (void*)display_test_run);
    menu_add_child(engineering_item, display_test_item, false);
    
    // Display a display testing screen - not available from the menu 
    item  = menu_get_item(APP_COMMAND_MENU_DISPLAY_TESTING);
    menu_initialise_item(item, APP_COMMAND_MENU_DISPLAY_TESTING, true);
    menu_add_draw_function(item, (void*)display_test_draw);
    
    item = menu_get_item(APP_COMMAND_MENU_LOG_LEVEL);
    menu_initialise_item(item, APP_COMMAND_MENU_LOG_LEVEL, true);
    menu_add_display_function(item, (void*)app_commands_get_logger_level_string);
    menu_add_edit_function(item, (void*)app_commands_edit_logger_level, true, false);
    menu_add_child(engineering_item, item, false);
    
    // Bias check is not available in the menu structure as it needs for there to
    // be no activity while it it being run. so has been given its own mode available
    // via RS232 commands
    item = menu_get_item(APP_COMMAND_MENU_BIAS_CHECK);
    menu_initialise_item(item, APP_COMMAND_MENU_BIAS_CHECK, true);
    menu_add_draw_function(item, (void*)app_commands_draw_bias_check);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_bias_check, (void*)app_commands_exit_bias_check);
    menu_add_child(engineering_item, item, false);
    
    item = menu_get_item(APP_COMMAND_MENU_SHORT_CIRCUIT_SENSE);
    menu_initialise_item(item, APP_COMMAND_MENU_SHORT_CIRCUIT_SENSE, true);
    menu_add_display_function(item, (void*)app_commands_get_short_circuit_sense_string);
    menu_add_edit_function(item, (void*)app_commands_edit_short_circuit_sense, true, false);
    menu_add_child(engineering_item, item, false);
  
    item = menu_get_item(APP_COMMAND_MENU_MQA_DECODER_LEVEL);
    menu_initialise_item(item, APP_COMMAND_MENU_MQA_DECODER_LEVEL, false);
    menu_add_display_function(item, (void*)avatar_message_get_mqa_decoder_level_string);
    menu_add_edit_function(item, (void*)app_commands_edit_mqa_decoder_level, true, false);
    menu_add_child(engineering_item, item, false);
    
    // Info menu
        
    // Display a display testing screen - not available from the menu 
    item  = menu_get_item(APP_COMMAND_MENU_INFO);
    menu_initialise_item(item, APP_COMMAND_MENU_INFO, true);
    menu_add_draw_function(item, (void*)app_commands_avatar_draw_info);
    menu_add_enter_exit_functions(item, (void*)app_commands_enter_info, (void*)NULL);
}

/* *****************************************************************************
 End of File
 */
